; -------------------------------------------------------------------------------------------------------------- ;
; 1541 - Directory Block Layout
; -------------------------------------------------------------------------------------------------------------- ;
; Zone 2 - Tracks 18-24 - 19 sectors per track (00-18)
;          BAM occupies one block ($00) - 18 blocks left for file entries
;          Max 144 files per diskette (18 blocks with 8 entries each)
; -------------------------------------------------------------------------------------------------------------- ;
DIR1ST            = $00        ; number 1st file entry - bytes $00-$1f ($20)
DIR1ST_OFF        = $00        ; starting offset
DIR1ST_LEN        = DIRENT_LEN ; entry length
; -------------------------------------------------------------------------------------------------------------- ;
; Directory Link Address
; -------------------------------------------------------------------------------------------------------------- ;
DIRLINK           = $00         ; link to the next directory block
DIRLINKTRK          = $00       ; track location of next directory block
DIRLINKSEC          = $01       ; sector location of next directory block
DIRLINKLASTID         = $00     ; last directory block
DIRLINKLASTCOUNT      = $ff     ; last directory block used byte count
; -------------------------------------------------------------------------------------------------------------- ;
; Directory 1st Data Byte
; -------------------------------------------------------------------------------------------------------------- ;
DIRDATA           = $02         ; directory data start
; -------------------------------------------------------------------------------------------------------------- ;
DIRFTYP           = $02         ; actual file type
DIRFTYP_MASK        = %00000111 ; 
DIRFTYP_MASK_LONG   = %00001111 ; 
DIRFTYP_SCR         = %00000000 ; scratched file entry
DIRFTYP_SEQ         = %00000001 ; sequential file entry
DIRFTYP_PRG         = %00000010 ; program file entry
DIRFTYP_USR         = %00000011 ; special access file entry
DIRFTYP_REL         = %00000100 ; direct access file entry
; -------------------------------------------------------------------------------------------------------------- ;
; Bit0-3: the actual filetype
;     : 000 (0) - DEL
;     : 001 (1) - SEQ
;     : 010 (2) - PRG
;     : 011 (3) - USR
;     : 100 (4) - REL
;                 values 5-15 are illegal - if used will produce very strange results 
;                 the 1541 is inconsistent in how it treats these bits
;                   - some routines use all 4 bits
;                   - others ignore bit 3 resulting in values from 0-7
; Bit4: not used
; Bit5: only present during a 'save-@' or "@0:" file replacement
; Bit6: locked flag (if set    : produces ">" locked files)
; Bit7: closed flag (if not set: produces "*" or "splat" files)
; -------------------------------------------------------------------------------------------------------------- ;
; $00-$04 - 'unclosed' filetype - '*PRG'
; -------------------------------------------------------------------------------------------------------------- ;
DIRFTYP_OPEN        = %00000000 ; $00 - open filetype (bit 7 = 0) --> *PRG
DIRFTYP_MASK_OPEN   = [DIRFTYP_OPEN   | DIRFTYP_MASK] ; %00000111 ; 
DIRFTYP_DEL_OPEN    = [DIRFTYP_OPEN   | DIRFTYP_SCR]  ; %00000000 ; $00 - *DEL
DIRFTYP_SEQ_OPEN    = [DIRFTYP_OPEN   | DIRFTYP_SEQ]  ; %00000001 ; $01 - *SEQ
DIRFTYP_PRG_OPEN    = [DIRFTYP_OPEN   | DIRFTYP_PRG]  ; %00000010 ; $02 - *PRG
DIRFTYP_USR_OPEN    = [DIRFTYP_OPEN   | DIRFTYP_USR]  ; %00000011 ; $03 - *USR
DIRFTYP_REL_OPEN    = [DIRFTYP_OPEN   | DIRFTYP_REL]  ; %00000100 ; $04 - *REL <-- cannot occur
; -------------------------------------------------------------------------------------------------------------- ;
; $80-$84 - closed filetype (bit 7 = 1)
; -------------------------------------------------------------------------------------------------------------- ;
DIRFTYP_CLOSED      = %10000000 ; $80 - closed filetype (bit 7 = 1) --> 'PRG'
DIRFTYP_CLOSED_NO   = %01111111 ; $7f
DIRFTYP_MASK_CLOSED = [DIRFTYP_CLOSED | DIRFTYP_MASK] ; %10000111 ; 
DIRFTYP_DEL_CLOS    = [DIRFTYP_CLOSED | DIRFTYP_SCR]  ; %10000000 ; $80 - DEL
DIRFTYP_SEQ_CLOS    = [DIRFTYP_CLOSED | DIRFTYP_SEQ]  ; %10000001 ; $81 - SEQ
DIRFTYP_PRG_CLOS    = [DIRFTYP_CLOSED | DIRFTYP_PRG]  ; %10000010 ; $82 - PRG
DIRFTYP_USR_CLOS    = [DIRFTYP_CLOSED | DIRFTYP_USR]  ; %10000011 ; $83 - USR
DIRFTYP_REL_CLOS    = [DIRFTYP_CLOSED | DIRFTYP_REL]  ; %10000100 ; $84 - REL
; -------------------------------------------------------------------------------------------------------------- ;
; $a0-$a4 - only present during a 'save-@' or "@0:" file replacement
; -------------------------------------------------------------------------------------------------------------- ;
DIRFTYP_SAVING      = %00100000 ; $20 - saving filetype (bit 5 = 1) --> '@PRG'
DIRFTYP_SAVING_NO   = %11011111 ; $df
DIRFTYP_MASK_SAVE   = [DIRFTYP_CLOSED | DIRFTYP_SAVING | DIRFTYP_MASK] ; %10100111 ; 
DIRFTYP_DEL_SAVE    = [DIRFTYP_CLOSED | DIRFTYP_SAVING | DIRFTYP_SCR]  ; %10100000 ; $a0 - DEL
DIRFTYP_SEQ_SAVE    = [DIRFTYP_CLOSED | DIRFTYP_SAVING | DIRFTYP_SEQ]  ; %10100001 ; $a1 - SEQ
DIRFTYP_PRG_SAVE    = [DIRFTYP_CLOSED | DIRFTYP_SAVING | DIRFTYP_PRG]  ; %10100010 ; $a2 - PRG
DIRFTYP_USR_SAVE    = [DIRFTYP_CLOSED | DIRFTYP_SAVING | DIRFTYP_USR]  ; %10100011 ; $a3 - USR
DIRFTYP_REL_SAVE    = [DIRFTYP_CLOSED | DIRFTYP_SAVING | DIRFTYP_REL]  ; %10100100 ; $a4 - REL <-- cannot occur
; -------------------------------------------------------------------------------------------------------------- ;
; $c0-$c4 - "locked" - 'PRG<'
; -------------------------------------------------------------------------------------------------------------- ;
DIRFTYP_LOCKED      = %01000000 ; $40 - locked filetype (bit 6 = 1) --> 'PRG<'
DIRFTYP_LOCKED_NO   = %10111111 ; $bf
DIRFTYP_MASK_LOCK   = [DIRFTYP_CLOSED | DIRFTYP_LOCKED | DIRFTYP_MASK] ; %11000111 ; 
DIRFTYP_DEL_LOCK    = [DIRFTYP_CLOSED | DIRFTYP_LOCKED | DIRFTYP_SCR]  ; %11000000 ; $c0 - DEL<
DIRFTYP_SEQ_LOCK    = [DIRFTYP_CLOSED | DIRFTYP_LOCKED | DIRFTYP_SEQ]  ; %11000001 ; $c1 - SEQ<
DIRFTYP_PRG_LOCK    = [DIRFTYP_CLOSED | DIRFTYP_LOCKED | DIRFTYP_PRG]  ; %11000010 ; $c2 - PRG<
DIRFTYP_USR_LOCK    = [DIRFTYP_CLOSED | DIRFTYP_LOCKED | DIRFTYP_USR]  ; %11000011 ; $c3 - USR<
DIRFTYP_REL_LOCK    = [DIRFTYP_CLOSED | DIRFTYP_LOCKED | DIRFTYP_REL]  ; %11000100 ; $c4 - REL<
; -------------------------------------------------------------------------------------------------------------- ;
DIRFILE           = $03         ; location of first block of file
DIRFILETRK          = $03       ; 1st track
DIRFILESEC          = $04       ; 1st sector
DIRFILENAM        = $05         ; 16 character filename in PETSCII padded with shift_space ($a0)
DIRFILENAM_LEN    = [DIRREL - DIRFILENAM - $01] ; 
DIRFILENAM_END    = $14         ; end of file name
; -------------------------------------------------------------------------------------------------------------- ;
; REL file side sector blocks start / record lengths (REL file only)
; -------------------------------------------------------------------------------------------------------------- ;
DIRREL            = $15         ; 
DIRRELSSTRK         = $15       ; REL file starting track  1st set of of side-sector blocks ($00)
DIRRELSSSEC         = $16       ; REL file starting sector 1st set of of side-sector blocks ($00)
DIRRELRECL          = $17       ; REL file record length                                    ($00)
; -------------------------------------------------------------------------------------------------------------- ;
DIRUNUSED         = $18         ; $00
DIRUNUSED1          = $18       ; $00
DIRUNUSED2          = $19       ; $00
DIRUNUSED3          = $1a       ; $00
DIRUNUSED4          = $1b       ; $00
; -------------------------------------------------------------------------------------------------------------- ;
DIRREPL           = $1c         ; 
DIRREPLTRK          = $1c       ; used during @-replacement - track  number start of the new replacement file
DIRREPLSEC          = $1d       ; used during @-replacement - sector number start of the new replacement file
; -------------------------------------------------------------------------------------------------------------- ;
DIRFILESIZE       = $1e         ; file size in blocks - filesize in bytes is <= #blocks * 254
DIRFILESIZE_LO      = $1e       ; 
DIRFILESIZE_HI      = $1f       ; 
; -------------------------------------------------------------------------------------------------------------- ;
DIRENT_LEN       = [DIRFILESIZE_HI + $01]  ; 
; -------------------------------------------------------------------------------------------------------------- ;
; from now on the DIRLINK bytes of each entry in this block should be $00/$00 as they are unused
; -------------------------------------------------------------------------------------------------------------- ;
DIR2ND            = $01                    ; number of 2nd file entry - bytes $20-$3f ($20
DIR2ND_OFF        = [DIRENT_LEN * DIR2ND]  ; starting offset
DIR2ND_LEN        = DIRENT_LEN             ; entry length
DIR3RD            = $02                    ; number of 3rd file entry - bytes $40-$5f ($20
DIR3RD_OFF        = [DIRENT_LEN * DIR3RD]  ; starting offset
DIR3RD_LEN        = DIRENT_LEN             ; entry length
DIR4TH            = $03                    ; number of 4th file entry - bytes $60-$7f ($20
DIR4TH_OFF        = [DIRENT_LEN * DIR4TH]  ; starting offset
DIR4TH_LEN        = DIRENT_LEN             ; entry length
DIR5TH            = $04                    ; number of 5th file entry - bytes $80-$9f ($20
DIR5TH_OFF        = [DIRENT_LEN * DIR5TH]  ; starting offset
DIR5TH_LEN        = DIRENT_LEN             ; entry length
DIR6TH            = $05                    ; number of 6th file entry - bytes $a0-$bf ($20
DIR6TH_OFF        = [DIRENT_LEN * DIR6TH]  ; starting offset
DIR6TH_LEN        = DIRENT_LEN             ; entry length
DIR7TH            = $06                    ; number of 7th file entry - bytes $c0-$df ($20
DIR7TH_OFF        = [DIRENT_LEN * DIR7TH]  ; starting offset
DIR7TH_LEN        = DIRENT_LEN             ; entry length
DIR8TH            = $07                    ; number of 8th file entry - bytes $e0-$ff ($20
DIR8TH_OFF        = [DIRENT_LEN * DIR8TH]  ; starting offset
DIR8TH_LEN        = DIRENT_LEN             ; entry length
; -------------------------------------------------------------------------------------------------------------- ;
DIRENT_MAX        = [DIR8TH + $01]          ; max number of directory entries
DIRALL_LEN        = [DIRENT_LEN * DIRENT_MAX - $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
