; -------------------------------------------------------------------------------------------------------------- ;
;  VC1541 - Standard Zeropage / Extended Zeropage / Stack / Vectors / Buffers
; -------------------------------------------------------------------------------------------------------------- ;
ZPAGE             = $00         ; Zeropage - JOB queue / Constants / Pointers / Work Area
; -------------------------------------------------------------------------------------------------------------- ;
; JOBSTCC0 - JOBSTCC5           ; Interface Main Program - Disc Controller
;                               ;   main program writes 
;                               ;     - job command codes
;                               ;     - track/sector numbers of the block to be processed (if necessary)
;                               ;   into the appropriate job cmd memory area
;                               ;     - bit7 set   - job will be executed with the next interrupt
;                               ;     - bit7 clear - bit0-bit6 contain the jobs status code
;                               ; 
;                               ;     Addresses and the assigned buffers
;                               ;   +-----+-------+--------+-------------+
;                               ;   ! JOB ! TRACK ! SECTOR !   BUFFER    !
;                               ;   +-----+-------+--------+-------------+
;                               ;   ! $00 !  $06  !  $07   ! $0300-$03ff !
;                               ;   ! $01 !  $08  !  $09   ! $0400-$04ff !
;                               ;   ! $02 !  $0a  !  $0b   ! $0500-$05ff !
;                               ;   ! $03 !  $0c  !  $0d   ! $0600-$06ff !
;                               ;   ! $04 !  $0e  !  $0f   ! $0700-$07ff !
;                               ;   ! $05 !  $10  !  $11   ! -no ram-    !
; -------------------------------------------------------------------------------------------------------------- ;
; Job Queue
; -------------------------------------------------------------------------------------------------------------- ;
JOBS              = $00         ; command code
JOBCC_ID            = %10000000 ; bit7=1  : is job   code
JOBCC_ERR           = %01111111 ; bit7=0  : is error code
JOBCC_MASK          = %11110000 ; bits 4-7: command  code
JOBCC_READ          = %10000000 ; $80 - Read   a sector
JOBCC_WRITE         = %10010000 ; $90 - Write  a sector
JOBCC_VERIFY        = %10100000 ; $a0 - Verify a sector
JOBCC_SEEK          = %10110000 ; $b0 - Seek   a sector
JOBCC_BUMP          = %11000000 ; $c0 - Bump head - find track 01
JOBCC_JUMP          = %11010000 ; $d0 - Exec program in buffer
JOBCC_EXECUTE       = %11100000 ; $e0 - Exec program in buffer / switch drive on and position head
; -------------------------------------------------------------------------------------------------------------- ;
JOBST_MASK          = %00001111 ; bits 0-3: error code
JOBST_OK            = %00000001 ; $01 - Everything OK                   00, OK
JOBST_ERR_NOHDR     = %00000010 ; $02 - Header block not found          20, READ ERROR
JOBST_ERR_NOSYNC    = %00000011 ; $03 - SYNC not found                  21, READ ERROR
JOBST_ERR_NODATA    = %00000100 ; $04 - Data block not found            22, READ ERROR
JOBST_ERR_CRCDATA   = %00000101 ; $05 - Checksum error in data block    23, READ ERROR
JOBST_ERR_FORMAT    = %00000110 ; $06 - Format error                    24, READ ERROR        - not used on the 1541
JOBST_ERR_VERIFY    = %00000111 ; $07 - Verify error                    25, WRITE ERROR
JOBST_ERR_WRTPROT   = %00001000 ; $08 - Disk write protected            26, WRITE PROTECT ON
JOBST_ERR_CRCHDR    = %00001001 ; $09 - Checksum error in header block  27, READ ERROR
JOBST_ERR_LONGDATA  = %00001100 ; $0a - Data block too long             28, READ ERROR        - not used on the 1541
JOBST_ERR_DSKCHG    = %00001011 ; $0b - Id mismatch                     29, DISK ID MISMATCH
JOBST_ERR_NODISC    = %00001111 ; $0f - Disk not inserted               74, DRIVE NOT READY
; -------------------------------------------------------------------------------------------------------------- ;
JOBS0               = $00       ; use BUFF0 - command and status registers - JOB0TRA/JOB0SEC
JOBS1               = $01       ; use BUFF1 - command and status registers - JOB1TRA/JOB1SEC
JOBS2               = $02       ; use BUFF2 - command and status registers - JOB2TRA/JOB2SEC
JOBS3               = $03       ; use BUFF3 - command and status registers - JOB3TRA/JOB3SEC
JOBS4               = $04       ; use BUFF4 - command and status registers - JOB4TRA/JOB4SEC
JOBS5               = $05       ; use BUFF5 - command and status registers - JOB5TRA/JOB5SEC - unused: no RAM
; -------------------------------------------------------------------------------------------------------------- ;
; Header Table
; -------------------------------------------------------------------------------------------------------------- ;
HDRS              = $06         ; table: track/sector area for JOBS
HDRSTRA             = $06       ; 
HDRSSEC             = $07       ; 

HDRS0TRA            = $06       ; BUFF0 track
HDRS0SEC            = $07       ; BUFF0 sector
HDRS1TRA            = $08       ; BUFF1 track
HDRS1SEC            = $09       ; BUFF1 sector
HDRS2TRA            = $0a       ; BUFF2 track
HDRS2SEC            = $0b       ; BUFF2 sector
HDRS3TRA            = $0c       ; BUFF3 track
HDRS3SEC            = $0d       ; BUFF3 sector
HDRS4TRA            = $0e       ; BUFF4 track
HDRS4SEC            = $0f       ; BUFF4 sector

HDRS5TRA            = $10       ; BUFF5 track  - unused: no RAM
HDRS5SEC            = $11       ; BUFF5 sector - unused: no RAM
; -------------------------------------------------------------------------------------------------------------- ;
; Master copy of disk ID (specified when disk was formatted) - updated with any JOBCC_SEEK
; -------------------------------------------------------------------------------------------------------------- ;
DSKID             = $12         ; drive #0 - expected sector header ID
DSKID_CHR1          = $12       ; 
DSKID_CHR2          = $13       ; 
DSKID1            = $14         ; drive #1 - expected sector header ID                          ---> unused <---
DSKID1_CHR1         = $14       ; 
DSKID1_CHR2         = $15       ; 
; -------------------------------------------------------------------------------------------------------------- ;
; Image of last HEADER read - stored on disk in the opposite sequence
; -------------------------------------------------------------------------------------------------------------- ;
HEADER            = $16         ; header block - from header of the sector last read from disk
HEADER_CHR1         = $16       ; header block ID     - 1st ID char
HEADER_CHR2         = $17       ; header block ID     - 2nd ID char
HEADER_TRK          = $18       ; header block track  - track  number from header of sector last read from disk
HEADER_SEC          = $19       ; header block sector - sector number from header of sector last read from disk
HEADER_CRC          = $1a       ; header block parity - checksum      from header of sector last read from disk
; -------------------------------------------------------------------------------------------------------------- ;
ACTJOB            = $1b         ; controllers active job                                        ---> unused <---
WPSW              = $1c         ; flag: drive #0 - disk change indicator
WPSW_FLAG           = %00000001 ; 
WPSW_SAME           = %00000000 ; disk has not changed
WPSW_DIFF           = $00000001 ; disk has changed
WPSW1             = $1d         ; flag: drive #1 - disk change indicator                        ---> unused <---
LWPT              = $1e         ; flag: drive #0 - last state of write protect photocell (write protect sense)
LWPT_ON             = %00010000 ; 
LWPT_OFF            = %11101111 ; 
LWPT1             = $1f         ; flag: drive #1 - previous status of write protect photocell   ---> unused <---
DRVST             = $20         ; table: drives current status (disk and step motor)
DRVST0              = $20       ; drive #0 - disk drive status
DRVST_MASK          = %11110000 ; 
DRVST_MSTOP         = %00010000 ; shut down drive motor  (1 = no , 0 = yes)
DRVST_MSTOP_NO      = %00010000 ; 
DRVST_MSTOP_YES     = %11101111 ;                                          
DRVST_MON           = %00100000 ; motor is on            (1 = yes, 0 = no )
DRVST_MON_YES       = %00100000 ; 
DRVST_MON_NO        = %11011111 ; 
DRVST_BUSY          = %01000000 ; read/write head moving (1 = yes, 0 = no )
DRVST_BUSY_YES      = %01000000 ; 
DRVST_BUSY_NO       = %10111111 ; 
DRVST_READY         = %10000000 ; disk drive ready       (1 = yes, 0 = no )
DRVST_READY_YES     = %10000000 ; 
DRVST_READY_NO      = %01111111 ; 
DRVST1            = $21         ; drive #1 - disk drive status                                  ---> unused <---
DRVTRK            = $22         ; drive #0 - current track under head
DRVTRK1           = $23         ; drive #1 - current track under head                           ---> unused <--- but: see next line
SBUSMODE          = $23         ; switch: serial bus communication speed (0=1541 /<>0=1540)
SBUS_C64            = $00       ; C64    mode - lower speed (extra waits are needed as compensation for the delays caused by sprite DMA in the host)
SBUS_VC20           = $01       ; VIC-20 mode - higher speed ($01-$ff)
; -------------------------------------------------------------------------------------------------------------- ;
; Scratch Pad Area of GCR conversion / storage for BIN -> GCR conversions
; -------------------------------------------------------------------------------------------------------------- ;
STAB              = $24         ; storage table for GCR conversion
STABX             = $2d         ; 
STAB_LEN          = [STABX - STAB - $01] ;  
; -------------------------------------------------------------------------------------------------------------- ;
SAVPNT            = $2e         ; pointer: current byte in buffer during GCR-encoding/decoding
SAVPNT_LO           = $2e       ; 
SAVPNT_HI           = $2f       ; 
BUFPNT            = $30         ; pointer: start of currently active buffer
BUFPNT_LO           = $30       ; 
BUFPNT_HI           = $31       ; 
HDRPNT            = $32         ; pointer: track and sector registers of current buffer
HDRPNT_TT           = $32       ; pointer: active track
HDRPNT_SS           = $33       ; pointer: active sector
GCRPNT            = $34         ; pointer: last converted byte during GCR-encoding/decoding
GCRERR            = $35         ; indicate GCR error code                                       ---> unused <---
BYTCNT            = $36         ; byte counter during GCR-encoding/decoding
BITCNT            = $37         ; bit counter                                                   ---> unused <---
BID               = $38         ; constant: ID start of data block
BID_DFLT            = $07       ; 
HBID              = $39         ; constant: ID start of block header
HBID_DFLT           = $08       ; 
CHKSUM            = $3a         ; computed data or header checksum

HINIB             = $3b         ;                                                               ---> unused <---
BYT               = $3c         ; "BYTE" impossible for DASM                                    ---> unused <---

DRIVE             = $3d         ; drive number
DRIVE0              = $00       ; 
DRIVE1              = $01       ;                                                               ---> unused <---
CDRIVE            = $3e         ; current drive number
CDRIVE0             = $00       ; 
CDRIVE1             = $01       ;                                                               ---> unused <---
CDRIVE_OFF          = $ff       ; motor is off - must spin up before seeking
JOBN              = $3f         ; current JOB number
JOBN0               = $00       ; 
JOBN1               = $01       ; 
JOBN2               = $02       ; 
JOBN3               = $03       ; 
JOBN4               = $04       ; 
JOBN5               = $05       ; buffer #5 does not exist                                      ---> unused <---
TRACC             = $40         ; work: track number
NXTJOB            = $41         ; position of next job in queue
NXTJOB0             = $00       ; 
NXTJOB1             = $01       ; 
NXTJOB2             = $02       ; 
NXTJOB3             = $03       ; 
NXTJOB4             = $04       ; 
NXTJOB5             = $05       ; buffer #5 does not exist                                      ---> unused <---
NXTRK             = $42         ; destination (next) track to move head to
SECTR             = $43         ; number of sectors per track for formatting
WORK              = $44         ; temporary workspace
JOB               = $45         ; temporary storage of job code

CTRACK            = $46         ;                                                               ---> unused <---

DBID              = $47         ; data block ID code - init to DBID_DFLT ($07) on reset drive
                                ;   - may be changed - write/read data blocks with different data block ID codes
                                ;   - 1st nibble of the data block ID code should always be a zero ($0-)
                                ;     otherwise the controller will have difficulty detecting the 
                                ;     end of the sync mark and the start of the DBID
                                ; if a sector is read whose DBID is different from the value stored here
                                ; the disk controller puts JOBST_NODATA_ER ($04) in the job queue and
                                ; the drive will report a #22 error (DATA BLOCK NOT FOUND)
DBID_DFLT           = $07       ; default
DBID_MIN            = $00       ; allowed values: $00-$0f
DBID_MAX            = $0f       ; allowed values: $00-$0f
ACLTIM            = $48         ; acceleration time delay - timer for acceleration of head movement
SAVSP             = $49         ; temporary save of stackpointer
STEPS             = $4a         ; number of steps to move head to desired track
                                ;   $00-$7f - move head out (from disk centre)
                                ;   $80-$ff - move head in  (to disk centre)
TMP               = $4b         ; retry counter for reading sector header / temporary storage during seeking
CSECT             = $4c         ; current sector - last sector read
NEXTS             = $4d         ; next sector to read
NXTBF             = $4e         ; pointer: next GCR source buffer to be changed to binary
NXTPNT            = $4f         ; pointer: next byte location in buffer
;NXTBF_HI           = $4e       ;   --> byte swapped !!
;NXTBF_LO           = $4f       ;   --> byte swapped !!
GCRFLG            = $50         ; flag: some buffer data still in GCR-encoded form
GCRFLG_BIN          = $00       ; $00    : Data in normal form
GCRFLG_GCR          = $01       ; $01-$FF: Data in GCR-encoded form - must be decoded
FTNUM             = $51         ; current track number for FORMAT
; -------------------------------------------------------------------------------------------------------------- ;
; Staging area for the 4 binary bytes being converted to GCR (PUT4BG) or from GCR (GET4BG)
; -------------------------------------------------------------------------------------------------------------- ;
BTAB              = $52         ; BIN table: temp area for 4 data bytes during GCR-BIN encoding/decoding
BTAB0               = $52       ; 
BTAB1               = $53       ; 
BTAB2               = $54       ; 
BTAB3               = $55       ; 
BTABX               = BTAB3     ; 
BTAB_LEN          = [BTABX - BTAB] ; 
; -------------------------------------------------------------------------------------------------------------- ;
; Staging area for the 5 GCR bytes being converted from binary (PUT4BG) or to binary (GET4BG)
; -------------------------------------------------------------------------------------------------------------- ;
GTAB              = $56         ; GCR table: temp area for data nybbles/5 GCR bytes during GCR-BIN encoding/decoding
GTAB0               = $56       ; 
GTAB1               = $57       ; 
GTAB2               = $58       ; 
GTAB3               = $59       ; 
GTAB4               = $5a       ; 
GTAB5               = $5b       ; 
GTAB6               = $5c       ; 
GTAB7               = $5d       ; 
GTABX             = GTAB7       ; 
GTAB_LEN          = [GTABX - GTAB] ; 
; -------------------------------------------------------------------------------------------------------------- ;
AS                = $5e         ; number of halftracks (steps) to accelerate/decelerate
AS_DFLT             = $04       ; twice this value must be less than value of MINSTP
AF                = $5f         ; acceleration/deceleration factor
AF_DFLT             = $04       ; value of DSKCNT(T1LH2) plus/minus value of AS times - this value must not be
                                ;   too low   below about 12-20 (depends on drive mechanics) or
                                ;   too high  above 255
ACLSTP            = $60         ; number of steps left to accelerate/decelerate when stepping the head
RSTEPS            = $61         ; number of steps left to step the head when in fast stepping (RUN) mode
NXTST             = $62         ; pointer: head stepping routine INACT($fa05)
NXTST_LO            = $62       ;
NXTST_HI            = $63       ;
MINSTP            = $64         ; minimum steps reqired for fast stepping (RUN) mode
MINSTP_DFLT         = $c8       ; 200
;UIPNT            = $65         ; pointer: warm start ('UI' command) routine [$EB22]
;UIPNT_LO           = $65       ; 
;UIPNT_HI           = $66       ; 
VNMI             = $65          ; pointer: warm start ('UI' command) routine [$EB22]
VNMI_LO            = $65        ;   --< all same name as in C64 mapping >-- use UIPNT instead to avoid conflicts
VNMI_HI            = $66        ; 
NMIFLG            = $67         ; flag: NMI in progress
AUTOFG            = $68         ; flag: enable/disable automatic disk init on ID MISMATCH
AUTOFG_YES          = $00       ; 
AUTOFG_NO           = $01       ; 
SECINC            = $69         ; soft interleave - distance (in sectors) for allocating the next sector for files
SECINC_DIR          = $03       ;   DIRECORY interleave
SECINC_FILE         = $0a       ;   FILE interleave
REVCNT            = $6a         ; number of retries on dos commands in case of an error
REVCNT_DFLT         = $05       ; number of retries
REVCNT_MASK         = %00111111 ; 
REVCNT_AHT          = %01000000 ; retry on adjacent halftracks
REVCNT_BUMP         = %10000000 ; bump head
REVCNT_AHT_NO       = %10111111 ; do not retry on adjacent halftracks
REVCNT_BUMP_NO      = %01111111 ; do not bump head
USRJMP            = $6b         ; pointer: start of user jump table for 'Ux' commands [$fff6]
USRJMP_LO           = $6b       ; 
USRJMP_HI           = $6c       ; 
BMPNT             = $6d         ; pointer: start of BAM bitmap [BUFF1] - set when a disk is initialized
BMPNT_LO            = $6d       ; 
BMPNT_HI            = $6e       ; 
; -------------------------------------------------------------------------------------------------------------- ;
; Temporary Work Area
; -------------------------------------------------------------------------------------------------------------- ;
TEMP              = $6f         ; 
T0                  = $6f       ; 
T1                  = $70       ; 
T2                  = $71       ; 
T3                  = $72       ; 
T4                  = $73       ; 
T5                  = $74       ; 
TEMPX               = T5        ; 
TEMP_LEN            = [TEMPX - TEMP] ; 
; -------------------------------------------------------------------------------------------------------------- ;
IP                = $75         ; indirect pointer: current byte during memory test
                                ;          exec address current 'Ux' user command
IP_LO               = $75       ; 
IP_HI               = $76       ; 
LSNADR            = $77         ; listener address (device number + $20) - $28 on reset
LSNADR_DFLT         = $28       ; 
TLKADR            = $78         ; talker   address (device number + $40) - $48 on reset
TLKADR_DFLT         = $48       ; 
LSNACT            = $79         ; flag: active listener
LSNACT_YES          = $01       ;       $01-$ff: LISTEN command currently active
LSNACT_NO           = $00       ;       $00    : no LISTEN command active
TLKACT            = $7a         ; flag: active talker
TLKACT_YES          = $01       ;       $01-$ff: TALK command currently active
TLKACT_NO           = $00       ;       $00    : no TALK command active
ADRSED            = $7b         ; flag: addressing mode
ATNPND            = $7c         ; flag: ATN pending - still receiving from serial bus
ATNPND_YES          = $01       ;       $01-$ff: ATN signal arrived
ATNPND_NO           = $00       ;       $00    : ATN inactive
ATNMOD            = $7d         ; flag: end of command (6502 in attention mode)
ATNMOD_YES          = $00       ;       $00    : Command fully arrived - ATN became inactive
ATNMOD_NO           = $80       ;       $01-$ff: Command still transferring
PRGTRK            = $7e         ; track number ($01-$ff) of previously opened file (used when opening '*')
PRGTRK_NONE         = $00       ; no file has been opened yet
DRVNUM            = $7f         ; current drive number (always 0 for 1541)
DRVNUM_MASK         = %00000001 ; 
DRVNUM_BAD          = %10000000 ; 
TRACK             = $80         ; current track number  ($00 after use)
SECTOR            = $81         ; current sector number ($00 after use)
LINDX             = $82         ; index: current channel number
LINDX_MASK          = %00111111 ; all buffers
LINDX0              = $00       ; buffer #00
LINDX1              = $01       ; buffer #01
LINDX2              = $02       ; buffer #02
LINDX3              = $03       ; buffer #03
LINDX4              = $04       ; buffer #04
LINDX5              = $05       ; buffer #05 - error message
SA                = $83         ; current secondary address
SA_MASK             = %00001111 ; 
SA_MAX              = LINTAB_LEN       ; 
SA_READ             = LINTAB_NUM_READ  ; internal read
SA_WRITE            = LINTAB_NUM_WRITE ; internal write
SA_CMD              = LINTAB_NUM_CMD   ; command channel
ORGSA             = $84         ; original secondary address
ORGSA_ID            = %01100000 ; id: secondary address = $60 + num($01-$0f)
ORGSA_DFLT          = [ORGSA_ID | ORGSA_CMD] ; $6f - command channel
ORGSA_MASK          = %00001111 ; 
ORGSA_LOAD_BAS      = %00000000 ; read data to BASIC start
ORGSA_LOAD_ABS      = %00000001 ; read data to the address stared in the 1st two bytes of the PRG file
ORGSA_CMD           = %00001111 ; $0f (15) - command channel
ORGSA_80            = %10000000 ; 
ORGSA_MASK_OC       = %11110000 ; mask OPEN/CLOSE commands
ORGSA_OPEN          = %11110000 ; 
ORGSA_CLOSE         = %11100000 ; 
DATA              = $85         ; temporary data byte
R0                = $86         ; temporary result
R1                = $87         ; temporary result
R2                = $88         ; temporary result
R3                = $89         ; temporary result
R4                = $8a         ; temporary result
; -------------------------------------------------------------------------------------------------------------- ;
; Result Area
; -------------------------------------------------------------------------------------------------------------- ;
RESULT            = $8b         ; 
RESULT0             = $8b       ; 
RESULT1             = $8c       ; 
RESULT2             = $8d       ; 
RESULT3             = $8e       ; 
RESULTX             = RESULT3   ; 
RESULT_LEN          = [RESULTX - RESULT] ; 
; -------------------------------------------------------------------------------------------------------------- ;
; Accumulator
; -------------------------------------------------------------------------------------------------------------- ;
ACCUM             = $8f         ; 
ACCUM0              = $8f       ;                
ACCUM1              = $90       ; 
ACCUM2              = $91       ; 
ACCUM3              = $92       ; 
ACCUM4              = $93       ; 
ACCUMX              = ACCUM4    ; 
ACCUM_LEN           = [ACCUMX - ACCUM] ; 
; -------------------------------------------------------------------------------------------------------------- ;
DIRBUF            = $94         ; pointer: current DIR entry - DIR buffer ($0205)
                                ; note   : DIRBUF does NOT point to the start of the data buffer ($0300, $0400, ...)
                                ;          but points to the first data byte ($0302, $0402, ...)
                                ;          as the entries are examined it is updated to point to the start of
                                ;          the entry ($02, $22, $42, ...)
DIRBUF_LO           = $94       ; 
DIRBUF_HI           = $95       ; 
ICMD              = $96         ; IEEE command in (not used on 1541)                            ---> unused <---
MYPA              = $97         ; MY PA flag $00  (not used on 1541)                            ---> unused <---
CONT              = $98         ; bit counter during serial bus input/output
; -------------------------------------------------------------------------------------------------------------- ;
; Buffer Byte Pointers - Point to the next byte to be used of each buffer - set by the B-P command
; -------------------------------------------------------------------------------------------------------------- ;
BUFTAB            = $99         ; table: pointer to next byte in buffers #0-#4 - normally: $0300-$0700
BUFTAB_LO           = $99       ; 
BUFTAB_HI           = $9a       ; 
;
BUFTAB0             = $99       ; pointer: next byte in buffer #0 - default : $0300
BUFTAB0_LO            = $99     ; 
BUFTAB0_HI            = $9a     ; 
BUFTAB1             = $9b       ; pointer: next byte in buffer #1 - default : $0400
BUFTAB1_LO            = $9b     ; 
BUFTAB1_HI            = $9c     ; 
BUFTAB2             = $9d       ; pointer: next byte in buffer #2 - default : $0500
BUFTAB2_LO            = $9d     ; 
BUFTAB2_HI            = $9e     ; 
BUFTAB3             = $9f       ; pointer: next byte in buffer #3 - default : $0600
BUFTAB3_LO            = $9f     ; 
BUFTAB3_HI            = $a0     ; 
BUFTAB4             = $a1       ; pointer: next byte in buffer #4 - default : $0700
BUFTAB4_LO            = $a1     ; 
BUFTAB4_HI            = $a2     ; 
BUFTABCMD           = $a3       ; pointer: next byte in command buffer - default: $0200
BUFTABCMD_LO          = $a3     ; 
BUFTABCMD_HI          = $a4     ; 
BUFTABERR           = $a5       ; pointer: next byte in error message buffer - default: $02D5
BUFTABERR_LO          = $a5     ; 
BUFTABERR_HI          = $a6     ; 
; -------------------------------------------------------------------------------------------------------------- ;
BUF0              = $a7         ; table: primary buffer number assigned to channels
BUF_NUM_MASK        = %00001111 ;        bit0-bit3: buffer number
BUF_FLAG_MASK       = %00111111 ;        bit6-but7: buffer status flags
BUF_ACT_NO          = %10000000 ;        bit7     : 1 = buffer used
BUF_DIRTY           = %01000000 ;        bit6     : 1 = buffer 'dirty'
BUF_DIRTY_NO        = %10111111 ;        bit6     : 0 = buffer good   
BUF_INACT           = %11111111 ; 
BUF00               = $a7       ; 
BUF01               = $a8       ; 
BUF02               = $a9       ; 
BUF03               = $aa       ; 
BUF04               = $ab       ; 
BUF05               = $ac       ; 
BUF06               = $ad       ; 
BUF0_MAX          = BUF06       ; 
BUF0_LEN          = [BUF0_MAX - BUF0 + $01]  ; 
; -------------------------------------------------------------------------------------------------------------- ;
BUF1              = $ae         ; table: secondary buffer number assigned to channels
BUF10               = $ae       ; 
BUF11               = $af       ; 
BUF12               = $b0       ; 
BUF13               = $b1       ; 
BUF14               = $b2       ; 
BUF15               = $b3       ; 
BUF16               = $b4       ; 
BUF1_MAX          = BUF16       ; 
BUF1_LEN          = [BUF1_MAX - BUF1 + $01]  ; 
BUF_MAX           = [BUF1_MAX - BUF0 + $01]  ; 
; -------------------------------------------------------------------------------------------------------------- ;
NBKL              = $b5         ; Number Of Blocks LO
; -------------------------------------------------------------------------------------------------------------- ;
RECL              = $b5         ; table: low byte - length of file assigned to channels / rel files: number of records
RECL0               = $b5       ; 
RECL1               = $b6       ; 
RECL2               = $b7       ; 
RECL3               = $b8       ; 
RECL4               = $b9       ; 
RECL5               = $ba       ; 
; -------------------------------------------------------------------------------------------------------------- ;
NBKH              = $bb         ; Number Of Blocks HI
; -------------------------------------------------------------------------------------------------------------- ;
RECH              = $bb         ; table: high byte - length of file assigned to channels / rel files: number of records
RECH0               = $bb       ; 
RECH1               = $bc       ; 
RECH2               = $bd       ; 
RECH3               = $be       ; 
RECH4               = $bf       ; 
RECH5               = $c0       ; 
; -------------------------------------------------------------------------------------------------------------- ;
NR                = $c1         ; table: offset of current byte in buffer assigned to channels
NR0                 = $c1       ; 
NR1                 = $c2       ; 
NR2                 = $c3       ; 
NR3                 = $c4       ; 
NR4                 = $c5       ; 
NR5                 = $c6       ; 
; -------------------------------------------------------------------------------------------------------------- ;
RS                = $c7         ; table: record length of relative file assigned to channels
RS0                 = $c7       ; 
RS1                 = $c8       ; 
RS2                 = $c9       ; 
RS3                 = $ca       ; 
RS4                 = $cb       ; 
RS5                 = $cc       ; 
; -------------------------------------------------------------------------------------------------------------- ;
SS                = $cd         ; table: buffer number holding side sector of relative file assigned to channels
SS_Num_Mask         = %01111111 ;        bit0-bit6: buffer number
SS_Act_Mask         = %10000000 ;        bit7     : 1 = no buffer assigned to channel
SS0               = $cd         ; 
SS1               = $ce         ; 
SS2               = $cf         ; 
SS3               = $d0         ; 
SS4               = $d1         ; 
SS5               = $d2         ; 
; -------------------------------------------------------------------------------------------------------------- ;
F1PTR             = $d3         ; work: input string
RECPTR            = $d4         ; offset: current byte in relative file record
SSNUM             = $d5         ; side sector number belonging to current REL file record
SSIND             = $d6         ; offset: track/sector number of current REL file record in side sector
RELPTR            = $d7         ; offset: record in relative file data sector
; -------------------------------------------------------------------------------------------------------------- ;
ENTSEC            = $d8         ; table: sector number of DIR entries
ENTSEC0             = $d8       ; 
ENTSEC1             = $d9       ; 
ENTSEC2             = $da       ; 
ENTSEC3             = $db       ; 
ENTSEC4             = $dc       ; 
; -------------------------------------------------------------------------------------------------------------- ;
ENTIND            = $dd         ; table: buffer offset of DIR entries
ENTIND0             = $dd       ; 
ENTIND1             = $de       ; 
ENTIND2             = $df       ; 
ENTIND3             = $e0       ; 
ENTIND4             = $e1       ; 
; -------------------------------------------------------------------------------------------------------------- ;
FILDRV            = $e2         ; table: drive number of files
FILDRV_NUM          = %00000001 ;        bit0: drive number
FILDRV_DFLT         = %10000000 ;        bit7: 1 = no valid drive number has been specified in command - must try both units
FILDRV0             = $e2       ; 
FILDRV1             = $e3       ; 
FILDRV2             = $e4       ; 
FILDRV3             = $e5       ; 
FILDRV4             = $e6       ; 
; -------------------------------------------------------------------------------------------------------------- ;
PATTYP            = $e7         ; table: file type / file status flages
PATTYP_TPYE_MASK    = %00000111 ; 
PATTYP_TYPE_DEL     = %00000000 ; $00
PATTYP_TYPE_SEQ     = %00000001 ; $01
PATTYP_TYPE_PRG     = %00000010 ; $02
PATTYP_TYPE_USR     = %00000011 ; $03
PATTYP_TYPE_REL     = %00000100 ; $04

PATTYP_CLOSED       = %11011111 ; 0 = file has been closed
PATTYP_OPEN         = %00100000 ; 
PATTYP_PROT_YES     = %01000000 ; 1 = file is write protected
PATTYP_PROT_NO      = %10111111 ; 
PATTYP_WICA_YES     = %10000000 ; 1 = wildcards present in file name
PATTYP_WICA_NO      = %01111111 ; 

PATTYP0           = $e7         ; 
PATTYP1           = $e8         ; 
PATTYP2           = $e9         ; 
PATTYP3           = $ea         ; 
PATTYP4           = $eb         ; 
PATTYPX           = PATTYP4     ; 
PATTYP_LEN          = [PATTYPX - PATTYP] ; 
; -------------------------------------------------------------------------------------------------------------- ;
FILTYP            = $ec         ; table: drive number / file type / file status flags
FILTYP_DRIVE        = %00000001 ; drive number

FILTYP_TYPE_MASK    = %00001110 ; file type
FILTYP_TYPE_DEL     = %00000000 ; $00
FILTYP_TYPE_SEQ     = %00000010 ; $01
FILTYP_TYPE_PRG     = %00000100 ; $02
FILTYP_TYPE_USR     = %00000110 ; $03
FILTYP_TYPE_REL     = %00001000 ; $04
FILTYP_TYPE_DDA     = %00001110 ; '#' - direct disk access

FILTYP_EOR          = %00100000 ; end of record
FILTYP_EOF          = %01000000 ; end of file
FILTYP_LRE          = %10000000 ; last record
FILTYP0           = $ec         ; 
FILTYP1           = $ed         ; 
FILTYP2           = $ee         ; 
FILTYP3           = $ef         ; 
FILTYP4           = $f0         ; 
FILTYP5           = $f1         ; 
; -------------------------------------------------------------------------------------------------------------- ;
CHNRDY            = $f2         ; table: channel input/output flags
CHNRDY_CLEAR      =   %00000000 ; reset all flags
CHNRDY_WRITE      =   %00000001 ; bit0 - 1 = write allowed
CHNRDY_EOI        =   %00001000 ; bit3 - 0 = end of input
CHNRDY_READ       =   %10000000 ; bit7 - 1 = read allowed
CHNRDY0           = $f2         ; 
CHNRDY1           = $f3         ; 
CHNRDY2           = $f4         ; 
CHNRDY3           = $f5         ; 
CHNRDY4           = $f6         ; 
CHNRDY5           = $f7         ; 
; -------------------------------------------------------------------------------------------------------------- ;
EOIFLG            = $f8         ; flag: end of file indicator current channel
EOIFLG_YES          = $00       ;       $00    : end of file
EOIFLG_NO           = $08       ;       $01-$FF: file has not ended yet
EOIFLG_NONE         = $80       ; 
JOBNUM            = $f9         ; current buffer number
; -------------------------------------------------------------------------------------------------------------- ;
LRUTBL            = $fa         ; table: temp store for channel numbers
LRUTBL0             = $fa       ; 
LRUTBL1             = $fb       ; 
LRUTBL2             = $fc       ; 
LRUTBL3             = $fd       ; 
LRUTBL4             = $fe       ; 
LRUTBLX           = LRUTBL4     ; 
LRUTBL_LEN        = [LRUTBLX - LRUTBL] ; 
; -------------------------------------------------------------------------------------------------------------- ;
NODRV             = $ff         ; flag: table: drive #0/#1 status
NODRV0            = $ff         ; flag: drive #0 status
NODRV_YES           = $00       ;       drive #0 ready
NODRV_NO            = $ff       ;       drive #0 not ready (no disk)
; -------------------------------------------------------------------------------------------------------------- ;
; Extended Zero Page - Stacks / Work Areas / GCR Overflow Buffer
; -------------------------------------------------------------------------------------------------------------- ;
ZPDRVEXT          = $0100       ; 
NODRV1              = $0100     ; flag: drive #1 status                                         ---> unused <---
DSKVER            = $0101       ; table: drives DOS version - taken from track 18 sector 00 BAMDOSVRS ($02)
DSKVER0             = $0101     ; drive #0 DOS version
DSKVER1             = $0102     ; drive #1 DOS version                                          ---> unused <---
; -------------------------------------------------------------------------------------------------------------- ;
ZPEND             = $0103       ;                                                               ---> unused <---
; -------------------------------------------------------------------------------------------------------------- ;
; Stack
; -------------------------------------------------------------------------------------------------------------- ;
STACK_DRV         = $0104       ; 
STACK_DRV_PTR_INI   = $45       ; offset top of stack
STACK_DRVX        = [ZPDRVEXT + STACK_DRV_PTR_INI] ; 
STACK_DRV_LEN     = [STACK_DRVX - STACK_DRV]    ; 

STACK_DRV_FREE    = [STACK_DRVX + $01] ; free - not used ($0146)
STACK_DRV_FREEX   = $01b9       ; 

BUFGCR            = $01ba       ; overflow buffer for GCR de/encoding
BUFGCR_OFF          = <BUFGCR           ; offset BUFGCR start
BUFGCR_LEN          = [CMDBUF - BUFGCR] ; 
; -------------------------------------------------------------------------------------------------------------- ;
; Command Buffer / Work
; -------------------------------------------------------------------------------------------------------------- ;
CMDBUF            = $0200       ; command buffer for input string from computer
CMDBUFX           = $0229       ; 
CMDBUF_LEN          = [CMDBUFX - CMDBUF] ; input buffer size
; -------------------------------------------------------------------------------------------------------------- ;
CMDNUM            = $022a       ; number of the cmd to be executed
CMDNUM_DOS_VAL      = $00       ; VALIDATE
CMDNUM_DOS_INI      = $01       ; INITIALIZE
CMDNUM_DOS_DUP      = $02       ; DUPLICATE (not used)
CMDNUM_DOS_MEM      = $03       ; MEMORY
CMDNUM_DOS_BLK      = $04       ; BLOCK
CMDNUM_DOS_USR      = $05       ; USER
CMDNUM_DOS_POS      = $06       ; POSITION
CMDNUM_DOS_UTI      = $07       ; UTILITY LOADER (&-Command)
CMDNUM_DOS_CPY      = $08       ; COPY
CMDNUM_DOS_REN      = $09       ; RENAME
CMDNUM_DOS_SCR      = $0a       ; SCRATCH
CMDNUM_DOS_NEW      = $0b       ; NEW
CMDNUM_DOS_DIR      = $0c       ; open '$'
                    
CMDNUM_DOS_BFL      = $80       ; $80-$fe: 'B-x' commands
CMDNUM_DOS_ERR      = $ff       ; error: command too long
; -------------------------------------------------------------------------------------------------------------- ;
LINTAB            = $022b       ; table: current status of each data channel (secondary address)
LINTAB_INACT        = %11111111 ;        inactive (no channel assigned)
LINTAB_R            = %00000000 ;        open for read
LINTAB_RW           = %01000000 ;        open for read/write
LINTAB_W            = %10000000 ;        open for write
LINTAB01            = $022b     ; logical index channel $01
LINTAB02            = $022c     ; logical index channel $02
LINTAB03            = $022d     ; logical index channel $03
LINTAB04            = $022e     ; logical index channel $04
LINTAB05            = $022f     ; logical index channel $05
LINTAB06            = $0230     ; logical index channel $06
LINTAB07            = $0231     ; logical index channel $07
LINTAB08            = $0232     ; logical index channel $08
LINTAB09            = $0233     ; logical index channel $09
LINTAB0A            = $0234     ; logical index channel $10
LINTAB0B            = $0235     ; logical index channel $11
LINTAB0C            = $0236     ; logical index channel $12
LINTAB0D            = $0237     ; logical index channel $13
LINTAB0E            = $0238     ; logical index channel $14
LINTAB0F            = $0239     ; logical index channel $15 - command channel
LINTAB_NUM_CMD      = [LINTAB0F - LINTAB + $01] ; 
LINTAB10            = $023a     ; logical index channel $16 - behind the command channel
LINTAB11            = $023b     ; logical index channel $17
LINTAB12            = $023c     ; logical index channel $18 - internal READ
LINTAB_NUM_READ     = [LINTAB12 - LINTAB] ; 
LINTAB13            = $023d     ; logical index channel $19 - internal WRITE
LINTAB_NUM_WRITE    = [LINTAB13 - LINTAB] ; 
LINTAB_LEN          = [LINTAB13 - LINTAB + $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
CHNDAT            = $023e       ; table: channel data byte - the most recent byte read or written for each channel
                                ; offset next data byte to be read or written from buffers #0-#4 for each channel
CHNDAT0             = $023e     ; BUFF0
CHNDAT1             = $023f     ; BUFF1
CHNDAT2             = $0240     ; BUFF2
CHNDAT3             = $0241     ; BUFF3
CHNDAT4             = $0242     ; BUFF4
CHNDAT5             = $0243     ; BUFF5 - Temporary area of next data byte to be written from error message buffer to serial bus
CHNDATX           = CHNDAT5     ; 
CHNDAT_LEN        = [CHNDATX - CHNDAT] ; 
; -------------------------------------------------------------------------------------------------------------- ;
LSTCHR            = $0244       ; table: channel last character pointer
                                ; offset last data byte read or written in buffers for each channel
LSTCHR0             = $0244     ; BUFF0
LSTCHR1             = $0245     ; BUFF1
LSTCHR2             = $0246     ; BUFF2
LSTCHR3             = $0247     ; BUFF3
LSTCHR4             = $0248     ; BUFF4
LSTCHR5             = $0249     ; BUFF5
LSTCHRX           = LSTCHR5     ; 
LSTCHR_LEN        = [LSTCHRX - LSTCHR] ; 
; -------------------------------------------------------------------------------------------------------------- ;
TYPE              = $024a       ; current file type
STRSIZ            = $024b       ; current file name length
TEMPSA            = $024c       ; temp area for secondary address
CMD               = $024d       ; temp area for DC command
LSTSEC            = $024e       ; work area to find the best sector
BUFUSE            = $024f       ; buffer allocation register
BUFUSE_LO           = $024f     ; buffer allocation register LO - drive #0
BUFUSE_HI           = $0250     ; buffer allocation register HI - drive #1
BUFUSE_FREE         = %11100000 ;    set all buffers to unused
BUFUSE0             = %00000001 ;    buffer #0 is being used
BUFUSE1             = %00000010 ;    buffer #1 is being used
BUFUSE2             = %00000100 ;    buffer #2 is being used
BUFUSE3             = %00001000 ;    buffer #3 is being used
BUFUSE4             = %00010000 ;    buffer #4 is being used
BUFUSEE             = %00100000 ;    error message buffer is being used
MDIRTY            = $0251       ; flag: BAM changed drives (dirty)
MDIRTY0             = $0251     ; flag: BAM changed drive #0
MDIRTY1             = $0252     ; flag: BAM changed drive #1                                    ---> unused <---
MDIRTY_NO           = $00       ;       BAM unchanged
MDIRTY_YES          = $01       ; - $ff BAM has been changed - must be written onto disk
ENTFND            = $0253       ; flag: DIR entry found
ENTFND_NO           = $ff       ;   bit7=1   : a file matching the name has NOT been found in the DIR
ENTFND_YES          = $00       ;   bit7=0   : a file matching the name has     been found in the DIR
                                ;   bit0-bit2: current entry number
DIRLST            = $0254       ; flag: DIR listing
DIRLST_YES          = $00       ;       DATA file is being LOADed
DIRLST_NO           = $01       ; -$ff: DIR is being LOADed
CMDWAT            = $0255       ; flag: command waiting
CMDWAT_YES          = $00       ;   command still arriving
CMDWAT_NO           = $01       ;   command fully arrived
LINUSE            = $0256       ; register: LINDX channel use - bit=1 - related channel free / bit=0 - related channel used
LINFRE_ALL          = %00001111 ;    all channels are free
LINFRE0             = %00000001 ;    channel #0 is free
LINFRE1             = %00000010 ;    channel #1 is free
LINFRE2             = %00000100 ;    channel #2 is free
LINFRE3             = %00001000 ;    channel #3 is free
LINUSE0             = %11111110 ;    channel #0 is being used
LINUSE1             = %11111101 ;    channel #1 is being used
LINUSE2             = %11111011 ;    channel #2 is being used
LINUSE3             = %11110111 ;    channel #3 is being used
LBUSED            = $0257       ; last used buffer / temp area for channel number
                                ;   $00-$06: assigned buffer number in primary   table BUF0 ($a7-$ad)
                                ;   $07-$0D: assigned buffer number in secondary table BUF1 ($ae-$b4)
REC               = $0258       ; record length of current relative file
TRKSS             = $0259       ; side sector track
SECSS             = $025a       ; side sector sector
; -------------------------------------------------------------------------------------------------------------- ;
LSTJOB            = $025b       ; table: last JOB codes for buffers #0-#4
LSTJOB_DRIVE_MASK   = %00000001 ; drive number
LSTJOB0             = $025b     ; 
LSTJOB1             = $025c     ; 
LSTJOB2             = $025d     ; 
LSTJOB3             = $025e     ; 
LSTJOB4             = $025f     ; 
LSTJOBX             = LSTJOB4   ; 
LSTJOB_LEN          = [LSTJOBX - LSTJOB] ; 
; -------------------------------------------------------------------------------------------------------------- ;
DSEC              = $0260       ; table: sector number of DIR entry of files specified in command
DSEC0               = $0260     ; 
DSEC1               = $0261     ; 
DSEC2               = $0262     ; 
DSEC3               = $0263     ; 
DSEC4               = $0264     ; 
DSEC5               = $0265     ; 
DSECX               = DSEC5     ; 
DSEC_LEN            = [DSECX - DSEC] ; 
; -------------------------------------------------------------------------------------------------------------- ;
DIND              = $0266       ; table: offset of DIR entry of files specified in command
DIND0               = $0266     ; 
DIND1               = $0267     ; 
DIND2               = $0268     ; 
DIND3               = $0269     ; 
DIND4               = $026a     ; 
DIND5               = $026b     ; 
DINDX               = DIND5     ; 
DIND_LEN            = [DINDX - DIND] ; 
; -------------------------------------------------------------------------------------------------------------- ;
ERWORD            = $026c       ; flag: an error has occurred
ERWORD_YES          = $01       ; $01-$ff: error - value used for LED flash timer
ERWORD_NO           = $00       ;          no error
ERLED             = $026d       ; flag: drive LED error flashing
ERLED_OFF           = $00       ;       switch LED off
ERLED_ON            = $08       ;       switch LED on
PRGDRV            = $026e       ; drive  number of last opened file (used when opening '*')
PRGSEC            = $026f       ; sector number of last opened file (used when opening '*')
WLINDX            = $0270       ; temp store for channel number (write LINDX)
RLINDX            = $0271       ; temp store for channel number (read  LINDX)
NBTEMP            = $0272       ; table: BASIC line number for entries sent to host during LOAD '$'
                                ;   for header: drive number
                                ;   for files : length of file in blocks
                                ;   for footer: number of free blocks
NBTEMP0             = $0272     ; 
NBTEMP1             = $0273     ; 
; -------------------------------------------------------------------------------------------------------------- ;
CMDSIZ            = $0274       ; length of input string
CMDSIZ_MAX          = CMDBUF_LEN ; 

CMDSIZ_ME_LEN       = $05       ; M-E AdrLo AdrHi                     - Memory-Execute
CMDSIZ_MR_LEN       = $06       ; M-R AdrLo AdrHi Num                 - Memory-Read
CMDSIZ_MW_LEN       = $06       ; M-W AdrLo AdrHi Num(max $23)        - Memory-Write
CMDSIZ_MW_MAX       = [CMDBUF_LEN - CMDSTR_MW_LEN] ; 

CMDSIZ_BR_LEN       = $07       ; B-R Channel# Drive# Track# Sector#  - Block-Read (do not use)
CMDSIZ_U1_LEN       = $06       ;  U1 Channel# Drive# Track# Sector#  - Block-Read

CMDSIZ_BW_LEN       = $07       ; B-R Channel# Drive# Track# Sector#  - Block-Write (do not use)
CMDSIZ_U2_LEN       = $06       ;  U2 Channel# Drive# Track# Sector#  - Block-Write

CMDSIZ_BA_LEN       = $06       ; B-A Drive# Track# Sector#           - Block-Allocate in BAM
CMDSIZ_BE_LEN       = $06       ; B-E Drive# Track# Sector#           - Block-Execute
CMDSIZ_BF_LEN       = $06       ; B-F Drive# Track# Sector#           - Block-Free in BAM
CMDSIZ_BP_LEN       = $03       ; B-P Channel# Position#              - Buffer-Pointer
; -------------------------------------------------------------------------------------------------------------- ;
CHAR              = $0275       ; actual character to search for in input buffer
LIMIT             = $0276       ; index: end of file name in command buffer (1st char after file name in command)
F1CNT             = $0277       ; work: count input string
F2CNT             = $0278       ; work: count input string
F2PTR             = $0279       ; work: pointer input string
; -------------------------------------------------------------------------------------------------------------- ;
; Parser Table Area ($027a - $0289)
; -------------------------------------------------------------------------------------------------------------- ;
FILTBL            = $027a       ; table: offsets file name in input string (last offset specifies end of command)
FILTBL_DEFAULT      = %10000000 ; flag: use default drive - no drive number in input string found for this file
FILTBL_DRIVENUM     = %00000001 ; drive number
FILTBL0           = $027a       ; 
FILTBL1           = $027b       ; 
FILTBL2           = $027c       ; 
FILTBL3           = $027d       ; 
FILTBL4           = $027e       ; 
FILTBL5           = $027f       ; 
FILTBLX           = FILTBL5     ; 
FILTBL_LEN          = [FILTBLX - FILTBL] ; 
; -------------------------------------------------------------------------------------------------------------- ;
FILTRK            = $0280       ; table: track number 1st file link
FILTRK_NFND         = $00       ;   $00    : file not yet found in DIR
                                ;   $01-$ff: track number of 1st file block
                                ; for 'B-x' cmds: high byte of parameters
FILTRK0           = $0280       ;
FILTRK1           = $0281       ;
FILTRK2           = $0282       ;
FILTRK3           = $0283       ;
FILTRK4           = $0284       ;
FILTRKX           = FILTRK4     ;
FILTRK_LEN          = [FILTRKX - FILTRK] ; 
; -------------------------------------------------------------------------------------------------------------- ;
FILSEC            = $0285       ; table: sector number 1st file link
                                ;   $00    : file not yet found in DIR
                                ;   $01-$ff: sector number of 1st file block
                                ; for 'B-x' cmds: low byte of parameters
FILSEC0             = $0285     ; 
FILSEC1             = $0286     ; 
FILSEC2             = $0287     ; 
FILSEC3             = $0288     ; 
FILSEC4             = $0289     ; 
FILSECX           = FILSEC4     ; 
FILSEC_LEN          = [FILSECX - FILSEC] ; 
; -------------------------------------------------------------------------------------------------------------- ;
PATFLG            = $028a       ; flag: wild card present (number of wildcards found in current file name)
PATFLG_NO           =%00000000  ; 
PATFLG_YES          =%10000000  ; 
PATFLG_COUNT        =%01111111  ; 
IMAGE             = $028b       ; flag: result of command syntax check
IMAGE_00          =   %00000000 ; init
                                ; --------------------------.  file #2
IMAGE_N2          =   %00000001 ; filename given             ! equation marks are present in the input string
IMAGE_D2          =   %00000010 ; Drive # specified          ! 
IMAGE_G2          =   %00000100 ; more then one file implied ! commas    after  equation marks are present in the input string
IMAGE_P2          =   %00001000 ; found wildcard             ! wildcards after  equation marks are present in the input string
                                ; ---------------------------+ file #1       
IMAGE_N1          =   %00010000 ; filename given             ! equation marks are present in the input string
IMAGE_D1          =   %00100000 ; Drive # specified          ! 
IMAGE_G1          =   %01000000 ; more then one file implied ! commas    before equation marks are present in the input string
IMAGE_P1          =   %10000000 ; found wildcard             ! wildcards before equation marks are present in the input string
                                ; --------------------------  

DRVCNT            = $028c       ; drives to process during DIR read
DRVCNT_MASK         = %00000011 ; 
DRVCNT_ONE          = %00000000 ;   one drive  in command
DRVCNT_TWO          = %00000001 ;   two drives in command
DRVCNT_2ND          = %00000010 ;   2nd drive available
DRVCNT_SEL          = %01000000 ;   bit6=1:take drive num from cmd string
                                ;   bit6=0:take drive num from SCHTBL bit7
DRVCNT_NUM          = %10000000 ;   drive number (0 or 1)
                                ;   
DRVFLG            = $028d       ; flag: drive search during DIR read
DRVFLG_ONE          = $00       ;   one drive available
DRVFLG_TWO          = $01       ;   2nd drive available
LSTDRV            = $028e       ; last drive number without error during reading the DIR (used as default drive)
FOUND             = $028f       ; flag: all files found in DIR searches (indicator to keep searching in the DIR)
FOUND_NO            = $00       ;       still more files to be searched for
FOUND_YES           = $ff       ;       all files have been found - no need to continue search
DIRSEC            = $0290       ; current DIR sector number
DELSEC            = $0291       ; DIR sector number to read (sector number of 1st available file)
DELSEC_1ST          = $00       ;   1st DIR sector is to be read
DELSEC_NEXT         = $01       ;   $01- $ff: sector number of next DIR to read
DELIND            = $0292       ; flag: search type - if found: offset current DIR entry in DIR sector (index of 1st available file)
DELIND_VLD          = $00       ;   look for a valid DIR entry
DELIND_DEL          = $01       ;   look for a free DIR entry (unused/deleted)
LSTBUF            = $0293       ; flag: end of DIR
LSTBUF_YES          = $00       ;   current DIR sector is the last one
LSTBUF_NO           = $01       ;   $01-$ff: more DIR sectors are present
INDEX             = $0294       ; offset: current DIR entry in buffer
FILCNT            = $0295       ; number of remaining DIR entries in DIR sector minus 1
                                ;   $00-$07: number of remaining DIR entries
                                ;   $ff    : no more entries left
TYPFLG            = $0296       ; flag: file type match of file being searched in DIR
TYPFLG_NO           = $00       ;   file type was NOT specified in command - any file type will match
TYPFLG_YES          = $01       ;   file type was     specified in command
MODE              = $0297       ; file open mode
MODE_R              = $00       ;   'R' - read   or LOAD
MODE_W              = $01       ;   'W' - write  or SAVE
MODE_A              = $02       ;   'A' - append
MODE_M              = $03       ;   'M' - modify or salvage improperly closed file
JOBRTN            = $0298       ; flag: message display for disk errors
JOBRTN_MSG_NO       = %00000000 ;   bit7=0: silently ignore the following errors when executing disk commands
                                ;           '26,WRITE PROTECT ON'
                                ;           '29,DISK ID MISMATCH'
                                ;           '74,DRIVE NOT READY' 
JOBRTN_MSG_YES      = %10000000 ;   bit7=1: display the error messages above
EPTR              = $0299       ; offset: current byte in halftrack seek table during retry disk operations on adjacent halftracks
TOFF              = $029a       ; offset: direction of seeking back to original halftrack during retry disk operations on adjacent halftracks
UBAM              = $029b       ; table: track number current BAM entry of drives (last BAM update pointer)
UBAM0               = $029b     ;   track number current BAM entry of drive #0
UBAM1               = $029c     ;   track number current BAM entry of drive #1                  ---> unused <---
TBAM              = $029d       ; table: track numbers of two cached BAM entries of drives
TBAM00              = $029d     ;   1st track number of two cached BAM entries of drive #0
TBAM01              = $029e     ;   2nd track number of two cached BAM entries of drive #0
TBAM10              = $029f     ;   1st track number of two cached BAM entries of drive #1      ---> unused <---
TBAM11              = $02a0     ;   2nd track number of two cached BAM entries of drive #1      ---> unused <---
; -------------------------------------------------------------------------------------------------------------- ;
; BAM Images Buffer
; -------------------------------------------------------------------------------------------------------------- ;
BAM               = $02a1       ; buffer: BAM image
BAMX              = $02b0       ; 
BAM_LEN           = [BAMX - BAM]; 
; -------------------------------------------------------------------------------------------------------------- ;
; DIR Buffer
; -------------------------------------------------------------------------------------------------------------- ;
NAMBUF            = $02b1       ; buffer: DIR listing line output
NAMBUFFIN         = $02c9       ; 
NAMBUF_LINE_LEN   = [NAMBUFFIN - NAMBUF] ; 
NAMBUFX           = $02d4       ; 
NAMBUF_LEN        = [NAMBUFX - NAMBUF] ; 
; -------------------------------------------------------------------------------------------------------------- ;
; Error Message Buffer
; -------------------------------------------------------------------------------------------------------------- ;
ERRBUF            = $02d5       ; buffer: error message output
ERRBUFX           = $02f8       ; 
ERRBUF_LEN        = [ERRBUFX - ERRBUF] ; 
; -------------------------------------------------------------------------------------------------------------- ;
WBAM              = $02f9       ; flag: 'do not write BAM' - reset to $00 before and after each command disk update upon BAM change switch
WBAM_WRITE_YES      = %00000000 ; write BAM
WBAM_WRITE_NO       = %00000001 ; don't write BAM
WBAM_DIRTY          = %01000000 ; 
WBAM_WRITE          = %10000000 ; 
NDBL              = $02fa       ; table: number of disk blocks free for drives   LO      
NDBL0               = $02fa     ;        number of disk blocks free for drive #0 LO      
NDBL1               = $02fb     ;        number of disk blocks free for drive #1 LO             ---> unused <---
NDBH              = $02fc       ; table: number of disk blocks free for drives   HI      
NDBH0               = $02fc     ;        number of disk blocks free for drive #0 HI      
NDBH1               = $02fd     ;        number of disk blocks free for drive #1 HI             ---> unused <---
PHASE             = $02fe       ; table: direction of seeking adjacent halftrack drives (current phase of head stepper motor)
PHASE0              = $02fe     ; direction of seeking adjacent halftrack drive #0
PHASE1              = $02ff     ; direction of seeking adjacent halftrack drive #1              ---> unused <---
PHASE_NO            = $00       ;   no seeking
PHASE_UP            = $01       ;   seek a halftrack upwards
PHASE_OK            = $02       ;   halftrack seeking complete
PHASE_DO            = $ff       ;   seek a halftrack downwards
; -------------------------------------------------------------------------------------------------------------- ;
; Data Buffers
; -------------------------------------------------------------------------------------------------------------- ;
BUFS              = $0300       ; start of data buffers
BUFS_MAX            = <[BUFSX - BUFS] ; max number of available buffers
FBUFS             = $0300       ; format download image
BUFF0               = $0300     ; main working buffer
BUFF1               = $0400     ; contains the actual part of the DIR
BUFF2               = $0500     ; USER buffer - normally free
BUFF3               = $0600     ; contains the last DIR block
CNT                   = $0620   ; error counter: decrements from CNT_MAX
CNT_MAX                 = $0a   ; 
FMTVAR                = $0620   ; format variable
NUM                   = $0621   ; number between sync and non-sync
NUM_ESTIMATE            = $0fa0 ; 1st estimation of a track half capacity (4000)
NUM_LO                  = $0621 ; 
NUM_HI                  = $0622 ; 
TRYS                  = $0623   ; number of tries in verify
TRYS_DFLT               = $c8   ; default tries (200)
TRAL                  = $0624   ; counter: number of bytes in data segment
TRAL_LO                 = $0624 ; 
TRAL_HI                 = $0625 ; 
DTRCK                 = $0626   ; distance to track
REMDR                 = $0627   ; remainder of size
SECT                  = $0628   ; sector number counter
BUFF4               = $0700     ; contains block 18,0 (BAM) after init
BUFF5               = $0700     ; unused - no RAM available - set to BUFF4
BUFSX             = BUFF4       ; end of buffers
; -------------------------------------------------------------------------------------------------------------- ;
END_OF_RAM        = $7fff       ; no ram from $0800 - $bfff
; -------------------------------------------------------------------------------------------------------------- ;
