; -------------------------------------------------------------------------------------------------------------- ;
; Snokie - Protected - snokie1.prg
; -------------------------------------------------------------------------------------------------------------- ;
; Memory Map
; -------------------------------------------------------------------------------------------------------------- ;
; $0400 - $07ff: Screen Start Menu
; $0800 - $0bff: Screen High Scores
; $0400 - $07ff: Screen Game
; 
; $1000 - $9fff: Game Code
; $713e        : MainEntry - called from SNOKIE.prg
; -------------------------------------------------------------------------------------------------------------- ;
; $1800 - $1fff: CharSet_GfxInfo
; $2000 - $27ff: CharSet_GfxGame
; $2800 - $2bff: CharSet_TextNormal
; $2c00 - $2fff: TabCharSet_TextReverse
; $3000 - $3fff: Sprite data
; $4000 - $6edf: Level Data
; $6ee0 - $713d: Level Check Data
; -------------------------------------------------------------------------------------------------------------- ;
                          * equ $1000                     ; 
; -------------------------------------------------------------------------------------------------------------- ;
; compiler settings
; -------------------------------------------------------------------------------------------------------------- ;
                          incdir  ..\inc                  ; C64 System Includes

C64CIA1                   include c64_cia1.asm            ; Complex Interface Adapter (CIA) #1 Registers  $dc00-$dc0f
C64CIA2                   include c64_cia2.asm            ; Complex Interface Adapter (CIA) #2 Registers  $dd00-$dd0f
C64SID                    include c64_sid.asm             ; Sound Interface Device (SID) Registers        $d400-$d41c
C64VicII                  include c64_vic.asm             ; Video Interface Chip (VIC-II) Registers       $d000-$d02e
C64Colors                 include c64_color.asm           ; Colour RAM Address / Colours
C64Memory                 include c64_mem.asm             ; Standard Zeropage / Stack / Vector constants
C64Basic                  include c64_basic.asm           ; Basic  Routines entry points                  $a000-$bfff
C64Kernal                 include c64_kernal.asm          ; Kernal Routines entry points $e000-$ffff

Game                      include inc\SN_VarsGame.asm     ; Game Variables
ZeroPage                  include inc\SN_Zpg.asm          ; Zero Page Addresses
; -------------------------------------------------------------------------------------------------------------- ;
GameHiScoreCompressOff    lda FlagHiScoreDeCompress       ; 
                          beq GameHiScoreCompress         ; 
                          
GameHiScoreCompressOffX   rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameHiScoreCompressOn   Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameHiScoreCompressOn     subroutine                      ; 
                          lda FlagHiScoreDeCompress       ; 
                          bne GameHiScoreCompress         ; 
                          
GameHiScoreCompressOnX    rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameHiScoreCompress     Does    : (De)Compress the Game High Score Table Entries
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameHiScoreCompress       subroutine                      ; 
.IniHiScoreLen            ldy #TabHiScoreEntries_Len      ; 
.GetNextHiScoreEntryByte  lda [TabHiScoreEntry_01 - $01],y; 
.FlipFlopHiScoreByte      eor #$1a                        ; ...##.#. - flip/flop compression
.SetHiScoreByte           sta [TabHiScoreEntry_01 - $01],y; 
                          
.DecHiScoreLen            dey                             ; 
.ChkHiScoreLen_Min        bne .GetNextHiScoreEntryByte    ; 
                          
                          lda FlagHiScoreDeCompress       ; 
.FlipFlopHiScoreFlag      eor #$ff                        ; 
                          sta FlagHiScoreDeCompress       ; 
                          
GameHiScoreCompressX      rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameHiScoreFileSave     Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameHiScoreFileSave       subroutine                      ; 
                          lda #$01                        ; set logical file number
                          ldx #$08                        ; set device number
                          ldy #$01                        ; LOAD a file to the address taken from the files disk header
                          jsr SETLFS                      ; Kernal($FFBA) Set logical file parameters ($FE00)
                          
                          lda #GameHiScoreFileName_Len    ; length
                          ldx #<GameHiScoreFileName       ; <name
                          ldy #>GameHiScoreFileName       ; >name
                          jsr SETNAM                      ; Kernal($FFBD) Set filename parameters ($FDF9)
                          
                          lda #<TabHiScoreEntries         ; <start adr
                          sta SNZ_TabHiScoreEntriesEnd_Lo ; 
                          lda #>TabHiScoreEntries         ; >start adr
                          sta SNZ_TabHiScoreEntriesEnd_Hi ; 
                          
                          lda #SNZ_TabHiScoreEntriesEnd_Lo; start adr ZP reg lo
                          ldx #<[TabHiScoreEntriesX + $01]; <END ADR * $01
                          ldy #>[TabHiScoreEntriesX + $01]; >END ADR * $01
                          jsr SAVE                        ; Kernal($FFD8) Save to a device (via $332 to $F5DD)
                          
GameHiScoreFileSaveX      rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameHiScoreFileLoad     Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameHiScoreFileLoad       subroutine                      ; 
                          lda #$01                        ; set logical file number
                          ldx #$08                        ; set device number
                          ldy #$ff                        ; no secondary address is used
                          jsr SETLFS                      ; Kernal($FFBA) Set logical file parameters ($FE00)
                          
                          lda #GameHiScoreFileName_Len    ; length
                          ldx #<GameHiScoreFileName       ; <name
                          ldy #>GameHiScoreFileName       ; >name
                          jsr SETNAM                      ; Kernal($FFBD) Set filename parameters ($FDF9)
                          
                          lda #$00                        ; flag: load
                          ldx #<TabHiScoreEntries         ; <start
                          ldy #>TabHiScoreEntries         ; >start
                          jsr LOAD                        ; Kernal($FFD5) Load from device (via $330 to $F49E)
                          
GameHiScoreFileLoadX      rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
GameHiScoreFileName       equ  *   ; 
                          dc.b $48 ; h
                          dc.b $49 ; i
                          dc.b $47 ; g
                          dc.b $48 ; h
                          dc.b $20 ; <space>
                          dc.b $53 ; s
                          dc.b $43 ; c
                          dc.b $4f ; o
                          dc.b $52 ; r
                          dc.b $45 ; e
                          dc.b $53 ; s
GameHiScoreFileName_Len   equ  [* - GameHiScoreFileName] ; 
; -------------------------------------------------------------------------------------------------------------- ;
; Data Includes
; -------------------------------------------------------------------------------------------------------------- ;
Unused                    include inc\SN_DataUnused.asm   ; Unused memory
CharsetGfx                include inc\SN_CharSetGfx.asm   ; GFX chars
CharsetText               include inc\SN_CharSetTxt.asm   ; Normal chars
Sprites                   include inc\SN_DataSprite.asm   ; Sprite data
LevelData                 include inc\SN_DataLevels.asm   ; Level structures
LevelCheck                include inc\SN_DataChecks.asm   ; Level check data for Snokie move
; -------------------------------------------------------------------------------------------------------------- ;
; MainEntry               Does    : Main entry point - called from SNOKIE.prg
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
MainEntry                 subroutine                      ; 
                          sei                             ; 
                          
                          jsr RESTOR                      ; Kernal($FF8A) Restore default I/O vectors ($FD15)
                          
                          lda #<NMI                       ; 
                          sta NMINV_LO                    ; Vector($0318/$0319): Hardware NMI interrupt address at ($FE47)
                          lda #>NMI                       ; 
                          sta NMINV_HI                    ; Vector($0318/$0319): Hardware NMI interrupt address at ($FE47)
                          
                          lda #$0c                        ; 
                          sta HIBASE                      ; High Byte of Screen Memory Address ($04)
                          
                          jsr GameHiScoreFileLoad         ; 
                          jsr GameHiScoreCompressOff      ; 
                          
.SetSpritePtrsTextTopTen  lda #[SpriteData_TextTopTen / VIC_Sprt_BlockLen] ; 
                          tax                             ; set sprite pointers
                          stx [SN_HiScoreSpritePtr_00]    ; 
                          inx                             ; 
                          stx [SN_HiScoreSpritePtr_01]    ; 
                          inx                             ; 
                          stx [SN_HiScoreSpritePtr_02]    ; 
                          inx                             ; 
                          stx [SN_HiScoreSpritePtr_03]    ; 
                          inx                             ; 
                          stx [SN_HiScoreSpritePtr_04]    ; 
                          inx                             ; 
                          stx [SN_HiScoreSpritePtr_05]    ; 
                          
.SetSpritePtrsOrbTiny     lda #[SpriteData_OrbTiny / VIC_Sprt_BlockLen] ; 
                          sta [SN_HiScoreSpritePtr_06]    ; 
                          sta [SN_HiScoreSpritePtr_07]    ; 
                          
                          lda SCROLX                      ; VIC($D016) VIC Control Register 2 (Horiz Fine Scroll)
                          and #VIC_SoftX_Clear            ; #####...
                          ora #VIC_SoftX_04               ; .....#..
                          sta SCROLX                      ; VIC($D016) VIC Control Register 2 (Horiz Fine Scroll)
                          
                          jsr InitSwapCharSetsHandler     ; 
                          jsr InitGameStartTuneSprite     ; 
                          
                          lda SCROLX                      ; VIC($D016) VIC Control Register 2 (Horiz Fine Scroll)
                          and #VIC_SoftX_Clear            ; #####...
                          sta SCROLX                      ; VIC($D016) VIC Control Register 2 (Horiz Fine Scroll)
; -------------------------------------------------------------------------------------------------------------- ;
; MainInit                Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
NMI                       subroutine                      ; 
                          nop                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; MainInit                Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
MainInit                  subroutine                      ; 
                          sei                             ; 
                          
                          jsr GameStartInputHandler       ; 
                          
.Start                    lda #$00                        ; return point 
                          sta SPENA                       ; VIC($D015) Sprite Enable Register
                          jsr InitColorRamToBlack         ; 
                          
                          lda #$00                        ; 
                          sta SPMC                        ; VIC($D01C) Sprite Multicolor
                          
                          lda #<IRQ_Game                  ; 
                          sta CINV_LO                     ; Vector($0314/$0315): Hardware IRQ Interrupt address at ($EA31)
                          lda #>IRQ_Game                  ; 
                          sta CINV_HI                     ; Vector($0314/$0315): Hardware IRQ Interrupt address at ($EA31)
                          
                          lda #VIC_IrqSetRaster           ; 
                          sta IRQMASK                     ; VIC($D01A) IRQ Mask Register
                          
                          lda SCROLY                      ; VIC($D011) VIC Control Register 1 (Vert Fine Scroll)
                          and #VIC_RasterHi_Off           ; .#######
                          sta SCROLY                      ; VIC($D011) VIC Control Register 1 (Vert Fine Scroll)
                          
                          lda #$00                        ; 
                          sta RASTER                      ; VIC($D012) Read : Current Raster Scan Line (Bit 8 in SCROLY = $D011)
                          
                          lda #CIA_IntMaskAll             ; .#######
                          sta CIAICR                      ; CIA1($DC0D) Interrupt Control
                          
                          jsr InitGameScreen              ; 
                          jsr InitGameSpritesSnokie       ; 
                          jsr InitGameSpritesTime         ; 
                          jsr InitGameVarsFull            ; 
                          jsr InitSwapCharSetsHandler     ; 
                          
                          cli                             ; 
                          
                          jsr InitVicScreenOn             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; MainLoop                Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
MainLoop                  subroutine                      ; 
                          jsr GameCheckKey_F1_F7          ; 
                          jsr GameSnokieCheckFinish       ; 
                          jsr GameSnokieCheckCollision    ; 
                          jsr GameHiScoreShow             ; 
                          jsr GameSetSpriteDataTime       ; 
                          jsr GameShowLevelTime           ; 
                          jsr GameMoveRocks_L04_05_07     ; 
                          jsr GameMoveRays_L06            ; 
                          jsr GameMoveBeams_L07           ; 
                          jsr GameMovePlates_L03          ; 
                          jsr GameMovePlates_L05          ; 
                          jsr GameSnokieCheckCollision    ; 
                          jsr GameTuneDeathHandler        ; 
                          jsr GameHiScoreHandler          ; 
                          jsr GameNextLevelHandler        ; 
                          jsr GameOrbBigMoveHandler       ; 
                          jsr GameOrbTinyMoveHandler      ; 
                          jsr GameSnokieSetJumpHeight     ; 
                          jsr GameSnokieMoveHandler       ; 
                          jsr GameOrbMoveSpriteHandler    ; 
                          jsr GameMoveDeferral            ; 
                          
__GetSpriteTimeData_Le_Lo equ [* + $01]                   ; 
                          lda #[SpriteData_TimeLe / VIC_Sprt_BlockLen] ; 
                          sta SN_GameSpritePtr_06         ; 
                          
__GetSpriteTimeData_Ri_Lo equ [* + $01]                   ; 
                          lda #[SpriteData_TimeRi / VIC_Sprt_BlockLen] ; 
                          sta SN_GameSpritePtr_07         ; 
                          
MainLoopX                 jmp MainLoop                    ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameMoveDeferral        Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameMoveDeferral          subroutine                      ; 
                          ldy TabLevelNumCurrent          ; 
                          
                          lda TabGameSpeed_P1             ; 
                          beq .GetWaitTime_Speed          ; 
                          
.GetWaitTime_Normal       lda TabGameMoveWaitSpeed,y      ; 
                          beq .ExitNoWait                 ; 
                          bne .IniMoveWaitTime            ; always
                          
.GetWaitTime_Speed        lda TabGameMoveWaitNormal,y     ; 
                          bne .IniMoveWaitTime            ; 
                          
.ExitNoWait               rts                             ; 
                          
.IniMoveWaitTime          tay                             ; 
.DecNextMoveWaitTime      dey                             ; 
                          bne .DecNextMoveWaitTime        ; 
                          
GameMoveDeferralX         rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabGameMoveWaitSpeed      equ  *   ; 
                          dc.b $90 ; 
                          dc.b $42 ; 
                          dc.b $42 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $1f ; 
                          dc.b $90 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabGameMoveWaitNormal     equ  *   ; 
                          dc.b $90 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $42 ; 
; -------------------------------------------------------------------------------------------------------------- ;
; IRQ_Game_ScrollLevel    Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
IRQ_Game_ScrollLevel      subroutine                      ; 
                          lda TabLevelSoftScrollValue     ; 
                          asl a                           ; 
                          sta ._SetSoftScrollX            ; 
                          
.GetSoftScrollX           lda SCROLX                      ; VIC($D016) VIC Control Register 2 (Horiz Fine Scroll)
                          and #VIC_SoftX_Clear            ; #####...
._SetSoftScrollX          equ [* + $01]                   ; 
.InsSoftScrollX           ora #$00                        ; 
.SetSoftScrollX           sta SCROLX                      ; VIC($D016) VIC Control Register 2 (Horiz Fine Scroll)
                          
                          lda TabLevelHardScrollValue     ; 
                          beq GamePutScreenLevelData      ; 
                          
                          clc                             ; 
                          lda TabGetLevelDataPtr_Lo       ; 
                          adc TabLevelHardScrollValue     ; 
                          sta TabGetLevelDataPtr_Lo       ; 
                          
                          lda TabGetLevelDataPtr_Hi       ; 
                          adc #$00                        ; 
                          sta TabGetLevelDataPtr_Hi       ; 
                          
                          lda #$00                        ; 
IRQ_Game_ScrollLevelX     sta TabLevelHardScrollValue     ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GamePutScreenLevelData  Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GamePutScreenLevelData    subroutine                      ; 
.IniGetPtrRowData         lda TabGetLevelDataPtr_Lo       ; 
                          sta ._GetScrnLevelDataByte_Lo   ; 
                          lda TabGetLevelDataPtr_Hi       ; 
                          sta ._GetScrnLevelDataByte_Hi   ; 
                          
.IniPutPtrRowScrn         lda #[SN_Scrn_RowLen * $05]     ; omit the 1st five screen rows
                          sta ._PutScrnLevelDataByte_Lo   ; 
                          lda #>SN_MemScreenGame          ; 
                          sta ._PutScrnLevelDataByte_Hi   ; 
                          
.IniScrnRowCount          ldx #[SN_Scrn_Rows - $05]       ; discount the 1st five screen rows
.IniNextScrnRowLen        ldy #[SN_Scrn_RowLen - $01]     ; 
._GetScrnLevelDataByte_Lo equ [* + $01]                   ; 
._GetScrnLevelDataByte_Hi equ [* + $02]                   ; 
.GetNextScrnDataByte      lda SN_GetDataDummy,y           ; 
._PutScrnLevelDataByte_Lo equ [* + $01]                   ; 
._PutScrnLevelDataByte_Hi equ [* + $02]                   ; 
                          sta SN_GetDataDummy,y           ; 
                          dey                             ; 
                          bpl .GetNextScrnDataByte        ; 
                          
.SetGetPtrToNextRowData   clc                             ; 
                          lda ._GetScrnLevelDataByte_Lo   ; 
                          adc #<[SN_Scrn_RowLen * $0f]    ; 
                          sta ._GetScrnLevelDataByte_Lo   ; 
                          
                          lda ._GetScrnLevelDataByte_Hi   ; 
                          adc #>[SN_Scrn_RowLen * $0f]    ; 
                          sta ._GetScrnLevelDataByte_Hi   ; 
                          
.SetPutPtrToNextRowScrn   clc                             ; 
                          lda ._PutScrnLevelDataByte_Lo   ; 
                          adc #<[SN_Scrn_RowLen * $01]    ; 
                          sta ._PutScrnLevelDataByte_Lo   ; 
                          
                          lda ._PutScrnLevelDataByte_Hi   ; 
                          adc #>[SN_Scrn_RowLen * $01]    ; 
                          sta ._PutScrnLevelDataByte_Hi   ; 
                          
.DecScrnRowCount          dex                             ; 
.ChkScrnRowCount_Min      bne .IniNextScrnRowLen          ; 
                          
GamePutScreenLevelDataX   rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameGetLevelCheckByte   Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameGetLevelCheckByte     subroutine                      ; 
                          clc                             ; 
                          lda #<TabTileChkData            ; 
                          adc SNZ_TileChkDataPtr_Lo       ; 
                          sta ._GetLvlDatAdr_Lo           ; 
                          
                          lda #>TabTileChkData            ; 
                          adc SNZ_TileChkDataPtr_Hi       ; 
                          sta ._GetLvlDatAdr_Hi           ; 
                          
._GetLvlDatAdr_Lo         equ [* + $01]                   ; 
._GetLvlDatAdr_Hi         equ [* + $02]                   ; 
.GetLvlChkByte            lda SN_GetDataDummy             ; 
                          tax                             ; 
.IsoLvlChkByte            and #$3f                        ; ..######
                          
GameGetLevelCheckByteX    rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameMoveSnokieRight     Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameMoveSnokieRight       subroutine                      ; 
.ChkLevel_Finish          lda TabLvlDatSnokiePosX_Lo_P1   ; 
                          cmp #<TabCheckDataPtr_L8        ; 
                          bne .IniChkMovePossible_Ri      ; 
                          
                          lda TabLvlDatSnokiePosX_Hi_P1   ; 
                          cmp #>TabCheckDataPtr_L8        ; 
                          bne .IniChkMovePossible_Ri      ; 
                          
                          lda #TabSnokieJumpHeight_Off    ; 
                          sta TabSnokieJumpHeight         ; 
                          
.ExitLevel_Finish         rts                             ; 
                          
.IniChkMovePossible_Ri    clc                             ; 
                          lda TabLvlDatSnokiePosX_Lo_P1   ; 
                          adc #<TabLvlDatSnokiePosX_Step  ; 
.SetSnokieStep_Ri_Lo      sta SNZ_TileChkDataPtr_Lo       ; step Snokie forward
                          
                          lda TabLvlDatSnokiePosX_Hi_P1   ; 
                          adc #>TabLvlDatSnokiePosX_Step  ; 
.SetSnokieStep_Ri_Hi      sta SNZ_TileChkDataPtr_Hi       ; 
                          
.GoGetChkByte_Ri          jsr GameGetLevelCheckByte       ; 
.SetChkByte_Ri            sta ._ChkSnokiePosY_Ri          ; 
                          
                          lda TabLvlSnokiePosY_P1         ; 
._ChkSnokiePosY_Ri        equ [* + $01]                   ; 
                          cmp #$00                        ; 
.ChkMovePossible_Ri       bcs .GetSnokieMoveStep_Ri       ; greater/equal - move Snokie
                          
.WasObstacle_Ri           jmp GameSfxSnokieBumpWall       ; lower
                          
.GetSnokieMoveStep_Ri     ldx TabSnokieMoveStep           ; 
.IncSnokieMoveStep_Ri     inx                             ; 
                          txa                             ; 
                          and #$03                        ; ......##
.SetSnokieMoveStep_Ri     sta TabSnokieMoveStep           ; 
.ChkSnokieMoveStep_Ri_Min bne .GetSnokieLvlScrollPosX     ; 
                          
.SetLevelAdrPtr_Ri        lda SNZ_TileChkDataPtr_Lo       ; 
                          sta TabLvlDatSnokiePosX_Lo_P1   ; 
                          
                          lda SNZ_TileChkDataPtr_Hi       ; 
                          sta TabLvlDatSnokiePosX_Hi_P1   ; 
; -------------------------------------------------------------------------------------------------------------- ;
; start scrolling if checkpoint is reached
; -------------------------------------------------------------------------------------------------------------- ;
.GetSnokieLvlScrollPosX   ldx TabSnokieLvlScrollPosX      ; 
.TstSnokieLvlScrollPosX   cpx #TabSnokieLvlScrollPosX_Do  ; level scroll check point
.ChkSnokieLvlScrollPosX   bcs .GetSoftScrollValX          ; greater/equal - scroll level data right
                          
.IncSnokieLvlScrollPosX   inx                             ; 
.SetSnokieLvlScrollPosX   stx TabSnokieLvlScrollPosX      ; 
                          rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetSoftScrollValX        ldx TabLevelSoftScrollValue     ; 
.DecSoftScrollValX        dex                             ; 
                          txa                             ; 
                          and #$03                        ; ......##
.SetSoftScrollValX        sta TabLevelSoftScrollValue     ; 
.ChkSoftScrollValX_Max    cmp #$03                        ; 
                          beq .IncHardScrollValX          ; 
                          
.Exit_NoHardScroll        rts                             ; 
                          
.IncHardScrollValX        inc TabLevelHardScrollValue     ; 
                          
.IncLvlProgressAdr        inc TabLvlScrollSteps_Lo        ; 
                          bne GameMoveSnokieRightX        ; 
                          inc TabLvlScrollSteps_Hi        ; 
                          
GameMoveSnokieRightX      jmp GameIncScores               ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameMoveSnokieLeft      Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameMoveSnokieLeft        subroutine                      ; 
                          lda #$14                        ; 
                          cmp TabSnokieLvlScrollPosX      ; 
                          bcc .IniChkMovePossible_Le      ; lower
                          
                          lda #TabSnokieJumpHeight_Off    ; 
                          sta TabSnokieJumpHeight         ; 
                          
                          lda TabSnokieStatus             ; 
                          cmp #TabSnokieStatus_Slide      ; 
                          bne .Exit_NoMove                ; 
                          
.SetSnokieStatusSlide     inc TabSnokieStatus             ; 
                          jmp GamePlayerDecNumLives       ; 
                          
.Exit_NoMove              rts                             ; 
                          
.IniChkMovePossible_Le    sec                             ; set adr ptr to previous byt
                          lda TabLvlDatSnokiePosX_Lo_P1   ; 
                          sbc #<TabLvlDatSnokiePosX_Step  ; 
.SetSnokieStep_Le_Lo      sta SNZ_TileChkDataPtr_Lo       ; step Snokie back
                          
                          lda TabLvlDatSnokiePosX_Hi_P1   ; 
                          sbc #>TabLvlDatSnokiePosX_Step  ; 
.SetSnokieStep_Le_Hi      sta SNZ_TileChkDataPtr_Hi       ; 
                          
.GoGetChkByte_Le          jsr GameGetLevelCheckByte       ; 
.SetChkByte_Le            sta ._ChkSnokiePosY_Le          ; 
                          
                          lda TabLvlSnokiePosY_P1         ; 
._ChkSnokiePosY_Le        equ [* + $01]                   ; 
                          cmp #$00                        ; 
                          bcs .GetSnokieMoveStep_Le       ; 
                          
.WasObstacle_Le           jmp GameSfxSnokieBumpWall       ; 
                          
.GetSnokieMoveStep_Le     ldx TabSnokieMoveStep           ; 
.IncSnokieMoveStep_Le     dex                             ; 
                          txa                             ; 
                          and #$03                        ; ......##
.SetSnokieMoveStep_Le     sta TabSnokieMoveStep           ; 
                          
.TstSnokieMoveStep_Le_Max cmp #$03                        ; 
.ChkSnokieMoveStep_Le_Max bne .DecLvlScrollPosX           ; 
                          
.SetLevelAdrPtr_Le        lda SNZ_TileChkDataPtr_Lo       ; 
                          sta TabLvlDatSnokiePosX_Lo_P1   ; 
                          
                          lda SNZ_TileChkDataPtr_Hi       ; 
                          sta TabLvlDatSnokiePosX_Hi_P1   ; 
                          
.DecLvlScrollPosX         dec TabSnokieLvlScrollPosX      ; 
                          
                          rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameSnokieGetJumpDir    Does    : 
;                         Expects : 
;                         Returns : .X=00 - no move left or right
;                                 : .X=ff - move left
;                                 : .X=01 - move right
; -------------------------------------------------------------------------------------------------------------- ;
GameSnokieGetJumpDir      subroutine                      ; 
.IniFlagMove_No           ldx #$00                        ; 
                          
.GetSnokieMove_Le         lda TabSnokieMove               ; 
                          and #CIA_Joy_Le                 ; .....#..
.ChkSnokieMove_Le         bne .GetSnokieMove_Ri           ; 
                          
.SetFlagMove_Le           dex                             ; -1
                          
.GetSnokieMove_Ri         lda TabSnokieMove               ; 
                          and #CIA_Joy_Ri                 ; ....#...
.ChkSnokieMove_Ri         bne GameSnokieGetJumpDirX       ; 
                          
.SetFlagMove_Ri           inx                             ; 1
                          
GameSnokieGetJumpDirX     rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameSnokieChkOnTilePosY Does   : Check if Snokie PosY is egal to a level tile PosY at actual Snokie PosX
;                         Expects : 
;                         Returns : .A=00 - Snokie PosY does not match level data PosY
;                                   .A=01 - Snokie PosY matches level data PosY
;                                   .X    - Check tile data
; -------------------------------------------------------------------------------------------------------------- ;
GameSnokieChkOnTilePosY   subroutine                      ; 
.SetSnokieLvlTileChk_Ptr  clc                             ; 
                          lda #<TabTileChkData            ; 
                          adc TabLvlDatSnokiePosX_Lo_P1   ; 
                          sta ._GetChkTileData_Lo         ; 
                          
                          lda #>TabTileChkData            ; 
                          adc TabLvlDatSnokiePosX_Hi_P1   ; 
                          sta ._GetChkTileData_Hi         ; 
                          
._GetChkTileData_Lo       equ [* + $01]                   ; 
._GetChkTileData_Hi       equ [* + $02]                   ; 
.GetChkTileData           lda SN_GetDataDummy             ; 
.SavChkTileData           tax                             ; 
.IsoChkTileData_Height    and #SN_TileChkDataHeight       ; ..######
.TstSnokie_Height         cmp TabLvlSnokiePosY_P1         ; 
.ChkSnokie_Height         beq .SetFlagMatchDataPosY_Yes   ; 
                          
.SetFlagMatchDataPosY_No  lda #$00                        ; 
                          rts                             ; 
                          
.SetFlagMatchDataPosY_Yes lda #$01                        ; 
GameSnokieChkOnTilePosYX  rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameGetSnokieJump       Does    : 
;                         Expects : 
;                         Returns : TabSnokieMove = #...##.. --> jump none
;                                 : TabSnokieMove = ....##.. --> jump up
;                                 : TabSnokieMove = ....#... --> jump left
;                                 : TabSnokieMove = .....#.. --> jump right
; -------------------------------------------------------------------------------------------------------------- ;
GameGetSnokieJump         subroutine                      ; 
                          lda TabFlagDemo                 ; 
.ChkDemo_On               bne GameDemoMoveHandler         ; handle demo moves
; -------------------------------------------------------------------------------------------------------------- ;
.IniJoystickChkFire_On    lda #$00                        ; ........
.SetJoystickChkFire_On    sta CIDDRA                      ; CIA1($DC02) Data Direction Register A
; -------------------------------------------------------------------------------------------------------------- ;
; check input joystick jump (Fire)
; -------------------------------------------------------------------------------------------------------------- ;
                          lda CIAPRA                      ; CIA1($DC00) Data Port Register A
.IsoJoystickFire          and #CIA_Joy_Fi                 ; 
.ChkJoystickFire          beq .GetWaitForButtonFire       ; 0=pressed - check debounce wait time
; -------------------------------------------------------------------------------------------------------------- ;
; check input keyboard jump (space key)
; -------------------------------------------------------------------------------------------------------------- ;
.IniJoystickChkFire_Off   lda #$ff                        ; ########
.SetJoystickChkFire_Off   sta CIDDRA                      ; CIA1($DC02) Data Direction Register A
                          
.GetKeyboardSelRow_7      lda #[$ff ^ CIA_KeySelRow_07]   ; .####### - keyboard scan row mask - row bit to check=0
.SelKeyboardSelRow_7      sta CIAPRA                      ; CIA1($DC00) Data Port Register A
                          
.GetKeyboardResCol_Space  lda CIAPRB                      ; CIA1($DC01) Data Port Register B
.SetKeyboardResCol_Space  and #[$ff ^ CIA_KeyWas_Space]   ; ###.####
.ChkKeyboardResCol_Space  beq .GetWaitForButtonFire       ; 0=pressed - check debounce wait time
; -------------------------------------------------------------------------------------------------------------- ;
                          lda TabWaitForNextButtonFire    ; 
                          beq .GetSnokieFlag_Jump_No      ; 
                          
.DecWaitForButtonFire     dec TabWaitForNextButtonFire    ; 
                          jmp .GetSnokieFlag_Jump_No      ; 
; -------------------------------------------------------------------------------------------------------------- ;
; debounce: check wait time
; -------------------------------------------------------------------------------------------------------------- ;
.GetWaitForButtonFire     lda TabWaitForNextButtonFire    ; 
.ChkWaitForButtonFire     beq .IniWaitForButtonFire_02    ; 
                          
.IniWaitForButtonFire_01  lda #$04                        ; 
.SetWaitForButtonFire_01  sta TabWaitForNextButtonFire    ; 
; -------------------------------------------------------------------------------------------------------------- ;
; set flag: no jump
; -------------------------------------------------------------------------------------------------------------- ;
.GetSnokieFlag_Jump_No    lda #$80                        ; #.......
                          bne .SetSnokieMove_Jump         ; always
; -------------------------------------------------------------------------------------------------------------- ;
; set flag: jump
; -------------------------------------------------------------------------------------------------------------- ;
.IniWaitForButtonFire_02  lda #$04                        ; 
.SetWaitForButtonFire_02  sta TabWaitForNextButtonFire    ; 
                          
.GetSnokieMove_Jump_Yes   lda #TabSnokieJump              ; ........
; -------------------------------------------------------------------------------------------------------------- ;
.SetSnokieMove_Jump       sta TabSnokieMove               ; 
; -------------------------------------------------------------------------------------------------------------- ;
; check input joystick jump direction
; -------------------------------------------------------------------------------------------------------------- ;
.IniJoystickChkMove_On    lda #$00                        ; ........
.SetJoystickChkMove_On    sta CIDDRA                      ; CIA1($DC02) Data Direction Register A
                          
                          lda CIAPRA                      ; CIA1($DC00) Data Port Register A
.IsoJoystickMove_LR       and #[CIA_Joy_Ri | CIA_Joy_Le]  ; ....##..
.InsJoystickMove          ora TabSnokieMove               ; 
.SetJoystickMove          sta TabSnokieMove               ; 
; -------------------------------------------------------------------------------------------------------------- ;
; check input keyboard move
; -------------------------------------------------------------------------------------------------------------- ;
.IniJoystickChkMove_Off   lda #$ff                        ; ########
.SetJoystickChkMove_Off   sta CIDDRA                      ; CIA1($DC02) Data Direction Register A
                          
.GetKeyboardSelRow_0      lda #[$ff ^ CIA_KeySelRow_00]   ; #######.
.SelKeyboardSelRow_0      sta CIAPRA                      ; CIA1($DC00) Data Port Register A
                          
.GetKeyboardResCol_CRSR   lda CIAPRB                      ; CIA1($DC01) Data Port Register B
.IsoKeyboardResCol_CRSR   and #[[$ff ^ CIA_KeyWas_Crsr_D] | [$ff ^ CIA_KeyWas_Crsr_R]] ; #....#..
.Asl_CRSR_UD              asl a                           ; ....#... - eventually set jump right
.Chk_CRSR_UD              bcc .Ini_Flag_Jump_No           ; 
                          
.Ins_CRSR_LR              ora #TabSnokieMove_Ri           ; .....#.. - insert jump left
; -------------------------------------------------------------------------------------------------------------- ;
; set overall result
; -------------------------------------------------------------------------------------------------------------- ;
.Ini_Flag_Jump_No         ora #TabSnokieJump_None         ; #....... - ini flag got no jump
                          and TabSnokieMove               ; 
                          sta TabSnokieMove               ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IniKeyboardChk_On        lda #$00                        ; ........
.SetKeyboardChk_On        sta CIDDRA                      ; CIA1($DC02) Data Direction Register A
                          
GameGetSnokieJumpX        rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameDemoMoveHandler     Does    : Eventually get next demo move data pair / advance demo data pointer then
;                         Expects : 
;                         Returns : TabSnokieMove = #...##.. --> jump none
;                                 : TabSnokieMove = ....##.. --> jump up
;                                 : TabSnokieMove = ....#... --> jump left
;                                 : TabSnokieMove = .....#.. --> jump right
; -------------------------------------------------------------------------------------------------------------- ;
GameDemoMoveHandler       subroutine                      ; 
.GetDemoMoveWaitTime      lda TabSnokieMoveWaitTime       ; 
                          bne .DecDemoMoveWaitTime        ; 
                          
.IniDemoDataOff_Move      ldy #$00                        ; 
.GetDemoData_Move         lda (SNZ_DemoMoveDataPtr),y     ; 
.SetDemoData_Move         sta TabSnokieMove               ; 
                          
.IniDemoDataOff_Time      iny                             ; 
.GetDemoData_Time         lda (SNZ_DemoMoveDataPtr),y     ; 
.SetDemoData_Time         sta TabSnokieMoveWaitTime       ; 
                          
.IncDemoDataPtr           clc                             ; advance pointer to next data pair
                          lda SNZ_DemoMoveDataPtr_Lo      ; 
                          adc #<[TabDemoMoveData_Move_Len + $01] ; 
                          sta SNZ_DemoMoveDataPtr_Lo      ; 
                          lda SNZ_DemoMoveDataPtr_Hi      ; 
                          adc #>[TabDemoMoveData_Move_Len + $01] ; 
                          sta SNZ_DemoMoveDataPtr_Hi      ; 
                          
.DecDemoMoveWaitTime      dec TabSnokieMoveWaitTime       ; 
                          
GameDemoMoveHandlerX      rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameSnokieMoveHandler   Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameSnokieMoveHandler     subroutine                      ; 
.GetSnokieWaitJump        ldx TabSnokieWaitTimeJump       ; 
.ChkSnokieWaitJump_Min_1  beq .GetStatus_Stand            ; 
                          
.DecSnokieWaitJump        dex                             ; 
.SetSnokieWaitJump        stx TabSnokieWaitTimeJump       ; 
.ChkSnokieWaitJump_Min_2  beq .GetStatus_Death            ; 
                          
.ExitNoJump               rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; check status - death
; -------------------------------------------------------------------------------------------------------------- ;
.GetStatus_Death          lda TabSnokieStatus             ; 
.TstStatus_Death          cmp #TabSnokieStatus_Death      ; 
.ChkStatus_Death          bcs .ExitNoJump                 ; greater/equal
                          
                          lda #TabSnokieStatus_Active     ; move/stand
                          sta TabSnokieStatus             ; 
                          
.Exit_Stand               rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; check status - active (move/stand)
; -------------------------------------------------------------------------------------------------------------- ;
.GetStatus_Stand          lda TabSnokieStatus             ; 
.TstStatus_Stand          cmp #TabSnokieStatus_Active     ; move/stand
.ChkStatus_Stand          beq .GoGetSnokieJump            ; 
                          
.GetStatus_Slide          cmp #TabSnokieStatus_Slide      ; 
.ChkStatus_Slide          bne .ExitNoJump                 ; 
                          
.WasStatus_Slide          jmp .GoGetLvlSlide_TilePosY     ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GoGetSnokieJump          jsr GameGetSnokieJump           ; 
.GoGetLvlJump_TilePosY    jsr GameSnokieChkOnTilePosY     ; A=00 - No / A=01 - Yes / .X=Level tile
.GetJumpResult            tay                             ; 
.ChkJumpResult_Yes        bne .GetSnokieJumpFlag_No       ; 
                          
.WasJumpResult_No         jmp .GetSnokieJumpHeight        ; handle Snokie ups and downs
; -------------------------------------------------------------------------------------------------------------- ;
.GetSnokieJumpFlag_No     lda #TabSnokieJumpFlag_No       ; 
.SetSnokieJumpFlag_No     sta TabSnokieJumpFlag           ; 
                          
.GetSnokieLvlTile         txa                             ; 
.IsoLvlTile_Death         and #SN_TileChkDataDeath        ; ##......
.ChkLvlTile_Death_No      beq .IniJumpToneHeight          ; snokie has not fallen into a death pit
                          
                          jmp .TstTileData_Death          ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IniJumpToneHeight        jsr InitJumpToneHeight          ; 
                          
.TstFlagMov               bit TabSnokieMove               ; #.......
.ChkFlagMov_Yes           bpl .GetSnokieFlagJump          ; 
                          
.GetSnokieMovFlag_Yes     lda #TabSnokieJumpDir_None      ; $00
.SetSnokieMovFlag         sta TabSnokieJumpDir            ; 
                          sta TabSnokieJumpHeight         ; TabSnokieJumpHeight_None
                          
.GoGetMovDir              jsr GameSnokieGetJumpDir        ; .X=00 - none / .X=ff - left / .X=01 - right
.GetMovDir                txa                             ; 
.ChkMovDir                bne .IniChkMovDir_LR            ; .X=ff - left / .X=01 - right
                          
.WasMovDir_None           lda #$03                        ; ......##
                          sta TabSnokieSfxTime            ; 
                          
.Exit_NoMove              rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; handle Snokie moves
; -------------------------------------------------------------------------------------------------------------- ;
.IniChkMovDir_LR          inc TabSnokieSfxTime            ; 
                          
.GetMovDir_LR             txa                             ; 
.ChkMovDir_LR             bpl .GetSnokieMovDir_Ri         ; .X=ff - left / .X=01 - right
                          
.GetSnokieMovDir_Le       lda #TabSnokieMoveDir_Le        ; 
.SetSnokieMovDir_Le       sta TabSnokieMoveDir            ; 
                          
.GoSnokieMov_Le           jmp GameMoveSnokieLeft          ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetSnokieMovDir_Ri       lda #TabSnokieMoveDir_Ri        ; 
                          sta TabSnokieMoveDir            ; 
                          
.GoSnokieMov_Ri           jmp GameMoveSnokieRight         ; 
; -------------------------------------------------------------------------------------------------------------- ;
; handle Snokie jumps
; -------------------------------------------------------------------------------------------------------------- ;
.GetSnokieFlagJump        lda #TabSnokieJumpFlag_Yes      ; 
.IniSnokieFlagJump        sta TabSnokieJumpFlag           ; 
                          
.GetJumpHeight_Max        lda TabSnokieJumpHeightMax      ; 
.IniJumpHeight            sta TabSnokieJumpHeight         ; 
                          
.GoGetJumpDir             jsr GameSnokieGetJumpDir        ; .X=00 - none / .X=ff - left / .X=01 - right
.SetJumpFlag              stx TabSnokieJumpDir            ; 
.GetJumpDir               txa                             ; 
.ChkJumpDir_LR_No         beq .IniSnokieSfxTime           ; $00=no move left/right
; -------------------------------------------------------------------------------------------------------------- ;
; create Snokie jump dir
; -------------------------------------------------------------------------------------------------------------- ;
.IncJumpDir_LR            inx                             ; .......# / ######## --> ......#. / ........
.GetJumpDir_LR            txa                             ;                         ......#. / ........
.CreJumpDir_LR            lsr a                           ;                         .......# / ........
.SetJumpDir_LR            sta TabSnokieMoveDir            ; $00 - TabSnokieMoveDir_Le / $01 - TabSnokieMoveDir_Ri
; -------------------------------------------------------------------------------------------------------------- ;
.IniSnokieSfxTime         lda #$04                        ; .....#..
                          sta TabSnokieSfxTime            ; 
                          bne .GetSnokieJumpHeight        ; always - handle Snokie ups and downs
; -------------------------------------------------------------------------------------------------------------- ;
.TstTileData_Death        cmp #SN_TileChkDataDeath        ; 
.ChkTileData_Death_No     bne .GetSnokieStatus_Slide      ; 
                          
.GetSnokieStatus_Death    lda #TabSnokieStatus_Death      ; 
                          sta TabSnokieStatus             ; 
                          
                          jsr InitJumpToneHeight          ; 
.Exit_Death               jmp GamePlayerDecNumLives       ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetSnokieStatus_Slide    lda #TabSnokieStatus_Slide      ; 
.GetSnokieStatus          sta TabSnokieStatus             ; 
                          
.GetSnokieJmpHeight_Off   lda #TabSnokieJumpHeight_Off    ; 
                          sta TabSnokieJumpHeight         ; 
                          
                          lda #TabSnokieJumpDir_None      ; 
                          sta TabSnokieJumpDir            ; 
                          
.GetLvlTileSlope          txa                             ; .X=Level tile
                          and #[SN_TileChkDataSlope_Le | SN_TileChkDataSlope_Ri] ; ##......
                          jmp .TstSnokieSlopeDir          ; 
; -------------------------------------------------------------------------------------------------------------- ;
; handle Snokie ups and downs
; -------------------------------------------------------------------------------------------------------------- ;
.GetSnokieJumpHeight      lda TabSnokieJumpHeight         ; 
.ChSnokieJumpHeight_None  beq .GetSnokieJumpHeightVal     ; 
.ChSnokieJumpHeight_Off   bpl .IncSnokiePosY              ; 
                          
.DecSnokiePosY            dec TabLvlSnokiePosY_P1         ; fall back down
                          jmp .GetSnokieJumpHeightVal     ; 
                          
.IncSnokiePosY            inc TabLvlSnokiePosY_P1         ; still jump up
; -------------------------------------------------------------------------------------------------------------- ;
.GetSnokieJumpHeightVal   ldx TabSnokieJumpHeight         ; 
.DecSnokieJumpHeightVal   dex                             ; 
.ChkSnokieJumpHeightVal   bpl .SetSnokieJumpHeight        ; 
                          
.GetSnokieJumpHeightOff   ldx #TabSnokieJumpHeight_Off    ; 
                          
.SetSnokieJumpHeight      stx TabSnokieJumpHeight         ; 
                          
                          jsr GameSfxSnokieJump           ; 
                          
                          lda TabSnokieJumpDir            ; 
.ChkSnokieMoveFlag_No     bne .ChkSnokieJumpDir_Ri        ; 
                          
.Exit_NoJumpLR            rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.ChkSnokieJumpDir_Ri      bpl .GoSnokieJump_Ri            ; TabSnokieJumpDir_Ri
                          
.GoSnokieJump_Le          jmp GameMoveSnokieLeft          ; 
                          
.GoSnokieJump_Ri          jmp GameMoveSnokieRight         ; 
; -------------------------------------------------------------------------------------------------------------- ;
; handle Snokie slides
; -------------------------------------------------------------------------------------------------------------- ;
.GoGetLvlSlide_TilePosY   jsr GameSnokieChkOnTilePosY     ; A=00 - No / A=01 - Yes / .X=Level tile
.GetSlideResult           tay                             ; 
.ChkSideResult_Yes        beq .GetSnokieJumpHeight        ; not on ground - handle Snokie (ups and) downs
                          
.WasSlideResult_No        txa                             ; 
.IsoSlideDir              and #[SN_TileChkDataSlope_Le | SN_TileChkDataSlope_Ri] ; ##......
.ChkSlideDir_End          bne .TstSnokieSlide_Death       ; try to continue sliding
                          
.GetWaitTimeJump          lda #$10                        ; finish sliding
.IniWaitTimeJump          sta TabSnokieWaitTimeJump       ; 
                          jsr InitJumpToneHeight          ; 
                          
                          lda TabSnokieMoveDir            ; 
.ChkSnokieSlideDir_Ri     bne .GoSlideSnokie_Ri_End       ; 
.ChkSnokieSlideDir_Le     beq .GoSlideSnokie_Le_End       ; always
; -------------------------------------------------------------------------------------------------------------- ;
.TstSnokieSlide_Death     cmp #SN_TileChkDataDeath        ; check death tile first
.ChkSnokieSlide_Death     bne .TstSnokieSlopeDir          ; continue sliding
                          
.WasSnokieSlide_Death     jmp .GetSnokieStatus_Death      ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GoSlideSnokie_Le_End     jsr GameMoveSnokieLeft          ; finish slide left
                          jsr GameMoveSnokieLeft          ; 
                          jmp GameMoveSnokieLeft          ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GoSlideSnokie_Ri_End     jsr GameMoveSnokieRight         ; finish slide right
                          jsr GameMoveSnokieRight         ; 
                          jmp GameMoveSnokieRight         ; 
; -------------------------------------------------------------------------------------------------------------- ;
.TstSnokieSlopeDir        cmp #SN_TileChkDataSlope_Le     ; continue sliding
.ChkSnokieSlope_Le        beq .WasSnokieSlope_Ri          ; 
                          
.WasSnokieSlope_Le        lda #TabSnokieMoveDir_Le        ; 
                          sta TabSnokieMoveDir            ; 
                          
.GoSlideSnokie_Le         jsr GameMoveSnokieLeft          ; 
                          jmp GameSfxSnokieSlide          ; 
; -------------------------------------------------------------------------------------------------------------- ;
.WasSnokieSlope_Ri        lda #TabSnokieMoveDir_Ri        ; 
                          sta TabSnokieMoveDir            ; 
                          
.GoSlideSnokie_Ri         jsr GameMoveSnokieRight         ; 
GameSnokieMoveHandlerX    jmp GameSfxSnokieSlide          ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GamePlayerDecNumLives   Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GamePlayerDecNumLives     subroutine                      ; 
.GetActivePlayer          lda TabActivePlayer             ; 
.TstActivePlayer_         cmp #TabActivePlayer_P1         ; 
.ChkActivePlayer__One     bne .DecNumLivesPlayer_02       ; 
                          
.DecNumLivesPlayer_01     dec TabNumLives_P1              ; 
                          rts                             ; 
                          
.DecNumLivesPlayer_02     dec TabNumLives_P2              ; 
GamePlayerDecNumLivesX    rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitJumpToneHeight      Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitJumpToneHeight        subroutine                      ; 
                          lda #VcRegTEST                  ; ....#... - 1=Lock Oscillator
                          sta VCREG1                      ; SID($D404) Oscillator 1 Control
                          
.GetTabJumpToneHeightIni  lda #$2d                        ; 
.SetTabJumpToneHeight     sta TabJumpToneHeight           ; 
                          
InitJumpToneHeightX       rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameSfxJumpHeightHandler Does   : Inc / dec jump tone height
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameSfxJumpHeightHandler  subroutine                      ; 
                          lda TabSnokieJumpHeight         ; 
                          beq .SetJumpToneHeight          ; 
                          bpl .IncJumpToneHeight          ; 
                          
.DecJumpToneHeight        dec TabJumpToneHeight           ; 
                          jmp .SetJumpToneHeight          ; 
                          
.IncJumpToneHeight        inc TabJumpToneHeight           ; 
                          
.SetJumpToneHeight        lda TabJumpToneHeight           ; 
                          sta FREHI1                      ; SID($D401) Oscillator 1 Freq Ctrl (high byte)
                          
                          lda #$00                        ; 
                          sta FRELO1                      ; SID($D400) Oscillator 1 Freq Ctrl (low byte)
                          
                          lda #[VcRegTRI | VcRegGATE]     ; ...#...#
                          sta VCREG1                      ; SID($D404) Oscillator 1 Control
                          
GameSfxJumpHeightHandlerX rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameSfxSnokieSlide      Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameSfxSnokieSlide        subroutine                      ; 
                          lda TabJumpToneHeight           ; 
                          sta FREHI1                      ; SID($D401) Oscillator 1 Freq Ctrl (high byte)
                          
                          lda #$00                        ; 
                          sta FRELO1                      ; SID($D400) Oscillator 1 Freq Ctrl (low byte)
                          
                          lda #[VcRegNOISE | VcRegGATE]   ; #......#
                          sta VCREG1                      ; SID($D404) Oscillator 1 Control
                          
GameSfxSnokieSlideX       rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameSfxSnokieJump       Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameSfxSnokieJump         subroutine                      ; 
                          lda TabSnokieJumpFlag           ; 
.ChkSnokieJumpFlag_No     beq .DecJumpTone                ; 
                          
.WasSnokieJumpFlag_Yes    jmp GameSfxJumpHeightHandler    ; Inc / dec jump tone height
                          
.DecJumpTone              dec TabJumpToneHeight           ; 
                          
.SetJumpToneOff           lda #VcRegTEST                  ; ....#... - 1=Lock Oscillator
                          sta VCREG1                      ; SID($D404) Oscillator 1 Control
                          
GameSfxSnokieJumpX        rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameSfxSnokieBumpWall   Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameSfxSnokieBumpWall     subroutine                      ; 
                          lda TabSnokieSfxTime            ; 
                          and #$04                        ; .....#..
                          beq .SetSfxSnokieBump           ; 
                          
.Exit_NoSfx               rts                             ; 
                          
.SetSfxSnokieBump         lda #$32                        ; 
                          sta FREHI1                      ; SID($D401) Oscillator 1 Freq Ctrl (high byte)
                          lda #$00                        ; 
                          sta FRELO1                      ; SID($D400) Oscillator 1 Freq Ctrl (low byte)
                          
                          lda #[VcRegNOISE | VcRegGATE]   ; #......#
                          sta VCREG1                      ; SID($D404) Oscillator 1 Control
                          
GameSfxSnokieBumpWallX    rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameSnokieSetJumpHeight Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameSnokieSetJumpHeight   subroutine                      ; 
                          sec                             ; 
                          lda TabLvlDatSnokiePosX_Lo_P1   ; 
                          sbc #<[TabTileChkData_L5_Start + $02] ; 
                          
                          lda TabLvlDatSnokiePosX_Hi_P1   ; 
                          sbc #>[TabTileChkData_L5_Start + $02] ; 
                          bmi GameSnokieSetJumpHeightX    ; 
                          
.GetCaveJumpHeight        lda #TabSnokieJumpHeight_Cave   ; 
.SetCaveJumpHeight        sta TabSnokieJumpHeightMax      ; 
                          
GameSnokieSetJumpHeightX  rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabPlate_L05_PosX_Lo      equ  * ; 
                          dc.b <[TabTileChkData_L5_Plate_1 + $01] ; 
                          dc.b <[TabTileChkData_L5_Plate_2 + $01] ; 
                          dc.b <[TabTileChkData_L5_Plate_3 + $01] ; 
                          dc.b <[TabTileChkData_L5_Plate_4 + $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabPlate_L05_PosX_Hi      equ  * ; 
                          dc.b >[TabTileChkData_L5_Plate_1 + $01] ; 
                          dc.b >[TabTileChkData_L5_Plate_2 + $01] ; 
                          dc.b >[TabTileChkData_L5_Plate_3 + $01] ; 
                          dc.b >[TabTileChkData_L5_Plate_4 + $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabPlate_L05_MoveDir      equ  *   ; 
TabPlate_L05_MoveDir_Le      = $01 ; 
TabPlate_L05_MoveDir_Ri      = $ff ; 
                          dc.b TabPlate_L05_MoveDir_Ri ; 
                          dc.b TabPlate_L05_MoveDir_Le ; 
                          dc.b TabPlate_L05_MoveDir_Ri ; 
                          dc.b TabPlate_L05_MoveDir_Le ; 
TabPlate_L05_Len          equ  [* - TabPlate_L05_MoveDir - $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabPlate_L05_MovePos      equ  *   ; 
TabPlate_L05_MovePos_Min     = $00 ; 
TabPlate_L05_MovePos_Max     = $0a ; 
                          dc.b $01 ; 
                          dc.b $06 ; 
                          dc.b $04 ; 
                          dc.b $0a ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabPlate_L05_TileNum      equ  *   ; 
TabPlate_L05_TileNum_Min     = $00 ; 
TabPlate_L05_TileNum_Max     = $03 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameMovePlates_L05      Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameMovePlates_L05        subroutine                      ; 
                          lda TabLevelNumCurrent          ; 
                          cmp #TabLevelNumCurrent_04      ; 
                          bcc .Exit_NoMove                ; lower
                          
                          cmp #TabLevelNumCurrent_07      ; 
                          bcs .Exit_NoMove                ; greater/equal
                          
.GetTilesMoveFlag         lda TabLvlTilesMoveFlag         ; 
.ChkTilesMoveFlag_Yes     beq .IniPlateTileCount          ; TabLvlTilesMoveFlag_Yes
                          
.Exit_NoMove              rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IniPlateTileCount        ldx #$00                        ; 
.GetNextPlateMovDir       lda TabPlate_L05_MoveDir,x      ; 
.ChkPlateMovDir_Le        bpl .GetPlateTileNum_Le         ; 
                          
.GetPlateTileNum_Ri       ldy TabPlate_L05_TileNum,x      ; 
.ChkPlateTileNum_Ri_Min   beq .GetPlateMovPosY_Min        ; 
                          
                          dey                             ; 
                          tya                             ; 
                          sta TabPlate_L05_TileNum,x      ; 
                          
                          jmp .GoSnokieAutoMove           ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetPlateMovPosY_Min      lda TabPlate_L05_MovePos,x      ; 
.ChkPlateMovPosY_Min      beq .RevPlateMovDir             ; 
                          
                          lda #TabPlate_L05_TileNum_Max   ; 
                          sta TabPlate_L05_TileNum,x      ; 
                          
                          dec TabPlate_L05_MovePos,x      ; 
                          
                          jmp .GoSnokieAutoMove           ; 
; -------------------------------------------------------------------------------------------------------------- ;
; flip flop the plate move dir an upper or lower end ($01->$ff / $ff->$01)
; -------------------------------------------------------------------------------------------------------------- ;
.RevPlateMovDir           sec                             ; 
                          lda #$00                        ; 
                          sbc TabPlate_L05_MoveDir,x      ; reverse
                          sta TabPlate_L05_MoveDir,x      ; 
                          
                          jmp .GoSnokieAutoMove           ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetPlateTileNum_Le       ldy TabPlate_L05_TileNum,x      ; 
.TstPlateTileNum_Le_Max   cpy #TabPlate_L05_TileNum_Max   ; 
.ChkPlateTileNum_Le_Min   beq .GetPlateMovPosY_Max        ; 
                          
.IncPlateTileNum_Le       iny                             ; 
                          tya                             ; 
                          sta TabPlate_L05_TileNum,x      ; 
                          
                          jmp .GoSnokieAutoMove           ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetPlateMovPosY_Max      lda TabPlate_L05_MovePos,x      ; 
                          cmp #$0b                        ; 
                          beq .RevPlateMovDir             ; 
                          
                          lda #TabPlate_L05_TileNum_Min   ; 
                          sta TabPlate_L05_TileNum,x      ; 
                          
                          inc TabPlate_L05_MovePos,x      ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GoSnokieAutoMove         jsr GameSnokieAutoMove_L05      ; 
; -------------------------------------------------------------------------------------------------------------- ;
                          clc                             ; 
                          lda #<TabLevelData_01_Row_0e    ; 
                          adc TabPlate_L05_PosX_Lo,x      ; 
                          sta SNZ_Plate_L05_DataOutPtr_Lo ; 
                          
                          lda #>TabLevelData_01_Row_0e    ; 
                          adc TabPlate_L05_PosX_Hi,x      ; 
                          sta SNZ_Plate_L05_DataOutPtr_Hi ; 
                          
                          clc                             ; 
                          lda #<TabTileChkData            ; 
                          adc TabPlate_L05_PosX_Lo,x      ; 
                          sta SNZ_TabLvlChkDataOutPtr_Lo  ; 
                          
                          lda #>TabTileChkData            ; 
                          adc TabPlate_L05_PosX_Hi,x      ; 
                          sta SNZ_TabLvlChkDataOutPtr_Hi  ; 
; -------------------------------------------------------------------------------------------------------------- ;
                          ldy #$00                        ; 
.GetNextTilePosOld_Le     tya                             ; 
.TstTilePosOld_Le         cmp TabPlate_L05_MovePos,x      ; 
.TstTilePosOld_Le_Min     beq .GetTileNumEnd_Le           ; 
                          
.GetTileNumClr_Le         lda #$50                        ; 
.SetTileNumClr_Le         sta (SNZ_Plate_L05_DataOutPtr),y; 
                          
.GetTileData_Water_Le     lda #SN_TileChkDataDeath        ; ##...... - death
                          sta (SNZ_TabLvlChkDataOutPtr),y ; 
                          
                          iny                             ; 
                          jmp .GetNextTilePosOld_Le       ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetTileNumEnd_Le         lda TabPlate_L05_TileNum,x      ; 
                          asl a                           ; 
                          ora #$40                        ; .#......
.SetTileNumEnd_Le         sta (SNZ_Plate_L05_DataOutPtr),y; 
                          
.GetTileData_Plate_Le     lda #$0f                        ; ....####
                          sta (SNZ_TabLvlChkDataOutPtr),y ; 
                          
                          iny                             ; 
                          
.GetTileMidCopyCount_Max  lda #$03                        ; 
.IniTileMidCopyCount      sta SNZ_CopyCount               ; 
.GetNextTileNumMid        lda #$52                        ; 
                          sta (SNZ_Plate_L05_DataOutPtr),y; 
                          
.GetTileData_Plate_Mid    lda #$0f                        ; ....####
                          sta (SNZ_TabLvlChkDataOutPtr),y ; 
                          
                          iny                             ; 
                          
.DecTileMidCopyCount      dec SNZ_CopyCount               ; 
.ChkTileMidCopyCount_Min  bne .GetNextTileNumMid          ; 
                          
.GetTileNumEnd_Ri         lda TabPlate_L05_TileNum,x      ; 
                          asl a                           ; 
                          ora #$48                        ; .#..#...
.SetTileNumEnd_Ri         sta (SNZ_Plate_L05_DataOutPtr),y; 
                          
.GetTileData_Plate_Ri     lda #$0f                        ; ....####
                          sta (SNZ_TabLvlChkDataOutPtr),y ; 
; -------------------------------------------------------------------------------------------------------------- ;
                          iny                             ; 
                          
.TstNextTilePosOld_Ri     cpy #$10                        ; 
.ChkNextTilePosOld_Ri_Max bcs .IncPlateTileCount          ; greater/equal
                          
.GetTileNumClr_Ri         lda #$50                        ; lower - clear remiders on right end of plate
.SetTileNumClr_Ri         sta (SNZ_Plate_L05_DataOutPtr),y; 
                          
.GetTileData_Water_Ri     lda #SN_TileChkDataDeath        ; ##...... - death
                          sta (SNZ_TabLvlChkDataOutPtr),y ; 
                          
                          iny                             ; 
                          jmp .TstNextTilePosOld_Ri       ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IncPlateTileCount        inx                             ; 
.TstPlateTileCount        cpx #[TabPlate_L05_Len + $01]   ; 
.ChkPlateTileCount_Max    beq GameMovePlates_L05X         ; 
                          
                          jmp .GetNextPlateMovDir         ; 
                          
GameMovePlates_L05X       rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameSnokieAutoMove_L05  Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameSnokieAutoMove_L05    subroutine                      ; 
                          sec                             ; 
                          lda TabLvlDatSnokiePosX_Lo_P1   ; 
                          sbc TabPlate_L05_PosX_Lo,x      ; 
                          sta SNZ_SnokiePlatePosX_Lo      ; 
                          
                          lda TabLvlDatSnokiePosX_Hi_P1   ; 
                          sbc TabPlate_L05_PosX_Hi,x      ; 
                          beq .GetChkAutoMovePosX_Max     ; 
                          
.Exit_NoMove              rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetChkAutoMovePosX_Max   lda SNZ_SnokiePlatePosX_Lo      ; 
.TstAutoMovePosX_Max      cmp #$10                        ; 
.ChkAutoMovePosX_Max      bcs .Exit_NoMove                ; greater/equal
                          
                          stx SNZ_L05_SaveXr              ; 
                          jsr GameSnokieChkOnTilePosY     ; A=00 - No / A=01 - Yes / .X=Level tile
                          ldx SNZ_L05_SaveXr              ; 
                          
                          tay                             ; 
.ChkSnokiePosY            beq .Exit_NoMove                ; 
                          
.GetSnokieStatus          lda TabSnokieStatus             ; 
.TstSnokieStatus_Stand    cmp #TabSnokieStatus_Active     ; move/stand
.ChkSnokieStatus_Stand    bne .Exit_NoMove                ; 
                          
.GetPlateMovDir           lda TabPlate_L05_MoveDir,x      ; 
.ChkPlateMovDir_Ri        bpl .GoMovSnokie_Ri             ; 
                          
.GoMovSnokie_Le           jsr GameMoveSnokieLeft          ; 
                          
                          ldx SNZ_L05_SaveXr              ; 
                          rts                             ; 
                          
.GoMovSnokie_Ri           jsr GameMoveSnokieRight         ; 
                          
                          ldx SNZ_L05_SaveXr              ; 
GameSnokieAutoMove_L05X   rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameSnokieCheckFinish   Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameSnokieCheckFinish     subroutine                      ; 
                          lda TabLvlTilesMoveFlag         ; 
                          eor TabGameSpeed_P1             ; will only be TabLvlTilesMoveFlag_No if TabGameSpeed_Fast
                          sta TabLvlTilesMoveFlag         ; 
                          
.GetFinish_Lo             lda TabLvlDatSnokiePosX_Lo_P1   ; 
.TstFinish_Lo             cmp #<TabCheckDataPtr_L8        ; 
.ChkFinish_Lo             bne GameSnokieCheckFinishX      ; 
                          
.GetFinish_Hi             lda TabLvlDatSnokiePosX_Hi_P1   ; 
.TstFinish_Hi             cmp #>TabCheckDataPtr_L8        ; 
.ChkFinish_Hi             bne GameSnokieCheckFinishX      ; 
                          
.ChkSnokieStatus          lda TabSnokieStatus             ; 
                          cmp #TabSnokieStatus_Active     ; move/stand
                          bne GameSnokieCheckFinishX      ; 
                          
                          jsr GameSnokieChkOnTilePosY     ; A=00 - No / A=01 - Yes / .X=Level tile
                          tay                             ; 
                          beq GameSnokieCheckFinishX      ; 
                          
                          lda TabSnokieSfxTime            ; 
                          and #$04                        ; .....#..
                          bne GameSnokieCheckFinishX      ; 
                          
.GetScoreBlink            lda TabHiScoreBlink             ; 
.TstScoreVisible          cmp #TabHiScoreBlink_On         ; greater/equal - invisible
.ChkScoreVisible          bcs GameSnokieCheckFinishX      ; 
; -------------------------------------------------------------------------------------------------------------- ;
; make all levels selectable from the main menu 
;   but only if the game was solved without a previous level selection
; -------------------------------------------------------------------------------------------------------------- ;
.GetNumLives_Select       lda TabSelectNumLives           ; 
.TstNumLives_Select       cmp #SN_SnokieNumLivesCheat     ; 
.ChkNumLives_Select       beq .GoGameFinishHandler        ; check if level was selected directly from main menu
; -------------------------------------------------------------------------------------------------------------- ;
.GetLevelSelect_Max       lda #TabLevelSelect_Max         ; 
.SetLevelSelect           sta TabLevelSelect              ; make all levels selectable from the main menu
; -------------------------------------------------------------------------------------------------------------- ;
.GoGameFinishHandler      jsr GameFinishHandler           ; 
                          
                          lda #TabGameSpeed_Fast          ; 
                          sta TabLvlTilesMoveFlag         ; TabLvlTilesMoveFlag_Yes
                          sta TabGameSpeed_P1             ; 
                          
.IniLvlDatSnokiePos       lda #<[TabCheckDataPtr_L1 + <TabLvlDatSnokiePosX_Start]; restart at level 01
                          sta TabLvlDatSnokiePosX_Lo_P1   ; 
                          
                          lda #>[TabCheckDataPtr_L1 + >TabLvlDatSnokiePosX_Start]; 
                          sta TabLvlDatSnokiePosX_Hi_P1   ; 
                          
                          jsr InitGameVarsPartial         ; 
                          
GameSnokieCheckFinishX    rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabLevelSelect            dc.b [TabLevelNumCurrent_01 + $01] ; 
TabLevelSelect_Max           = [TabLevelNumCurrent_07 + $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameFinishHandler       Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameFinishHandler         subroutine                      ; 
.IniSprtHeartPieces       lda #[SpriteData_HeartLe / VIC_Sprt_BlockLen] ; 
                          sta SN_GameSpritePtr_06         ; 
                          lda #[SpriteData_HeartRi / VIC_Sprt_BlockLen] ; 
                          sta SN_GameSpritePtr_07         ; 
                          
                          lda SPMC                        ; VIC($D01C) Sprite Multicolor
                          and #$3f                        ; ..######
                          sta SPMC                        ; VIC($D01C) Sprite Multicolor
                          
                          lda #LT_RED                     ; 
                          sta SP6COL                      ; VIC($D02D) Color Sprite 6
                          sta SP7COL                      ; VIC($D02E) Color Sprite 7
                          
                          lda #$66                        ; 
                          sta SP6Y                        ; VIC($D00D) Sprite 6 Y-Pos (Bits 0-7)
                          sta SP7Y                        ; VIC($D00F) Sprite 7 Y-Pos (Bits 0-7)
                          
                          lda #$94                        ; 
                          sta SP6X                        ; VIC($D00C) Sprite 6 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          
                          lda #$d2                        ; 
                          sta SP7X                        ; VIC($D00E) Sprite 7 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          
.IniWaitCount             ldx #$1f                        ; 
.WaitNextWhile            jsr WaitAWhile                  ; 
                          
                          dec SP7X                        ; VIC($D00E) Sprite 7 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          inc SP6X                        ; VIC($D00C) Sprite 6 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          
.DecWaitCount             dex                             ; 
.ChkWaitCount_Min         bne .WaitNextWhile              ; 
                          
                          jsr GameTuneFinishHandler       ; 
                          jsr InitGameSpritesTime         ; 
                          
GameFinishHandlerX        rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; WaitAWhile              Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
WaitAWhile                subroutine                      ; 
.Sav_X                    txa                             ; 
                          
                          ldx #$14                        ; 
                          ldy #$00                        ; 
.DecNextWaitVal           dey                             ; 
                          bne .DecNextWaitVal             ; 
                          dex                             ; 
                          bne .DecNextWaitVal             ; 
                          
.Rst_X                    tax                             ; 
                          
WaitAWhileX               rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabPlatePhase_LeRi        dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabPlateTilesL3           equ  *   ; 
TabPlateTilesL3_Start     equ  *   ; 
TabPlateChk_Start         dc.b $00 ; 
TabPlateChk_Mid           dc.b $00 ; $00
TabPlateChk_End           dc.b $01 ; $01
                          dc.b $01 ; $02
                          dc.b $01 ; $03
; -------------------------------------------------------------------------------------------------------------- ;
TabPlateTilesL3_Out       equ  *   ; 
                          dc.b $01 ; $04 - $00
                          dc.b $01 ; $05 - $01
                          dc.b $01 ; $06 - $02
                          dc.b $00 ; $07 - $03
                          dc.b $00 ; $08 - $04
                          dc.b $01 ; $09 - $05
                          dc.b $01 ; $0a - $06
                          dc.b $01 ; $0b - $07
                          dc.b $00 ; $0c - $08
                          dc.b $00 ; $0d - $09
                          dc.b $01 ; $0e - $0a
                          dc.b $01 ; $0f - $0b
                          dc.b $01 ; $10 - $0c
                          dc.b $01 ; $11 - $0d
                          dc.b $01 ; $12 - $0e
                          dc.b $01 ; $13 - $0f
                          dc.b $01 ; $14 - $10
                          dc.b $01 ; $15 - $11
                          dc.b $01 ; $16 - $12
                          dc.b $01 ; $17 - $13
                          dc.b $01 ; $18 - $14
                          dc.b $01 ; $19 - $15
                          dc.b $00 ; $1a - $16
                          dc.b $00 ; $1b - $17
                          dc.b $01 ; $1c - $18
                          dc.b $01 ; $1d - $19
                          dc.b $01 ; $1e - $1a
                          dc.b $01 ; $1f - $1b
                          dc.b $01 ; $20 - $1c
                          dc.b $01 ; $21 - $1d
                          dc.b $00 ; $22 - $1e
                          dc.b $00 ; $23 - $1f
                          dc.b $01 ; $24 - $20
                          dc.b $01 ; $25 - $21
                          dc.b $01 ; $26 - $22
TabPlateTilesL3_Out_Len   equ  [* - TabPlateTilesL3_Out - $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
                          dc.b $01 ; $27 - 
                          dc.b $01 ; $28 - 
                          dc.b $00 ; $29 - 
                          dc.b $00 ; $2a - 
TabPlateTilesL3_End       equ  *   ; 
TabPlateTilesL3_Len       equ  [* - [TabPlateTilesL3 + $01] - $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameMovePlates_L03      Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameMovePlates_L03        subroutine                      ; 
                          lda TabLevelNumCurrent          ; 
.ChkLvl_01                beq .Exit_NoMove                ; TabLevelNumCurrent_01
                          
.ChkLvl_05                cmp #TabLevelNumCurrent_05      ; 
                          bcs .Exit_NoMove                ; greater/equal
                          
.GetTilesMoveFlag         lda TabLvlTilesMoveFlag         ; 
.ChkTilesMoveFlag_Yes     beq .MovPlates                  ; TabLvlTilesMoveFlag_Yes
                          
.Exit_NoMove              rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.MovPlates                ldx TabPlatePhase_LeRi          ; 
                          dex                             ; 
                          txa                             ; 
                          and #$03                        ; ......##
                          sta TabPlatePhase_LeRi          ; 
                          
.TstPlatePhase_LeRi_Max   cmp #$03                        ; 
.ChkPlatePhase_LeRi_Max   bne .GoSnokieAutoMove           ; 
; -------------------------------------------------------------------------------------------------------------- ;
; move the platform tile table one pos to the left
; -------------------------------------------------------------------------------------------------------------- ;
.IniPlateTileCount        ldx #$00                        ; 
.GetNextPlateTile         lda [TabPlateTilesL3 + $01],x   ; 
.SetPlateTile             sta [TabPlateTilesL3 + $00],x   ; copy platform tiles
.IncPlateTileCount        inx                             ; 
.TstPlateTileCount_Max    cpx #TabPlateTilesL3_Len        ; 
.ChkPlateTileCount_Max    bne .GetNextPlateTile           ; 
                          
                          lda [TabPlateTilesL3_Start + $00] ; copy top tiles to table end
                          sta [TabPlateTilesL3_End   - $02] ; 
                          
                          lda [TabPlateTilesL3_Start + $01] ; 
                          sta [TabPlateTilesL3_End   - $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
; update level tile check data
; -------------------------------------------------------------------------------------------------------------- ;
.IniTileDataPtr           clc                             ; 
                          lda #<TabTileChkData            ; 
                          adc #<[TabCheckDataPtr_L3 + [TabTileChkData_L3_Hdr_Len + $01]]; 
                          sta SNZ_TileChkDataPtr_Lo       ; 
                          
                          lda #>TabTileChkData            ; 
                          adc #>[TabCheckDataPtr_L3 + [TabTileChkData_L3_Hdr_Len + $01]]; 
                          sta SNZ_TileChkDataPtr_Hi       ; 
                          
.IniOutTableCount         ldy #$00                        ; 
.GetNextOutTableByte      lda TabPlateTilesL3_Out,y       ; 
                          bne .GetTileData_Plate          ; 
                          
.GetTileData_Water        lda #SN_TileChkDataDeath        ; ##...... - death
                          bne .SetTileCheckData           ; always
                          
.GetTileData_Plate        lda #$1f                        ; ...#####
                          
.SetTileCheckData         sta (SNZ_TileChkDataPtr),y      ; 
                          
.IncOutTableCount         iny                             ; 
.TstOutTableCount_Max     cpy #TabPlateTilesL3_Out_Len    ; 
.ChkOutTableCount_Max     bne .GetNextOutTableByte        ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GoSnokieAutoMove         jsr GameSnokieAutoMove_L03      ; 
; -------------------------------------------------------------------------------------------------------------- ;
; update screen level data
; -------------------------------------------------------------------------------------------------------------- ;
.IniLevelDataPtr          clc                             ; 
                          lda #<TabLevelData_01_Row_0a    ; 
                          adc #<[SN_Scrn_RowLen * $07]    ; TabLevelData_03_Row_11
                          sta SNZ_Plate_L03_DataOutPtr_Lo ; 
                          
                          lda #>TabLevelData_01_Row_0a    ; 
                          adc #>[SN_Scrn_RowLen * $07]    ; 
                          sta SNZ_Plate_L03_DataOutPtr_Hi ; 
                          
.IniPlateTileTabLen       ldy #$00                        ; 
.GetNextPlatTileTabByte   lda [TabPlateTilesL3 + $01],y   ; 
                          bne .GetTilePlate_Start         ; 
                          
.GetTile_Void             lda #$50                        ; 
.OutTile_Void             jmp .OutPlateTile               ; 
                          
.GetTilePlate_Start       lda TabPlateChk_Start,y         ; 
.ChkTilePlate_Start_Void  bne .GetTilePlate_End           ; 
                          
                          lda TabPlatePhase_LeRi          ; 
                          asl a                           ; *2
.SetTilePlateStart        ora #$40                        ; .#......
.OutTilePlateStart        jmp .OutPlateTile               ; 
                          
.GetTilePlate_End         lda TabPlateChk_End,y           ; 
.ChkTilePlate_End_Void    bne .GetTilePlate_Mid           ; 
                          
                          lda TabPlatePhase_LeRi          ; 
                          asl a                           ; 
.SetTilePlate_End         ora #$48                        ; .#..#...
.OutTilePlate_End         jmp .OutPlateTile               ; 
                          
.GetTilePlate_Mid         lda #$52                        ; 
                          
.OutPlateTile             sta (SNZ_Plate_L03_DataOutPtr),y; 
                          
.IncPlateTileTabLen       iny                             ; 
.TstPlateTileTabLen       cpy #TabPlateTilesL3_Len        ; 
.ChkPlateTileTabLen_Max   bne .GetNextPlatTileTabByte     ; 
                          
GameMovePlates_L03X       rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameSnokieAutoMove_L03  Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameSnokieAutoMove_L03    subroutine                      ; 
                          sec                             ; 
                          lda TabLvlDatSnokiePosX_Lo_P1   ; 
                          sbc #<[TabTileChkData_L3_Start + $01] ; 
                          sta SNZ_SnokiePlatePosX_Lo      ; 
                          
                          lda TabLvlDatSnokiePosX_Hi_P1   ; 
                          sbc #>[TabTileChkData_L3_Start + $01] ; 
                          beq .GetChkAutoMovePosX_Max     ; 
                          
.Exit_NoAutoMove          rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetChkAutoMovePosX_Max   lda SNZ_SnokiePlatePosX_Lo      ; 
.TstAutoMovePosX_Max      cmp #$22                        ; 
.ChkAutoMovePosX_Max      bcs .Exit_NoAutoMove            ; greater/equal
                          
                          jsr GameSnokieChkOnTilePosY     ; A=00 - No / A=01 - Yes / .X=Level tile
                          tay                             ; 
.ChkSnokiePosY            beq .Exit_NoAutoMove            ; 
                          
.GetSnokieStatus          lda TabSnokieStatus             ; 
.TstSnokieStatus_Stand    cmp #TabSnokieStatus_Active     ; move/stand
.ChkSnokieStatus_Stand    bne .Exit_NoAutoMove            ; move olny if stand
                          
GameSnokieAutoMove_L03X   jmp GameMoveSnokieLeft          ; move Snokie left
; -------------------------------------------------------------------------------------------------------------- ;
TabAdrLvlDatRows          equ  *      ; 
                          dc.w TabLevelData_01_Row_00 ; 
                          dc.w TabLevelData_01_Row_02 ; 
                          dc.w TabLevelData_01_Row_04 ; 
                          dc.w TabLevelData_01_Row_06 ; 
                          dc.w TabLevelData_01_Row_08 ; 
                          dc.w TabLevelData_01_Row_0a ; 
                          dc.w TabLevelData_01_Row_0c ; 
                          dc.w TabLevelData_01_Row_0e ; 
                          dc.w TabLevelData_01_Row_10 ; 
                          dc.w TabLevelData_01_Row_12 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabBeamLvlDatRowOff_Top   equ  *   ; 
                          dc.b $02 ; 
                          dc.b $01 ; 
                          dc.b $01 ; 
                          dc.b $01 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabBeamLvlDatRowOff_Bot   equ  *   ; 
                          dc.b $06 ; 
                          dc.b $06 ; 
                          dc.b $04 ; 
                          dc.b $03 ; 
; -------------------------------------------------------------------------------------------------------------- ;
Tab_L07_BeamStatus        equ  *   ; 
Tab_L07_BeamStatus_On        = $01 ; 
Tab_L07_BeamStatus_Off       = $00 ; 
Tab_L07_BeamStatus_01     dc.b Tab_L07_BeamStatus_On ; 
Tab_L07_BeamStatus_02     dc.b Tab_L07_BeamStatus_On ; 
Tab_L07_BeamStatus_03     dc.b Tab_L07_BeamStatus_On ; 
Tab_L07_BeamStatus_04     dc.b Tab_L07_BeamStatus_On ; 
; -------------------------------------------------------------------------------------------------------------- ;
Tab_L07_BeamStatusTime    equ  *   ; 
                          dc.b $1f ; 
                          dc.b $17 ; 
                          dc.b $0f ; 
                          dc.b $07 ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameMoveBeams_L07       Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameMoveBeams_L07         subroutine                      ; 
                          lda TabLevelNumCurrent          ; 
                          cmp #TabLevelNumCurrent_06      ; 
                          bcc .Exit_NoMove                ; lower
                          
.GetTilesMoveFlag         lda TabLvlTilesMoveFlag         ; 
.ChkTilesMoveFlag_Yes     beq .IniBeamTilePtr             ; TabLvlTilesMoveFlag_Yes
                          
.Exit_NoMove              rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IniBeamTilePtr           lda #<[TabCheckDataPtr_L7 + [TabTileChkData_L7_Hdr_Len + $01]] ; 
                          sta SNZ_TileChkDataPtr_Lo       ; 
                          
                          lda #>[TabCheckDataPtr_L7 + [TabTileChkData_L7_Hdr_Len + $01]] ; 
                          sta SNZ_TileChkDataPtr_Hi       ; 
                          
.IniBeamStatusCount       ldx #$00                        ; 
.DecNextBeamStatusTime    dec Tab_L07_BeamStatusTime,x    ; 
                          bne .IncBeamTileDataPtr         ; 
                          
                          lda #$1f                        ; 
.IniBeamStatusTime        sta Tab_L07_BeamStatusTime,x    ; 
                          
.GetStatusBeam            lda Tab_L07_BeamStatus,x        ; 
.FlipFlopStatusBeam       eor #Tab_L07_BeamStatus_On      ; 
.SetStatusBeam            sta Tab_L07_BeamStatus,x        ; 
.ChkStatusBeam            bne .GoIniBeamsTileOn           ; 
                          
.GoIniBeamsTileOff        jsr GameMoveBeamsIniTileOff     ; 
                          jmp .IncBeamTileDataPtr         ; 
                          
.GoIniBeamsTileOn         jsr GameMoveBeamsIniTileOn      ; 
                          
.IncBeamTileDataPtr       clc                             ; 
                          lda SNZ_TileChkDataPtr_Lo       ; 
                          adc #<SN_BeamDistTileChkData    ; 
                          sta SNZ_TileChkDataPtr_Lo       ; 
                          
                          lda SNZ_TileChkDataPtr_Hi       ; 
                          adc #>SN_BeamDistTileChkData    ; 
                          sta SNZ_TileChkDataPtr_Hi       ; 
                          
.IncBeamStatusCount       inx                             ; 
.TstBeamStatusCount       cpx #[TabBeam_Len + $01]        ; 
.ChkBeamStatusCount_Max   bne .DecNextBeamStatusTime      ; 
                          
GameMoveBeams_L07X        rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameDataBeamsSet        Does    : set a 4*4 char beam part
;                         Expects : .A=num level data row tab row bottom
;                                 : .Y=num level data row tab row top
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameDataBeamsSet          subroutine                      ; 
                          tya                             ; 
.Sav_Yr                   pha                             ; 
                          
                          asl a                           ; *2
                          tay                             ; 
                          
.IniAdrLevelDataPtr_Top   clc                             ; 
                          lda [TabAdrLvlDatRows + $00],y  ; 
                          adc SNZ_TileChkDataPtr_Lo       ; 
                          sta SNZ_Beam_L07_DataOutPtr_Lo  ; 
                          
                          lda [TabAdrLvlDatRows + $01],y  ; 
                          adc SNZ_TileChkDataPtr_Hi       ; 
                          sta SNZ_Beam_L07_DataOutPtr_Hi  ; 
                          
.IniLevelDataByte_Top     ldy #$00                        ; 
                          lda SNZ_BeamDataTile            ; 
.SetLevelDataByte_Top_Le  sta (SNZ_Beam_L07_DataOutPtr),y ; 
                          iny                             ; 
.SetLevelDataByte_Top_Ri  sta (SNZ_Beam_L07_DataOutPtr),y ; 
                          
.IniAdrLevelDataPtr_Bot   clc                             ; 
                          lda SNZ_Beam_L07_DataOutPtr_Lo  ; 
                          adc #<[SN_Scrn_RowLen * $0f]    ; 
                          sta SNZ_Beam_L07_DataOutPtr_Lo  ; 
                          
                          lda SNZ_Beam_L07_DataOutPtr_Hi  ; 
                          adc #>[SN_Scrn_RowLen * $0f]    ; 
                          sta SNZ_Beam_L07_DataOutPtr_Hi  ; 
                          
.IniLevelDataByte_Bot     ldy #$00                        ; 
                          lda SNZ_BeamDataTile            ; 
                          clc                             ; 
                          adc #$01                        ; 
.SetLevelDataByte_Bot_Le  sta (SNZ_Beam_L07_DataOutPtr),y ; 
                          iny                             ; 
.SetLevelDataByte_Bot_Ri  sta (SNZ_Beam_L07_DataOutPtr),y ; 
                          
                          pla                             ; 
.Rst_Yr                   tay                             ; 
                          
GameDataBeamsSetX         rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameMoveBeamsIniTileOn  Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameMoveBeamsIniTileOn    subroutine                      ; 
                          lda #$0c                        ; 
                          sta SNZ_BeamDataTile            ; 
                          
.IniBeamTile_Bot_On       lda #$0e                        ; 
                          
.TstBeam_1st              cpx #$00                        ; 
.ChkBeam_1st              bne .SetBeamTile_Bot_1st_On     ; 
                          
.GetBeamTile_Bot_1st_On   lda #$78                        ; 
                          
.SetBeamTile_Bot_1st_On   sta SNZ_BeamDataTile_Bot        ; 
GameMoveBeamsIniTileOnX   jmp GameDataBeamsHandler        ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameMoveBeamsIniTileOff Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameMoveBeamsIniTileOff   subroutine                      ; 
                          lda #$00                        ; 
                          sta SNZ_BeamDataTile            ; 
                          
.IniBeamTile_Bot_Off      lda #$0a                        ; 
                          
.TstBeam_1st              cpx #$00                        ; 
.ChkBeam_1st              bne GameMoveBeamsIniTileOffX    ; 
                          
.GetBeamTile_Bot_1st_Off  lda #$8e                        ; 
                          
GameMoveBeamsIniTileOffX  sta SNZ_BeamDataTile_Bot        ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameDataBeamsHandler    Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameDataBeamsHandler      subroutine                      ; 
                          ldy TabBeamLvlDatRowOff_Top,x   ; 
                          lda TabBeamLvlDatRowOff_Bot,x   ; 
                          
.SavNextBeamDataByte      pha                             ; 
                          
.GoSetBeamDataByte        jsr GameDataBeamsSet            ; 
                          
                          iny                             ; 
                          
.RstBeamDataByte          pla                             ; 
                          
                          sec                             ; 
.DecRowNumber             sbc #$01                        ; 
.ChkRowNumber_Min         bne .SavNextBeamDataByte        ; 
                          
.TstBeam_3rd              cpx #$02                        ; 
.ChkBeam_3rd              beq .GoSetBeamDataByteBot       ; 
                          
.GetBeamDataByteBot       lda SNZ_BeamDataTile_Bot        ; 
.SetBeamDataByte          sta SNZ_BeamDataTile            ; 
                          
.GoSetBeamDataByteBot     jsr GameDataBeamsSet            ; 
                          
GameDataBeamsHandlerX     rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameMoveRays_L06        Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameMoveRays_L06          subroutine                      ; 
                          lda TabLevelNumCurrent          ; 
                          cmp #TabLevelNumCurrent_05      ; 
                          bcc .Exit_NoMove                ; lower
                          
.GetTilesMoveFlag         lda TabLvlTilesMoveFlag         ; 
.ChkTilesMoveFlag_Yes     beq .IniRayCount                ; TabLvlTilesMoveFlag_Yes
                          
.Exit_NoMove              rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IniRayCount              ldx #$00                        ; 
                          
                          lda #<[TabCheckDataPtr_L6 + [TabTileChkData_L6_Hdr_Len + $01]] ; 
                          sta SNZ_TileChkDataPtr_Lo       ; 
                          
                          lda #>[TabCheckDataPtr_L6 + [TabTileChkData_L6_Hdr_Len + $01]] ; 
                          sta SNZ_TileChkDataPtr_Hi       ; 
; -------------------------------------------------------------------------------------------------------------- ;
; move ray up
; -------------------------------------------------------------------------------------------------------------- ;
.GetNextRayPhase          ldy TabRayPhase,x               ; 
.ChkRayPhase_00           bne .SetRayPhase_01             ; 
                          
.GoSetRayUp               jsr GameMoveRayIniTileOn        ; 
                          
                          dec TabRayLevelDataRowsOff,x    ; 
                          bne .GoIncRayTileDataPtr_Ph1    ; 
                          
                          lda #$07                        ; 
                          sta TabRayLevelDataRowsOff,x    ; 
                          
.IncRayPhase_01           inc TabRayPhase,x               ; 
                          
.GoIncRayTileDataPtr_Ph1  jmp .IncRayTileDataPtr          ; 
; -------------------------------------------------------------------------------------------------------------- ;
.SetRayPhase_01           dey                             ; 
.ChkRayPhase_01           bne .SetRayPhase_02             ; 
                          
.GoClrRayUp               jsr GameMoveRayIniTileOff       ; 
                          
                          dec TabRayLevelDataRowsOff,x    ; 
                          bne .GoIncRayTileDataPtr_Ph2    ; 
                          
                          lda #$07                        ; 
                          sta TabRayLevelDataRowsOff,x    ; 
                          
.IncRayPhase_02           inc TabRayPhase,x               ; 
.GoIncRayTileDataPtr_Ph2  jmp .IncRayTileDataPtr          ; 
; -------------------------------------------------------------------------------------------------------------- ;
.SetRayPhase_02           dey                             ; 
.ChkRayPhase_02           bne .SetRayPhase_03             ; 
                          
                          dec TabRayLevelDataRowsOff,x    ; 
                          bne .GoIncRayTileDataPtr_Ph3    ; 
                          
.GetRowNum_Min            lda #$01                        ; 
.SetRowNum_Min            sta TabRayLevelDataRowsOff,x    ; 
                          
.IncRayPhase_03           inc TabRayPhase,x               ; 
.GoIncRayTileDataPtr_Ph3  jmp .IncRayTileDataPtr          ; 
; -------------------------------------------------------------------------------------------------------------- ;
; move ray down
; -------------------------------------------------------------------------------------------------------------- ;
.SetRayPhase_03           dey                             ; 
.ChkRayPhase_03           bne .SetRayPhase_04             ; 
                          
.GoSetRayDown             jsr GameMoveRayIniTileOn        ; 
                          
                          inc TabRayLevelDataRowsOff,x    ; 
                          
                          lda TabRayLevelDataRowsOff,x    ; 
                          cmp #$08                        ; 
                          bne .GoIncRayTileDataPtr_Ph4    ; 
                          
                          lda #$01                        ; 
                          sta TabRayLevelDataRowsOff,x    ; 
                          
.IncRayPhase_04           inc TabRayPhase,x               ; 
.GoIncRayTileDataPtr_Ph4  jmp .IncRayTileDataPtr          ; 
; -------------------------------------------------------------------------------------------------------------- ;
.SetRayPhase_04           dey                             ; 
.ChkRayPhase_04           bne .DecRayPhase_00_Ini         ; 
                          
.GoClrRayDown             jsr GameMoveRayIniTileOff       ; 
                          
                          inc TabRayLevelDataRowsOff,x    ; 
                          
                          lda TabRayLevelDataRowsOff,x    ; 
                          cmp #$08                        ; 
                          bne .GoIncRayTileDataPtr_Ph5    ; 
                          
                          lda #$0f                        ; 
                          sta TabRayLevelDataRowsOff,x    ; 
                          
                          inc TabRayPhase,x               ; 
.GoIncRayTileDataPtr_Ph5  jmp .IncRayTileDataPtr          ; 
; -------------------------------------------------------------------------------------------------------------- ;
.DecRayPhase_00_Ini       dec TabRayLevelDataRowsOff,x    ; 
                          bne .IncRayTileDataPtr          ; 
                          
                          lda #$07                        ; 
                          sta TabRayLevelDataRowsOff,x    ; 
                          
.GetRayPhase_00_Ini       lda #$00                        ; 
.SetRayPhase_00_Ini       sta TabRayPhase,x               ; 
; -------------------------------------------------------------------------------------------------------------- ;
; set next ray beam
; -------------------------------------------------------------------------------------------------------------- ;
.IncRayTileDataPtr        clc                             ; 
                          lda SNZ_TileChkDataPtr_Lo       ; 
                          adc #<SN_RayDistTileChkData     ; 
                          sta SNZ_TileChkDataPtr_Lo       ; 
                          
                          lda SNZ_TileChkDataPtr_Hi       ; 
                          adc #>SN_RayDistTileChkData     ; 
                          sta SNZ_TileChkDataPtr_Hi       ; 
                          
.IncRayCount              inx                             ; 
.TstRayCount              cpx #[TabRayData_Len + $01]     ; 
.ChkRayCount_Max          beq GameMoveRays_L06X           ; 
                          
                          jmp .GetNextRayPhase            ; 
                          
GameMoveRays_L06X         rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameMoveRayIniTileOn    Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameMoveRayIniTileOn      subroutine                      ; 
                          lda #$80                        ; 
                          sta SNZ_RayDataTile_Le          ; 
                          
                          lda #$82                        ; 
                          sta SNZ_RayDataTile_Ri          ; 
GameMoveRayIniTileOnX     bne GameDataRaysHandler         ; always
; -------------------------------------------------------------------------------------------------------------- ;
; GameMoveRayIniTileOff   Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameMoveRayIniTileOff     subroutine                      ; 
                          lda #$00                        ; 
                          sta SNZ_RayDataTile_Le          ; 
GameMoveRayIniTileOffX    sta SNZ_RayDataTile_Ri          ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameDataRaysHandler     Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameDataRaysHandler       subroutine                      ; 
                          lda TabRayLevelDataRowsOff,x    ; 
                          asl a                           ; *2
                          tay                             ; 
                          
.IniAdrLevelDataPtr_Top   clc                             ; 
                          lda [TabAdrLvlDatRows + $00],y  ; 
                          adc SNZ_TileChkDataPtr_Lo       ; 
                          sta SNZ_Ray_L06_DataOutPtr_Lo   ; 
                          
                          lda [TabAdrLvlDatRows + $01],y  ; 
                          adc SNZ_TileChkDataPtr_Hi       ; 
                          sta SNZ_Ray_L06_DataOutPtr_Hi   ; 
                          
.IniLevelDataByte_Le      ldy #$00                        ; 
                          lda SNZ_RayDataTile_Le          ; 
.SetLevelDataByte_Top_Le  sta (SNZ_Ray_L06_DataOutPtr),y  ; 
                          iny                             ; 
.SetLevelDataByte_Top_Ri  lda SNZ_RayDataTile_Ri          ; 
                          sta (SNZ_Ray_L06_DataOutPtr),y  ; 
                          
.IniAdrLevelDataPtr_Bot   clc                             ; 
                          lda SNZ_Ray_L06_DataOutPtr_Lo   ; 
                          adc #<[SN_Scrn_RowLen * $0f]    ; 
                          sta SNZ_Ray_L06_DataOutPtr_Lo   ; 
                          
                          lda SNZ_Ray_L06_DataOutPtr_Hi   ; 
                          adc #>[SN_Scrn_RowLen * $0f]    ; 
                          sta SNZ_Ray_L06_DataOutPtr_Hi   ; 
                          
.IniLevelDataByte_Bot     ldy #$00                        ; 
                          lda SNZ_RayDataTile_Le          ; 
                          clc                             ; 
                          adc #$01                        ; 
.SetLevelDataByte_Bot_Le  sta (SNZ_Ray_L06_DataOutPtr),y  ; 
                          iny                             ; 
                          lda SNZ_RayDataTile_Ri          ; 
                          clc                             ; 
                          adc #$01                        ; 
.SetLevelDataByte_Bot_Ri  sta (SNZ_Ray_L06_DataOutPtr),y  ; 
                          
GameDataRaysHandlerX      rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabRayPhase               equ  *   ; 
                          dc.b $00 ; 
                          dc.b $01 ; 
                          dc.b $02 ; 
                          dc.b $03 ; 
                          dc.b $04 ; 
                          dc.b $05 ; 
TabRayData_Len            equ  [* - TabRayPhase - $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabRayLevelDataRowsOff    equ  *   ; 
                          dc.b $07 ; 
                          dc.b $07 ; 
                          dc.b $07 ; 
                          dc.b $01 ; 
                          dc.b $01 ; 
                          dc.b $0f ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabRockLvlDatPosX_Lo      equ  *   ; 
                          dc.b <[TabTileChkData_L4_Rock_1 + $01] ; level 4
                          dc.b <[TabTileChkData_L4_Rock_2 + $01] ; 
                          dc.b <[TabTileChkData_L4_Rock_3 + $01] ; 
                          dc.b <[TabTileChkData_L4_Rock_4 + $01] ; 
                          dc.b <[TabTileChkData_L4_Rock_5 + $01] ; 
                          dc.b <[TabTileChkData_L4_Rock_6 + $01] ; 
                          dc.b <[TabTileChkData_L4_Rock_7 + $01] ; 
                          dc.b <[TabTileChkData_L4_Rock_8 + $01] ; 
                          dc.b <[TabTileChkData_L5_Rock_1 + $01] ; level 5
                          dc.b <[TabTileChkData_L5_Rock_2 + $01] ; 
                          dc.b <[TabTileChkData_L7_Rock_1 + $01] ; level 7
                          dc.b <[TabTileChkData_L7_Rock_2 + $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabRockLvlDatPosX_Hi      equ  *   ; 
                          dc.b >[TabTileChkData_L4_Rock_1 + $01] ; level 4
                          dc.b >[TabTileChkData_L4_Rock_2 + $01] ; 
                          dc.b >[TabTileChkData_L4_Rock_3 + $01] ; 
                          dc.b >[TabTileChkData_L4_Rock_4 + $01] ; 
                          dc.b >[TabTileChkData_L4_Rock_5 + $01] ; 
                          dc.b >[TabTileChkData_L4_Rock_6 + $01] ; 
                          dc.b >[TabTileChkData_L4_Rock_7 + $01] ; 
                          dc.b >[TabTileChkData_L4_Rock_8 + $01] ; 
                          dc.b >[TabTileChkData_L5_Rock_1 + $01] ; level 5
                          dc.b >[TabTileChkData_L5_Rock_2 + $01] ; 
                          dc.b >[TabTileChkData_L7_Rock_1 + $01] ; level 7
                          dc.b >[TabTileChkData_L7_Rock_2 + $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabRockFallRowIni         equ  *   ; 
                          dc.b $00 ; L4_Rock_1 - level 4
                          dc.b $01 ; L4_Rock_2
                          dc.b $00 ; L4_Rock_3
                          dc.b $01 ; L4_Rock_4
                          dc.b $01 ; L4_Rock_5
                          dc.b $00 ; L4_Rock_6
                          dc.b $00 ; L4_Rock_7
                          dc.b $01 ; L4_Rock_8
                          dc.b $01 ; L5_Rock_1 - level 5
                          dc.b $01 ; L5_Rock_2
                          dc.b $01 ; L7_Rock_1 - lev3l 7
                          dc.b $00 ; L7_Rock_2
; -------------------------------------------------------------------------------------------------------------- ;
TabRockFallHeight         equ  *   ; 
TabRockFallHeight_Fallen     = $ff ; flag: rock no longer active
                          dc.b $05 ; 
                          dc.b $05 ; 
                          dc.b $06 ; 
                          dc.b $04 ; 
                          dc.b $04 ; 
                          dc.b $06 ; 
                          dc.b $06 ; 
                          dc.b $05 ; 
                          dc.b $04 ; 
                          dc.b $04 ; 
                          dc.b $04 ; 
                          dc.b $05 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabRockFallRowWrk         equ  *   ; 
                          dc.b $00 ; 
                          dc.b $01 ; 
                          dc.b $00 ; 
                          dc.b $01 ; 
                          dc.b $01 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $01 ; 
                          dc.b $01 ; 
                          dc.b $01 ; 
                          dc.b $01 ; 
                          dc.b $00 ; 
TabRockTileDataTab_Len    equ  [* - TabRockFallRowWrk - $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabRockTileDataTabOff     equ  *   ; 
                          dc.b TabRockTileData_Len ; 
                          dc.b TabRockTileData_Len ; 
                          dc.b TabRockTileData_Len ; 
                          dc.b TabRockTileData_Len ; 
                          dc.b TabRockTileData_Len ; 
                          dc.b TabRockTileData_Len ; 
                          dc.b TabRockTileData_Len ; 
                          dc.b TabRockTileData_Len ; 
                          dc.b TabRockTileData_Len ; 
                          dc.b TabRockTileData_Len ; 
                          dc.b TabRockTileData_Len ; 
                          dc.b TabRockTileData_Len ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabRockTileData_Top_Le    equ  *   ; 
                          dc.b $2d ; 
                          dc.b $31 ; 
                          dc.b $35 ; 
                          dc.b $00 ; 
TabRockTileData_Len       equ  [* - TabRockTileData_Top_Le - $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabRockTileData_Top_Ri    equ  *   ; 
                          dc.b $2e ; 
                          dc.b $32 ; 
                          dc.b $36 ; 
                          dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabRockTileData_Bot_Le    equ  *   ; 
                          dc.b $2f ; 
                          dc.b $33 ; 
                          dc.b $37 ; 
                          dc.b $2b ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabRockTileData_Bot_Ri    equ  *   ; 
                          dc.b $30 ; 
                          dc.b $34 ; 
                          dc.b $38 ; 
                          dc.b $2c ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitRocks_L04_L05       Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitRocks_L04_L05         subroutine                      ; 
.IniRockStatusLen         ldx #$00                        ; 
.GetNextRockDataTabOff    lda #TabRockTileData_Len        ; init tab data offset
.SetRockDataTabOff        sta TabRockTileDataTabOff,x     ; 
                          
                          lda TabRockFallRowIni,x         ; 
                          sta TabRockFallRowWrk,x         ; 
                          
                          tay                             ; 
                          iny                             ; 
                          
.IniRockStart             lda #$2b                        ; 
                          sta SNZ_RockDataTile_Le         ; 
                          lda #$2c                        ; 
                          sta SNZ_RockDataTile_Ri         ; 
                          
.SetRockStart             jsr GameRockFallTileOut         ; 
                          
                          iny                             ; 
                          
.IniRockClear             lda #$00                        ; 
                          sta SNZ_RockDataTile_Le         ; 
                          sta SNZ_RockDataTile_Ri         ; 
                          
                          lda TabRockFallHeight,x         ; 
                          
.SavNextRockFallHeight    pha                             ; 
                          
.ClrRockFallPath          jsr GameRockFallTileOut         ; 
                          
                          iny                             ; 
                          
.RstRockFallHeight        pla                             ; 
                          sec                             ; 
.DecRockFallHeight        sbc #$01                        ; 
.ChkRockFallHeight_Min    bne .SavNextRockFallHeight      ; 
                          
.IncRockStatusLen         inx                             ; 
.TstRockStatusLen         cpx #[TabRockTileDataTab_Len + $01] ; 
.ChkRockStatusLen_Max     bne .GetNextRockDataTabOff      ; 
                          
InitRocks_L04_L05X        rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameRockFallTileOut     Does    : 
;                         Expects : .Y=num level data row tab row
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameRockFallTileOut       subroutine                      ; 
                          tya                             ; 
.Sav_Yr                   pha                             ; 
                          
                          asl a                           ; *2
                          tay                             ; 
                          
.IniAdrLevelDataPtr_Top   clc                             ; 
                          lda [TabAdrLvlDatRows + $00],y  ; 
                          adc TabRockLvlDatPosX_Lo,x      ; 
                          sta SNZ_Rock_L457_DataOutPtr_Lo ; 
                          
                          lda [TabAdrLvlDatRows + $01],y  ; 
                          adc TabRockLvlDatPosX_Hi,x      ; 
                          sta SNZ_Rock_L457_DataOutPtr_Hi ; 
                          
.IniLevelDataByte_Top     ldy #$00                        ; 
                          
                          lda SNZ_RockDataTile_Le         ; 
                          asl a                           ; *2
.SetLevelDataByte_Top_Le  sta (SNZ_Rock_L457_DataOutPtr),y; 
                          
                          iny                             ; 
                          
                          lda SNZ_RockDataTile_Ri         ; 
                          asl a                           ; 
.SetLevelDataByte_Top_Ri  sta (SNZ_Rock_L457_DataOutPtr),y; 
                          
.IniAdrLevelDataPtr_Bot   clc                             ; 
                          lda SNZ_Rock_L457_DataOutPtr_Lo ; 
                          adc #<[SN_Scrn_RowLen * $0f]    ; 
                          sta SNZ_Rock_L457_DataOutPtr_Lo ; 
                          
                          lda SNZ_Rock_L457_DataOutPtr_Hi ; 
                          adc #>[SN_Scrn_RowLen * $0f]    ; 
                          sta SNZ_Rock_L457_DataOutPtr_Hi ; 
                          
.IniLevelDataByte_Bot     ldy #$00                        ; 
                          
                          lda SNZ_RockDataTile_Le         ; 
                          asl a                           ; 
                          clc                             ; 
                          adc #$01                        ; 
.SetLevelDataByte_Bot_Le  sta (SNZ_Rock_L457_DataOutPtr),y; 
                          
                          iny                             ; 
                          
                          lda SNZ_RockDataTile_Ri         ; 
                          asl a                           ; 
                          clc                             ; 
                          adc #$01                        ; 
.SetLevelDataByte_Bot_Ri  sta (SNZ_Rock_L457_DataOutPtr),y; 
                          
                          pla                             ; 
.Rst_Yr                   tay                             ; 
                          
GameRockFallTileOutX      rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameMoveRocks_L04_05_07 Does  : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameMoveRocks_L04_05_07   subroutine                      ; 
                          lda TabLevelNumCurrent          ; 
                          cmp #TabLevelNumCurrent_04      ; 
                          bcc .Exit_NoFall                ; lower
                          
                          cmp #TabLevelNumCurrent_06      ; 
                          beq .Exit_NoFall                ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IniRockCount             ldx #$00                        ; 
.GetNextRockStatus        lda TabRockFallRowWrk,x         ; 
.TstRockStatus            cmp #TabRockFallHeight_Fallen   ; 
.ChkRockStatus_Active     bne .IniRockSnokieDistChk       ; 
                          
.IncNextRockCount         inx                             ; 
.TstRockCount             cpx #[TabRockTileDataTab_Len + $01] ; 
.ChkRockCount_Max         bne .GetNextRockStatus          ; 
                          
.Exit_NoFall              rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IniRockSnokieDistChk     cmp TabRockFallRowIni,x         ; 
                          bne .IncTabRockTileOff          ; 
                          
.IniSnokieRockDist        sec                             ; 
                          lda TabRockLvlDatPosX_Lo,x      ; 
                          sbc TabLvlDatSnokiePosX_Lo_P1   ; 
.SetRockSnokieDist        sta SNZ_SnokieRockDistX_Lo      ; 
                          
                          lda TabRockLvlDatPosX_Hi,x      ; 
                          sbc TabLvlDatSnokiePosX_Hi_P1   ; 
                          bmi .IncTabRockTileOff          ; 
                          bne .Exit_NoFall                ; 
                          
.GetRockSnokieDist        lda SNZ_SnokieRockDistX_Lo      ; 
.TstRockSnokieDist        cmp #SN_SnokieRockDist_Min      ; 
.ChkRockSnokieDist_Min    bcs .Exit_NoFall                ; greater/equal
; -------------------------------------------------------------------------------------------------------------- ;
.IncTabRockTileOff        inc TabRockTileDataTabOff,x     ; point to next rock data
                          
.GetTabRockTileOff        lda TabRockTileDataTabOff,x     ; 
.LimTabRockTileOff        and #TabRockTileData_Len        ; limit to max length
.SetTabRockTileOff        sta TabRockTileDataTabOff,x     ; 
.ChkTabRockTileOff_Min    bne .MovRockPart_Top            ; 
                          
.ChkRockFallen            sec                             ; 
                          lda TabRockFallRowWrk,x         ; 
                          sbc TabRockFallRowIni,x         ; 
                          sec                             ; 
                          sbc #$01                        ; 
                          cmp TabRockFallHeight,x         ; 
                          beq .GetRockFallen              ; 
                          
.IncRockFallRow           inc TabRockFallRowWrk,x         ; 
                          jmp .MovRockPart_Top            ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetRockFallen            lda #TabRockFallHeight_Fallen   ; 
.SetRockFallen            sta TabRockFallRowWrk,x         ; 
                          jmp .IncNextRockCount           ; 
; -------------------------------------------------------------------------------------------------------------- ;
.MovRockPart_Top          ldy TabRockTileDataTabOff,x     ; 
                          lda TabRockTileData_Top_Le,y    ; 
                          sta SNZ_RockDataTile_Le         ; 
                          
                          lda TabRockTileData_Top_Ri,y    ; 
                          sta SNZ_RockDataTile_Ri         ; 
                          
                          ldy TabRockFallRowWrk,x         ; 
                          jsr GameRockFallTileOut         ; 
                          
                          sec                             ; 
                          lda TabRockFallRowWrk,x         ; 
                          sbc TabRockFallRowIni,x         ; 
                          sec                             ; 
                          sbc #$01                        ; 
                          cmp TabRockFallHeight,x         ; 
                          beq .IncNextRockCount           ; 
; -------------------------------------------------------------------------------------------------------------- ;
.MovRockPart_Bot          ldy TabRockTileDataTabOff,x     ; 
                          lda TabRockTileData_Bot_Le,y    ; 
                          sta SNZ_RockDataTile_Le         ; 
                          
                          lda TabRockTileData_Bot_Ri,y    ; 
                          sta SNZ_RockDataTile_Ri         ; 
                          
                          ldy TabRockFallRowWrk,x         ; 
                          iny                             ; 
                          jsr GameRockFallTileOut         ; 
                          
GameMoveRocks_L04_05_07X  jmp .IncNextRockCount           ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameSnokieCheckCollision Does   : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameSnokieCheckCollision  subroutine                      ; 
                          lda TabSnokieStatus             ; 
.ChkSnokieStatus_Hidden   bne .ChkSnokieStatus_Death      ; not hidden
; -------------------------------------------------------------------------------------------------------------- ;
; snokie status hidden: game not started
; -------------------------------------------------------------------------------------------------------------- ;
                          jsr InitJumpToneHeight          ; 
                          
.ClrSprtCollision         lda SPSPCL                      ; VIC($D01E) Sprite-Sprite Collision - Cleared on read
                          lda SPBGCL                      ; VIC($D01F) Sprite-Background Collision - Cleared on read
                          
                          rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.ChkSnokieStatus_Death    cmp #TabSnokieStatus_Death      ; 
                          bcs .ClrSprtCollision           ; greater/equal
; -------------------------------------------------------------------------------------------------------------- ;
.GetSprtColl_Backg        lda SPBGCL                      ; VIC($D01F) Sprite-Background Collision - Cleared on read
                          and #[VIC_Sprt1_CollBg | VIC_Sprt0_CollBg] ; ......##
                          bne .ChkDat_L07_Beam_Pos        ; 
                          
                          jmp .GetSprtSprtCollision       ; 
; -------------------------------------------------------------------------------------------------------------- ;
; level 7 - vertical beam - leftmost
; -------------------------------------------------------------------------------------------------------------- ;
.ChkDat_L07_Beam_Pos      lda TabLvlDatSnokiePosX_Hi_P1   ; 
                          cmp #$02                        ; 
                          bne .ChkDat_L03_Water_Pos       ; 
                          
                          lda TabLvlDatSnokiePosX_Lo_P1   ; 
.ChkBeamPos_Lo_End_Ri     cmp #$31                        ; 
                          bcc .GetBeamPos_Lo_Start_Le     ; lower
                          
.HasPassedTheBeams        jmp .GetSprtSprtCollision       ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetBeamPos_Lo_Start_Le   lda TabLvlDatSnokiePosX_Lo_P1   ; 
.ChkBeamPos_Lo_Start_Le   cmp #$1c                        ; 
                          bne .ChkDat_L03_Water_Pos       ; 
                          
.GetBeam_01_Status        lda Tab_L07_BeamStatus_01       ; 
.ChkBeam_01_Status        beq .ChkDat_L03_Water_Pos       ; Tab_L07_BeamStatus_Off - beam is off
                          
                          jmp .GetSnokieStatus_Death      ; 
; -------------------------------------------------------------------------------------------------------------- ;
; level 3 - water surface
; -------------------------------------------------------------------------------------------------------------- ;
.ChkDat_L03_Water_Pos     lda TabLvlDatSnokiePosX_Hi_P1   ; 
                          cmp #$01                        ; 
                          bne .ChkLvlDat_07_Beam_Pos      ; 
                          
                          lda TabLvlDatSnokiePosX_Lo_P1   ; 
.ChkWaterPos_Lo_Start_Le  cmp #$19                        ; 
                          bcs .ChkWaterPos_Lo_End_Ri      ; higher/equal
                          
                          jmp .GetSprtSprtCollision       ; 
; -------------------------------------------------------------------------------------------------------------- ;
.ChkWaterPos_Lo_End_Ri    cmp #$46                        ; 
                          bcs .ChkLvlDat_07_Beam_Pos      ; greater/equal
                          
.GetSnokiePosY            lda TabLvlSnokiePosY_P1         ; 
.TstSnokiePosY_Surface    cmp #$1a                        ; 
.ChkSnokiePosY_Surface    bcs .ChkLvlDat_07_Beam_Pos      ; higher/equal
                          
.WasSnokiePosY_Surface    jmp .GetSnokieStatus_Death      ; drowned
; -------------------------------------------------------------------------------------------------------------- ;
; level 7 - vertical beams
; -------------------------------------------------------------------------------------------------------------- ;
.ChkLvlDat_07_Beam_Pos    ldx TabLvlDatSnokiePosX_Hi_P1   ; 
                          cmp #$02                        ; 
                          bne .ChkDat_L06_Ray_Pos         ; 
                          
.IniStatusBeamCount       ldy #$00                        ; 
.GetNextStatusBeam        lda Tab_L07_BeamStatus,y        ; 
.ChkStatusBeam            beq .IncStatusBeamCount         ; Tab_L07_BeamStatus_Off - beam is off
                          
                          sec                             ; 
.GetBeamPos_Lo_Le         lda TabLvlDatSnokiePosX_Lo_P1   ; 
                          sbc TabBeamPosX_Lo,y            ; 
.ChkBeamPos_Lo_Le         bcc .ChkDat_L06_Ray_Pos         ; lower
                          
                          cmp #$04                        ; 
                          bcc .GetSnokieStatus_Death      ; lower
                          
.IncStatusBeamCount       iny                             ; 
.TstStatusBeamCount_Max   cpy #[TabBeam_Len + $01]        ; 
.ChkStatusBeamCount_Max   bcc .GetNextStatusBeam          ; lower
; -------------------------------------------------------------------------------------------------------------- ;
; level 6 - vertical rays
; -------------------------------------------------------------------------------------------------------------- ;
.ChkDat_L06_Ray_Pos       ldx #$00                        ; IniRayCount
                          
.SetRayDatPtr_Last        lda #<[TabTileChkData_L6_Start + $01] ; 
                          sta SNZ_TileChkDataPtr_Lo       ; 
                          lda #>[TabTileChkData_L6_Start + $01] ; 
                          sta SNZ_TileChkDataPtr_Hi       ; 
                          
.GetNextRayChkPos         sec                             ; 
                          lda TabLvlDatSnokiePosX_Lo_P1   ; 
                          sbc SNZ_TileChkDataPtr_Lo       ; 
                          sta ._GetRayPosChkVal           ; 
                          
                          lda TabLvlDatSnokiePosX_Hi_P1   ; 
                          sbc SNZ_TileChkDataPtr_Hi       ; 
                          bcc .ChkDat_L05_Pos_Start       ; lower
                          
                          cmp #$00                        ; 
                          bne .ChkDat_L05_Pos_Start       ; 
                          
._GetRayPosChkVal         equ [* + $01]                   ; 
.GetRayPos                lda #$00                        ; 
.TstRayPos                cmp #$01                        ; 
.ChkRayPos                bcc .GetSnokieStatus_Death      ; lower
                          bne .SetRayDatPtr_Next          ; higher
                          
                          lda TabSnokieMoveStep           ; equal
                          beq .GetSnokieStatus_Death      ; 
                          
.SetRayDatPtr_Next        clc                             ; 
                          lda SNZ_TileChkDataPtr_Lo       ; 
                          adc #<SN_RayDistTileChkData     ; 
                          sta SNZ_TileChkDataPtr_Lo       ; 
                          bcc .IncRayCount                ; 
                          inc SNZ_TileChkDataPtr_Hi       ; 
                          
.IncRayCount              inx                             ; 
.TstRayCount_Max          cpx #[TabRayData_Len + $01]     ; 
.ChkRayCount_Max          bcc .GetNextRayChkPos           ; 
; -------------------------------------------------------------------------------------------------------------- ;
.ChkDat_L05_Pos_Start     lda TabLvlDatSnokiePosX_Hi_P1   ; 
                          cmp #>[TabTileChkData_L5_Start + $02] ; 
                          bne .ChkDat_L04_L05_Rock_Pos    ; 
                          
                          lda TabLvlDatSnokiePosX_Lo_P1   ; 
                          cmp #<[TabTileChkData_L5_Start + $02] ; 
                          bcc .ChkDat_L04_L05_Rock_Pos    ; lower
                          
.ChkDat_L05_Pos_End       cmp #<[TabTileChkData_L5_End   + $06] ; 
                          bcs .ChkDat_L04_L05_Rock_Pos    ; higher/equal
                          
.GetSnokie_L05_PosY       lda TabLvlSnokiePosY_P1         ; 
.TstSnokie_L05_PosY_Min   cmp #$0a                        ; 
                          bcc .GetSnokieStatus_Death      ; lower
                          bcs .ChkDat_L04_L05_Rock_Pos    ; higher/equal - always
; -------------------------------------------------------------------------------------------------------------- ;
.GetSprtSprtCollision     lda SPSPCL                      ; VIC($D01E) Sprite-Sprite Collision - Cleared on read
                          and #[VIC_Sprt2_CollSp | VIC_Sprt1_CollSp | VIC_Sprt0_CollSp] ; .....###
                          cmp #[VIC_Sprt2_CollSp | VIC_Sprt1_CollSp]                    ; .....##.
                          beq .GetSnokieStatus_Death      ; 
                          
                          cmp #[VIC_Sprt2_CollSp | VIC_Sprt0_CollSp]                    ; .....#.#
                          beq .GetSnokieStatus_Death      ; 
                          
                          cmp #[VIC_Sprt2_CollSp | VIC_Sprt1_CollSp | VIC_Sprt0_CollSp] ; .....###
                          beq .GetSnokieStatus_Death      ; 
                          
                          rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetSnokieStatus_Death    lda #TabSnokieStatus_Death      ; 
                          sta TabSnokieStatus             ; 
                          
                          jsr InitJumpToneHeight          ; 
                          jmp GamePlayerDecNumLives       ; 
; -------------------------------------------------------------------------------------------------------------- ;
; level 4 and 5 and 7 - rocks
; -------------------------------------------------------------------------------------------------------------- ;
.ChkDat_L04_L05_Rock_Pos  ldx #$00                        ; IniRockCount
                          
.IniNextRockRowAdjust     sec                             ; 
                          lda TabRockLvlDatPosX_Lo,x      ; 
                          sbc #$01                        ; 
                          sta ._SubLvlDatRockAdr_Lo       ; 
                          
                          lda TabRockLvlDatPosX_Hi,x      ; 
                          sbc #$00                        ; 
                          sta ._SubLvlDatRockAdr_Hi       ; 
                          
                          sec                             ; 
                          lda TabLvlDatSnokiePosX_Lo_P1   ; 
._SubLvlDatRockAdr_Lo     equ [* + $01]                   ; 
                          sbc #$00                        ; 
                          sta ._GetLvlDatRockAdr_Lo       ; 
                          
                          lda TabLvlDatSnokiePosX_Hi_P1   ; 
._SubLvlDatRockAdr_Hi     equ [* + $01]                   ; 
                          sbc #$00                        ; 
                          bcs .ChkLvlDatRockAdr_Min       ; 
                          
.IncNextRockCount         inx                             ; 
.TstRockCount             cpx #[TabRockTileDataTab_Len + $01] ; 
.ChkRockCount             bne .IniNextRockRowAdjust       ; 
.ChkRockCount_Max         beq .GetSprtSprtCollision       ; always
                          
.ChkLvlDatRockAdr_Min     cmp #$00                        ; 
                          bne .IncNextRockCount           ; 
                          
._GetLvlDatRockAdr_Lo     equ [* + $01]                   ; 
.GetLvlDatRockAdr_Lo      lda #$00                        ; 
.TstLvlDatRockAdr         cmp #$03                        ; 
.ChkLvlDatRockAdr_Max     bcs .IncNextRockCount           ; higher/equal
                          
.GetRockFallRow           lda TabRockFallRowWrk,x         ; 
.TstRockFallRow           cmp #TabRockFallHeight_Fallen   ; 
.ChkRockFallRow_Fallen    beq .IncNextRockCount           ; 
                          
                          asl a                           ; *2
                          asl a                           ; *4
                          asl a                           ; *8
                          sta ._SubRockFallRow            ; 
                          
                          lda TabRockTileDataTabOff,x     ; 
                          asl a                           ; *2
                          sta ._SubRockDataTabOff         ; 
                          
                          sec                             ; 
                          lda #$45                        ; 
._SubRockFallRow          equ [* + $01]                   ; 
.SubRockFallRow           sbc #$00                        ; 
                          sec                             ; 
._SubRockDataTabOff       equ [* + $01]                   ; 
.SubRockDataTabOff        sbc #$00                        ; 
                          cmp TabLvlSnokiePosY_P1         ; 
                          bcc .IncNextRockCount           ; lower
                          
                          sec                             ; 
                          sbc #$07                        ; 
                          adc #$07                        ; 
                          sta ._TstSnokieHit              ; 
                          
                          lda TabSnokieStatus             ; 
                          cmp #TabSnokieStatus_Active     ; move/stand
                          bne .GetAddSnokieHeight_Slide   ; 
                          
.GetAddSnokieHeight_Stand lda #$0f                        ; 
                          bne .SetAddSnokieHeight         ; always
                          
.GetAddSnokieHeight_Slide lda #$0c                        ; 
                          
.SetAddSnokieHeight       sta ._AddSnokieHeight           ; 
                          
                          clc                             ; 
                          lda TabLvlSnokiePosY_P1         ; 
._AddSnokieHeight         equ [* + $01]                   ; 
                          adc #$00                        ; 
._TstSnokieHit            equ [* + $01]                   ; 
.TstChkSnokieHit          cmp #$00                        ; 
.ChkChkSnokieHit          bcc .IncNextRockCount           ; lower
                          
GameSnokieCheckCollisionX jmp .GetSnokieStatus_Death      ; higher/equal
; -------------------------------------------------------------------------------------------------------------- ;
TabBeamPosX_Lo            equ  *   ; 
                          dc.b <TabTileChkData_L7_Beam_1 ; 
                          dc.b <TabTileChkData_L7_Beam_2 ; 
                          dc.b <TabTileChkData_L7_Beam_3 ; 
                          dc.b <TabTileChkData_L7_Beam_4 ; 
TabBeam_Len               equ  [* - TabBeamPosX_Lo - $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabOrbBigDataPosX_Lo_Ini  equ  *      ; 
                          dc.b <[TabTileChkData_L1_Orb_1 + $01] ; level 1
                          dc.b <[TabTileChkData_L1_Orb_2 + $01] ; 
                          dc.b <[TabTileChkData_L1_Orb_3 + $01] ; 
                          dc.b <[TabTileChkData_L2_Orb_1 + $01] ; level 2
TabOrbBig_Len             equ [* - TabOrbBigDataPosX_Lo_Ini - $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabOrbBigDataPosX_Hi_Ini  equ  *   ; 
                          dc.b >[TabTileChkData_L1_Orb_1 + $01] ; level 1
                          dc.b >[TabTileChkData_L1_Orb_2 + $01] ; 
                          dc.b >[TabTileChkData_L1_Orb_3 + $01] ; 
                          dc.b >[TabTileChkData_L2_Orb_1 + $01] ; level 2
; -------------------------------------------------------------------------------------------------------------- ;
TabOrbBigRollPosY_Ini     equ  *   ; 
                          dc.b $13 ; 
                          dc.b $0b ; 
                          dc.b $1f ; 
                          dc.b $1f ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabOrbBigRollDir_Ini      equ  *   ; 
TabOrbBigRollDir_Ini_Ri      = $01 ; 
TabOrbBigRollDir_Ini_Le      = $00 ; 
                          dc.b TabOrbBigRollDir_Ini_Ri ; 
                          dc.b TabOrbBigRollDir_Ini_Ri ; 
                          dc.b TabOrbBigRollDir_Ini_Ri ; 
                          dc.b TabOrbBigRollDir_Ini_Le ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabOrbBigRollPhase        equ  *   ; 
TabOrbBigRollPhase_Min       = $01 ; 
TabOrbBigRollPhase_Max       = $04 ; 
TabOrbBigRollPhase_Off       = $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabOrbBigDataPosX_Hi_Wrk  equ  *   ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabOrbBigDataPosX_Lo_Wrk  equ  *   ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabOrbBigRollPosY_Wrk     equ  *   ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabOrbBigRollDir_Wrk      equ  *   ; 
TabOrbBigRollDir_Wrk_Le      = $ff ; 
TabOrbBigRollDir_Wrk_None    = $00 ; 
TabOrbBigRollDir_Wrk_Ri      = $01 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabOrbBigRollUnused       equ  *   ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabOrbBigRollSteps        equ  *   ; 
TabOrbBigRollSteps_Max       = $03 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitOrbsBig_L01_L02     Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitOrbsBig_L01_L02       subroutine                      ; 
.IniTabByteCount          ldx #TabOrbBig_Len              ; 
.GetNextTabByte           lda #TabOrbBigRollPhase_Max     ; 
                          sta TabOrbBigRollPhase,x        ; 
                          
                          lda TabOrbBigDataPosX_Hi_Ini,x  ; 
                          sta TabOrbBigDataPosX_Hi_Wrk,x  ; 
                          
                          lda TabOrbBigDataPosX_Lo_Ini,x  ; 
                          sta TabOrbBigDataPosX_Lo_Wrk,x  ; 
                          
                          lda TabOrbBigRollPosY_Ini,x     ; 
                          asl a                           ; 
                          sta TabOrbBigRollPosY_Wrk,x     ; 
                          
                          lda #TabOrbBigRollDir_Wrk_None  ; 
                          sta TabOrbBigRollDir_Wrk,x      ; 
                          sta TabOrbBigRollUnused,x       ; 
                          
                          lda #TabOrbBigRollSteps_Max     ; 
                          sta TabOrbBigRollSteps,x        ; 
                          
.DecTabByteCount          dex                             ; 
.ChkTabByteCount_Min      bpl .GetNextTabByte             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameOrbsSpriteOff       Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameOrbsSpriteOff         subroutine                      ; 
.GetOrbsSprt_Off          lda SPENA                       ; VIC($D015) Sprite Enable Register
.InsOrbsSprt_Off          and #$fb                        ; #####.##
.SetOrbsSprt_Off          sta SPENA                       ; VIC($D015) Sprite Enable Register
                          
GameOrbsSpriteOffX        rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameOrbMoveSpriteHandler Does   : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameOrbMoveSpriteHandler  subroutine                      ; 
                          lda TabLevelNumCurrent          ; 
.TstLvl_03                cmp #TabLevelNumCurrent_03      ; 
.ChkLvl_03                bcs GameOrbMoveSpriteHandlerX   ; greater/equal - not level 01 or 02
                          
.IniOrbBigCount           ldx #$00                        ; was level 01 or 02
.GetNextOrbBigRollPhase   lda TabOrbBigRollPhase,x        ; 
                          bne GameOrbsBigSprt_L01_L02     ; roll big orb sprites
                          
IncNextOrbBigCount        inx                             ; global return point
.TstOrbBigCount           cpx #[TabOrbBig_Len + $01]      ; 
.ChkOrbBigCount_Max       bne .GetNextOrbBigRollPhase     ; 
                          
GameOrbMoveSpriteHandlerX jmp GameOrbTinySprt_L04_L05     ; fly tiny orb sprites
; -------------------------------------------------------------------------------------------------------------- ;
; GameOrbsBigSprt_L01_L02 Does    : Roll big orb sprites
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameOrbsBigSprt_L01_L02   subroutine                      ; 
.GetOrbBigPosX            sec                             ; 
                          lda TabOrbBigDataPosX_Lo_Wrk,x  ; 
                          sbc TabLvlScrollSteps_Lo        ; 
                          sta SNZ_OrbBigSprt_PosX_Lo      ; 
                          
                          lda TabOrbBigDataPosX_Hi_Wrk,x  ; 
                          sbc TabLvlScrollSteps_Hi        ; 
                          sta SNZ_OrbBigSprt_PosX_Hi      ; 
                          
                          asl SNZ_OrbBigSprt_PosX_Lo      ; 
                          rol SNZ_OrbBigSprt_PosX_Hi      ; *2
                          asl SNZ_OrbBigSprt_PosX_Lo      ; 
                          rol SNZ_OrbBigSprt_PosX_Hi      ; *4
                          
                          lda SNZ_OrbBigSprt_PosX_Hi      ; 
.ChkOrbBig_PosX_Hi_Min    beq .GetOrbBig_PosX_Lo_Min      ; 
; -------------------------------------------------------------------------------------------------------------- ;
; check if big orb rolls out of the left side of the screen
; -------------------------------------------------------------------------------------------------------------- ;
.TstOrbBig_PosX_Hi_Max    cmp #$ff                        ; 
.ChkOrbBig_PosX_Hi_Max    bne IncNextOrbBigCount          ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetOrbBig_PosX_Lo_Max    lda SNZ_OrbBigSprt_PosX_Lo      ; 
.TstOrbBig_PosX_Lo_Max    cmp #$fd                        ; 
                          bcc IncNextOrbBigCount          ; lower
                          bcs .IniTstScollOrbBigPosX      ; higher/equal - always
; -------------------------------------------------------------------------------------------------------------- ;
.GetOrbBig_PosX_Lo_Min    lda SNZ_OrbBigSprt_PosX_Lo      ; 
.TstOrbBig_PosX_Lo_Min    cmp #$bc                        ; 
                          bcs IncNextOrbBigCount          ; higher/equal
; -------------------------------------------------------------------------------------------------------------- ;
; check if big orb sits inside the screen limits
; -------------------------------------------------------------------------------------------------------------- ;
.IniTstScollOrbBigPosX    clc                             ; 
                          adc TabOrbBigRollSteps,x        ; 
                          clc                             ; 
                          adc TabLevelSoftScrollValue     ; 
                          clc                             ; 
                          adc #$1d                        ; 
                          
.TstOrbBigPosX_Le         cmp #$20                        ; 
.ChkOrbBigPosX_Min        bcc IncNextOrbBigCount          ; lower
                          
.TstOrbBigPosX_Ri         cmp #$d9                        ; 
.ChkOrbBigPosX_Max        bcs IncNextOrbBigCount          ; higher/euqal
; -------------------------------------------------------------------------------------------------------------- ;
; step big orb sprite forward
; -------------------------------------------------------------------------------------------------------------- ;
.SetOrbBigSprt_PosX_Tmp   sta SNZ_OrbBigSprt_PosX_Lo      ; 
                          
                          sec                             ; 
                          lda #$d6                        ; 
                          sbc TabOrbBigRollPosY_Wrk,x     ; 
.SetOrbBigSprt_PosY       sta SP2Y                        ; VIC($D005) Sprite 2 Y-Pos (Bits 0-7)
                          
                          lda TabOrbBigRollPhase,x        ; 
                          clc                             ; 
                          adc #[[SpriteData_OrbBig / VIC_Sprt_BlockLen] - $01] ; 
.SetOrbBigSprt_Ptr        sta SN_GameSpritePtr_02         ; 
                          
                          lda SNZ_OrbBigSprt_PosX_Lo      ; 
                          asl a                           ; *2
.SetOrbBigSprt_PosX_Wrk   sta SNZ_OrbBigSprt_PosX_Lo      ; 
                          lda #$00                        ; 
                          rol a                           ; 
.SetOrbBigSprt_PosX_MSB   sta SNZ_OrbBigSprt_PosX_Hi      ; 
                          
                          sec                             ; 
                          lda SNZ_OrbBigSprt_PosX_Lo      ; 
                          sbc #$2a                        ; 
.SetOrbBigSprt_PosX       sta SP2X                        ; VIC($D004) Sprite 2 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          
                          lda SNZ_OrbBigSprt_PosX_Hi      ; 
                          sbc #$00                        ; 
                          asl a                           ; 
                          asl a                           ; 
                          sta ._InsOrbBigSprt_MSB         ; 
                          
.GetOrbBigSprt_MSB        lda MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
.IsoOrbBigSprt_MSB        and #$fb                        ; #####.##
._InsOrbBigSprt_MSB       equ [* + $01]                   ; 
.InsOrbBigSprt_MSB        ora #$00                        ; 
.SetOrbBigSprt_MSB        sta MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
                          
.GetOrbBigSprt_On         lda SPENA                       ; VIC($D015) Sprite Enable Register
.InsOrbBigSprt_On         ora #$04                        ; .....#..
.SetOrbBigSprt_On         sta SPENA                       ; VIC($D015) Sprite Enable Register
                          
GameOrbsBigSprt_L01_L02X  rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabOrbBigRollPhaseTime    dc.b $00 ; 
TabOrbBigRollPhaseTime_Max   = $03 ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameOrbBigMoveHandler   Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameOrbBigMoveHandler     subroutine                      ; 
                          lda TabLevelNumCurrent          ; 
                          cmp #TabLevelNumCurrent_03      ; 
.ChkLvl_01_02             bcs .Exit_NoRoll                ; greater/equal - only for level 01 and 02
                          
.GetRollTimer             ldx TabOrbBigRollPhaseTime      ; 
.IncRollTimer             inx                             ; 
                          txa                             ; 
                          and #TabOrbBigRollPhaseTime_Max ; ......## - limit to maximum
.SetRollTimer             sta TabOrbBigRollPhaseTime      ; 
                          
.IniOrbBigCount           ldx #$00                        ; 
.GetNextOrbBigRollPhase   lda TabOrbBigRollPhase,x        ; 
.ChkOrbBigRoll_Off        bne .IniOrbBigChkRollDir_Le     ; TabOrbBigRollPhase_Off = $00
                          
.IncNextOrbBigCount       inx                             ; 
.TstOrbBigCount           cpx #[TabOrbBig_Len + $01]      ; 
.ChkOrbBigCount_Max       bne .GetNextOrbBigRollPhase     ; 
                          
.Exit_NoRoll              rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IniOrbBigChkRollDir_Le   lda TabOrbBigRollDir_Wrk,x      ; 
.ChkOrbBigRollDir_None    bne .IniTileChkData             ; orb rolls already - check ground tile
                          
.TstOrbBigRollDir_Ini     lda TabOrbBigRollDir_Ini,x      ; was TabOrbBigRollDir_Wrk_None
.ChkOrbBigRollDir_Ini_Le  bne .IniOrbBigRollStart_Ri      ; $00 / $01 = Left / Right
                          
.IniOrbBigRollStart_Le    sec                             ; 
                          lda TabOrbBigDataPosX_Lo_Wrk,x  ; 
                          sbc TabLvlDatSnokiePosX_Lo_P1   ; 
.SetOrbBigRollGapX_Le     sta SNZ_OrbBigSnokieGapX        ; 
                          
                          lda TabOrbBigDataPosX_Hi_Wrk,x  ; 
                          sbc TabLvlDatSnokiePosX_Hi_P1   ; 
                          bmi .IniOrbBigRollDir           ; 
                          bne .IncNextOrbBigCount         ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetOrbBigRollGapX_Le     lda SNZ_OrbBigSnokieGapX        ; 
.TstOrbBigRollGapX_Le     cmp #SN_SnokieOrbBigGap_Le_Max  ; 
                          bcs .IncNextOrbBigCount         ; greater/equal
.TstOrbBigRollGapX_Le_Min bcc .IniOrbBigRollDir           ; lower - start roll left
; -------------------------------------------------------------------------------------------------------------- ;
.IniOrbBigRollStart_Ri    sec                             ; 
                          lda TabLvlDatSnokiePosX_Lo_P1   ; 
                          sbc TabOrbBigDataPosX_Lo_Wrk,x  ; 
.SetOrbBigRollGapX_Ri     sta SNZ_OrbBigSnokieGapX        ; 
                          
                          lda TabLvlDatSnokiePosX_Hi_P1   ; 
                          sbc TabOrbBigDataPosX_Hi_Wrk,x  ; 
                          bmi .IncNextOrbBigCount         ; greater/equal
                          bne .IniOrbBigRollDir           ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetOrbBigRollGapX_Ri     lda SNZ_OrbBigSnokieGapX        ; 
.TstOrbBigRollGapX_Ri     cmp #SN_SnokieOrbBigGap_Ri_Max  ; 
.ChkOrbBigRollGapX_Ri_Min bcc .IncNextOrbBigCount         ; lower
; -------------------------------------------------------------------------------------------------------------- ;
; start roll orb big
; -------------------------------------------------------------------------------------------------------------- ;
.IniOrbBigRollDir         lda TabOrbBigRollDir_Ini,x      ; $00 / $01 = Left / Right
                          asl a                           ; *2
                          sec                             ; 
                          sbc #$01                        ; 
.SetOrbBigRollDir         sta TabOrbBigRollDir_Wrk,x      ; $01 / $ff = Right / Left
; -------------------------------------------------------------------------------------------------------------- ;
.IniTileChkData           clc                             ; 
                          lda #<TabTileChkData            ; 
                          adc TabOrbBigDataPosX_Lo_Wrk,x  ; 
                          sta ._GetLvlTileData_Lo         ; 
                          
                          lda #>TabTileChkData            ; 
                          adc TabOrbBigDataPosX_Hi_Wrk,x  ; 
                          sta ._GetLvlTileData_Hi         ; 
                          
._GetLvlTileData_Lo       equ [* + $01]                   ; 
._GetLvlTileData_Hi       equ [* + $02]                   ; 
.GetLvlTileData           lda SN_GetDataDummy             ; 
                          tay                             ; 
                          and #$3f                        ; ..######
                          asl a                           ; *2
                          cmp TabOrbBigRollPosY_Wrk,x     ; 
                          bcs .IniTileChkData_Death       ; greater/equal
                          
.MovOrbBig_Down           dec TabOrbBigRollPosY_Wrk,x     ; lower
; -------------------------------------------------------------------------------------------------------------- ;
.IniOrbBigChkRollDir_Ri   lda TabOrbBigRollDir_Wrk,x      ; 
.ChkOrBigRollDir_Ri       bpl .GoOrbBigStep_Ri            ; 
                          
.GoOrbBigStep_Le          jsr GameOrbBigStepLeft          ; 
                          
                          lda #TabOrbBigRollDir_Wrk_Le    ; 
                          sta TabOrbBigRollDir_Wrk,x      ; 
                          
.GetRollPhaseTime_Le      lda TabOrbBigRollPhaseTime      ; 
.ChkRollPhaseTime_Le_Min  bne .GoNextOrbBigCount_Le       ; 
                          
.IncRollPhase             inc TabOrbBigRollPhase,x        ; 
                          
.GetRollPhase             lda TabOrbBigRollPhase,x        ; 
.TstRollPhase             cmp #[TabOrbBigRollPhase_Max + $01] ; 
.ChkRollPhase_Max         bne .GoNextOrbBigCount_Le       ; 
                          
.GetRollPhase_Min         lda #TabOrbBigRollPhase_Min     ; 
.SetRollPhase_Min         sta TabOrbBigRollPhase,x        ; 
                          
.GoNextOrbBigCount_Le     jmp .IncNextOrbBigCount         ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GoOrbBigStep_Ri          jsr GameOrbBigStepRight         ; 
                          
                          lda #TabOrbBigRollDir_Wrk_Ri    ; 
                          sta TabOrbBigRollDir_Wrk,x      ; 
                          
.GetRollPhaseTime_Re      lda TabOrbBigRollPhaseTime      ; 
.GetRollPhaseTime_Re_Min  bne .GoNextOrbBigCount_Ri       ; 
                          
.DecRollPhase             dec TabOrbBigRollPhase,x        ; 
.ChkRollPhase_Min         bne .GoNextOrbBigCount_Ri       ; 
                          
.GetRollPhase_Max         lda #TabOrbBigRollPhase_Max     ; 
.SetRollPhase_Max         sta TabOrbBigRollPhase,x        ; 
                          
.GoNextOrbBigCount_Ri     jmp .IncNextOrbBigCount         ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IniTileChkData_Death     tya                             ; 
.IsoTileChkData_Death     and #SN_TileChkDataDeath        ; ##......
.ChkTileChkData_Free      beq .IniOrbBigChkRollDir_Ri     ; plain ground
                          
.TstTileChkData           cmp #SN_TileChkDataDeath        ; ##......
.TstTileChkData_Death     bne .TstTileChkData_Slope       ; slope left/right
                          
.GetOrbBigPhase_Off       lda #TabOrbBigRollPhase_Off     ; big orb has rolled into a death pit
.SetOrbBigPhase_Off       sta TabOrbBigRollPhase,x        ; 
                          
                          jmp .IncNextOrbBigCount         ; 
; -------------------------------------------------------------------------------------------------------------- ;
.TstTileChkData_Slope     cmp #$80                        ; #....... - slope down right
.TstTileChkData_Slope_Ri  beq .GoOrbBigStep_Ri            ; 
GameOrbBigMoveHandlerX    bne .GoOrbBigStep_Le            ; .#...... - slope down left
; -------------------------------------------------------------------------------------------------------------- ;
; GameOrbBigStepLeft      Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameOrbBigStepLeft        subroutine                      ; 
                          ldy TabOrbBigRollSteps,x        ; 
                          dey                             ; 
                          tya                             ; 
                          and #TabOrbBigRollSteps_Max     ; 
                          sta TabOrbBigRollSteps,x        ; 
                          cmp #TabOrbBigRollSteps_Max     ; 
                          bne GameOrbBigStepLeftX         ; 
                          
.DecOrbStep_Le_Lo         dec TabOrbBigDataPosX_Lo_Wrk,x  ; 
                          lda TabOrbBigDataPosX_Lo_Wrk,x  ; 
.TstOrbStep_Le_Lo         cmp #$ff                        ; 
.ChkOrbStep_Le_Lo_Min     bne GameOrbBigStepLeftX         ; 
.DecOrbStep_Le_Hi         dec TabOrbBigDataPosX_Hi_Wrk,x  ; 
                          
GameOrbBigStepLeftX       rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameOrbBigStepRight     Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameOrbBigStepRight       subroutine                      ; 
                          ldy TabOrbBigRollSteps,x        ; 
                          iny                             ; 
                          tya                             ; 
                          and #TabOrbBigRollSteps_Max     ; 
                          sta TabOrbBigRollSteps,x        ; 
                          bne GameOrbBigStepRightX        ; 
                          
.IncOrbStep_Ri_Lo         inc TabOrbBigDataPosX_Lo_Wrk,x  ; 
.ChkOrbStep_Ri_Lo_Max     bne GameOrbBigStepRightX        ; 
.IncOrbStep_Ri_Hi         inc TabOrbBigDataPosX_Hi_Wrk,x  ; 
                          
GameOrbBigStepRightX      rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabOrbTinyDataPosX_Lo_Ini equ  *   ; 
                          dc.b <[TabTileChkData_L4_Orb_1 + $01] ; level 1
                          dc.b <[TabTileChkData_L5_Orb_1 + $01] ; 
                          dc.b <[TabTileChkData_L5_Orb_2 + $01] ; level 2
TabOrbTiny_Len            equ  [* - TabOrbTinyDataPosX_Lo_Ini - $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabOrbTinyDataPosX_Hi_Ini equ  *   ; 
                          dc.b >[TabTileChkData_L4_Orb_1 + $01] ; level 1
                          dc.b >[TabTileChkData_L5_Orb_1 + $01] ; 
                          dc.b >[TabTileChkData_L5_Orb_2 + $01] ; level 2
; -------------------------------------------------------------------------------------------------------------- ;
TabOrbTinyPosY_Min        equ  *   ; 
                          dc.b $13 ; 
                          dc.b $13 ; 
                          dc.b $13 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabOrbTinySnokieGapX_Min  equ  *   ; 
                          dc.b $11 ; 
                          dc.b $18 ; 
                          dc.b $0d ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabOrbTinyStatus          equ  *   ; 
TabOrbTinyStatus_Off         = $00 ; 
TabOrbTinyStatus_Wait        = $01 ; 
TabOrbTinyStatus_Fly_DE      = $02 ; down-east
TabOrbTinyStatus_Fly_DW      = $03 ; down-west
TabOrbTinyStatus_Fly_W       = $04 ; west
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabOrbTinyDataPosX_Lo_Wrk equ  *   ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabOrbTinyDataPosX_Hi_Wrk equ  *   ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabOrbTinySubSprtPosY     equ  *   ; 
TabOrbTinySubSprtPosY_Ini    = $37 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabOrbTinyFlyWestTime     equ  *   ; 
TabOrbTinyFlyWestTime_Max    = $c0 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabOrbTinyFlySteps        equ  *   ; 
TabOrbTinyFlySteps_Min       = $00 ; 
TabOrbTinyFlySteps_Max       = $03 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
                          dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitOrbsTiny_L04_L05    Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitOrbsTiny_L04_L05      subroutine                      ; 
.IniTabByteCount          ldx #TabOrbTiny_Len             ; 
.GetNextTabByte           lda #TabOrbTinyStatus_Wait      ; 
                          sta TabOrbTinyStatus,x          ; 
                          
                          lda TabOrbTinyDataPosX_Lo_Ini,x ; 
                          sta TabOrbTinyDataPosX_Lo_Wrk,x ; 
                          
                          lda TabOrbTinyDataPosX_Hi_Ini,x ; 
                          sta TabOrbTinyDataPosX_Hi_Wrk,x ; 
                          
                          lda #TabOrbTinySubSprtPosY_Ini  ; 
                          sta TabOrbTinySubSprtPosY,x     ; 
                          
                          lda #TabOrbTinyFlyWestTime_Max  ; 
                          sta TabOrbTinyFlyWestTime,x     ; 
                          
                          lda #TabOrbTinyFlySteps_Min     ; 
                          sta TabOrbTinyFlySteps,x        ; 
                          
.DecTabByteCount          dex                             ; 
.ChkTabByteCount_Min      bpl .GetNextTabByte             ; 
                          
InitOrbsTiny_L04_L05X     rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameOrbTinySprt_L04_L05 Does    : Fly tiny orb sprites
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameOrbTinySprt_L04_L05   subroutine                      ; 
                          lda TabLevelNumCurrent          ; 
.Tst_Lvl_04               cmp #TabLevelNumCurrent_04      ; 
                          bcc .ExitOrbsSprtOff            ; lower
                          
.Tst_Lvl_06               cmp #TabLevelNumCurrent_06      ; 
                          bcs .ExitOrbsSprtOff            ; higher/equal
                          
.IniOrbTinyCount          ldx #$00                        ; was level 03 or 04
.GetNextOrbTinyStatus     lda TabOrbTinyStatus,x          ; 
                          bne .GetOrbTiny_PosX            ; 
                          
.IncNextOrbTinyCount      inx                             ; 
.TstOrbTinyTabCount       cpx #[TabOrbTiny_Len + $01]     ; 
.ChkOrbTinyTabCount_Max   bne .GetNextOrbTinyStatus       ; 
                          
.ExitOrbsSprtOff          jmp GameOrbsSpriteOff           ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetOrbTiny_PosX          sec                             ; 
                          lda TabOrbTinyDataPosX_Lo_Wrk,x ; 
                          sbc TabLvlScrollSteps_Lo        ; 
                          sta SNZ_OrbTinySprt_PosX_Lo     ; 
                          
                          lda TabOrbTinyDataPosX_Hi_Wrk,x ; 
                          sbc TabLvlScrollSteps_Hi        ; 
                          sta SNZ_OrbTinySprt_PosX_Hi     ; 
                          
                          asl SNZ_OrbTinySprt_PosX_Lo     ; *2
                          rol SNZ_OrbTinySprt_PosX_Hi     ; 
                          asl SNZ_OrbTinySprt_PosX_Lo     ; *4
                          rol SNZ_OrbTinySprt_PosX_Hi     ; 
                          
                          lda SNZ_OrbTinySprt_PosX_Hi     ; 
.ChkOrbTiny_PosX_Hi_Min   beq .GetOrbTiny_PosX_Lo_Min     ; 
; -------------------------------------------------------------------------------------------------------------- ;
; check if small orb flies out of the left side of the screen
; -------------------------------------------------------------------------------------------------------------- ;
.TstOrbTiny_PosX_Hi_Max   cmp #$ff                        ; 
.ChkOrbTiny_PosX_Hi_Max   bne .IncNextOrbTinyCount        ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetOrbTiny_PosX_Lo_Max   lda SNZ_OrbTinySprt_PosX_Lo     ; 
.TstOrbTiny_PosX_Lo_Max   cmp #$fd                        ; 
                          bcc .IncNextOrbTinyCount        ; lower
                          bcs .IniTstScollOrbTinyPosX     ; higher/equal - always
; -------------------------------------------------------------------------------------------------------------- ;
.GetOrbTiny_PosX_Lo_Min   lda SNZ_OrbTinySprt_PosX_Lo     ; 
.TstOrbTiny_PosX_Lo_Min   cmp #$bc                        ; 
                          bcs .IncNextOrbTinyCount        ; 
; -------------------------------------------------------------------------------------------------------------- ;
; check if big orb sits inside the screen limits
; -------------------------------------------------------------------------------------------------------------- ;
.IniTstScollOrbTinyPosX   clc                             ; 
                          adc TabOrbTinyFlySteps,x        ; 
                          clc                             ; 
                          adc TabLevelSoftScrollValue     ; 
                          clc                             ; 
                          adc #$20                        ; 
                          
.TstOrbTinyPosX_Min       cmp #$20                        ; 
.ChkOrbTinyPosX_Min       bcc .IncNextOrbTinyCount        ; lower
                          
.TstOrbTinyPosX_Max       cmp #$d9                        ; 
.ChkOrbTinyPosX_Max       bcs .IncNextOrbTinyCount        ; greater/equal
; -------------------------------------------------------------------------------------------------------------- ;
                          sta SNZ_OrbTinySprt_PosX_Lo     ; 
                          
                          lda TabOrbTinySubSprtPosY,x     ; 
                          asl a                           ; *2
                          sta ._SubOrbTinySprtPosY        ; 
                          
                          sec                             ; 
                          lda #$e0                        ; 
._SubOrbTinySprtPosY      equ [* + $01]                   ; 
                          sbc #$00                        ; 
                          sta SP2Y                        ; VIC($D005) Sprite 2 Y-Pos (Bits 0-7)
                          
                          lda SNZ_OrbTinySprt_PosX_Lo     ; 
                          asl a                           ; *2
                          sta SNZ_OrbTinySprt_PosX_Lo     ; 
                          lda #$00                        ; 
                          rol a                           ; *2
                          sta SNZ_OrbTinySprt_PosX_Hi     ; 
                          
                          sec                             ; 
                          lda SNZ_OrbTinySprt_PosX_Lo     ; 
                          sbc #$2a                        ; 
                          sta SP2X                        ; VIC($D004) Sprite 2 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          
                          lda SNZ_OrbTinySprt_PosX_Hi     ; 
                          sbc #$00                        ; 
                          asl a                           ; 
                          asl a                           ; 
                          sta ._InsOrbTinySprt_MSIGX      ; 
                          
                          lda MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
                          and #$fb                        ; #####.##
._InsOrbTinySprt_MSIGX    equ [* + $01]                   ; 
.InsOrbTinySprt_MSIGX     ora #$00                        ; 
.SetOrbTinySprt_MSIGX     sta MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
                          
                          lda #[SpriteData_OrbTiny / VIC_Sprt_BlockLen] ; 
                          sta SN_GameSpritePtr_02         ; 
                          
.GetOrbTinySprt_On        lda SPENA                       ; VIC($D015) Sprite Enable Register
.InsOrbTinySprt_On        ora #$04                        ; .....#..
.SetOrbTinySprt_On        sta SPENA                       ; VIC($D015) Sprite Enable Register
                          
GameOrbTinySprt_L04_L05X  rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameShowLevelTime       Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameShowLevelTime         subroutine                      ; 
.IniStageTimeSprtOff      ldx #$00                        ; 
                          
.GetSectTimeVal           lda TabLevelTime                ; 
                          lsr a                           ; /2
                          
.SavNextSectTimeVal       pha                             ; 
.TstSectTimeVal_Max       cmp #[TabTimeSpriteGfx_Len + $01] ; 
.ChkSectTimeeVal_Max      bcc .GetSectTimeGfxTabOff       ; lower
                          
.GetSectTimeGfxTabOff_Max lda #TabTimeSpriteGfx_Len       ; 
                          
.GetSectTimeGfxTabOff     tay                             ; 
.GetSectTimeGfx           lda TabTimeSpriteGfx,y          ; 
                          
.GetSectTimeSprtGfxOff    ldy TabSpriteGfxTimeOff,x       ; 
.SetSectTimeSprtGfx_Row1  sta [SpriteData_TimeLe + $00],y ; 
.SetSectTimeSprtGfx_Row2  sta [SpriteData_TimeLe + $03],y ; 
                          
.RstSectTimeVal           pla                             ; 
                          cmp #[TabTimeSpriteGfx_Len + $01] ; 
                          bcc .SetSectTimeVal_Min         ; lower
                          
                          sec                             ; 
                          sbc #TabTimeSpriteGfx_Len       ; 
                          jmp .IncSectTimeSprtOff         ; 
                          
.SetSectTimeVal_Min       lda #$00                        ; 
                          
.IncSectTimeSprtOff       inx                             ; 
.TstSectTimeSprtOff_Max   cpx #[TabSpriteGfxTimeOff_Len + $01] ; 
.ChkSectTimeSprtOff_Max   bcc .SavNextSectTimeVal         ; lower
                          
                          lda TabSnokieStatus             ; 
.ChkSnokieStatus_Hidden   beq .GetSectTime_White          ; snokie still hidden from display
                          
                          cmp #TabSnokieStatus_Death      ; 
                          bcc .GetSectTime_Color          ; lower
                          
.GetSectTime_White        lda #$01                        ; 
                          bne .SetSectTime                ; always
                          
.GetSectTime_Color        lda TabLevelTime                ; 
.TstSectTime_Color_Min    cmp #$15                        ; 
.ChkSectTime_Color_Min    bcc .GetSectTime_Red            ; lower
                          
.TstSectTime_Color_Mid    cmp #$29                        ; 
.ChkSectTime_Color_Mid    bcc .GetSectTime_Yellow         ; lower
                          
.GetSectTime_Green        lda #GREEN                      ; 
                          bne .SetSectTime                ; always
                          
.GetSectTime_Yellow       lda #YELLOW                     ; 
                          bne .SetSectTime                ; always
                          
.GetSectTime_Red          lda #RED                        ; 
                          
.SetSectTime              sta SP6COL                      ; VIC($D02D) Color Sprite 6
                          sta SP7COL                      ; VIC($D02E) Color Sprite 7
                          
GameShowLevelTimeX        rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabTimeSpriteGfx          equ  *   ; 
                          dc.b $00 ; ........
                          dc.b $80 ; #.......
                          dc.b $a0 ; #.#.....
                          dc.b $a8 ; #.#.#...
                          dc.b $aa ; #.#.#.#.
TabTimeSpriteGfx_Len      equ [* - TabTimeSpriteGfx - $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabSpriteGfxTimeOff       equ  *   ; 
                          dc.b [SpriteData_TimeLe_Row_3 + $00] ; 
                          dc.b [SpriteData_TimeLe_Row_3 + $01] ; 
                          dc.b [SpriteData_TimeLe_Row_3 + $02] ; 
                          
                          dc.b [SpriteData_TimeRi_Row_3 + $00] ; 
                          dc.b [SpriteData_TimeRi_Row_3 + $01] ; 
                          dc.b [SpriteData_TimeRi_Row_3 + $02] ; 
                          
                          dc.b [SpriteData_TimeLe_Row_2 + $00] ; 
                          dc.b [SpriteData_TimeLe_Row_2 + $01] ; 
                          dc.b [SpriteData_TimeLe_Row_2 + $02] ; 
                          
                          dc.b [SpriteData_TimeRi_Row_2 + $00] ; 
                          dc.b [SpriteData_TimeRi_Row_2 + $01] ; 
                          dc.b [SpriteData_TimeRi_Row_2 + $02] ; 
                          
                          dc.b [SpriteData_TimeLe_Row_1 + $00] ; 
                          dc.b [SpriteData_TimeLe_Row_1 + $01] ; 
                          dc.b [SpriteData_TimeLe_Row_1 + $02] ; 
                          
                          dc.b [SpriteData_TimeRi_Row_1 + $00] ; 
                          dc.b [SpriteData_TimeRi_Row_1 + $01] ; 
                          dc.b [SpriteData_TimeRi_Row_1 + $02] ; 
TabSpriteGfxTimeOff_Len   equ  [* - TabSpriteGfxTimeOff - $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameCheckKey_F1_F7      Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameCheckKey_F1_F7        subroutine                      ; 
                          lda #$ff                        ; ########
                          sta CIDDRA                      ; CIA1($DC02) Data Direction Register A
                          
                          lda #[CIA_Key_None ^ CIA_KeySelRow_00] ; #######.
                          sta CIAPRA                      ; CIA1($DC00) Data Port Register A
                          
                          lda CIAPRB                      ; CIA1($DC01) Data Port Register B
                          and #[$ff ^ CIA_KeyWas_F1]      ; 
                          beq .GetWaitTime_F1             ; 
                          
                          lda TabWaitForNextKeyF1         ; 
                          beq .ChkKey_F7                  ; 
                          
.DecWaitTime_F1           dec TabWaitForNextKeyF1         ; 
                          jmp .ChkKey_F7                  ; 
                          
.GetWaitTime_F1           lda TabWaitForNextKeyF1         ; 
                          beq .IniWaitTime_F1_Main        ; 
                          
.IniWaitTime_F1           lda #$05                        ; 
.SetWaitTime_F1           sta TabWaitForNextKeyF1         ; 
                          bne .ChkKey_F7                  ; always
; -------------------------------------------------------------------------------------------------------------- ;
; F1 - Restart Game at Main Menu
; -------------------------------------------------------------------------------------------------------------- ;
.IniWaitTime_F1_Main      lda #$05                        ; 
.SetWaitTime_F1_Main      sta TabWaitForNextKeyF1         ; 
                          
                          jmp MainInit                    ; 
; -------------------------------------------------------------------------------------------------------------- ;
; F7 - Pause Game
; -------------------------------------------------------------------------------------------------------------- ;
.ChkKey_F7                lda CIAPRB                      ; CIA1($DC01) Data Port Register B
                          and #[$ff ^ CIA_KeyWas_F7]      ; 
                          bne .GetResetDDRA               ; 
                          
                          jsr InitLockOscAll              ; Lock oscillators 1-3
                          
.IniWaitCount_01          ldy #$80                        ; 
.DecWaitCount_01          dey                             ; 
.ChkWaitCount_01_Min      bne .DecWaitCount_01            ; 
                          
.GetKey_F7_01             lda CIAPRB                      ; CIA1($DC01) Data Port Register B
.IsoKey_F7_01             and #[$ff ^ CIA_KeyWas_F7]      ; 
.ChkKey_F7_01             beq .GetKey_F7_01               ; 
                          
.IniWaitCount_02          ldy #$80                        ; 
.DecWaitCount_02          dey                             ; 
.ChkWaitCount_02_Min      bne .DecWaitCount_02            ; 
                          
.GetKey_F7_02             lda CIAPRB                      ; CIA1($DC01) Data Port Register B
.IsoKey_F7_02             and #[$ff ^ CIA_KeyWas_F7]      ; 
.ChkKey_F7_02             bne .GetKey_F7_02               ; 
                          
.IniWaitCount_03          ldy #$80                        ; 
.DecWaitCount_03          dey                             ; 
.ChkWaitCount_03_Min      bne .DecWaitCount_03            ; 
                          
.GetKey_F7_03             lda CIAPRB                      ; CIA1($DC01) Data Port Register B
.IsoKey_F7_03             and #[$ff ^ CIA_KeyWas_F7]      ; 
.ChkKey_F7_03             beq .GetKey_F7_03               ; 
                          
.GetResetDDRA             lda #$00                        ; 
.SetResetDDRA             sta CIDDRA                      ; CIA1($DC02) Data Direction Register A
                          
GameCheckKey_F1_F7X       rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitLockOscAll          Does    : Lock oscillators 1-3
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitLockOscAll            subroutine                      ; 
                          lda #VcRegTEST                  ; ....#... - 1=Lock Oscillator
                          sta VCREG1                      ; SID($D404) Oscillator 1 Control
                          sta VCREG2                      ; SID($D40B) Oscillator 2 Control
                          sta VCREG3                      ; SID($D412) Oscillator 3 Control
                          
InitLockOscAllX           rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitMemory              Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitMemory                subroutine                      ; 
                          lda #$00                        ; ........
                          sta SPENA                       ; VIC($D015) Sprite Enable Register
                          sta BGCOL0                      ; VIC($D021) Background Color 0 - black
                          
                          lda #LT_BLUE                    ; 
                          sta EXTCOL                      ; VIC($D020) Border Color
                          
.GetScrn_MainMenu         lda #[VIC_Video_0400 | VIC_CharS_2800] ; 
.SetScrn_MainMenu         sta VMCSB                       ; VIC($D018) VIC Chip Memory Control
                          
                          lda SCROLX                      ; VIC($D016) VIC Control Register 2 (Horiz Fine Scroll)
                          and #VIC_MCM_Off                ; ###.####
                          sta SCROLX                      ; VIC($D016) VIC Control Register 2 (Horiz Fine Scroll)
                          
                          lda #$20                        ; 
                          ldy #[SN_Scrn_RowLen * $05]     ; 
.IniNextMemByte           sta [SN_Scrn_Game_Row_01 - $01],y ; 
                          sta [SN_Scrn_Game_Row_06 - $01],y ; 
                          sta [SN_Scrn_Game_Row_0b - $01],y ; 
                          sta [SN_Scrn_Game_Row_10 - $01],y ; 
                          sta [SN_Scrn_Game_Row_15 - $01],y ; 
                          
                          lda #$80                        ; 
                          sta [SN_Scrn_HiScore_Row_01] - $01],y ; 
                          sta [SN_Scrn_HiScore_Row_06] - $01],y ; 
                          sta [SN_Scrn_HiScore_Row_0b] - $01],y ; 
                          sta [SN_Scrn_HiScore_Row_10] - $01],y ; 
                          sta [SN_Scrn_HiScore_Row_15] - $01],y ; 
                          
                          lda #WHITE                      ; 
                          sta [[COLORAM - $01] + [SN_Scrn_RowLen * $05]],y ; 
                          sta [[COLORAM - $01] + [SN_Scrn_RowLen * $0a]],y ; 
                          sta [[COLORAM - $01] + [SN_Scrn_RowLen * $0f]],y ; 
                          sta [[COLORAM - $01] + [SN_Scrn_RowLen * $00]],y ; 
                          sta [[COLORAM - $01] + [SN_Scrn_RowLen * $14]],y ; 
                          
                          lda #$20                        ; 
                          
                          dey                             ; 
                          bne .IniNextMemByte             ; 
                          
InitMemoryX               rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameStartInputHandler   Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameStartInputHandler     subroutine                      ; 
                          jsr InitLockOscAll              ; Lock oscillators 1-3
                          jsr InitMemory                  ; 
                          jsr InitSwapCharSetsHandler     ; 
                          jsr InitVicScreenOn             ; 
                          jsr InitMoveTxtSpriteSnokie     ; 
                          jsr InitOutStartMenuTxt         ; 
                          jsr InitOutStartMenuTxtSpeed    ; 
                          jsr InitOutStartMenuTxtPlayer   ; 
                          jsr GameHiScoreTableDisplay     ; 
                          
.GetNumLives_Game         lda #SN_SnokieNumLivesGame      ; 
.SetNumLives_Game         sta TabSelectNumLives           ; init to regular start with 3 Lives
                          
                          lda #$00                        ; 
                          sta TabSelectNumLevel           ; 
                          sta TabFlagDemo                 ; TabFlagDemo_Off
                          
                          lda #$00                        ; 
                          sta TabWaitCount_Lo             ; 
                          lda #$80                        ; 
                          sta TabWaitCount_Hi             ; 
                          
                          lda #$00                        ; 
                          sta SNZ_InfoTxtSnokiePos        ; 
                          sta SNZ_InfoTxtSnokieWaitTime   ; 
                          
                          lda #$05                        ; 
                          sta TabWaitForNextKeyF3         ; 
                          sta TabWaitForNextKeyF5         ; 
                          sta TabWaitForNextKeyF1         ; 
                          sta TabWaitForNextButtonFire    ; 
; -------------------------------------------------------------------------------------------------------------- ;
; Wait for selection input on start screen
; -------------------------------------------------------------------------------------------------------------- ;
.GoScrollScrnText_Snokie  jsr StartScreenTextRowScroll    ; scroll top/bottom rows text "* snokie"
                          jsr GameHiScoreOrbMoveOne       ; 
                          jsr GameHiScoreOrbMoveTwo       ; 
                          
.IniWaitTime              ldx #$00                        ; 
.DecNextWaitTime          dex                             ; 
.ChkWaitTime_Min          bne .DecNextWaitTime            ; 
                          
                          jsr GameStartGetKeyHandler      ; 
                          
                          lda #$ff                        ; ########
                          sta CIDDRA                      ; CIA1($DC02) Data Direction Register A
                          
.ChkKeybd_Row_00          lda #[CIA_Key_None ^ CIA_KeySelRow_00] ; $fe - #######.
                          sta CIAPRA                      ; CIA1($DC00) Data Port Register A
                          
.GetKeybd_Col_Result_F3   lda CIAPRB                      ; CIA1($DC01) Data Port Register B
.ChkKeybd_Col_Result_F3   and #[CIA_Key_None ^ CIA_KeyWas_F3] ; $20 - ..#.....
                          beq .KeyWas_F3                  ; 
                          
                          lda TabWaitForNextKeyF3         ; 
                          beq .GetKeybd_Col_Result_F5     ; 
                          
                          dec TabWaitForNextKeyF3         ; 
                          jmp .GetKeybd_Col_Result_F5     ; 
; -------------------------------------------------------------------------------------------------------------- ;
; f3 - flip flop game speed
; -------------------------------------------------------------------------------------------------------------- ;
.KeyWas_F3                lda TabWaitForNextKeyF3         ; 
                          beq .GetWait_F3                 ; 
                          
                          lda #$05                        ; 
                          sta TabWaitForNextKeyF3         ; 
                          bne .GetKeybd_Col_Result_F5     ; always
                          
.GetWait_F3               lda #$05                        ; 
.SetWait_F3               sta TabWaitForNextKeyF3         ; 
                          
                          lda TabGameSpeed                ; 
.FlipFlopGameSpeed        eor #TabGameSpeed_Slow          ; 
                          sta TabGameSpeed                ; 
                          beq .IniVarTxtSpeedLen          ; 
                          
.IniVarTxtNormalLen       ldx #TabTxtInfoScrnVar_03_Len   ; 
.GetNextVarTxtNormal      lda TabTxtInfoScrnVar_03,x      ; 'NORMAL'
                          sta TabTxtInfoScrnVarOut1,x     ; 
.DecVarTxtNormalLen       dex                             ; 
.ChkVarTxtNormalLen_Min   bpl .GetNextVarTxtNormal        ; 
                          bmi .GoOutTxtDifficulty         ; always
                          
.IniVarTxtSpeedLen        ldx #TabTxtInfoScrnVar_04_Len   ; 
.GetNextVarTxtSpeed       lda TabTxtInfoScrnVar_04,x      ; 'HIGH SPEED'
                          sta TabTxtInfoScrnVarOut1,x     ; 
.DecVarTxtSpeedLen        dex                             ; 
.ChkVarTxtSpeedLen_Min    bpl .GetNextVarTxtSpeed         ; 
                          
.GoOutTxtDifficulty       jsr InitOutStartMenuTxtSpeed    ; 
; -------------------------------------------------------------------------------------------------------------- ;
; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetKeybd_Col_Result_F5   lda CIAPRB                      ; CIA1($DC01) Data Port Register B
.ChkKeybd_Col_Result_F5   and #[CIA_Key_None ^ CIA_KeyWas_F5] ; $40 - .#......
                          beq .KeyWas_F5                  ; 
                          
                          lda TabWaitForNextKeyF5         ; 
                          beq .GetKeybd_Col_Result_F1     ; 
                          
                          dec TabWaitForNextKeyF5         ; 
                          jmp .GetKeybd_Col_Result_F1     ; 
; -------------------------------------------------------------------------------------------------------------- ;
; f5 - one or two players game
; -------------------------------------------------------------------------------------------------------------- ;
.KeyWas_F5                lda TabWaitForNextKeyF5         ; 
                          beq .GetWaitTime_F5             ; 
                          
                          lda #$05                        ; 
                          sta TabWaitForNextKeyF5         ; 
                          bne .GetKeybd_Col_Result_F1     ; always
                          
.GetWaitTime_F5           lda #$05                        ; 
.SetWaitTime_F5           sta TabWaitForNextKeyF5         ; 
                          
                          sec                             ; 
.IniPlayerCount_Max       lda #[TabNumPlayers_Two + $01]  ; 
.SubPlayerCount_Actual    sbc TabNumPlayers               ; flip-flop player count
.SetPlayerCount_New       sta TabNumPlayers               ; 
                          cmp #TabNumPlayers_One          ; 
                          bne .IniVarTxtTwoPlrLen         ; 
                          
.IniVarTxtOnePlrLen       ldx #TabTxtInfoScrnVar_01_Len   ; 
.GetNextVarTxtOnePlr      lda TabTxtInfoScrnVar_01,x      ; 
                          sta TabTxtInfoScrnVarOut2,x     ; 
.DecVarTxtOnePlrLen       dex                             ; 
.ChkVarTxtOnePlrLen_Min   bpl .GetNextVarTxtOnePlr        ; 
                          bmi .GoOutTxtPlayers            ; always
                          
.IniVarTxtTwoPlrLen       ldx #$0a                        ; 
.GetNextVarTxtTwoPlr      lda TabTxtInfoScrnVar_02,x      ; 
                          sta TabTxtInfoScrnVarOut2,x     ; 
.DecVarTxtTwoPlrLen       dex                             ; 
.ChkVarTxtTwoPlrLen_Min   bpl .GetNextVarTxtTwoPlr        ; 
                          
.GoOutTxtPlayers          jsr InitOutStartMenuTxtPlayer   ; 
; -------------------------------------------------------------------------------------------------------------- ;
; f1 - start game
; -------------------------------------------------------------------------------------------------------------- ;
.GetKeybd_Col_Result_F1   lda CIAPRB                      ; CIA1($DC01) Data Port Register B
.ChkKeybd_Col_Result_F1   and #[CIA_Key_None ^ CIA_KeyWas_F1] ; $10 - ...#....
                          beq .GetWait_F1                 ; 
                          
                          lda TabWaitForNextKeyF1         ; 
                          beq .IniGet_Fire                ; 
                          
                          dec TabWaitForNextKeyF1         ; 
                          jmp .IniGet_Fire                ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetWait_F1               lda TabWaitForNextKeyF1         ; 
.ChkWait_F1               beq GameStartInputHandlerX      ; 
                          
                          lda #$05                        ; 
                          sta TabWaitForNextKeyF1         ; 
; -------------------------------------------------------------------------------------------------------------- ;
; fire - start game
; -------------------------------------------------------------------------------------------------------------- ;
.IniGet_Fire              lda #$00                        ; 
                          sta CIDDRA                      ; CIA1($DC02) Data Direction Register A
                          
                          lda CIAPRA                      ; CIA1($DC00) Data Port Register A
                          and #CIA_Joy_Fi                 ; $10 - ...#....
                          beq .KeyWas_F1                  ; 
                          
                          lda TabWaitForNextButtonFire    ; 
                          beq .GoNextInputLoop            ; 
                          
.DecWait_Fire             dec TabWaitForNextButtonFire    ; 
                          
.GoNextInputLoop          jmp .GoScrollScrnText_Snokie    ; 
; -------------------------------------------------------------------------------------------------------------- ;
.KeyWas_F1                lda TabWaitForNextButtonFire    ; 
                          beq GameStartInputHandlerX      ; 
                          
.GetWaitTime_F1           lda #$05                        ; 
                          sta TabWaitForNextButtonFire    ; 
                          bne .GoNextInputLoop            ; always
                          
GameStartInputHandlerX    rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
InfoScreenTextVariable    include inc\SN_TextInfoScreenVar.asm ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitGameSound           Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitGameSound             subroutine                      ; 
                          lda #$08                        ; 
                          sta VCREG2                      ; SID($D40B) Oscillator 2 Control
                          
                          lda #$0f                        ; 
                          sta ATDCY1                      ; SID($D405) Oscillator 1 Attack/Decay2
                          
                          lda #$07                        ; 
                          sta ATDCY2                      ; SID($D40C) Oscillator 2 Attack/Decay
                          sta SUREL2                      ; SID($D40D) Oscillator 2 Sustain/Release
                          
                          lda #$ff                        ; 
                          sta SUREL1                      ; SID($D406) Oscillator 1 Sustain/Release
                          
InitGameSoundX            rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitMoveTxtSpriteSnokie Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitMoveTxtSpriteSnokie   subroutine                      ; 
                          jsr InitSprites                 ; 
                          
.IniSprtNum_Mov_Ri        ldx #$06                        ; 
.IniSprtNum_Mov_Le        ldy #$04                        ; 
.MovNextTxtSprtSnokie     jsr InitMoveTxtSpriteSnokieDo   ; move a sprite pair down
                          jsr InitMoveTxtSpriteSnokieLR   ; move a sprite pair left right
                          
.DecSprtNum_Pair_Ri       inx                             ; 
                          inx                             ; 
                          
.DecSprtNum_Pair_Le       dey                             ; 
                          dey                             ; 
                          
.TstSprtNum_Pair_Max      cpx #$0c                        ; 
.ChkSprtNum_Pair_Max      bne .MovNextTxtSprtSnokie       ; 
                          
InitMoveTxtSpriteSnokieX  rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitMoveTxtSpriteSnokieDo Does  : move a snokie text sprite pair down
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitMoveTxtSpriteSnokieDo subroutine                      ; 
                          lda #$3a                        ; 
                          sta [TabMoveSpriteMaxPosX + $05]; 
                          
.IncNextSprite_PosX       inc SP0Y,x                      ; VIC($D001) Sprite 0 Y-Pos (Bits 0-7)
                          
                          txa                             ; 
.Sav_SpriteNum_PosX       pha                             ; 
                          
.Get_SpriteNum_PosY       tya                             ; 
                          tax                             ; 
                          
.IncSprite_PosY_          inc SP0Y,x                      ; VIC($D001) Sprite 0 Y-Pos (Bits 0-7)
                          
                          pla                             ; 
.Rst_SpriteNum_PosX       tax                             ; 
                          
                          jsr WaitForNextSpriteMove       ; 
                          
                          dec [TabMoveSpriteMaxPosX + $05]; 
                          bne .IncNextSprite_PosX         ; 
                          
InitMoveTxtSpriteSnokieDoX rts                            ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitMoveTxtSpriteSnokieLR Does  : move a snokie text sprite pair left/right
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitMoveTxtSpriteSnokieLR subroutine                      ; 
.MovNextSprt_Ri           clc                             ; 
                          lda SP0X,x                      ; VIC($D000) Sprite 0 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          adc #$01                        ; 
                          sta SP0X,x                      ; VIC($D000) Sprite 0 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          bcc .MovNextSprt_Le             ; 
                          
                          txa                             ; 
.Sav_SpriteNum_PosX       pha                             ; 
                          
                          lsr a                           ; care for PosX high bit
                          tax                             ; 
                          lda TabMoveSpriteHiPosX,x       ; 
                          ora MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
                          sta MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
                          
                          pla                             ; 
.Rst_SpriteNum_PosX       tax                             ; 
                          
.MovNextSprt_Le           sec                             ; 
                          lda SP0X,y                      ; VIC($D000) Sprite 0 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          sbc #$01                        ; 
                          sta SP0X,y                      ; VIC($D000) Sprite 0 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          bcs .WaitForNextMove            ; 
                          
.Sav_SpriteNum_PosY       tya                             ; 
                          pha                             ; 
                          
                          lsr a                           ; care for PosX high bit
                          tay                             ; 
                          lda TabMoveSpriteHiPosX,y       ; 
                          and MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
                          sta MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
                          
                          pla                             ; 
.Rst_SpriteNum_PosY       tay                             ; 
                          
.WaitForNextMove          jsr WaitForNextSpriteMove       ; 
                          
                          lda TabMoveSpriteMaxPosX,y      ; 
                          cmp SP0X,y                      ; VIC($D000) Sprite 0 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          bne .MovNextSprt_Ri             ; 
                          
InitMoveTxtSpriteSnokieLRX rts                            ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabMoveSpriteHiPosX       equ  *   ; 
                          dc.b $fe ; #######.
                          dc.b $fd ; ######.#
                          dc.b $fb ; #####.##
                          
                          dc.b $08 ; ....#...
                          dc.b $10 ; ...#....
                          dc.b $20 ; ..#.....
; -------------------------------------------------------------------------------------------------------------- ;
TabMoveSpriteMaxPosX      equ  *   ; 
                          dc.b $3a ; 
                          dc.b $00 ; 
                          dc.b $68 ; 
                          dc.b $00 ; 
                          dc.b $96 ; 
                          dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabWaitTimeSpriteMoveHi   dc.b $00 ; 
TabWaitTimeSpriteMoveLo   dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
; WaitForNextSpriteMove   Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
WaitForNextSpriteMove     subroutine                      ; 
.GetWaitTimeHi            lda #$02                        ; 
.IniWaitTimeHi            sta TabWaitTimeSpriteMoveHi     ; 
                          
.GetNextWaitTimeLo        lda #$ff                        ; 
.SetWaitTimeLo            sta TabWaitTimeSpriteMoveLo     ; 
                          
.DecNextWaitTimeLo        dec TabWaitTimeSpriteMoveLo     ; 
.ChkNextWaitTimeLo_Mic    bne .DecNextWaitTimeLo          ; 
                          
.DecWaitTimeHi            dec TabWaitTimeSpriteMoveHi     ; 
.ChkWaitTimeHi_Min        bne .GetNextWaitTimeLo          ; 
                          
WaitForNextSpriteMoveX    rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitSprites             Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitSprites               subroutine                      ; 
                          lda #$3f                        ; ..######
                          sta SPMC                        ; VIC($D01C) Sprite Multicolor
                          sta YXPAND                      ; VIC($D017) Sprite Scale Double Width
                          sta XXPAND                      ; VIC($D01D) Sprite Double Height
                          
                          lda #LT_BLUE                    ; 
                          sta SP0COL                      ; VIC($D027) Color Sprite 0
                          sta SP1COL                      ; VIC($D028) Color Sprite 1
                          sta SP2COL                      ; VIC($D029) Color Sprite 2
                          sta SP3COL                      ; VIC($D02A) Color Sprite 3
                          sta SP4COL                      ; VIC($D02B) Color Sprite 4
                          sta SP5COL                      ; VIC($D02C) Color Sprite 5
                          
                          lda #WHITE                      ; .......#
                          sta SPMC0                       ; VIC($D025) Sprite Multicolor Register 0
                          sta SPMC1                       ; VIC($D026) Sprite Multicolor Register 1
                          
                          lda #$08                        ; 
                          sta SP0Y                        ; VIC($D001) Sprite 0 Y-Pos (Bits 0-7)
                          sta SP1Y                        ; VIC($D003) Sprite 1 Y-Pos (Bits 0-7)
                          sta SP2Y                        ; VIC($D005) Sprite 2 Y-Pos (Bits 0-7)
                          sta SP3Y                        ; VIC($D007) Sprite 3 Y-Pos (Bits 0-7)
                          sta SP4Y                        ; VIC($D009) Sprite 4 Y-Pos (Bits 0-7)
                          sta SP5Y                        ; VIC($D00B) Sprite 5 Y-Pos (Bits 0-7)
                          
                          lda #$07                        ; .....###
                          sta MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
                          
                          lda #$18                        ; 
                          sta SP0X                        ; VIC($D000) Sprite 0 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          
                          lda #$34                        ; 
                          sta SP1X                        ; VIC($D002) Sprite 1 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          sta SP2X                        ; VIC($D004) Sprite 2 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          
                          lda #$26                        ; 
                          sta SP3X                        ; VIC($D006) Sprite 3 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          sta SP4X                        ; VIC($D008) Sprite 4 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          sta SP5X                        ; VIC($D00A) Sprite 5 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          
                          ldx #[SpriteData_Text_Snokie / VIC_Sprt_BlockLen] ; 
                          stx SN_GameSpritePtr_00         ; 
                          inx                             ; 
                          stx SN_GameSpritePtr_01         ; 
                          inx                             ; 
                          stx SN_GameSpritePtr_02         ; 
                          inx                             ; 
                          stx SN_GameSpritePtr_03         ; 
                          inx                             ; 
                          stx SN_GameSpritePtr_04         ; 
                          inx                             ; 
                          stx SN_GameSpritePtr_05         ; 
                          
                          lda #$3f                        ; ..######
                          sta SPENA                       ; VIC($D015) Sprite Enable Register
                          
InitSpritesX              rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; StartScreenTextRowScroll Does   : scroll top/bottom rows text "* snokie"
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
StartScreenTextRowScroll  subroutine                      ; 
                          inc SNZ_InfoTxtSnokieWaitTime   ; 
                          
                          lda SNZ_InfoTxtSnokieWaitTime   ; 
                          and #$3f                        ; ..######
                          sta SNZ_InfoTxtSnokieWaitTime   ; 
                          beq .GetSnokieScrollPos         ; 
                          
.Exit_NoScroll            rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; scroll '* SNOKIE' on top and bottom info screen line
; -------------------------------------------------------------------------------------------------------------- ;
.GetSnokieScrollPos       ldx SNZ_InfoTxtSnokiePos        ; 
.DecSnokieScrollPos       dex                             ; 
.ChkSnokieScrollPos_Min   bpl .SetSnokieScrollPos         ; 
                          
.IniSnokieScrollPos_Max   ldx #[TxtInfoScrn_Snokie_Len - $01] ; 
                          
.SetSnokieScrollPos       stx SNZ_InfoTxtSnokiePos        ; 
; -------------------------------------------------------------------------------------------------------------- ;
; scroll top row forward
; -------------------------------------------------------------------------------------------------------------- ;
.IniScrnRowPos_Top        ldy #$00                        ; 
.GetNextTxtSnokie_Top     lda TxtInfoScrn_Snokie,x        ; 
                          and #$bf                        ; #.###### - set text screen code
.SetTxtSnokie_Top         sta [SN_Scrn_Game + [SN_Scrn_RowLen * $00]],y ; top screen row
                          
.IncScrnRowPos_Top        iny                             ; 
                          
.IncSnokieTxtPos_Top      inx                             ; 
.TstSnokieTxtPos_Top      cpx #TxtInfoScrn_Snokie_Len     ; 
.ChkSnokieTxtPos_Top_Min  bcc .TstSnokie_Top_Row          ; 
                          
.IniSnokieTxtPos_Min      ldx #$00                        ; 
                          
.TstSnokie_Top_Row        cpy #SN_Scrn_RowLen             ; 
.ChkSnokie_Top_Row_Max    bcc .GetNextTxtSnokie_Top       ; lower
                          
                          sec                             ; 
                          lda #[TxtInfoScrn_Snokie_Len - $01] ; 
                          sbc SNZ_InfoTxtSnokiePos        ; 
                          tax                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; scroll bottom row backward
; -------------------------------------------------------------------------------------------------------------- ;
.IniScrnRowPos_Bot        ldy #$00                        ; 
.GetNextTxtSnokie_Bot     lda TxtInfoScrn_Snokie,x        ; 
                          and #$bf                        ; #.###### - set text screen code
                          sta [SN_Scrn_Game + [SN_Scrn_RowLen * $18]],y ; bottom screen row
                          
.IniccrnRowPos_Bot        iny                             ; 
                          
.IncSnokieTxtPos_Bot      inx                             ; 
.TstSnokieTxtPos_Bot      cpx #TxtInfoScrn_Snokie_Len     ; 
.ChkSnokieTxtPos_Bot_Min  bcc .TstSnokie_Bot_Row          ; 
                          
                          ldx #$00                        ; 
                          
.TstSnokie_Bot_Row        cpy #SN_Scrn_RowLen             ; 
.ChkSnokie_Bot_Row_Max    bcc .GetNextTxtSnokie_Bot       ; lower
                          
                          jsr GameHiScoreFlashSprite      ; 
                          jsr GameHiScoreFlashLeader      ; 
                          
StartScreenTextRowScrollX rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
TxtInfoScrn_Snokie        equ  *   ; 
                          dc.b $2a ; *
                          dc.b $20 ; _
                          dc.b $53 ; S
                          dc.b $4e ; N
                          dc.b $4f ; O
                          dc.b $4b ; K
                          dc.b $49 ; I
                          dc.b $45 ; E
                          dc.b $20 ; _
TxtInfoScrn_Snokie_Len    equ  [* - TxtInfoScrn_Snokie] ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameOrbTinyMoveHandler  Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameOrbTinyMoveHandler    subroutine                      ; 
                          lda TabLevelNumCurrent          ; 
                          cmp #TabLevelNumCurrent_04      ; 
                          bcc .Exit_NoMove                ; lower
                          
                          cmp #TabLevelNumCurrent_06      ; 
                          bcs .Exit_NoMove                ; greater/equal
                          
.IniOrbTinyCount          ldx #$00                        ; 
.GetNextOrbTinyStatus     lda TabOrbTinyStatus,x          ; 
                          bne .TstOrbTinyStatus_Wait      ; 
                          
.IncNextOrbTinyCount      inx                             ; 
.TstOrbTinyTabCount       cpx #[TabOrbTiny_Len + $01]     ; 
.ChkOrbTinyTabCount_Max   bne .GetNextOrbTinyStatus       ; 
                          
.Exit_NoMove              rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.TstOrbTinyStatus_Wait    cmp #TabOrbTinyStatus_Wait      ; 
.ChkOrbTinyStatus_Wait    bne .TstOrbTinyStatus_Fly_DE    ; 
                          
.IniChkOrbTinySnokieGapX  sec                             ; 
                          lda TabOrbTinyDataPosX_Lo_Wrk,x ; 
                          sbc TabLvlDatSnokiePosX_Lo_P1   ; 
                          sta SNZ_OrbTinySnokieGapX       ; 
                          
                          lda TabOrbTinyDataPosX_Hi_Wrk,x ; 
                          sbc TabLvlDatSnokiePosX_Hi_P1   ; 
                          bmi .SetOrbTinyStatus_Fly_DE    ; 
                          bne .IncNextOrbTinyCount        ; 
                          
.GetOrbTinySnokieGapX_Min lda TabOrbTinySnokieGapX_Min,x  ; 
.TstOrbTinySnokieGapX     cmp SNZ_OrbTinySnokieGapX       ; 
.ChkOrbTinySnokieGapX_Min bcc .IncNextOrbTinyCount        ; lower
                          
.SetOrbTinyStatus_Fly_DE  inc TabOrbTinyStatus,x          ; down-east
.GetOrbTinyStatus         lda TabOrbTinyStatus,x          ; 
; -------------------------------------------------------------------------------------------------------------- ;
.TstOrbTinyStatus_Fly_DE  cmp #TabOrbTinyStatus_Fly_DE    ; down-east
.ChkOrbTinyStatus_Fly_DE  bne .TstOrbTinyStatus_Fly_DW    ; 
                          
                          dec TabOrbTinySubSprtPosY,x     ; 
                          
.DecOrbTinyFlyTime_DE     inc TabOrbTinyFlySteps,x        ; 
                          lda TabOrbTinyFlySteps,x        ; 
                          and #TabOrbTinyFlySteps_Max     ; 
                          sta TabOrbTinyFlySteps,x        ; 
                          bne .IniOrbTinyTstDown_Max      ; 
                          
.SetOrbTinyFly_East       inc TabOrbTinyDataPosX_Lo_Wrk,x ; 
                          bne .IniOrbTinyTstDown_Max      ; 
                          inc TabOrbTinyDataPosX_Hi_Wrk,x ; 
                          
.IniOrbTinyTstDown_Max    sec                             ; 
                          lda #$37                        ; 
                          sbc TabOrbTinyPosY_Min,x        ; 
                          lsr a                           ; 
                          clc                             ; 
                          adc TabOrbTinyPosY_Min,x        ; 
.TstOrbTinyTst_DE         cmp TabOrbTinySubSprtPosY,x     ; 
.ChkOrbTinyTst_DE_Max     bcc .IncNextOrbTinyCount        ; lower
                          
.SetOrbTinyStatus_Fly_DW  inc TabOrbTinyStatus,x          ; down-west
                          
.GoIncOrbTinyTabCount_DE  jmp .IncNextOrbTinyCount        ; 
; -------------------------------------------------------------------------------------------------------------- ;
.TstOrbTinyStatus_Fly_DW  cmp #TabOrbTinyStatus_Fly_DW    ; down-west
.ChkOrbTinyStatus_Fly_DW  bne .DecOrbTinyFlyTime_W        ; 
                          
                          dec TabOrbTinySubSprtPosY,x     ; 
                          
.DecOrbTinyFlyTime_DW     dec TabOrbTinyFlySteps,x        ; 
                          lda TabOrbTinyFlySteps,x        ; 
                          and #TabOrbTinyFlySteps_Max     ; 
                          sta TabOrbTinyFlySteps,x        ; 
                          cmp #TabOrbTinyFlySteps_Max     ; 
                          bne .GetOrbTinyPosY_Min         ; 
                          
.SetOrbTinyFly_Down_West  dec TabOrbTinyDataPosX_Lo_Wrk,x ; 
                          lda TabOrbTinyDataPosX_Lo_Wrk,x ; 
                          cmp #$ff                        ; -1
                          bne .GetOrbTinyPosY_Min         ; 
                          dec TabOrbTinyDataPosX_Hi_Wrk,x ; 
                          
.GetOrbTinyPosY_Min       lda TabOrbTinyPosY_Min,x        ; 
                          cmp TabOrbTinySubSprtPosY,x     ; 
                          bcc .GoIncOrbTinyTabCount_DW    ; lower
                          
.SetOrbTinyStatus_Fly_W   inc TabOrbTinyStatus,x          ; 
                          
.GoIncOrbTinyTabCount_DW  jmp .IncNextOrbTinyCount        ; 
; -------------------------------------------------------------------------------------------------------------- ;
; TabOrbTinyStatus_Fly_W - fly tiny orb west
; -------------------------------------------------------------------------------------------------------------- ;
.DecOrbTinyFlyTime_W      dec TabOrbTinyFlySteps,x        ; 
                          lda TabOrbTinyFlySteps,x        ; 
                          and #TabOrbTinyFlySteps_Max     ; 
                          sta TabOrbTinyFlySteps,x        ; 
.TstOrbTinyFlyTime_W      cmp #TabOrbTinyFlySteps_Max     ; 
.ChkOrbTinyFlyTime_W_Max  bne .DecOrbTinyFly_W_Time       ; 
                          
.DecOrbTinyFlyPosX_W_Lo   dec TabOrbTinyDataPosX_Lo_Wrk,x ; 
                          lda TabOrbTinyDataPosX_Lo_Wrk,x ; 
.TstOrbTinyFlyPosX_W      cmp #$ff                        ; -1
.ChkOrbTinyFlyPosX_W_Min  bne .DecOrbTinyFly_W_Time       ; 
.DecOrbTinyFlyPosX_W_Hi   dec TabOrbTinyDataPosX_Hi_Wrk,x ; 
                          
.DecOrbTinyFly_W_Time     dec TabOrbTinyFlyWestTime,x     ; 
.ChkOrbTinyFly_W_Time_Min bne GameOrbTinyMoveHandlerX     ; 
                          
.GetOrbTinyStatus_Off     lda #TabOrbTinyStatus_Off       ; leftmost level border reached
.SetOrbTinyStatus_Off     sta TabOrbTinyStatus,x          ; 
                          
GameOrbTinyMoveHandlerX   jmp .IncNextOrbTinyCount        ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitGameSpritesScore    Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitGameSpritesScore      subroutine                      ; 
                          lda #$00                        ; 
.IniSprtScoreOff          ldy #SpriteData_Score_Len       ; 
.ClrNextSprtScoreByte     sta [SpriteData_Score - $01],y  ; 
.DecSprtScoreOff          dey                             ; 
.ChkSprtScoreOff_Min      bne .ClrNextSprtScoreByte       ; 
                          
.SetSprtColor             lda #WHITE                      ; 
                          sta SP3COL                      ; VIC($D02A) Color Sprite 3
                          sta SP4COL                      ; VIC($D02B) Color Sprite 4
                          sta SP5COL                      ; VIC($D02C) Color Sprite 5
                          
.SetSprtSize              lda YXPAND                      ; VIC($D017) Sprite Scale Double Width
                          and #$c0                        ; ##......
                          sta YXPAND                      ; VIC($D017) Sprite Scale Double Width
                          
                          lda XXPAND                      ; VIC($D01D) Sprite Double Height
                          and #$c0                        ; ##......
                          sta XXPAND                      ; VIC($D01D) Sprite Double Height
                          
.SetSprtPtr               ldx #[SpriteData_Score / VIC_Sprt_BlockLen] ; 
                          stx SN_GameSpritePtr_03         ; 
                          inx                             ; 
                          stx SN_GameSpritePtr_04         ; 
                          inx                             ; 
                          stx SN_GameSpritePtr_05         ; 
                          
.SetSprtPos               lda #$28                        ; 
                          sta SP3X                        ; VIC($D006) Sprite 3 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          
                          lda #$3a                        ; 
                          sta SP3Y                        ; VIC($D007) Sprite 3 Y-Pos (Bits 0-7)
                          sta SP4Y                        ; VIC($D009) Sprite 4 Y-Pos (Bits 0-7)
                          sta SP5Y                        ; VIC($D00B) Sprite 5 Y-Pos (Bits 0-7)
                          
                          lda #$40                        ; 
                          sta SP4X                        ; VIC($D008) Sprite 4 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          
                          lda #$58                        ; 
                          sta SP5X                        ; VIC($D00A) Sprite 5 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          
                          lda SPENA                       ; VIC($D015) Sprite Enable Register
                          ora #$38                        ; ..###...
                          sta SPENA                       ; VIC($D015) Sprite Enable Register
                          
                          lda #$00                        ; 
                          sta MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
                          
InitGameSpritesScoreX     rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitGameSpritesTime     Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitGameSpritesTime       subroutine                      ; 
                          lda #[SpriteData_TimeLe / VIC_Sprt_BlockLen] ; 
                          sta SN_GameSpritePtr_06         ; 
                          
                          lda #[SpriteData_TimeRi / VIC_Sprt_BlockLen] ; 
                          sta SN_GameSpritePtr_07         ; 
                          
                          lda #GREEN                      ; 
                          sta SP6COL                      ; VIC($D02D) Color Sprite 6
                          sta SP7COL                      ; VIC($D02E) Color Sprite 7
                          
                          lda SPMC                        ; VIC($D01C) Sprite Multicolor
                          ora #$c0                        ; ##......
                          sta SPMC                        ; VIC($D01C) Sprite Multicolor
                          
                          lda YXPAND                      ; VIC($D017) Sprite Scale Double Width
                          ora #$c0                        ; ##......
                          sta YXPAND                      ; VIC($D017) Sprite Scale Double Width
                          sta XXPAND                      ; VIC($D01D) Sprite Double Height
                          
                          lda #$c8                        ; 
                          sta SP6X                        ; VIC($D00C) Sprite 6 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          lda #$f8                        ; 
                          sta SP7X                        ; VIC($D00E) Sprite 7 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          
                          lda #$32                        ; 
                          sta SP6Y                        ; VIC($D00D) Sprite 6 Y-Pos (Bits 0-7)
                          sta SP7Y                        ; VIC($D00F) Sprite 7 Y-Pos (Bits 0-7)
                          
                          lda #WHITE                        ; 
                          sta SPMC0                       ; VIC($D025) Sprite Multicolor Register 0
                          
                          lda SPENA                       ; VIC($D015) Sprite Enable Register
                          ora #$c0                        ; ##......
                          sta SPENA                       ; VIC($D015) Sprite Enable Register
                          
InitGameSpritesTimeX      rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameNextLevelHandler    Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameNextLevelHandler      subroutine                      ; 
                          lda TabSnokieStatus             ; 
.ChkSnokieStatus_Hidden   beq .Exit_StayInCurrentLevel    ; snokie still hidden from display
                          
                          cmp #TabSnokieStatus_Death      ; 
                          bcc .DecLevelWaitTime           ; 
                          
.Exit_StayInCurrentLevel  rts                             ; 
                          
.DecLevelWaitTime         dec TabLevelWaitTime            ; 
                          bne .GetLevelNum_Current        ; 
; -------------------------------------------------------------------------------------------------------------- ;
                          ldx TabLevelNumCurrent          ; 
.GetLevelWaitTime         lda TabLevelIniWaitTime,x       ; 
.IniLevelWaitTime         sta TabLevelWaitTime            ; 
; -------------------------------------------------------------------------------------------------------------- ;
.DecLevelTime             dec TabLevelTime                ; 
.ChkLevelTime_Min         bne .GetLevelNum_Current        ; 
                          
.RanOutOfTime             lda #TabSnokieStatus_Death      ; 
                          sta TabSnokieStatus             ; 
                          
                          jmp GamePlayerDecNumLives       ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetLevelNum_Current      ldx TabLevelNumCurrent          ; 
.TstLevelNum_Current_Max  cpx #TabLevelNumCurrent_07      ; final level
.ChkLevelNum_Current_Max  beq .Exit_StayInCurrentLevel    ; 
                          
.IncLevelNum_Current      inx                             ; 
                          
.SubLvlDatSnokiePosX_Strt sec                             ; 
                          lda TabLvlDatSnokiePosX_Lo_P1   ; 
                          sbc #<TabLvlDatSnokiePosX_Start ; 
                          sta SNZ_TileChkDataPtr_Lo       ; 
                          
                          lda TabLvlDatSnokiePosX_Hi_P1   ; 
                          sbc #>TabLvlDatSnokiePosX_Start ; 
                          sta SNZ_TileChkDataPtr_Hi       ; 
                          
                          sec                             ; 
                          lda SNZ_TileChkDataPtr_Lo       ; 
                          sbc TabIniCheckDataPtrs_Lo,x    ; 
                          
                          lda SNZ_TileChkDataPtr_Hi       ; 
                          sbc TabIniCheckDataPtrs_Hi,x    ; 
.ChkStartLevel_Next       bmi .Exit_StayInCurrentLevel    ; 
                          
.IniStartLevelPing        lda #$07                        ; 
                          sta ATDCY2                      ; SID($D40C) Oscillator 2 Attack/Decay
                          lda #$23                        ; 
                          sta FREHI2                      ; SID($D408) Oscillator 2 Freq Ctrl (high byte)
                          lda #$00                        ; 
                          sta FRELO2                      ; SID($D407) Oscillator 2 Freq Ctrl (low byte)
                          lda #$10                        ; 
                          sta VCREG2                      ; SID($D40B) Oscillator 2 Control
                          lda #$11                        ; 
                          sta VCREG2                      ; SID($D40B) Oscillator 2 Control
                          lda #$10                        ; 
                          sta VCREG2                      ; SID($D40B) Oscillator 2 Control
                          
.SetLevelNum_Current      stx TabLevelNumCurrent          ; 
                          
                          lda TabLevelIniWaitTime,x       ; 
                          sta TabLevelWaitTime            ; 
                          
.GetLevelTime_Max         lda #$84                        ; 
.IniLevelTime             sta TabLevelTime                ; 
; -------------------------------------------------------------------------------------------------------------- ;
; at main menu and after a game over:
; each previously solved level is directly selectable with its number key (1-7) - but with one life only
; -------------------------------------------------------------------------------------------------------------- ;
.GetNumLives_Select       lda TabSelectNumLives           ; 
.TstNumLives_Select       cmp #SN_SnokieNumLivesCheat     ; 
.ChkNumLives_Select       beq .Exit_StayInCurrentLevel    ; level was selected - do not update level selection feature
                          
.TstLevelNum_Select       cpx TabLevelSelect              ; 
.ChkLevelNum_Select       bcc .Exit_StayInCurrentLevel    ; lower - do not update level selection feature
                          
.SetLevelNum_Select       stx TabLevelSelect              ; higher/equal - make solved level selectable
                          
GameNextLevelHandlerX     rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
GameVariables             include inc\SN_TabVars.asm      ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameTuneDeathHandler    Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameTuneDeathHandler      subroutine                      ; 
                          lda TabSnokieStatus             ; 
                          cmp #TabSnokieStatus_Death      ; 
                          bne .Exit                       ; 
                          
.DecSnokieDeathTime       dec TabSnokieDeathTime          ; 
.ChkSnokieDeathTime_Min   bne .GetDeathJingleTime         ; 
                          
.SetSnokieStatus_Restart  inc TabSnokieStatus             ; 
                          
                          jsr IRQ_Game_SnokieMove         ; 
                          
.GetActivePlayer          lda TabActivePlayer             ; 
.TstActivePlayer          cmp #TabActivePlayer_P1         ; 
.ChkActivePlayer_One      bne .GetNumLives_P2             ; 
                          
.GetNumLives_P1           lda TabNumLives_P1              ; 
                          jmp .ChkNumLives                ; 
                          
.GetNumLives_P2           lda TabNumLives_P2              ; 
                          
.ChkNumLives              bne .GoGameTuneDeath            ; 
                          
.GoPlayDeathBlues         jsr GameTuneDeath               ; 
                          
.Exit                     rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GoGameTuneDeath          jmp GameTuneDeath               ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetDeathJingleTime       lda TabSnokieDeathTime          ; 
                          cmp #$4e                        ; 
                          bcs .GetDeathJingle_End         ; greater/equal
                          
                          cmp #$32                        ; greater/equal
                          bcs .ChkDeathJingleTime         ; 
                          
.GetDeathJingle_End       lda #VcRegTEST                  ; ....#... - 1=Lock Oscillator
.SetDeathJingle_End       sta VCREG1                      ; SID($D404) Oscillator 1 Control
                          rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.ChkDeathJingleTime       cmp #$46                        ; 
                          bcc .SetDeathJingleTune_Hi      ; lower
                          
.SetDeathJingleTune_Lo    sec                             ; 
                          sbc #$1e                        ; 
                          jmp .SetDeathJingle             ; 
                          
.SetDeathJingleTune_Hi    sec                             ; 
                          lda #$82                        ; 
                          sbc TabSnokieDeathTime          ; 
                          
.SetDeathJingle           sbc #$67                        ; 
                          eor #$ff                        ; 
                          sta FREHI1                      ; SID($D401) Oscillator 1 Freq Ctrl (high byte)
                          lda #$00                        ; 
                          sta FRELO1                      ; SID($D400) Oscillator 1 Freq Ctrl (low byte)
                          
                          lda #[VcRegTRI | VcRegGATE]     ; ...#...#
                          sta VCREG1                      ; SID($D404) Oscillator 1 Control
                          
GameTuneDeathHandlerX     rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameHiScoreHandler      Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameHiScoreHandler        subroutine                      ; 
                          lda TabSnokieStatus             ; 
                          cmp #TabSnokieStatus_Restart    ; end of death skull sequence
                          beq .IniNumLivesChk             ; 
                          
.Exit_NoScore             rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IniNumLivesChk           lda #$00                        ; 
                          
.GetNumPlayers_01         ldx TabNumPlayers               ; 
.TstNumPlayers_01         cpx #TabNumPlayers_One          ; 
.ChkNumPlayers_01_One     beq .InsNumLivesChk_P1          ; 
                          
.GetNumLivesChk_P2        lda TabNumLives_P2              ; 
                          
.InsNumLivesChk_P1        ora TabNumLives_P1              ; 
                          beq .GetFlagDemo                ; 
                          
                          jmp .GetNumPlayers_03           ; 
                          
.GetFlagDemo              lda TabFlagDemo                 ; 
.ChkFlagDemo_Off          beq .GetNumPlayers_02           ; 
                          
.ChkFlagDemo_On           jmp MainInit                    ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetNumPlayers_02         lda TabNumPlayers               ; 
.TstNumPlayers_02         cmp #TabNumPlayers_One          ; 
.ChkNumPlayers_02_One     bne .IniScorePlayerCount        ; 2 players - try to find the highest score
                          
                          lda #TabGameRtsOnlyFlag_No      ; 
                          sta TabGameRtsOnlyFlag          ; 
                          jsr GameHiScoreGetPlayerName    ; 
                          
                          lda TabGameRtsOnlyFlag          ; 
                          beq .GoMainInit_02              ; 
                          
                          jsr GameRtsOnly                 ; 
.GoMainInit_02            jmp MainInit                    ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IniScorePlayerCount      ldx #$00                        ; 
.GetNextScorePlayer_02    lda TabScorePlayer_02,x         ; compare player scores
.TstScorePlayer_01        cmp TabScorePlayer_01,x         ; 
                          bcc .GetScorePlayer_01          ; lower
                          bne .GetScorePlayer_02          ; higher
                          
.IncScorePlayerCount      inx                             ; equal - get next score byte
.TstScorePlayerCount      cpx #[TabScorePlayer_Len + $01] ; 
.ChkScorePlayerCount_Max  bne .GetNextScorePlayer_02      ; 
                          
.GetScorePlayer_01        lda #TabActivePlayer_P1         ; 
                          bne .SetScorePlayer             ; always
                          
.GetScorePlayer_02        lda #TabActivePlayer_P2         ; 
                          
.SetScorePlayer           sta TabActivePlayer             ; 
                          
                          lda #TabGameRtsOnlyFlag_No      ; 
                          sta TabGameRtsOnlyFlag          ; 
                          jsr GameHiScoreGetPlayerName    ; 
                          
                          sec                             ; switch active player
                          lda #[TabActivePlayer_P2 + $01] ; 
                          sbc TabActivePlayer             ; 
                          sta TabActivePlayer             ; 
                          jsr GameHiScoreGetPlayerName    ; 
                          
                          lda TabGameRtsOnlyFlag          ; 
                          beq .GoMainInit                 ; 
                          
                          jsr GameRtsOnly                 ; 
.GoMainInit               jmp MainInit                    ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetNumPlayers_03         lda TabNumPlayers               ; 
.TstNumPlayers_03         cmp #TabNumPlayers_One          ; 
.ChkNumPlayers_03_One     bne .GetActivePlayer            ; 
                          
                          jsr InitGameVarsPartial         ; 
                          rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetActivePlayer          lda TabActivePlayer             ; 
.TstActivePlayer          cmp #TabActivePlayer_P1         ; 
.ChkActivePlayer_One      beq .GetNumLives_P2             ; 
                          
.GetNumLives_P1           lda TabNumLives_P1              ; 
                          jmp .TstNumLives                ; 
                          
.GetNumLives_P2           lda TabNumLives_P2              ; 
                          
.TstNumLives              cmp #$00                        ; 
.ChkNumLives_Min          bne .IniSwapPlayerProgress      ; 
                          
.WasNumLives_Min          jsr InitGameVarsPartial         ; 
                          rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IniSwapPlayerProgress    sec                             ; 
                          lda #[TabActivePlayer_P2 + $01] ; 
                          sbc TabActivePlayer             ; 
                          sta TabActivePlayer             ; 
                          
                          jsr SwapPlayerLevelProgress     ; 
                          jsr InitGameVarsPartial         ; 
GameHiScoreHandlerX       rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameHiScoreGetPlayerName Does   : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameHiScoreGetPlayerName  subroutine                      ; 
                          jsr GameHiScoreGetPlayerPtr     ; 
; -------------------------------------------------------------------------------------------------------------- ;
; check if score is still zero
; -------------------------------------------------------------------------------------------------------------- ;
.GetChkScoreEntryLen      ldy #TabHiScoreEntry_Score_Len  ; 
.IniChkScore              lda #$00                        ; 
.InsNextChkPlayerScore    ora (SNZ_PlayerScorePtr),y      ; 
.DecChkScoreEntryLen      dey                             ; 
.ChkChkScoreEntryLen_Min  bpl .InsNextChkPlayerScore      ; 
                          
.TstPlayerScore_00        cmp #$30                        ; 
.ChkPlayerScore_00        beq .ExitNoScore                ; score still zero
; -------------------------------------------------------------------------------------------------------------- ;
.GetScoreEntryCount_Max   lda #SN_HiScoreEntry_Max        ; 
.SetScoreEntryCount       sta SNZ_HiScoreEntryCount       ; 
                          
                          lda #$00                        ; 
                          
.SetNextScoreEntryOff     sta SNZ_HiScoreEntryOff         ; 
                          
.GetEntryOff              tax                             ;
.GetScoreOff              ldy #$00                        ; 
.GetNextEntryHiScoreByte  lda TabHiScoreEntry_Score,x     ; 
.TstHiScoreByte           cmp #" "                        ; 
.ChkHiScoreByte_Empty     bne .TstScorePlayer             ; 
                          
.IniScorePlayer_Zero      lda #"0"                        ; 
                          
.TstScorePlayer           cmp (SNZ_PlayerScorePtr),y      ; compare with actual score
.TstScorePlayer_Low       bcc .MovHiScoreEntriesOneDown   ; lower
.TstScorePlayer_Equal     bne .GetScoreEntryOff           ; higher
                          
.IncEntryOff              inx                             ; equal
.IncScoreOff              iny                             ; 
.TstScoreOff              cpy #[TabHiScoreEntry_Score_Len + $01] ; 
.ChkScoreOff_Max          bne .GetNextEntryHiScoreByte    ; 
                          
.GetScoreEntryOff         lda SNZ_HiScoreEntryOff         ; advance to next high score enty
                          clc                             ; 
.SetScoreEntryOff_Next    adc #[TabHiScoreEntry_Len + $01]; 
.DecScoreEntryCount       dec SNZ_HiScoreEntryCount       ; 
.ChkScoreEntryCount_Min   bne .SetNextScoreEntryOff       ; 
                          
.ExitNoScore              rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; copy high score entries one pos down 
; -------------------------------------------------------------------------------------------------------------- ;
.MovHiScoreEntriesOneDown lda #[[TabHiScoreEntry_Len + $01] * $08]; 
.IniEntryOff              sta SNZ_HiScoreEntryOffCopy     ; 
.SetNextEntryCount        dec SNZ_HiScoreEntryCount       ; 
.ChkNextEntryCount_Min    beq .ClrHiScoreEntry            ; 
                          
.GetCopyEntryOff          ldy SNZ_HiScoreEntryOffCopy     ; 
.GetCopyScoreOff          ldx #[TabHiScoreEntry_Len + $01]; 
.GetNextCopyScoreByte     lda [TabHiScoreEntries + [[TabHiScoreEntry_Len + $01] * $00]],y ; 
                          sta [TabHiScoreEntries + [[TabHiScoreEntry_Len + $01] * $01]],y ; 
.IncCopyEntryOff          iny                             ; 
.DecCopyScoreOff          dex                             ; 
.ChkCopyScoreOff_Min      bne .GetNextCopyScoreByte       ; 
                          
                          sec                             ; 
                          lda SNZ_HiScoreEntryOffCopy     ; 
.SubEntryLen              sbc #[TabHiScoreEntry_Len + $01]; 
.SetEntryOff_Next         sta SNZ_HiScoreEntryOffCopy     ; 
                          
                          jmp .SetNextEntryCount          ; 
; -------------------------------------------------------------------------------------------------------------- ;
; clear selected high score entry
; -------------------------------------------------------------------------------------------------------------- ;
.ClrHiScoreEntry          ldx #[TabHiScoreEntry_Len + $01]; 
.GetHiScoreEntry_Clear    lda #" "                        ; 
.GetHiScoreEntryOff       ldy SNZ_HiScoreEntryOff         ; 
.ClrNextHiScoreEntryByte  sta TabHiScoreEntries,y         ; 
.IncHiScoreEntryOff       iny                             ; 
.DecHiScoreEntryLen       dex                             ; 
.ChkHiScoreEntryLen_Min   bne .ClrNextHiScoreEntryByte    ; 
; -------------------------------------------------------------------------------------------------------------- ;
; Find first non zero byte in new high score
; -------------------------------------------------------------------------------------------------------------- ;
.GetNewScoreEntryOff      ldx SNZ_HiScoreEntryOff         ; 
.GetNewScoreEntryLen      ldy #$00                        ; 
.GetNextNewScoreByteChk   lda (SNZ_PlayerScorePtr),y      ; 
.IncNewScoreEntryLen      iny                             ; 
.TstNewScoreByte          cmp #"0"                        ; 
.ChkNewScoreByte_Zero     bne .SetNewScoreByte            ; 
                          
.IncNewScoreEntryOff      inx                             ; 
.TstNewScoreEntryLen      cpy #[TabHiScoreEntry_Score_Len + $01] ; 
.ChkNewScoreEntryLen_Min  bne .GetNextNewScoreByteChk     ; 
                          
                          sta [TabHiScoreEntry_Score - $01],x ; 
                          beq .GoGetScorerName            ; always
; -------------------------------------------------------------------------------------------------------------- ;
; insert new high score entry
; -------------------------------------------------------------------------------------------------------------- ;
.GetNextNewScoreByteCopy  lda (SNZ_PlayerScorePtr),y      ; 
.IncNewScoreLenCopy       iny                             ; 
.SetNewScoreByte          sta TabHiScoreEntry_Score,x     ; 
.IncNewScoreOffCopy       inx                             ; 
.TstNewScoreLenCopy       cpy #[TabHiScoreEntry_Score_Len + $01] ; 
.TstNewScoreLenCpy_Min    bne .GetNextNewScoreByteCopy    ; 
; -------------------------------------------------------------------------------------------------------------- ;
; get new scorer name
; -------------------------------------------------------------------------------------------------------------- ;
.GoGetScorerName          jsr GameHiScoreInputName        ; 
; -------------------------------------------------------------------------------------------------------------- ;
; write new high scrore file
; -------------------------------------------------------------------------------------------------------------- ;
                          jsr GameHiScoreFileScratch      ; 
                          jsr GameHiScoreCompressOn       ; 
                          jsr GameHiScoreFileSave         ; 
                          
GameHiScoreGetPlayerNameX jmp GameHiScoreCompressOff      ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameHiScoreGetPlayerPtr Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameHiScoreGetPlayerPtr   subroutine                      ; 
.GetActivePlayer          lda TabActivePlayer             ; 
.TstActivePlayer          cmp #TabActivePlayer_P1         ; 
.ChkActivePlayer_One      bne .SetScorePoiner_P2          ; 
                          
.SetScorePoiner_P1        lda #<TabScorePlayer_01         ; 
                          sta SNZ_PlayerScorePtr_Lo       ; 
                          lda #>TabScorePlayer_01         ; 
                          sta SNZ_PlayerScorePtr_Hi       ; 
                          rts                             ; 
                          
.SetScorePoiner_P2        lda #<TabScorePlayer_02         ; 
                          sta SNZ_PlayerScorePtr_Lo       ; 
                          lda #>TabScorePlayer_02         ; 
                          sta SNZ_PlayerScorePtr_Hi       ; 
GameHiScoreGetPlayerPtrX  rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameHiScoreInputName    Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameHiScoreInputName      subroutine                      ; 
                          jsr InitLockOscAll              ; Lock oscillators 1-3
                          
                          sei                             ; 
                          
                          lda #TabGameRtsOnlyFlag_Yes     ; 
                          sta TabGameRtsOnlyFlag          ; 
                          
.GetScoreEntryOff         lda SNZ_HiScoreEntryOff         ; 
.SavScoreEntryOff         pha                             ; 
                          
                          jsr InitColorRamToBlack         ; 
                          
                          lda #[VIC_Video_0800 | VIC_CharS_2800] ;
                          sta VMCSB                       ; VIC($D018) VIC Chip Memory Control
                          
                          lda #$80                        ; TabCharSet_GfxInfo_80 - blank
.IniScrnTopTen            jsr GameHiScoreClearScreen      ; 
                          jsr GameHiScoreTableDisplay     ; 
                          jsr GameHiScoreSpritesInit      ; 
                          
.IniSprtSmallOrbsPosXY    lda #$33                        ; 
                          sta SP6Y                        ; VIC($D00D) Sprite 6 Y-Pos (Bits 0-7)
                          lda #$19                        ; 
                          sta SP6X                        ; VIC($D00C) Sprite 6 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          lda #$44                        ; 
                          sta SP7X                        ; VIC($D00E) Sprite 7 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          lda #$f1                        ; 
                          sta SP7Y                        ; VIC($D00F) Sprite 7 Y-Pos (Bits 0-7)
                          
                          lda MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
                          ora #$80                        ; #.......
                          sta MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
                          
.IniSprtSmallOrbsColor    lda #YELLOW                     ; 
                          sta SP6COL                      ; VIC($D02D) Color Sprite 6
                          sta SP7COL                      ; VIC($D02E) Color Sprite 7
                          
.IniSprtSmallOrbsMoveDir  lda #TabHiScoreOrbMovDir_Ri     ; 
                          sta TabHiScoreOrbMovDirOne      ; 
                          
                          lda #TabHiScoreOrbMovDir_Le     ; 
                          sta TabHiScoreOrbMovDirTwo      ; 
                          
.IniSprtSmallOrbsEnab     lda SPENA                       ; VIC($D015) Sprite Enable Register
                          ora #$c0                        ; ##......
                          sta SPENA                       ; VIC($D015) Sprite Enable Register
                          
.GetNewCharSet            jsr InitSwapCharSetsHandler     ; 
                          
.SetActivePlayerNumOut_Le lda #<[SN_Scrn_HiScore_Row_02 + $03] ; 
                          sta SNZ_HiScoreCharOutPtr_Lo    ; 
                          lda #>SN_MemScreenHiScore       ; 
                          sta SNZ_HiScoreCharOutPtr_Hi    ; 
                          
                          clc                             ; 
.GetActivePlayer_Le       lda TabActivePlayer             ; 
.SetActivePlayer_Le_Digit adc #"0"                        ; 
                          jsr GameHiScoreCharOut          ; 
                          
.SetActivePlayerNumOut_Ri lda #<[SN_Scrn_HiScore_Row_02 + $23] ; 
                          sta SNZ_HiScoreCharOutPtr_Lo    ; 
                          
                          clc                             ; 
.GetActivePlayer_Ri       lda TabActivePlayer             ; 
.SetActivePlayer_Ri_Digit adc #"0"                        ; 
                          jsr GameHiScoreCharOut          ; 
                          
                          jsr InitColorRamToWhite         ; 
                          
.RstScoreEntryOff         pla                             ; 
.SetScoreEntryOff         sta SNZ_HiScoreEntryOff         ; 
                          
                          lda #$00                        ; 
                          sta TabWaitCount_Lo             ; 
                          lda #$40                        ; 
                          sta TabWaitCount_Hi             ; 
                          
                          lda #CIA_Key_None               ; 
.IniKeyGot                sta TabKeyboardKeyGot           ; 
                          
.GetHiScoreEntryOff       ldy SNZ_HiScoreEntryOff         ; 
                          
.GetNextHiScoreChar_Crsr  lda #"*"                        ; TabCharSet_TextNormal_2a - cursor char "*"
                          jsr GameHiScoreOutCursor        ; 
.MovNextSprtSmallOrbs     jsr GameHiScoreOrbMoveOne       ; 
                          jsr GameHiScoreOrbMoveTwo       ; 
                          
                          tya                             ; 
.SavHiScoreEntryOff       pha                             ; 
.GoGetInputKey            jsr KeyboardGetKey              ; 
                          pla                             ; 
.RstHiScoreEntryOff       tay                             ; 
                          
.GetInputKey              lda TabKeyboardKeyGot           ; 
.TstInputKey              cmp #CIA_Key_None               ; 
.ChkInputKey_None         bne .IniChkKey_Enter            ; 
; -------------------------------------------------------------------------------------------------------------- ;
; no key pressed so far - wait a while
; -------------------------------------------------------------------------------------------------------------- ;
.IniWaitSomeTime_Hi       lda #$02                        ; 
.SavNextWaitSomeTime_Hi   pha                             ; 
.IniWaitSomeTime_Lo       ldx #$00                        ; 
.DecNextWaitSomeTime      dex                             ; 
.ChkWaitSomeTime_Min      bne .DecNextWaitSomeTime        ; 
                          pla                             ; 
                          sec                             ; 
.DecWaitSomeTime_Hi       sbc #$01                        ; 
.ChkWaitSomeTime_Hi_Min   bne .SavNextWaitSomeTime_Hi     ; 
; -------------------------------------------------------------------------------------------------------------- ;
; check wait for input time
; -------------------------------------------------------------------------------------------------------------- ;
.GetWaitTimeInput_Lo      lda TabWaitCount_Lo             ; 
                          bne .DecWaitTimeInput_Lo        ; 
                          
.DecWaitTimeInput_Hi      dec TabWaitCount_Hi             ; 
.ChkWaitTimeInput_Hi_Min  bmi .IniLeaveGetScorerName      ; wait time expired - leave
                          
.DecWaitTimeInput_Lo      dec TabWaitCount_Lo             ; 
.ChkWaitTimeInput_Lo_Min  beq .IniPlayerNumBlink_On       ; 
; -------------------------------------------------------------------------------------------------------------- ;
; check player number blink time
; -------------------------------------------------------------------------------------------------------------- ;
.GetWaitTimePlayer        lda TabWaitCount_Lo             ; 
.TstWaitTimePlayer        cmp #$80                        ; blink cursor/player num independently
                          bne .GetCrsrBlinkTime           ; 
                          
.GetPlayerNumBlink_Off    lda #" "                        ; 
.GoPlayerNumBlink         jsr GameHiScoreClrPlayerNum     ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetCrsrBlinkTime         lda TabWaitCount_Lo             ; 
                          and #$7f                        ; .#######
.TstCrsrBlinkTime         cmp #$0a                        ; 
.ChkCrsrBlinkTime_Off     beq .GetCrsrBlink_Off           ; 
                          
.ChkCrsrBlinkTime_On      cmp #$4a                        ; 
                          bne .MovNextSprtSmallOrbs       ; 
                          
.GetCrsrBlink_On          lda #"*"                        ; 
                          bne .GoCrsrBlinkCharOut         ; always
                          
.GetCrsrBlink_Off         lda #" "                        ; 
.GoCrsrBlinkCharOut       jsr GameHiScoreOutCursor        ; 
.GoFlickerSprtTopTen      jsr GameHiScoreFlashSprite      ; 
.GoFlickerTheLeader       jsr GameHiScoreFlashLeader      ; 
                          
                          jmp .MovNextSprtSmallOrbs       ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IniPlayerNumBlink_On     clc                             ; 
.GetPlayerNumBlink_On     lda TabActivePlayer             ; 
                          adc #"0"                        ; 
                          bne .GoPlayerNumBlink           ; always
; -------------------------------------------------------------------------------------------------------------- ;
.IniChkKey_Enter          and #TabKeyGotDecode_Len        ; ..######
.TstChkKey_Enter          cmp #$3e                        ; 
.ChkChkKey_Enter          bne .TstChkKey_Delete           ; 
; -------------------------------------------------------------------------------------------------------------- ;
; wait for input time expired or <enter> key pressed - leave high score input
; -------------------------------------------------------------------------------------------------------------- ;
.IniLeaveGetScorerName    lda #" "                        ; clear cursor char
                          jsr GameHiScoreOutCursor        ; 
                          jsr InitColorRamToBlack         ; 
                          
                          lda #$80                        ; TabCharSet_GfxInfo_80 - blank
                          jsr GameHiScoreClearScreen      ; 
.Exit_WaitTimeExpired     rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.TstChkKey_Delete         cmp #$3f                        ; 
.ChkChkKey_Delete         bne .SetKeyGotDecode_Off        ; 
                          
.TstPosLeft_Max           cpy SNZ_HiScoreEntryOff         ; 
.ChkPosLeft_Max           beq .GetInputScoreChar_None     ; no deletes if on 1st input positon
                          
.GetInputClear            lda #" "                        ; 
.GoOutInputClear          jsr GameHiScoreOutCursor        ; 
                          
.DecHiScoreEntryOff       dey                             ; 
                          sta TabHiScoreEntries,y         ; 
                          jmp .IniGoHiScoreChar_Crsr      ; 
; -------------------------------------------------------------------------------------------------------------- ;
.SetKeyGotDecode_Off      tax                             ; key pressed is offset to decode table
                          
                          sec                             ; 
.GetInputScoreEntryOff    tya                             ; 
                          sbc SNZ_HiScoreEntryOff         ; 
                          cmp #$08                        ; 
                          beq .GetInputScoreChar_None     ; 
                          
.GetInputScoreNam_Char    lda TabKeyGotDecode,x           ; 
.ChkInputScoreNam_Char_@  beq .SetInputScoreNam_Char      ; 
                          
.TstInputScoreNam         cmp #$1c                        ; 
.ChkInputScoreNam_Char_No bcs .SetInputScoreNam_Char      ; greater/equal - numbers and special chars
                          
.TstInputScoreChar        bit TabKeyboardKeyGot           ; 
.TstInputScoreChar_Shift  bvc .SetInputScoreNam_Char      ; test if shift/C= are pressed
                          
.SetKeyGotDecode_Off_Bad  tax                             ; replace char wit blank then
.GetKeyGotDecode_Char_Bad lda #" "                        ; 
                          
.SetInputScoreNam_Char    sta TabHiScoreEntries,y         ; 
                          jsr GameHiScoreOutCursor        ; 
                          
.IncInputScoreEntryOff    iny                             ; advance to next input pos
                          
.IniGoHiScoreChar_Crsr    nop                             ; obsolete
.GoOutInputScoreChar      jmp .GetNextHiScoreChar_Crsr    ; 
                          
.GetInputScoreChar_None   lda #$fa                        ; 
GameHiScoreInputNameX     bne .GoOutInputScoreChar        ; always
; -------------------------------------------------------------------------------------------------------------- ;
; GameHiScoreClrPlayerNum Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameHiScoreClrPlayerNum   subroutine                      ; 
.Sav_Yr                   sty ._Sav_Yr                    ; 
.SavCharOut               sta ._GetCharOut                ; 
                          
.SetPlayerNumPtr_Le       ldy #<[SN_Scrn_HiScore_Row_02 + $03] ; 
                          sty SNZ_HiScoreCharOutPtr_Lo    ; 
                          ldy #>[SN_Scrn_HiScore_Row_02 + $03] ; 
                          sty SNZ_HiScoreCharOutPtr_Hi    ; 
                          
.ClrPlayerNum_Le          jsr GameHiScoreCharOut          ; 
                          
.SetPlayerNumPtr_Ri       ldy #<[SN_Scrn_HiScore_Row_02 + $23] ; 
                          sty SNZ_HiScoreCharOutPtr_Lo    ; 
                          
._GetCharOut              equ [* + $01]                   ; 
.RstCharOut               lda #$00                        ; 
.ClrPlayerNum_Ri          jsr GameHiScoreCharOut          ; 
                          
._Sav_Yr                  equ [* + $01]                   ; 
.Rst_Yr                   ldy #$00                        ; 
GameHiScoreClrPlayerNumX  rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameHiScoreOutCursor    Does    : 
;                         Expects : .A=Char num
;                                 : .Y=HiScoreEntryOff
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameHiScoreOutCursor      subroutine                      ; 
.SavCharOut               sta ._GetCharOut                ; 
.Sav_Yr                   sty ._Sav_Yr                    ; 
                          
.SetPlayerNamPtr_Top      lda #<[SN_Scrn_HiScore_Row_05 + $04] ; start with top high score list screen row
                          sta SNZ_HiScoreCharOutPtr_Lo    ; 
                          lda #>[SN_Scrn_HiScore_Row_05 + $04] ; 
                          sta SNZ_HiScoreCharOutPtr_Hi    ; 
                          
.GetScoreEntryOff         tya                             ; get the score entry offset
; -------------------------------------------------------------------------------------------------------------- ;
.TstNextScoreEntryLen     cmp #[TabHiScoreEntry_Len + $01]; check if already in the input row
.ChkScoreEntryLen_Min     bcc .GetEntryPtr                ; lower - iput row addressed
                          
.Sav_ScoreEntryOff        tay                             ; higher/equal - iput row not yet addressed
                          
.SetPlayerNamPtr_Next     clc                             ; locate the actual high score list screen row
                          lda SNZ_HiScoreCharOutPtr_Lo    ; 
                          adc #[SN_Scrn_RowLen * $02]     ; 
                          sta SNZ_HiScoreCharOutPtr_Lo    ; 
                          bcc .Rst_ScoreEntryOff          ; 
                          inc SNZ_HiScoreCharOutPtr_Hi    ; 
                          
.Rst_ScoreEntryOff        tya                             ; subtract next score entry length
                          sec                             ; 
                          sbc #[TabHiScoreEntry_Len + $01]; 
                          jmp .TstNextScoreEntryLen       ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetEntryPtr              asl a                           ; *2
                          clc                             ; 
                          adc SNZ_HiScoreCharOutPtr_Lo    ; 
.SetEntryPtr              sta SNZ_HiScoreCharOutPtr_Lo    ; locate input column
                          bcc .RstCharOut                 ; 
                          inc SNZ_HiScoreCharOutPtr_Hi    ; 
                          
._GetCharOut              equ [* + $01]                   ; 
.RstCharOut               lda #$00                        ; 
.GoOutPlayerNamChar       jsr GameHiScoreCharOut          ; 
                          
                          lda ._GetCharOut                ; 
._Sav_Yr                  equ [* + $01]                   ; 
.Rst_Yr                   ldy #$00                        ; 
                          
GameHiScoreOutCursorX     rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameRtsOnly             Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameRtsOnly               subroutine                      ; 
GameRtsOnlyX              rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameHiScoreClearScreen  Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameHiScoreClearScreen    subroutine                      ; 
                          ldy #[SN_Scrn_RowLen * $05]  ; 
.FillNextHiScoreScrnPos   sta [SN_Scrn_HiScore_Row_01 - $01],y ; 
                          sta [SN_Scrn_HiScore_Row_06 - $01],y ; 
                          sta [SN_Scrn_HiScore_Row_0b - $01],y ; 
                          sta [SN_Scrn_HiScore_Row_10 - $01],y ; 
                          sta [SN_Scrn_HiScore_Row_15 - $01],y ; 
                          dey                             ; 
                          bne .FillNextHiScoreScrnPos     ; 
                          
GameHiScoreClearScreenX   rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; ScoreShowCurrent        Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
ScoreShowCurrent          subroutine                      ; 
.IniScoreLen              ldx #TabScoreDisplay_Len        ; 
.GetNextScoreByte         lda TabScoreDisplay,x           ; 
                          jsr GameScoreShow               ; 
.DecScoreLen              dex                             ; 
.ChkScoreLen_Min          bpl .GetNextScoreByte           ; 
                          
ScoreShowCurrentX         rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameScoreShow           Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameScoreShow             subroutine                      ; 
.IniSprtDataRowCount      ldy #$00                        ; 
                          sty ._TabCharSetOff             ; 
                          
                          asl a                           ; 
                          rol ._TabCharSetOff             ; *2
                          asl a                           ; 
                          rol ._TabCharSetOff             ; *4
                          asl a                           ; 
                          rol ._TabCharSetOff             ; *8
                          
.GetCharSetAdr_Lo         sta ._TabCharSetAdr_Lo          ; 
                          
                          clc                             ; 
.GetCharSetAdr_Hi         lda #>TabCharSet_Text           ; 
._TabCharSetOff           equ [* + $01]                   ; 
                          adc #$00                        ; 
.SetCharSetAdr_Hi         sta ._TabCharSetAdr_Hi          ; 
                          
                          lda TabSpriteGfxScoreOff,x      ; 
                          
.Sav_Xr                   stx ._Sav_Xr                    ; 
                          
                          tax                             ; 
._TabCharSetAdr_Lo        equ [* + $01]                   ; 
._TabCharSetAdr_Hi        equ [* + $02]                   ; 
.GetNextDataScore         lda SN_GetDataDummy,y           ; 
.SetNextDataScore         sta SpriteData_Score,x          ; 
                          
.GetNextDataScoreOff      txa                             ; 
                          clc                             ; 
.AdNextDataScoreRowLen    adc #$03                        ; 
.SetNextDataScoreOff      tax                             ; 
                          
.IncSprtDataRowCount      iny                             ; 
.TstSprtDataRowCount_Max  cpy #[SpriteData_Score_Rows + $01] ; 
.ChkSprtDataRowCount_Max  bcc .GetNextDataScore           ; lower
                          
._Sav_Xr                  equ [* + $01]                   ; 
.Rst_Xr                   ldx #$00                        ; 
                          
GameScoreShowX            rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabSpriteGfxScoreOff      equ  *   ; 
                          dc.b [SpriteData_ScoreLe_Row_1 + $00] ; 
                          dc.b [SpriteData_ScoreLe_Row_1 + $01] ; 
                          dc.b [SpriteData_ScoreLe_Row_1 + $02] ; 
                          
                          dc.b [SpriteData_ScoreMid_Row_1 + $00] ; 
                          dc.b [SpriteData_ScoreMid_Row_1 + $01] ; 
                          dc.b [SpriteData_ScoreMid_Row_1 + $02] ; 
                          
                          dc.b [SpriteData_ScoreRi_Row_1 + $00] ; 
                          dc.b [SpriteData_ScoreRi_Row_1 + $01] ; 
                          dc.b [SpriteData_ScoreRi_Row_1 + $02] ; 
                          
                          dc.b [SpriteData_ScoreLe_Row_2 + $00] ; 
                          dc.b [SpriteData_ScoreLe_Row_2 + $01] ; 
                          dc.b [SpriteData_ScoreLe_Row_2 + $02] ; 
                          
                          dc.b [SpriteData_ScoreMid_Row_2 + $00] ; 
                          dc.b [SpriteData_ScoreMid_Row_2 + $01] ; 
                          dc.b [SpriteData_ScoreMid_Row_2 + $02] ; 
                          
                          dc.b [SpriteData_ScoreRi_Row_2 + $00] ; 
                          dc.b [SpriteData_ScoreRi_Row_2 + $01] ; 
                          dc.b [SpriteData_ScoreRi_Row_2 + $02] ; 
TabSpriteGfxScoreOff_Len  equ  [* - TabSpriteGfxScoreOff - $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabScoreDisplay           equ  *   ; 
                          dc.b $20 ; 
                          
TabScoreDisplay_Plr_01    dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 

TabScoreDisplay_Sep_01    dc.b $20 ; ":" - seperator
TabScoreDisplay_Num_01    dc.b $20 ; player number

TabScoreDisplay_02        dc.b $20 ; 
TabScoreDisplay_Plr_02    dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          
TabScoreDisplay_Sep_02    dc.b $20 ; ":" - seperator
TabScoreDisplay_Num_02    dc.b $20 ; player number
TabScoreDisplay_Len       equ  [* - TabScoreDisplay - $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameHiScoreShow         Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameHiScoreShow           subroutine                      ; 
.IniScoreDisplayLen       ldx #TabScoreDisplay_Len        ; 
.GetScoreDisplayIni       lda #" "                        ; 
.SetNextScoreDisplayByte  sta TabScoreDisplay,x           ; 
.DecScoreDisplayLen       dex                             ; 
.ChkScoreDisplayLen_Min   bpl .SetNextScoreDisplayByte    ; 
                          
.IncScoreBlink            ldx TabHiScoreBlink             ; 
                          inx                             ; 
                          txa                             ; 
                          and #$1f                        ; ...##### - 31
                          sta TabHiScoreBlink             ; 
                          
                          jsr GameTuneXtraLife            ; 
                          
.GetHiScoreByte_P1        lda [TabScorePlayer_01 + $01]   ; 
.TstChkHiScoreXtra_P1     cmp TabScoreChkXtraLive_P1      ; 
.ChkChkHiScoreXtra_P1     beq .GetHiScoreByte_P2          ; already got one xtra live
                          
                          sta TabScoreChkXtraLive_P1      ; 
                          
.IncNumLives_P1           inc TabNumLives_P1              ; only one time
                          jsr GameTuneXtraLifeInit        ; 
                          
.GetHiScoreByte_P2        lda [TabScorePlayer_02 + $01]   ; 
.TstChkHiScoreXtra_P2     cmp TabScoreChkXtraLive_P2      ; 
.ChkChkHiScoreXtra_P2     beq .GetActivePlayerScore       ; already got one xtra live
                          
                          sta TabScoreChkXtraLive_P2      ; 
                          
.IncNumLives_P2           inc TabNumLives_P2              ; only one time
                          jsr GameTuneXtraLifeInit        ; 
                          
.GetActivePlayerScore     lda TabActivePlayer             ; 
.TstActivePlayerScore     cmp #TabActivePlayer_P1         ; 
.ChkActivePlayerScore_One bne .IniChkScore_P1             ; 
                          
.GetScoreBlink_P1         lda TabHiScoreBlink             ; 
.TstScoreVisible_P1       cmp #TabHiScoreBlink_On         ; 
.ChkScoreVisible_P1       bcs .GetNumPlayers              ; greater/equal - invisible
                          
.IniChkScore_P1           ldx #$00                        ; 
                          ldy #$00                        ; 
.GetNextChkScore_P1       lda TabScorePlayer_01,x         ; 
                          inx                             ; 
                          cmp #"0"                        ; 
                          bne .SetScorePlayer_01          ; 
                          
                          iny                             ; 
                          
                          cpx #[TabScorePlayer_Len + $01] ; 
                          bne .GetNextChkScore_P1         ; 
                          
                          sta TabScoreDisplay,y           ; 
                          beq .GetNumLives_P1             ; 
                          
.GetNextScoreByte_P1      lda TabScorePlayer_01,x         ; 
                          inx                             ; 
.SetScorePlayer_01        sta TabScoreDisplay_Plr_01,y    ; 
                          
                          iny                             ; 
                          
                          cpx #[TabScorePlayer_Len + $01] ; 
                          bne .GetNextScoreByte_P1        ; 
                          
.GetNumLives_P1           lda TabNumLives_P1              ; 
.TstNumLives_Game_Max_P1  cmp #$0a                        ; 
.ChkNumLives_Game_Max_P1  bcc .CreNumPlayer_01            ; 
                          
.GetNumLives_Game_Max_P1  lda #$09                        ; 
                          
.CreNumPlayer_01          ora #"0"                        ; make digit
.SetNumPlayer_01          sta TabScoreDisplay_Num_01      ; 
                          
                          lda #":"                        ; 
                          sta TabScoreDisplay_Sep_01      ; 
                          
.GetNumPlayers            lda TabNumPlayers               ; 
.TstNumPlayers            cmp #TabNumPlayers_One          ; 
.ChkNumPlayers_One        beq GameHiScoreShowX            ; 
                          
.GetActivePlayer          lda TabActivePlayer             ; 
.TstActivePlayer          cmp #TabActivePlayer_P2         ; 
.ChkActivePlayer_Two      bne .IniChkScore_P2             ; 
                          
.GetScoreBlink_P2         lda TabHiScoreBlink             ; 
.TstScoreVisible_P2       cmp #TabHiScoreBlink_On         ; 
.ChkScoreVisible_P2       bcs GameHiScoreShowX            ; greater/equal - invisible
                          
.IniChkScore_P2           ldx #$00                        ; 
                          ldy #$00                        ; 
.GetNextChkScore_P2       lda TabScorePlayer_02,x         ; 
                          inx                             ; 
                          cmp #"0"                        ; 
                          bne .SetScorePlayer_P2          ; 
                          
                          iny                             ; 
                          cpx #[TabScorePlayer_Len + $01] ; 
                          bne .GetNextChkScore_P2         ; 
                          
                          sta TabScoreDisplay_02,y        ; 
                          beq .GetNumLives_P2             ; 
                          
.GetNextScoreByte_P2      lda TabScorePlayer_02,x         ; 
                          inx                             ; 
.SetScorePlayer_P2        sta TabScoreDisplay_Plr_02,y    ; 
                          
                          iny                             ; 
                          
                          cpx #[TabScorePlayer_Len + $01] ; 
                          bne .GetNextScoreByte_P2        ; 
                          
.GetNumLives_P2           lda TabNumLives_P2              ; 
.TstNumLives_Game_Max_P2  cmp #$0a                        ; 
.ChkNumLives_Game_Max_P2  bcc .CreNumPlayer_P2            ; 
                          
                          lda #$09                        ; 
                          
.CreNumPlayer_P2          ora #"0"                        ; make digit
.SetNumPlayer_P2          sta TabScoreDisplay_Num_02      ; 
                          
                          lda #":"                        ; 
                          sta TabScoreDisplay_Sep_02      ; 
                          
GameHiScoreShowX          jmp ScoreShowCurrent            ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameTuneXtraLifeInit    Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameTuneXtraLifeInit      subroutine                      ; 
                          lda #$01                        ; 
                          sta TabTuneXtraLifeTime_Lo      ; 
                          
                          lda #$09                        ; 
GameTuneXtraLifeInitX     sta TabTuneXtraLifeTime_Hi      ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameTuneXtraLife        Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameTuneXtraLife          subroutine                      ; 
                          lda TabTuneXtraLifeTime_Hi      ; 
                          bne .DecTuneTime_Lo             ; 
                          
.Exit_NoTune              rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.DecTuneTime_Lo           dec TabTuneXtraLifeTime_Lo      ; 
                          bne .Exit_NoTune                ; 
                          
.GetTuneTime_Lo_Max       lda #$08                        ; 
.SetTuneTime_Lo           sta TabTuneXtraLifeTime_Lo      ; 
                          
.DecTuneTime_Hi           dec TabTuneXtraLifeTime_Hi      ; 
.ChkTuneTime_Hi_Min       bne .GetWavTriangle             ; 
                          
                          lda #VcRegTEST                  ; ....#... - 1=Lock Oscillator
                          sta VCREG2                      ; SID($D40B) Oscillator 2 Control
                          rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetWavTriangle           lda #VcRegTRI                   ; ...#....
                          sta VCREG2                      ; SID($D40B) Oscillator 2 Control
                          lda #[VcRegTRI | VcRegGATE]     ; ...#...#
                          sta VCREG2                      ; SID($D40B) Oscillator 2 Control
                          
                          lda TabTuneXtraLifeTime_Hi      ; 
                          and #$01                        ; .......#
                          beq .GetFreHi_50                ; 
                          
.GetFreHi_32              lda #$32                        ; 
                          bne .SetFreHi                   ; always
                          
.GetFreHi_50              lda #$50                        ; 
                          
.SetFreHi                 sta FREHI2                      ; SID($D408) Oscillator 2 Freq Ctrl (high byte)
                          
                          lda #$00                        ; 
                          sta FRELO2                      ; SID($D407) Oscillator 2 Freq Ctrl (low byte)
                          
GameTuneXtraLifeX         rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameIncScores           Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameIncScores             subroutine                      ; 
                          lda TabSelectNumLives           ; 
                          cmp #SN_SnokieNumLivesCheat     ; 
                          beq GameIncScoresX              ; 
                          
.GetActivePlayer          lda TabActivePlayer             ; 
.TstActivePlayer          cmp #TabActivePlayer_P1         ; 
.ChkActivePlayer_One      bne .GetScorePtrPlayer_02       ; 
                          
.GetScorePtrPlayer_01     lda #<TabScorePlayer_01         ; 
                          ldy #>TabScorePlayer_01         ; 
                          jmp .IniScorePtr                ; 
                          
.GetScorePtrPlayer_02     lda #<TabScorePlayer_02         ; 
                          ldy #>TabScorePlayer_02         ; 
                          
.IniScorePtr              sta SNZ_ScorePlayerPtr_Lo       ; 
                          sty SNZ_ScorePlayerPtr_Hi       ; 
                          
.IniScoreAdd_02           lda #$02                        ; 
.IniScoreLen              ldy #[TabScorePlayer_Len - $01] ; last byte remains 0
.AddNextScoreVal          clc                             ; 
                          adc (SNZ_ScorePlayerPtr),y      ; 
.TstScoreByte_Max         cmp #$3a                        ; 
.ChkScoreByte_Max         bcs .IniSub_10                  ; greater/equal
                          
.SetScoreByte             sta (SNZ_ScorePlayerPtr),y      ; lower
                          rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IniSub_10                sec                             ; 
                          sbc #$0a                        ; 
.SetScoreByte_00          sta (SNZ_ScorePlayerPtr),y      ; 
                          
.IniScoreAddNext          lda #$01                        ; 
                          
.DecScoreLen              dey                             ; 
.ChkScoreLen_Min          bpl .AddNextScoreVal            ; 
                          
GameIncScoresX            rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitGameVarsFull        Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitGameVarsFull          subroutine                      ; 
.IniTabVarCount           ldx #TabGameVarsInits_Len       ; init full game variable table
.GetNextVarVal            lda TabGameVarsInits,x          ; 
                          sta TabGameVars,x               ; 
.DecTabVarCount           dex                             ; 
.ChkTabVarCount_Min       bpl .GetNextVarVal              ; 
                          
                          lda TabGameSpeed                ; 
                          sta TabGameSpeed_P1             ; 
                          sta TabGameSpeed_P2             ; 
                          
.GetNumLives              lda TabSelectNumLives           ; 
.SetNumLives_P1           sta TabNumLives_P1              ; 
.SetNumLives_P2           sta TabNumLives_P2              ; 
                          
                          ldx TabSelectNumLevel           ; 
                          clc                             ; 
.AddLvlDatSnokiePosx_Strt lda TabIniCheckDataPtrs_Lo,x    ; 
                          adc #<TabLvlDatSnokiePosX_Start ; 
                          sta TabLvlDatSnokiePosX_Lo_P1   ; 
                          sta TabLvlDatSnokiePosX_Lo_P2   ; 
                          
                          lda TabIniCheckDataPtrs_Hi,x    ; 
                          adc #>TabLvlDatSnokiePosX_Start ; 
                          sta TabLvlDatSnokiePosX_Hi_P1   ; 
                          sta TabLvlDatSnokiePosX_Hi_P2   ; 
                          
                          lda #<TabDemoMoveData           ; 
                          sta SNZ_DemoMoveDataPtr_Lo      ; 
                          lda #>TabDemoMoveData           ; 
                          sta SNZ_DemoMoveDataPtr_Hi      ; 
                          
                          lda #$00                        ; 
                          sta TabSnokieMoveWaitTime       ; 
                          
                          jmp InitGameLevel               ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitGameVarsPartial     Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitGameVarsPartial       subroutine                      ; 
.IniTabVarCount           ldx #TabGameVarsInitsHalf_Len   ; init 2nd half of game variable table
.GetNextVarVal            lda TabGameVarsInitsHalf,x      ; 
                          sta TabGameVarsHalf,x           ; 
.DecTabVarCount           dex                             ; 
.ChkTabVarCount_Min       bpl .GetNextVarVal              ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitGameLevel           Does    : (Re)Start a level
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitGameLevel             subroutine                      ; 
                          sec                             ; 
.GetCurrLevelProgress_Lo  lda TabLvlDatSnokiePosX_Lo_P1   ; current level progress
.SubSnokiePos_Lo          sbc #<TabLvlDatSnokiePosX_Start ; 
                          sta SNZ_TileChkDataPtr_Lo       ; 
                          
.GetCurrLevelProgress_Hi  lda TabLvlDatSnokiePosX_Hi_P1   ; 
.SubSnokiePos_Hi          sbc #>TabLvlDatSnokiePosX_Start ; 
                          sta SNZ_TileChkDataPtr_Hi       ; 
                          
.IniTabLvlDatStartLen     ldx #TabLvlDatStartPtr_Len      ; search current level start
.IniFindNextLevelStartPtr sec                             ; 
                          lda SNZ_TileChkDataPtr_Lo       ; 
                          sbc TabIniCheckDataPtrs_Lo,x    ; 
                          lda SNZ_TileChkDataPtr_Hi       ; 
                          sbc TabIniCheckDataPtrs_Hi,x    ; 
.ChkCurrLevelStart        bpl .IniLevelStartAdr           ; found
                          
.DecTabLvlDatStartLen     dex                             ; 
.ChkTabLvlDatStartLen_Min bne .IniFindNextLevelStartPtr   ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IniLevelStartAdr         clc                             ; 
                          lda TabIniCheckDataPtrs_Lo,x    ; 
                          sta SNZ_TileChkDataPtr_Lo       ; 
.IniLevelStartAdr_Lo      sta TabLvlScrollSteps_Lo        ; 
                          
.AddSnokiePos_Lo          adc #<TabLvlDatSnokiePosX_Start ; 
                          sta TabLvlDatSnokiePosX_Lo_P1   ; 
                          
                          lda TabIniCheckDataPtrs_Hi,x    ; 
                          sta SNZ_TileChkDataPtr_Hi       ; 
.IniLevelStartAdr_Hi      sta TabLvlScrollSteps_Hi        ; 
                          
.AddSnokiePos_Hi          adc #>TabLvlDatSnokiePosX_Start ; 
                          sta TabLvlDatSnokiePosX_Hi_P1   ; 
                          
                          lda TabLvlSnokieStartPosY,x     ; 
.IniSnokieLevelStartPosY  sta TabLvlSnokiePosY_P1         ; 
                          
                          stx TabLevelNumCurrent          ; 
                          
                          ldx #$00                        ; 
                          ldy #$00                        ; 
                          
                          clc                             ; 
                          lda [TabAdrLvlDatRows + $00]    ; 
                          adc SNZ_TileChkDataPtr_Lo       ; 
.IniProgressPtr_Lo        sta TabGetLevelDataPtr_Lo       ; 
                          
                          lda [TabAdrLvlDatRows + $01]    ; 
                          adc SNZ_TileChkDataPtr_Hi       ; 
.IniProgressPtr_Hi        sta TabGetLevelDataPtr_Hi       ; 
                          
                          jsr InitRocks_L04_L05           ; 
                          jsr InitOrbsBig_L01_L02         ; 
                          jsr InitOrbsTiny_L04_L05        ; 
                          jsr InitGameSound               ; 
                          jsr GamePutScreenLevelData      ; 
                          
InitGameLevelX            jmp InitGameSpritesScore        ; 
; -------------------------------------------------------------------------------------------------------------- ;
; SwapPlayerLevelProgress Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
SwapPlayerLevelProgress   subroutine                      ; 
.IniLvlDatCount           ldx #TabLvlDat_Len              ; 
.GetNextLvlDatProgrData   lda TabLvlDat_P1,x              ; 
                          ldy TabLvlDat_P2,x              ; 
                          
.SwapLvlDatProgrData      sta TabLvlDat_P2,x ; 
                          tya                             ; 
                          sta TabLvlDat_P1,x ; 
                          
.DecLvlDatCount           dex                             ; 
.IniLvlDatCount_Min       bpl .GetNextLvlDatProgrData     ; 
                          
SwapPlayerLevelProgressX  rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; IRQ_Game_SnokieMove     Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
IRQ_Game_SnokieMove       subroutine                      ; 
                          lda TabSnokieStatus             ; 
.ChkSnokieStatus_Hidden   bne .ChkSnokieStatus_Stand      ; not hidden
                          
.SetSprt_Off              lda SPENA                       ; VIC($D015) Sprite Enable Register
                          and #$fc                        ; ######..
                          sta SPENA                       ; VIC($D015) Sprite Enable Register
                          
                          rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.ChkSnokieStatus_Stand    cmp #TabSnokieStatus_Active     ; move/stand
                          beq .IniSnokieSprtStand         ; 
                          
                          jmp .ChkSnokieStatus_Slide      ; 
                          
.IniSnokieSprtStand       lda SPENA                       ; VIC($D015) Sprite Enable Register
                          ora #$03                        ; ......##
                          sta SPENA                       ; VIC($D015) Sprite Enable Register
                          
                          lda SPMC                        ; VIC($D01C) Sprite Multicolor
                          ora #$03                        ; ......##
                          sta SPMC                        ; VIC($D01C) Sprite Multicolor
                          
                          lda #WHITE                      ; 
                          sta SPMC0                       ; VIC($D025) Sprite Multicolor Register 0
                          
                          lda #LT_RED                     ; 
                          sta SPMC1                       ; VIC($D026) Sprite Multicolor Register 1
                          
                          lda #BLACK                      ; 
                          sta SP0COL                      ; VIC($D027) Color Sprite 0
                          
.GetSnokieSprtNum_Min     lda #$00                        ; 
.SetSnokieSprtNum_Min     sta ._SnokieAddSprtNumStand     ; 
                          
                          lda TabSnokieMoveDir            ; 
.ChkSnokieMoveDir_Le      beq .GetSnokieSprtPtr           ; 
                          
.GetSnokieSprtNum_Max     lda #$04                        ; 
.SetSnokieSprtNum_Max     sta ._SnokieAddSprtNumStand     ; 
                          
.GetSnokieSprtPtr         lda #[SpriteData_Snokie_Le_1 / VIC_Sprt_BlockLen] ; 
                          clc                             ; 
._SnokieAddSprtNumStand   equ [* + $01]                   ; 
.AddSnokieSprtNum         adc #$00                        ; 
.SavSnokieSprtPtr         tax                             ; 
                          
                          lda #$00                        ; 
                          sta ._SnokieAddSprtNumGo        ; 
                          
                          lda TabSnokieSfxTime            ; 
                          and #$04                        ; .....#..
                          beq .RstSnokieSprtPtr           ; 
                          
                          lda #$02                        ; 
                          sta ._SnokieAddSprtNumGo        ; 
                          
.RstSnokieSprtPtr         txa                             ; 
                          clc                             ; 
._SnokieAddSprtNumGo      equ [* + $01]                   ; 
                          adc #$00                        ; 
                          
.SetSnokieSprtPtr_00      sta SN_GameSpritePtr_00         ; 
                          
                          clc                             ; 
                          adc #$01                        ; 
.SetSnokieSprtPtr_01      sta SN_GameSpritePtr_01         ; 
                          
                          lda TabLvlSnokiePosY_P1         ; 
                          asl a                           ; 
                          sta ._SubSnokieSpritePosY       ; 
                          
                          sec                             ; 
                          lda #$c9                        ; 
._SubSnokieSpritePosY     equ [* + $01]                   ; 
                          sbc #$00                        ; 
.SetSnokieSprtPosY_01     sta SP0Y                        ; VIC($D001) Sprite 0 Y-Pos (Bits 0-7)
                          
                          clc                             ; 
                          adc #$15                        ; 
.SetSnokieSprtPosY_02     sta SP1Y                        ; VIC($D003) Sprite 1 Y-Pos (Bits 0-7)
                          
                          lda TabSnokieStatus             ; 
                          cmp #TabSnokieStatus_Slide      ; 
                          beq .GetSnokieMovDir            ; 
                          
.GetSnokieLvlPosX         lda TabSnokieLvlScrollPosX      ; 
                          asl a                           ; 
                          
                          sec                             ; 
._SubSnokieSpritePosX     equ [* + $01]                   ; 
                          sbc #$05                        ; 
                          sta SP0X                        ; VIC($D000) Sprite 0 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          sta SP1X                        ; VIC($D002) Sprite 1 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          
                          rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetSnokieMovDir          lda TabSnokieMoveDir            ; 
.ChkSnokieMoveDir_Ri      bne .GetSnokieLvlPosX           ; 
                          
                          lda #$04                        ; 
                          sta ._SubSnokieSpritePosX       ; 
                          
                          jmp .GetSnokieLvlPosX           ; 
; -------------------------------------------------------------------------------------------------------------- ;
.ChkSnokieStatus_Slide    cmp #TabSnokieStatus_Slide      ; 
                          bne .ChkSnokieStatus_Death      ; 
                          
.GetSnokieSprtPtrSit_Le   ldx #[SpriteData_SnokieSit_Le_1 / VIC_Sprt_BlockLen] ; 
                          
                          lda TabSnokieMoveDir            ; 
.ChkSnokieSpriteDir_Le    beq .GetSnokieSprtPtrSit        ; 
                          
.GetSnokieSprtPtrSit_Ri   ldx #[SpriteData_SnokieSit_Ri_1 / VIC_Sprt_BlockLen] ; 
                          
.GetSnokieSprtPtrSit      txa                             ; 
                          jmp .SetSnokieSprtPtr_00        ; 
; -------------------------------------------------------------------------------------------------------------- ;
.ChkSnokieStatus_Death    cmp #TabSnokieStatus_Death      ; 
                          beq .GetSnokieDeathTime         ; 
                          
                          jmp .SetSprt_Off                ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetSnokieDeathTime       lda TabSnokieDeathTime          ; 
                          cmp #TabSnokieDeathTime_Phase1  ; 
.ChkDeathTwinklePhase_1   bcs .IniSprt_SkullTwinkle       ; lower
                          
                          cmp #TabSnokieDeathTime_Phase2  ; 
.ChkDeathTwinklePhase_2   bcc .IniSprt_SkullTwinkle       ; lower
                          
                          cmp #TabSnokieDeathTime_Phase3  ; 
.ChkDeathTwinklePhase_3   bcs .GetSprtPtr_TwinkleHalf     ; greater/equal
                          
                          cmp #TabSnokieDeathTime_Phase4  ; 
.ChkDeathTwinklePhase_4   bcc .GetSprtPtr_TwinkleHalf     ; lower
                          
.GetSprtPtr_TwinkleFull   lda #[SpriteData_Skull_Phase_03 / VIC_Sprt_BlockLen] ; twinkle full
                          
.SetSprtPtr_SkullTwinkle  sta SN_GameSpritePtr_00         ; 
                          
                          rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetSprtPtr_TwinkleHalf   lda #[SpriteData_Skull_Phase_02 / VIC_Sprt_BlockLen] ; twinkle half
                          bne .SetSprtPtr_SkullTwinkle    ; always
; -------------------------------------------------------------------------------------------------------------- ;
.IniSprt_SkullTwinkle     lda SPENA                       ; VIC($D015) Sprite Enable Register
                          and #$fd                        ; ######.#
                          sta SPENA                       ; VIC($D015) Sprite Enable Register
                          
                          lda YXPAND                      ; VIC($D017) Sprite Scale Double Width
                          ora #$01                        ; .......#
                          sta YXPAND                      ; VIC($D017) Sprite Scale Double Width
                          
                          lda XXPAND                      ; VIC($D01D) Sprite Double Height
                          ora #$01                        ; .......#
                          sta XXPAND                      ; VIC($D01D) Sprite Double Height
                          
                          lda #WHITE                      ; 
                          sta SP0COL                      ; VIC($D027) Color Sprite 0
                          
                          lda SPMC                        ; VIC($D01C) Sprite Multicolor
                          and #$fe                        ; #######.
                          sta SPMC                        ; VIC($D01C) Sprite Multicolor
                          
                          lda TabSnokieLvlScrollPosX      ; 
                          asl a                           ; 
                          sec                             ; 
                          sbc #$0f                        ; 
                          sta SP0X                        ; VIC($D000) Sprite 0 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          
.GetSprtPtr_TwinkleStart  lda #[SpriteData_Skull / VIC_Sprt_BlockLen] ; 
.SetSprtPtr_TwinkleStart  sta SN_GameSpritePtr_00         ; 
                          
IRQ_Game_SnokieMoveX      rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitGameSpritesSnokie   Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitGameSpritesSnokie     subroutine                      ; 
                          lda #[SpriteData_Snokie_Ri_1 / VIC_Sprt_BlockLen] ; 
                          sta SN_GameSpritePtr_00         ; 
                          
                          lda #[SpriteData_Snokie_Ri_2 / VIC_Sprt_BlockLen] ; 
                          sta SN_GameSpritePtr_01         ; 
                          
                          lda YXPAND                      ; VIC($D017) Sprite Scale Double Width
                          and #$fc                        ; ######..
                          sta YXPAND                      ; VIC($D017) Sprite Scale Double Width
                          
                          lda YXPAND                      ; VIC($D017) Sprite Scale Double Width
                          and #$fc                        ; ######..
                          sta XXPAND                      ; VIC($D01D) Sprite Double Height
                          
                          sta SP0COL                      ; VIC($D027) Color Sprite 0
                          sta SP1COL                      ; VIC($D028) Color Sprite 1
                          
                          lda SPMC                        ; VIC($D01C) Sprite Multicolor
                          ora #$03                        ; ......##
                          sta SPMC                        ; VIC($D01C) Sprite Multicolor
                          sta SPMC0                       ; VIC($D025) Sprite Multicolor Register 0
                          
                          lda #LT_RED                     ; 
                          sta SPMC1                       ; VIC($D026) Sprite Multicolor Register 1
                          
                          lda #$00                        ; 
                          sta SP0X                        ; VIC($D000) Sprite 0 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          sta SP1X                        ; VIC($D002) Sprite 1 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          
                          lda SPENA                       ; VIC($D015) Sprite Enable Register
                          ora #$03                        ; ......##
                          sta SPENA                       ; VIC($D015) Sprite Enable Register
                          
InitGameSpritesSnokieX    rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitGameScreen          Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitGameScreen            subroutine                      ; 
                          lda SCROLX                      ; VIC($D016) VIC Control Register 2 (Horiz Fine Scroll)
                          and #VIC_Cols38                 ; ####.###
                          ora #VIC_SoftX_07               ; .....###
                          sta SCROLX                      ; VIC($D016) VIC Control Register 2 (Horiz Fine Scroll)
                          
.GetScrn_Game             lda #[VIC_Video_0400 | VIC_CharS_2000] ; 
.SetScrn_Game             sta VMCSB                       ; VIC($D018) VIC Chip Memory Control
                          
                          lda #BLACK                        ; 
                          sta EXTCOL                      ; VIC($D020) Border Color
                          
                          lda #BLACK                      ; 
                          sta BGCOL0                      ; VIC($D021) Background Color 0
                          
                          lda #LT_BLUE                    ; 
                          sta BGCOL2                      ; VIC($D023) Background Color 2
                          
                          lda #LT_GREY                    ; 
                          sta BGCOL1                      ; VIC($D022) Background Color 1
                          
                          lda #LT_BLUE                    ; 
                          ldy #$c8                        ; 
.IniNextColoRamByte       sta [COLORAM + [$c8 * $01] - $01],y ; 
                          sta [COLORAM + [$c8 * $02] - $01],y ; 
                          sta [COLORAM + [$c8 * $03] - $01],y ; 
                          sta [COLORAM + [$c8 * $04] - $01],y ; 
                          sta [COLORAM + [$c8 * $00] - $01],y ; 
                          
                          lda #$00                        ; 
                          sta [SN_Scrn_Game + [$c8 * $00] - $01],y ; 
                          
                          lda #LT_BLUE                    ; 
                          dey                             ; 
                          bne .IniNextColoRamByte         ; 
                          
                          lda SCROLX                      ; VIC($D016) VIC Control Register 2 (Horiz Fine Scroll)
                          ora #VIC_MCM_On                  ; ...#....
                          sta SCROLX                      ; VIC($D016) VIC Control Register 2 (Horiz Fine Scroll)
                          
.GetColorSprtBall         lda #ORANGE                     ; 
.SetColorSprtBall         sta SP2COL                      ; VIC($D029) Color Sprite 2
                          
InitGameScreenX           rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; IRQ_Game                Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
IRQ_Game                  subroutine                      ; 
                          lda #$01                        ; 
                          sta VICIRQ                      ; VIC($D019) Interrupt Flags - Latched flags cleared if set to 1
                          
                          lda Tab_IRQ_Game_Count          ; 
                          bne .Ini_IRQ_Count              ; 
                          
                          inc Tab_IRQ_Game_Count          ; 
                          
                          lda #$00                        ; 
                          sta RASTER                      ; VIC($D012) Read : Current Raster Scan Line (Bit 8 in SCROLY = $D011)
                          
                          lda SCROLY                      ; VIC($D011) VIC Control Register 1 (Vert Fine Scroll)
                          ora #VIC_RasterHi               ; #.......
                          sta SCROLY                      ; VIC($D011) VIC Control Register 1 (Vert Fine Scroll)
                          
.IRQ_Exit                 jmp [_IRQ_MAIN + $4d]           ; read CIA1($DC0D) Interrupt Control and return from interrupt
; -------------------------------------------------------------------------------------------------------------- ;
.Ini_IRQ_Count            lda #$00                        ; 
.Set_IRQ_Count            sta Tab_IRQ_Game_Count          ; 
                          
                          jsr IRQ_Game_ScrollLevel        ; 
                          jsr IRQ_Game_SnokieMove         ; 
                          
                          lda SCROLY                      ; VIC($D011) VIC Control Register 1 (Vert Fine Scroll)
                          and #VIC_RasterHi_Off           ; .#######
                          sta SCROLY                      ; VIC($D011) VIC Control Register 1 (Vert Fine Scroll)
                          
IRQ_Game_X                jmp [_IRQ_MAIN + $4d]           ; read CIA1($DC0D) Interrupt Control and return from interrupt
; -------------------------------------------------------------------------------------------------------------- ;
Tab_IRQ_Game_Count        dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameSetSpriteDataTime   Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameSetSpriteDataTime     subroutine                      ; 
                          lda TabSnokieStatus             ; 
.ChkSnokieStatus_Hidden   bne .ChkSnokieStatus_Death      ; not hidden
                          
.SetSprt_Off              lda SPMC                        ; VIC($D01C) Sprite Multicolor
                          and #$3f                        ; ..######
                          sta SPMC                        ; VIC($D01C) Sprite Multicolor
; -------------------------------------------------------------------------------------------------------------- ;
.GetActivePlayer_01       lda TabActivePlayer             ; 
.TstActivePlayer_01_P1    cmp #TabActivePlayer_P1         ; 
.ChkActivePlayer_01_P1    bne .IniGetSprtData_Player_2    ; 
                          
..IniGetSprtData_Player_1 lda #[SpriteData_TextPlayer1Le / VIC_Sprt_BlockLen] ; 
                          sta __GetSpriteTimeData_Le_Lo   ; 
                          
                          lda #[SpriteData_TextPlayer1Ri / VIC_Sprt_BlockLen] ; 
                          sta __GetSpriteTimeData_Ri_Lo   ; 
                          rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IniGetSprtData_Player_2  lda #[SpriteData_TextPlayer2Le / VIC_Sprt_BlockLen] ; 
                          sta __GetSpriteTimeData_Le_Lo   ; 
                          
                          lda #[SpriteData_TextPlayer2Ri / VIC_Sprt_BlockLen] ; 
                          sta __GetSpriteTimeData_Ri_Lo   ; 
                          rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.ChkSnokieStatus_Death    cmp #TabSnokieStatus_Death      ; 
                          bne .IniGetSprtDataTime         ; 
                          
.GetActivePlayer_02       lda TabActivePlayer             ; 
.TstActivePlayer_02_P1    cmp #TabActivePlayer_P1         ; 
.ChkActivePlayer_02_P1    bne .GetNumLives_P2             ; 
                          
.GetNumLives_P1           lda TabNumLives_P1              ; 
                          jmp .GetNumLives                ; 
                          
.GetNumLives_P2           lda TabNumLives_P2              ; 
                          
.GetNumLives              tay                             ; 
.ChkNumLives_Min          bne .IniGetSprtDataTime         ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IniGetSprtData_GameOver  lda SPMC                        ; VIC($D01C) Sprite Multicolor
                          and #$3f                        ; ..######
                          sta SPMC                        ; VIC($D01C) Sprite Multicolor
                          
                          lda #WHITE                      ; 
                          sta SP0COL                      ; VIC($D027) Color Sprite 0
                          
                          lda #[SpriteData_TextGameOverLe / VIC_Sprt_BlockLen] ; 
                          sta __GetSpriteTimeData_Le_Lo   ; 
                          
                          lda #[SpriteData_TextGameOverRi / VIC_Sprt_BlockLen] ; 
                          sta __GetSpriteTimeData_Ri_Lo   ; 
                          rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IniGetSprtDataTime       lda #[SpriteData_TimeLe / VIC_Sprt_BlockLen] ; 
                          sta __GetSpriteTimeData_Le_Lo   ; 
                          
                          lda #[SpriteData_TimeRi / VIC_Sprt_BlockLen] ; 
                          sta __GetSpriteTimeData_Ri_Lo   ; 
; -------------------------------------------------------------------------------------------------------------- ;
.SetSprt_On               lda SPMC                        ; VIC($D01C) Sprite Multicolor
                          ora #$c0                        ; ##......
                          sta SPMC                        ; VIC($D01C) Sprite Multicolor
                          
GameSetSpriteDataTimeX    rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitOutStartMenuTxt     Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitOutStartMenuTxt       subroutine                      ; 
                          ldy #TabTxtInfoScrnRow_01_Len   ; "by troy lyndon and yves lempereur"
.OutNextInfoByte_Authors  lda [TabTxtInfoScrnRow_01 - $01],y ; 
                          and #$bf                        ; #.######
                          sta [SN_Scrn_Game + [SN_Scrn_RowLen * $08] + $02],y ; 
                          dey                             ; 
                          bne .OutNextInfoByte_Authors    ; 
                          
                          ldy #TabTxtInfoScrnRow_02_Len   ; "copyright (c) 1983, funsoft inc."
.OutNextInfoByte_Company  lda [TabTxtInfoScrnRow_02 - $01],y ; 
                          and #$bf                        ; #.######
                          sta [SN_Scrn_Game + [SN_Scrn_RowLen * $0c] + $03],y ; 
                          
                          lda [TabTxtInfoScrnRow_03 - $01],y ; "28611 canwood, agoura, ca  93010"
                          and #$bf                        ; #.######
                          sta [SN_Scrn_Game + [SN_Scrn_RowLen * $0d] + $03],y ; 
                          
                          lda [TabTxtInfoScrnRow_04 - $01],y ; "game concept by a. marsily"
                          and #$bf                        ; #.######
                          sta [SN_Scrn_Game + [SN_Scrn_RowLen * $0a] + $03],y ; 
                          
                          dey                             ; 
                          bne .OutNextInfoByte_Company    ; 
                          
                          ldy #TabTxtInfoScrnRow_05_Len   ; 
.OutNextInfoByte_Phone    lda [TabTxtInfoScrnRow_05 - $01],y ; "(213) 991-6540"
                          and #$bf                        ; #.######
                          sta [SN_Scrn_Game + [SN_Scrn_RowLen * $0e] + $0c],y ; 
                          dey                             ; 
                          bne .OutNextInfoByte_Phone      ; 
                          
                          ldy #TabTxtInfoScrnRow_06_Len   ; 
.OutNextInfoByte_Header   lda [TabTxtInfoScrnRow_06 - $01],y ; "-- i n s t r u c t i o n s --"
                          and #$bf                        ; #.######
                          sta [SN_Scrn_Game + [SN_Scrn_RowLen * $10] + $04],y ; 
                          dey                             ; 
                          bne .OutNextInfoByte_Header     ; 
                          
                          ldy #TabTxtInfoScrnRow_07_Len   ; 
.OutNextInfoByte_Keys     lda [TabTxtInfoScrnRow_07 - $01],y ; "< f1 >"
                          and #$bf                        ; #.######
                          sta [SN_Scrn_Game + [SN_Scrn_RowLen * $12] + $03],y ; 
                          
                          lda [TabTxtInfoScrnRow_08 - $01],y ; "< f3 >"
                          and #$bf                        ; #.######
                          sta [SN_Scrn_Game + [SN_Scrn_RowLen * $14] + $03],y ; 
                          
                          lda [TabTxtInfoScrnRow_09 - $01],y ; "< f5 >"
                          and #$bf                        ; #.######
                          sta [SN_Scrn_Game + [SN_Scrn_RowLen * $16] + $03],y ; 
                          
                          dey                             ; 
                          bne .OutNextInfoByte_Keys       ; 
                          
                          ldy #TabTxtInfoScrnRow_10_Len   ; 
.OutNextInfoByte_Txt_F1   lda [TabTxtInfoScrnRow_10 - $01],y ; "b e g i n  g a m e"
                          and #$bf                        ; 
                          sta [SN_Scrn_Game + [SN_Scrn_RowLen * $12] + $0f],y ; 
                          dey                             ; 
                          bne .OutNextInfoByte_Txt_F1     ; 
                          
InitOutStartMenuTxtX      rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitOutStartMenuTxtPlayer Does  : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitOutStartMenuTxtPlayer subroutine                      ; 
.IniTxtCount              ldy #$00                        ; 
                          ldx #$00                        ; 
.OutNextInfoByte_Players  lda TabTxtInfoScrnVarOut2,y     ; 
                          and #$bf                        ; #.######
                          sta [SN_Scrn_Game + [SN_Scrn_RowLen * $16] + $0e],x ; 
                          
.IncTxtCount              iny                             ; 
                          
.IncTxtOut_Off            inx                             ; 
                          inx                             ; 
                          
.TstTxtCount_Max          cpy #[TabTxtInfoScrnVarOut1_Len + $01] ; 
.ChkTxtCount_Max          bcc .OutNextInfoByte_Players    ; 
                          
InitOutStartMenuTxtPlayerX rts                            ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitOutStartMenuTxtSpeed Does   : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitOutStartMenuTxtSpeed  subroutine                      ; 
.IniTxtCount              ldy #$00                        ; 
                          ldx #$00                        ; 
.OutNextInfoByte_Speed    lda TabTxtInfoScrnVarOut1,y     ; "normal"
                          and #$bf                        ; #.######
                          sta [SN_Scrn_Game + [SN_Scrn_RowLen * $14] + $0e],x ; 
                          
.IncTxtCount              iny                             ; 
                          
.IncTxtOut_Off            inx                             ; 
                          inx                             ; 
                          
.TstTxtCount_Max          cpy #[TabTxtInfoScrnVarOut1_Len + $01] ; 
.ChkTxtCount_Max          bcc .OutNextInfoByte_Speed      ; lower
                          
InitOutStartMenuTxtSpeedX rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
InfoScreenText            include inc\SN_TextInfoScreen.asm ; 
; -------------------------------------------------------------------------------------------------------------- ;
; KeyboardGetKey          Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
KeyboardGetKey            subroutine                      ; 
                          lda #$ff                        ; ########
                          sta CIDDRA                      ; CIA1($DC02) Data Direction Register A
                          
.IniKeyboardRowCount      ldx #$00                        ; 
.GetNextKeyboardRowBit    lda TabKeyboardBitRowChk,x      ; 
                          sta CIAPRA                      ; CIA1($DC00) Data Port Register A
                          
                          lda TabKeyboardBitColCheck,x    ; 
                          sta ._EorBitColCheck            ; 
                          
                          lda CIAPRB                      ; CIA1($DC01) Data Port Register B
                          eor #$ff                        ; ########
                          sta TabKeyboardBitColCheck,x    ; 
                          
._EorBitColCheck          equ [* + $01]                   ; 
                          eor #$00                        ; 
                          and TabKeyboardBitColCheck,x    ; 
.ChkKeyboardKeyPressed    bne .IniKeyboardColCount        ; some was key pressed - check which
                          
.IncKeyboardRowCount      inx                             ; 
.TstKeyboardRowCount_Max  cpx #[TabKeyboardBitRowChk_Len + $01] ; 
.ChkKeyboardRowCount_Max  bcc .GetNextKeyboardRowBit      ; lower
; -------------------------------------------------------------------------------------------------------------- ;
.IniExit_NoKey            ldx #$00                        ; 
                          stx CIDDRA                      ; CIA1($DC02) Data Direction Register A
                          
                          lda #CIA_Key_None               ; 
                          sta TabKeyboardKeyGot           ; 
                          
.Exit_NoKey               rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IniKeyboardColCount      ldy #$00                        ; 
.GetNextKeyboardColBit    asl a                           ; 
                          bcs .GetKeyGot_Row              ; 
                          
.IncKeyboardColCount      iny                             ; 
.TstKeyboardColCount_Max  cpy #$08                        ; 
.ChkKeyboardColCount_Max  bcc .GetNextKeyboardColBit      ; lower - try next col
                          bcs .IniExit_NoKey              ; higher/equal - always
; -------------------------------------------------------------------------------------------------------------- ;
.GetKeyGot_Row            txa                             ; .....### - get keyboard key
                          asl a                           ; ....###. 
                          asl a                           ; ...###..
                          asl a                           ; ..###...
                          
.SetKeyGot_Col            sty ._AddKeyGotRow              ; 
                          
                          clc                             ; 
._AddKeyGotRow            equ [* + $01]                   ; 
                          adc #$00                        ; 
                          
                          cmp #$05                        ; 
                          beq .IniExit_NoKey              ; 
                          
                          cmp #$0b                        ; 
                          beq .IniExit_NoKey              ; 
                          
                          cmp #$30                        ; 
                          beq .IniExit_NoKey              ; 
                          
                          cmp #$39                        ; a
                          bcc .SavComposedKeyGot          ; lower
                          
                          cmp #$3d                        ; 
                          bcc .IniExit_NoKey              ; lower
; -------------------------------------------------------------------------------------------------------------- ;
.SavComposedKeyGot        pha                             ; 
                          
                          lda #[CIA_Key_None ^ CIA_KeySelRow_06] ; #.######
                          sta CIAPRA                      ; CIA1($DC00) Data Port Register A
                          
                          lda CIAPRB                      ; CIA1($DC01) Data Port Register B
                          and #[CIA_Key_None ^ CIA_KeyWas_Shft_R] ; ...#....
.ChkKey_Shift_Right       beq .GetKeyShift                ; 
                          
                          lda #[CIA_Key_None ^ CIA_KeySelRow_01] ; ######.#
                          sta CIAPRA                      ; CIA1($DC00) Data Port Register A
                          
                          lda CIAPRB                      ; CIA1($DC01) Data Port Register B
                          and #[CIA_Key_None ^ CIA_KeyWas_Shft_L] ; #.......
.ChkKey_Shift_Left        beq .GetKeyShift                ; 
                          
.RstComposedKeyGot        pla                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IniWaitTime              ldx #$00                        ; 
.DecNextWaitTime          dex                             ; 
.ChkWaitTime_Min          bne .DecNextWaitTime            ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IniDDRA                  stx CIDDRA                      ; CIA1($DC02) Data Direction Register A
                          
.SetKeyboarKeyGot         sta TabKeyboardKeyGot           ; 
                          
.Exit_GotKey              rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetKeyShift              pla                             ; 
.InsKeyShift              ora #$40                        ; .#......
KeyboardGetKeyX           bne .IniWaitTime                ; always
; -------------------------------------------------------------------------------------------------------------- ;
TabKeyboardBitRowChk      equ  *   ; row bit to check=0 / row bit to ignore=1
                          dc.b $7f ; .#######
                          dc.b $bf ; #.######
                          dc.b $df ; ##.#####
                          dc.b $ef ; ###.####
                          dc.b $f7 ; ####.###
                          dc.b $fb ; #####.##
                          dc.b $fd ; ######.#
                          dc.b $fe ; #######.
TabKeyboardBitRowChk_Len  equ [* - TabKeyboardBitRowChk - $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabKeyboardBitColCheck    equ  *   ; 
                          dc.b $00 ; ........
                          dc.b $00 ; ........
                          dc.b $00 ; ........
                          dc.b $00 ; ........
                          dc.b $00 ; ........
                          dc.b $00 ; ........
                          dc.b $00 ; ........
                          dc.b $00 ; ........
; -------------------------------------------------------------------------------------------------------------- ;
DemoMoves                 include inc\SN_DemoMoves.asm    ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabKeyGotDecode           equ  *   ;                      sorted
                          dc.b $20 ; $00 -                $00 ; $11 - @
                          dc.b $11 ; $01 - Q              $01 ; $35 - A
                          dc.b $20 ; $02 -                $02 ; $23 - B
                          dc.b $20 ; $03 -                $03 ; $2b - C
                          dc.b $32 ; $04 - 2              $04 ; $2d - D
                          dc.b $20 ; $05 -                $05 ; $31 - E
                          dc.b $1f ; $06 - <arrow left>   $06 ; $2a - F
                          dc.b $31 ; $07 - 1              $07 ; $25 - G
                          dc.b $2f ; $08 - ?              $08 ; $22 - H
                          dc.b $1e ; $09 - <arrow up>     $09 ; $1e - I
                          dc.b $3d ; $0a - =              $0a ; $1d - J
                          dc.b $20 ; $0b -                $0b ; $1a - K
                          dc.b $20 ; $0c -                $0c ; $15 - L
                          dc.b $3b ; $0d - ;              $0d ; $1b - M
                          dc.b $2a ; $0e - *              $0e ; $18 - N
                          dc.b $1c ; $0f - <pound>        $0f ; $19 - O
                          dc.b $2c ; $10 - ,              $10 ; $16 - P
                          dc.b $00 ; $11 - @              $11 ; $01 - Q
                          dc.b $3a ; $12 - :              $12 ; $2e - R
                          dc.b $2e ; $13 - .              $13 ; $32 - S
                          dc.b $2d ; $14 - -              $14 ; $29 - T
                          dc.b $0c ; $15 - L              $15 ; $21 - U
                          dc.b $10 ; $16 - P              $16 ; $20 - V
                          dc.b $2b ; $17 - +              $17 ; $36 - W
                          dc.b $0e ; $18 - N              $18 ; $28 - X
                          dc.b $0f ; $19 - O              $19 ; $26 - Y
                          dc.b $0b ; $1a - K              $1a ; $33 - Z
                          dc.b $0d ; $1b - M              $1c ; $0f - <pound>
                          dc.b $30 ; $1c - 0              $1e ; $09 - <arrow up>
                          dc.b $0a ; $1d - J              $1f ; $06 - <arrow left>
                          dc.b $09 ; $1e - I              $20 ; $00 - 
                          dc.b $39 ; $1f - 9              $20 ; $02 - 
                          dc.b $16 ; $20 - V              $20 ; $03 - 
                          dc.b $15 ; $21 - U              $20 ; $05 - 
                          dc.b $08 ; $22 - H              $20 ; $0b - 
                          dc.b $02 ; $23 - B              $20 ; $0c - 
                          dc.b $38 ; $24 - 8              $20 ; $30 - 
                          dc.b $07 ; $25 - G              $20 ; $38 - 
                          dc.b $19 ; $26 - Y              $20 ; $39 - 
                          dc.b $37 ; $27 - 7              $20 ; $3a - 
                          dc.b $18 ; $28 - X              $20 ; $3b - 
                          dc.b $14 ; $29 - T              $20 ; $3c - 
                          dc.b $06 ; $2a - F              $20 ; $3d - 
                          dc.b $03 ; $2b - C              $20 ; $3e - <enter>
                          dc.b $36 ; $2c - 6              $20 ; $3f - <delete>
                          dc.b $04 ; $2d - D              $2a ; $0e - *
                          dc.b $12 ; $2e - R              $2b ; $17 - +
                          dc.b $35 ; $2f - 5              $2c ; $10 - ,
                          dc.b $20 ; $30 -                $2d ; $14 - -
                          dc.b $05 ; $31 - E              $2e ; $13 - .
                          dc.b $13 ; $32 - S              $2f ; $08 - ?
                          dc.b $1a ; $33 - Z              $30 ; $1c - 0
                          dc.b $34 ; $34 - 4              $31 ; $07 - 1
                          dc.b $01 ; $35 - A              $32 ; $04 - 2
                          dc.b $17 ; $36 - W              $33 ; $37 - 3
                          dc.b $33 ; $37 - 3              $34 ; $34 - 4
                          dc.b $20 ; $38 -                $35 ; $2f - 5
                          dc.b $20 ; $39 -                $36 ; $2c - 6
                          dc.b $20 ; $3a -                $37 ; $27 - 7
                          dc.b $20 ; $3b -                $38 ; $24 - 8
                          dc.b $20 ; $3c -                $39 ; $1f - 9
                          dc.b $20 ; $3d -                $3a ; $12 - :
                          dc.b $20 ; $3e - <enter>        $3b ; $0d - ;
                          dc.b $20 ; $3f - <delete>       $3d ; $0a - =
TabKeyGotDecode_Len       equ  [* - TabKeyGotDecode - $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabHiScoreEntries         equ  *   ; 
TabHiScoreEntry_01        equ  *   ; 
TabHiScoreEntry_Name      equ  *   ; 
                          dc.b $14 ; t
                          dc.b $12 ; r
                          dc.b $0f ; o
                          dc.b $19 ; y
                          dc.b $20 ; <space>
                          dc.b $0c ; l
                          dc.b $2e ; .
TabHiScoreEntry_Name_Len  = [* - TabHiScoreEntry_Name - $01] ; 
                          dc.b $20 ; <space>
TabHiScoreEntry_Score     dc.b $20 ; 
                          dc.b $31 ; 1
                          dc.b $30 ; 0
                          dc.b $30 ; 0
                          dc.b $30 ; 0
                          dc.b $30 ; 0
TabHiScoreEntry_Score_Len = [* - TabHiScoreEntry_Score - $01] ; 
TabHiScoreEntry_Len       = [* - TabHiScoreEntry_Name  - $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabHiScoreEntry_02        equ  *   ; 
                          dc.b $19 ; y
                          dc.b $16 ; v
                          dc.b $05 ; e
                          dc.b $13 ; s
                          dc.b $20 ; <space>
                          dc.b $0c ; l
                          dc.b $2e ; .
                          dc.b $20 ; <space>
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $39 ; 9
                          dc.b $30 ; 0
                          dc.b $30 ; 0
                          dc.b $30 ; 0
; -------------------------------------------------------------------------------------------------------------- ;
TabHiScoreEntry_03        equ  *   ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabHiScoreEntry_04        equ  *   ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabHiScoreEntry_05        equ  *   ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabHiScoreEntry_06        equ  *   ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabHiScoreEntry_07        equ  *   ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabHiScoreEntry_08        equ  *   ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabHiScoreEntry_09        equ  *   ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabHiScoreEntry_10        equ  *   ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
                          dc.b $20 ; 
TabHiScoreEntriesX        equ  *   ; 
TabHiScoreEntries_Len     equ  [* - TabHiScoreEntries] ; 
; -------------------------------------------------------------------------------------------------------------- ;
FlagHiScoreDeCompress     dc.b $ff ; ########
; -------------------------------------------------------------------------------------------------------------- ;
; GameHiScoreTableDisplay Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameHiScoreTableDisplay   subroutine                      ; 
                          lda #<[SN_Scrn_HiScore_Row_05 + $04] ; 
                          sta SNZ_HiScoreCharOutPtr_Lo    ; 
                          lda #>SN_MemScreenHiScore       ; 
                          sta SNZ_HiScoreCharOutPtr_Hi    ; 
                          
.IniHiScoreTabOff         ldx #$00                        ; 
.SetNextHiScoreNamLenMax  ldy #$08                        ; 
.SetNextHiScoreNamLen     sty ._HiScoreNamLen             ; 
                          lda TabHiScoreEntries,x         ; 
                          jsr GameHiScoreCharOut          ; 
                          
                          clc                             ; 
                          lda SNZ_HiScoreCharOutPtr_Lo    ; 
                          adc #$02                        ; 
                          sta SNZ_HiScoreCharOutPtr_Lo    ; 
                          bcc .IncHiScoreTabOff_Nam       ; 
                          inc SNZ_HiScoreCharOutPtr_Hi    ; 
                          
.IncHiScoreTabOff_Nam     inx                             ; 
._HiScoreNamLen           equ [* + $01]                   ; 
.GetHiScoreNamLen         ldy #$00                        ; 
.DecHiScoreNamLen         dey                             ; 
.ChkHiScoreNamLen_Min     bne .SetNextHiScoreNamLen       ; 
                          
                          clc                             ; 
                          lda SNZ_HiScoreCharOutPtr_Lo    ; 
                          adc #>SN_MemScreenGame          ; 
                          sta SNZ_HiScoreCharOutPtr_Lo    ; 
                          bcc .IniHiScoreValLenMax        ; 
                          inc SNZ_HiScoreCharOutPtr_Hi    ; 
                          
.IniHiScoreValLenMax      ldy #$06                        ; 
.SetNextScoreValLen       sty ._HiScoreValLen             ; 
                          lda TabHiScoreEntries,x         ; 
                          jsr GameHiScoreCharOut          ; 
                          
                          clc                             ; 
                          lda SNZ_HiScoreCharOutPtr_Lo    ; 
                          adc #$02                        ; 
                          sta SNZ_HiScoreCharOutPtr_Lo    ; 
                          bcc .IncHiScoreTabOff           ; 
                          inc SNZ_HiScoreCharOutPtr_Hi    ; 
                          
.IncHiScoreTabOff         inx                             ; 
._HiScoreValLen           equ [* + $01]                   ; 
.GetHiScoreValLen         ldy #$00                        ; 
.DecHiScoreValLen         dey                             ; 
.ChkHiScoreValLen_Min     bne .SetNextScoreValLen         ; 
                          
.SetPointer_NextRow       clc                             ; 
                          lda SNZ_HiScoreCharOutPtr_Lo    ; 
                          adc #[SN_Scrn_RowLen + $08]     ; 
                          sta SNZ_HiScoreCharOutPtr_Lo    ; 
                          bcc .TstHiScoreTabOff_Max       ; 
                          inc SNZ_HiScoreCharOutPtr_Hi    ; 
                          
.TstHiScoreTabOff_Max     cpx #$8c                        ; 
.ChkHiScoreTabOff_Max     bcc .SetNextHiScoreNamLenMax    ; lower
                          
GameHiScoreTableDisplayX  rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameHiScoreCharOut      Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameHiScoreCharOut        subroutine                      ; 
                          asl a                           ; *2
                          asl a                           ; *4
                          
.SetOff_Row_1_Pos_1       ldy #$00                        ; 
                          sta (SNZ_HiScoreCharOutPtr),y   ; 
                          
                          clc                             ; 
                          adc #$01                        ; 
.SetOff_Row_1_Pos_2       iny                             ; 
                          sta (SNZ_HiScoreCharOutPtr),y   ; 
                          
                          clc                             ; 
                          adc #$01                        ; 
.SetOff_Row_2_Pos_1       ldy #SN_Scrn_RowLen             ; 
                          sta (SNZ_HiScoreCharOutPtr),y   ; 
                          
                          clc                             ; 
                          adc #$01                        ; 
.SetOff_Row_2_Pos_2       iny                             ; 
                          sta (SNZ_HiScoreCharOutPtr),y   ; 
                          
GameHiScoreCharOutX       rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitSwapCharSetsHandler Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitSwapCharSetsHandler   subroutine                      ; 
                          lda VMCSB                       ; VIC($D018) VIC Chip Memory Control
                          and #[$ff ^ VIC_VideoMask]      ; ####....
.TstScrn_Menu             cmp #VIC_Video_0400             ; 
.ChkScrn_Menu             bne .Ini1stCharByte_Gfx         ; 
                          
.Ini1stCharByte_Txt       lda #$78                        ; 1st byte TabCharSet_Text
                          bne .Tst1stCharByte             ; always
                          
.Ini1stCharByte_Gfx       lda #$00                        ; 1st byte TabCharSet_Gfx
                          
.Tst1stCharByte           cmp TabCharSet_Text             ; 
                          bne InitSwapCharTileSets        ; 
                          
InitSwapCharSetsHandlerX  rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitSwapCharTileSets    Does    : Swap txt and gfx charsets
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitSwapCharTileSets      subroutine                      ; 
.IniCharSetPtr_Txt        lda #<TabCharSet_Text           ; $2800 - $2fff - active display charset
                          sta ._GetCharSet_Adr_Lo         ; 
                          sta ._SetCharSet_Adr_Lo         ; 
                          
                          lda #>TabCharSet_Text           ; $2800 - $2fff
                          sta ._GetCharSet_Adr_Hi         ; 
                          sta ._SetCharSet_Adr_Hi         ; 
                          
.IniCharSetPtr_Gfx        lda #<TabCharSet_Gfx            ; $1800 - $1fff - inactive display charset
                          sta ._GetGfxTile_Adr_Lo         ; 
                          sta ._SetGfxTile_Adr_Lo         ; 
                          
                          lda #>TabCharSet_Gfx            ; $1800 - $1fff
                          sta ._GetGfxTile_Adr_Hi         ; 
                          sta ._SetGfxTile_Adr_Hi         ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IniCountPage             ldx #$08                        ; 
.SetNextCharByteCount     stx ._GetCountPage              ; 
                          
.IniCountByte             ldy #$00                        ; 
; -------------------------------------------------------------------------------------------------------------- ;
._GetCharSet_Adr_Lo       equ [* + $01]                   ; 
._GetCharSet_Adr_Hi       equ [* + $02]                   ; 
.GetNextCharSetByte       lda TabCharSet_Text,y           ; 
.SavCharSetByte_Txt       sta ._GetGfxTile_Val_Out        ; save txt charset byte
                          
._GetGfxTile_Adr_Lo       equ [* + $01]                   ; 
._GetGfxTile_Adr_Hi       equ [* + $02]                   ; 
.GetCharSetByte_Gfx       lda TabCharSet_Gfx,y            ; 
                          
._SetCharSet_Adr_Lo       equ [* + $01]                   ; 
._SetCharSet_Adr_Hi       equ [* + $02]                   ; 
.SetCharSetByte_Gfx       sta TabCharSet_Text,y           ; swap gfx charset byte
                          
._GetGfxTile_Val_Out      equ [* + $01]                   ; 
.GetCharSetByte_Txt       lda #$00                        ; 
                          
._SetGfxTile_Adr_Lo       equ [* + $01]                   ; 
._SetGfxTile_Adr_Hi       equ [* + $02]                   ; 
.SetCharSetByte_Txt       sta TabCharSet_Gfx,y            ; swap txt charset byte
                          
.IncCountByte             iny                             ; 
.ChkCountByte_Max         bne .GetNextCharSetByte         ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IncAdrPage               inc ._GetCharSet_Adr_Hi         ; 
                          inc ._GetGfxTile_Adr_Hi         ; 
                          inc ._SetCharSet_Adr_Hi         ; 
                          inc ._SetGfxTile_Adr_Hi         ; 
                          
._GetCountPage            equ [* + $01]                   ; 
.GetCountPage             ldx #$00                        ; 
.DecCountPage             dex                             ; 
.ChkCountPage_Min         bne .SetNextCharByteCount       ; 
                          
InitSwapCharTileSetsX     rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitColorRamToBlack     Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitColorRamToBlack       subroutine                      ; 
                          lda SCROLY                      ; VIC($D011) VIC Control Register 1 (Vert Fine Scroll)
                          and #CIA_IntMaskFlag_Off        ; ###.####
                          sta SCROLY                      ; VIC($D011) VIC Control Register 1 (Vert Fine Scroll)
                          
.IniColoramLen            ldy #$c8                        ; 
                          lda #$00                        ; BLACK
.SetSpritesOff            sta SPENA                       ; VIC($D015) Sprite Enable Register
.SetNextColoramByte       sta [COLORAM + [[SN_Scrn_RowLen * $05] * $00] - $01],y ; start at row #5
                          sta [COLORAM + [[SN_Scrn_RowLen * $05] * $01] - $01],y ; 
                          sta [COLORAM + [[SN_Scrn_RowLen * $05] * $02] - $01],y ; 
                          sta [COLORAM + [[SN_Scrn_RowLen * $05] * $03] - $01],y ; 
                          sta [COLORAM + [[SN_Scrn_RowLen * $05] * $04] - $01],y ; 
.DecColoramLen            dey                             ; 
.ChkColoramLen_Min        bne .SetNextColoramByte         ; 
                          
InitColorRamToBlackX      rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitColorRamToWhite     Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitColorRamToWhite       subroutine                      ; 
.IniColoramLen            ldy #$c8                        ; 
                          lda #WHITE                      ; 
.SetNextColoramByte       sta [COLORAM + [[SN_Scrn_RowLen * $05] * $00] - $01],y ; start at row #5
                          sta [COLORAM + [[SN_Scrn_RowLen * $05] * $01] - $01],y ; 
                          sta [COLORAM + [[SN_Scrn_RowLen * $05] * $02] - $01],y ; 
                          sta [COLORAM + [[SN_Scrn_RowLen * $05] * $03] - $01],y ; 
                          sta [COLORAM + [[SN_Scrn_RowLen * $05] * $04] - $01],y ; 
.DecColoramLen            dey                             ; 
.ChkColoramLen_Min        bne .SetNextColoramByte         ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitVicScreenOn         Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitVicScreenOn           subroutine                      ; 
                          lda SCROLY                      ; VIC($D011) VIC Control Register 1 (Vert Fine Scroll)
                          ora #VIC_Screen_On              ; ...#....
                          sta SCROLY                      ; VIC($D011) VIC Control Register 1 (Vert Fine Scroll)
                          
InitVicScreenOnX          rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameHiScoreSpritesInit  Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameHiScoreSpritesInit    subroutine                      ; 
                          lda #$3f                        ; ..######
                          sta SPENA                       ; VIC($D015) Sprite Enable Register
                          sta SPMC                        ; VIC($D01C) Sprite Multicolor
                          
                          lda VMCSB                       ; VIC($D018) VIC Chip Memory Control
                          and #[$ff ^ VIC_VideoMask]      ; ####....
                          cmp #VIC_Video_0400             ; ...#....
                          bne .IniSprtPosXYLen_Rst        ; 
                          
                          lda #$3f                        ; ..######
                          sta YXPAND                      ; VIC($D017) Sprite Scale Double Width
                          sta XXPAND                      ; VIC($D01D) Sprite Double Height
                          
.IniSprtPosXYLen_Sav      ldy #TabSavSprtPosXY_Len        ; 
.GetNextSprtPosXY_Sav     lda [TabSavSprtPosXY - $01],y   ; 
.SetSprtPosXY_Sav         sta [SP0X - $01],y              ; 
.DecSprtPosXYLen_Sav      dey                             ; 
.ChkSprtPosXYLen_Sav_Min  bne .GetNextSprtPosXY_Sav       ; 
                          
                          lda #$20                        ; ..#.....
                          sta MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
                          
                          rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.IniSprtPosXYLen_Rst      ldy #TabSavSprtPosXY_Len        ; 
.GetNextSprtPosXY_Rst     lda [SP0X - $01],y              ; 
.SetSprtPosXY_Rst         sta [TabSavSprtPosXY - $01],y   ; 
.DecSprtPosXYLen_Rst      dey                             ; 
.ChkSprtPosXYLen_Rst_Min  bne .GetNextSprtPosXY_Rst       ; 
                          
                          lda #$20                        ; 
                          sta MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
                          
                          lda #$00                        ; 
                          sta YXPAND                      ; VIC($D017) Sprite Scale Double Width
                          sta XXPAND                      ; VIC($D01D) Sprite Double Height
                          sta MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
                          
                          lda #$3b                        ; 
                          sta SP0Y                        ; VIC($D001) Sprite 0 Y-Pos (Bits 0-7)
                          sta SP1Y                        ; VIC($D003) Sprite 1 Y-Pos (Bits 0-7)
                          sta SP2Y                        ; VIC($D005) Sprite 2 Y-Pos (Bits 0-7)
                          sta SP3Y                        ; VIC($D007) Sprite 3 Y-Pos (Bits 0-7)
                          sta SP4Y                        ; VIC($D009) Sprite 4 Y-Pos (Bits 0-7)
                          sta SP5Y                        ; VIC($D00B) Sprite 5 Y-Pos (Bits 0-7)
                          
                          lda #$66                        ; 
                          sta SP0X                        ; VIC($D000) Sprite 0 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          lda #$7e                        ; 
                          sta SP1X                        ; VIC($D002) Sprite 1 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          lda #$96                        ; 
                          sta SP2X                        ; VIC($D004) Sprite 2 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          lda #$c6                        ; 
                          sta SP3X                        ; VIC($D006) Sprite 3 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          lda #$de                        ; 
                          sta SP4X                        ; VIC($D008) Sprite 4 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          lda #$f6                        ; 
                          sta SP5X                        ; VIC($D00A) Sprite 5 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          
                          jsr GameHiScoreOrbSprtInit      ; 
                          
GameHiScoreSpritesInitX   rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabSavSprtPosXY           equ  *   ; 
                          dc.b $00 ; $00
                          dc.b $00 ; $01
                          dc.b $00 ; $02
                          dc.b $00 ; $03
                          dc.b $00 ; $04
                          dc.b $00 ; $05
                          dc.b $00 ; $06
                          dc.b $00 ; $07
                          dc.b $00 ; $08
                          dc.b $00 ; $09
                          dc.b $00 ; $0a
                          dc.b $00 ; $0b
                          dc.b $00 ; $0c
TabSavSprtPosXY_Len       equ  [* - TabSavSprtPosXY - $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameHiScoreFlashSprite  Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameHiScoreFlashSprite    subroutine                      ; 
.SetNextSpriteColor       dec TabSpriteColor              ; 
                          
.GetActualSpriteColor     lda TabSpriteColor              ; 
                          and #$0f                        ; ....#### - limit sprite colors
.SetActualSpriteColor     sta TabSpriteColor              ; 
                          
                          tax                             ; 
                          
.SetColor_S               stx SP0COL                      ; VIC($D027) Color Sprite 0
                          inx                             ; 
.SetColor_N               stx SP1COL                      ; VIC($D028) Color Sprite 1
                          inx                             ; 
.SetColor_O               stx SP2COL                      ; VIC($D029) Color Sprite 2
                          inx                             ; 
.SetColor_K               stx SP3COL                      ; VIC($D02A) Color Sprite 3
                          inx                             ; 
.SetColor_I               stx SP4COL                      ; VIC($D02B) Color Sprite 4
                          inx                             ; 
.SetColor_E               stx SP5COL                      ; VIC($D02C) Color Sprite 5
                          
GameHiScoreFlashSpriteX   rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameHiScoreFlashLeader  Does    : changes color of top scorer row in high scrore list
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameHiScoreFlashLeader    subroutine                      ; 
.GetColorFromFirstSprite  lda SP0COL                      ; VIC($D027) Color Sprite 0
.IniTopScorerLen          ldx #[SN_Scrn_RowLen * $02]     ; 
.SetNextColorRamByte      sta [COLORAM + [SN_Scrn_RowLen * $04]],x ; 
.DecTopScorerLen          dex                             ; 
.ChkTopScorerLen_Min      bne .SetNextColorRamByte        ; 
                          
GameHiScoreFlashLeaderX   rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabSpriteColor            dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameHiScoreOrbMoveOne   Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameHiScoreOrbMoveOne     subroutine                      ; 
.GetMovDir                lda TabHiScoreOrbMovDirOne      ; 
.LimMovDir                and #TabHiScoreOrbMovDir_Max    ; limit to max
.SetMovDir                sta TabHiScoreOrbMovDirOne      ; 
                          
                          cmp #TabHiScoreOrbMovDir_Do     ; 
                          beq .IncMovDir_Do               ; 
                          
                          cmp #TabHiScoreOrbMovDir_Le     ; 
                          beq .DecMovDir_Le               ; 
                          
                          cmp #TabHiScoreOrbMovDir_Up     ; 
                          beq .DecMovDir_Up               ; 
; -------------------------------------------------------------------------------------------------------------- ;
; move tiny orb one right
; -------------------------------------------------------------------------------------------------------------- ;
.IncMovDir_Ri             inc SP6X                        ; VIC($D00C) Sprite 6 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
.ChkMovDir_Ri_Max_Inc     bne .GetMovDir_Ri_MSB_Chk       ; 
                          
.GetMovDir_Ri_MSB         lda MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
.InsMovDir_Ri_MSB         ora #$40                        ; .#......
.SetMovDir_Ri_MSB         sta MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
                          
.GetMovDir_Ri_MSB_Chk     lda MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
.IsoMovDir_Ri_MSB_Chk     and #$40                        ; .#......
.ChkMovDir_Ri_MSB_Chk     beq .ExitMovOrbTiny_Ri          ; 
                          
.GetMovDir_Ri_Max         lda #$44                        ; 
.TstMovDir_Ri_Max         cmp SP6X                        ; VIC($D00C) Sprite 6 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
.ChkMovDir_Ri_Max         bne .ExitMovOrbTiny_Ri          ; 
                          
.SetMovDir_Do             inc TabHiScoreOrbMovDirOne      ; next move dir down
                          
.ExitMovOrbTiny_Ri        rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; move tiny orb one down
; -------------------------------------------------------------------------------------------------------------- ;
.IncMovDir_Do             inc SP6Y                        ; VIC($D00D) Sprite 6 Y-Pos (Bits 0-7)
                          
.GetMovDir_Do             lda SP6Y                        ; VIC($D00D) Sprite 6 Y-Pos (Bits 0-7)
.TstMovDir_Do             cmp #$f1                        ; 
.ChkMovDir_Do_Max         bne .ExitMovOrbTiny_Ri          ; 
                          
.SetMovDir_Le             inc TabHiScoreOrbMovDirOne      ; next move dir left
                          
.ExitMovOrbTiny_Do        rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; move tiny orb one left
; -------------------------------------------------------------------------------------------------------------- ;
.DecMovDir_Le             dec SP6X                        ; VIC($D00C) Sprite 6 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          
.GetMovDir_Le             lda SP6X                        ; VIC($D00C) Sprite 6 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
.ChkMovDir_Le             cmp #$ff                        ; 
.ChkMovDir_Le_Min_Dec     bne .GetMovDir_Le_MSB_Chk       ; 
                          
.GetMovDir_Le_MSB         lda MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
.ClrMovDir_Le_MSB         and #$bf                        ; #.######
.SetMovDir_Le_MSB         sta MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
                          
.GetMovDir_Le_MSB_Chk     lda MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
.IsoMovDir_Le_MSB_Chk     and #$40                        ; .#......
.ChkMovDir_Le_MSB_Chk     bne .ExitMovOrbTiny_Ri          ; 
                          
.GetMovDir_Le_Min         lda #$1e                        ; 
.TstMovDir_Le_Min         cmp SP6X                        ; VIC($D00C) Sprite 6 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
.ChkMovDir_Le_Min         bne .ExitMovOrbTiny_Ri          ; 
                          
.SetMovDir_Up             inc TabHiScoreOrbMovDirOne      ; next move dir up
                          
.ExitMovOrbTiny_Le        rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; move tiny orb one up
; -------------------------------------------------------------------------------------------------------------- ;
.DecMovDir_Up             dec SP6Y                        ; VIC($D00D) Sprite 6 Y-Pos (Bits 0-7)
                          
.GetMovDir_Up             lda SP6Y                        ; VIC($D00D) Sprite 6 Y-Pos (Bits 0-7)
.TstMovDir_Up             cmp #$33                        ; 
.ChkMovDir_Up_Min         bne .ExitMovOrbTiny_Ri          ; 
                          
.SetMovDir_Ri             inc TabHiScoreOrbMovDirOne      ; next move dir right
                          
GameHiScoreOrbMoveOneX    rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameHiScoreOrbMoveTwo   Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameHiScoreOrbMoveTwo     subroutine                      ; 
.GetMovDir                lda TabHiScoreOrbMovDirTwo      ; 
.LimMovDir                and #TabHiScoreOrbMovDir_Max    ; limit to max
.SetMovDir                sta TabHiScoreOrbMovDirTwo      ; 
                          
                          cmp #TabHiScoreOrbMovDir_Do     ; 
                          beq .IncMovDir_Do               ; 
                          
                          cmp #TabHiScoreOrbMovDir_Le     ; 
                          beq .DecMovDir_Le               ; 
                          
                          cmp #TabHiScoreOrbMovDir_Up     ; 
                          beq .DecMovDir_Up               ; 
; -------------------------------------------------------------------------------------------------------------- ;
; move tiny orb two right
; -------------------------------------------------------------------------------------------------------------- ;
.IncMovDir_Ri             inc SP7X                        ; VIC($D00E) Sprite 7 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
.ChkMovDir_Ri_Max_Inc     bne .GetMovDir_Ri_MSB_Chk       ; 
                          
.GetMovDir_Ri_MSB         lda MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
.InsMovDir_Ri_MSB         ora #$80                        ; 
.SetMovDir_Ri_MSB         sta MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
                          
.GetMovDir_Ri_MSB_Chk     lda MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
.IsoMovDir_Ri_MSB_Chk     and #$80                        ; #.......
.ChkMovDir_Ri_MSB_Chk     beq .ExitMovOrbTiny_Ri          ; 
                          
.GetMovDir_Ri_Max         lda #$44                        ; 
.TstMovDir_Ri_Max         cmp SP7X                        ; VIC($D00E) Sprite 7 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
.ChkMovDir_Ri_Max         bne .ExitMovOrbTiny_Ri          ; 
                          
.SetMovDir_Do             inc TabHiScoreOrbMovDirTwo      ; next move dir down
                          
.ExitMovOrbTiny_Ri        rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; move tiny orb two down
; -------------------------------------------------------------------------------------------------------------- ;
.IncMovDir_Do             inc SP7Y                        ; VIC($D00F) Sprite 7 Y-Pos (Bits 0-7)
                          
.GetMovDir_Do             lda SP7Y                        ; VIC($D00F) Sprite 7 Y-Pos (Bits 0-7)
.TstMovDir_Do             cmp #$f1                        ; 
.ChkMovDir_Do_Max         bne .ExitMovOrbTiny_Ri          ; 
                          
.SetMovDir_Le             inc TabHiScoreOrbMovDirTwo      ; next move dir left
                          
.ExitMovOrbTiny_Do        rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; move tiny orb two left
; -------------------------------------------------------------------------------------------------------------- ;
.DecMovDir_Le             dec SP7X                        ; VIC($D00E) Sprite 7 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          
.GetMovDir_Le             lda SP7X                        ; VIC($D00E) Sprite 7 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
.ChkMovDir_Le             cmp #$ff                        ; 
.ChkMovDir_Le_Min_Dec     bne .GetMovDir_Le_MSB_Chk       ; 
                          
.GetMovDir_Le_MSB         lda MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
.ClrMovDir_Le_MSB         and #$7f                        ; .#######
.SetMovDir_Le_MSB         sta MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
                          
.GetMovDir_Le_MSB_Chk     lda MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
.IsoMovDir_Le_MSB_Chk     and #$80                        ; #.......
.ChkMovDir_Le_MSB_Chk     bne .ExitMovOrbTiny_Ri          ; 
                          
.GetMovDir_Le_Min         lda #$1e                        ; 
.TstMovDir_Le_Min         cmp SP7X                        ; VIC($D00E) Sprite 7 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
.ChkMovDir_Le_Min         bne .ExitMovOrbTiny_Ri          ; 
                          
.SetMovDir_Up             inc TabHiScoreOrbMovDirTwo      ; next move dir up
                          
.ExitMovOrbTiny_Le        rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; move tiny orb one down
; -------------------------------------------------------------------------------------------------------------- ;
.DecMovDir_Up             dec SP7Y                        ; VIC($D00F) Sprite 7 Y-Pos (Bits 0-7)
                          
.GetMovDir_Up             lda SP7Y                        ; VIC($D00F) Sprite 7 Y-Pos (Bits 0-7)
.TstMovDir_Up             cmp #$33                        ; 
.ChkMovDir_Up_Min         bne .ExitMovOrbTiny_Ri          ; 
                          
.SetMovDir_Ri             inc TabHiScoreOrbMovDirTwo      ; next move dir right
                          
GameHiScoreOrbMoveTwoX    rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameStartGetKeyHandler  Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameStartGetKeyHandler    subroutine                      ; 
                          jsr KeyboardGetKey              ; 
                          
                          lda TabKeyboardKeyGot           ; 
                          cmp #CIA_Key_None               ; 
                          beq .DecDemoWaitCount           ; 
; -------------------------------------------------------------------------------------------------------------- ;
; get digit 1-7 input for a possible level selection
; -------------------------------------------------------------------------------------------------------------- ;
                          and #TabKeyGotDecode_Len        ; limit to table length
                          tax                             ; 
                          lda TabKeyGotDecode,x           ; 
                          sec                             ; 
.SetLevelNum              sbc #$31                        ; gives a possible level number of 0-6
                          tax                             ; 
.TstLevelNum              cpx #$07                        ; 
.ChkLevelNum              bcc .ChkLevelSelect             ; lower - number key 1-7 pressed
; -------------------------------------------------------------------------------------------------------------- ;
                          jsr InitColorRamToBlack         ; 
                          
.GetScrn                  lda VMCSB                       ; VIC($D018) VIC Chip Memory Control
                          and #[$ff ^ VIC_VideoMask]      ; ####....
.TstScrnInfoTxt           cmp #VIC_Video_0400             ; ...#....
.ChkScrnInfoTxt           bne .GetScrnInfoTxt             ; 
                          
.GetScrnHighScore         lda VMCSB                       ; VIC($D018) VIC Chip Memory Control
                          and #VIC_VideoMask              ; ....####
.SetScrnHighScore         ora #VIC_Video_0800             ; ..#.....
                          bne .SetScrn                    ; always
                          
.GetScrnInfoTxt           lda VMCSB                       ; VIC($D018) VIC Chip Memory Control
                          and #VIC_VideoMask              ; ....####
.SetScrnInfoTxt           ora #VIC_Video_0400             ; ...#....
                          
.SetScrn                  sta VMCSB                       ; VIC($D018) VIC Chip Memory Control
                          
                          jsr InitSwapCharSetsHandler     ; 
                          jsr GameHiScoreSpritesInit      ; 
                          jsr InitColorRamToWhite         ; 
                          
                          lda #$00                        ; 
                          sta TabWaitCount_Lo             ; 
                          
                          lda #$40                        ; 
                          sta TabWaitCount_Hi             ; 
                          
                          lda #CIA_Key_None               ; 
                          sta TabKeyboardKeyGot           ; 
                          
                          rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.DecDemoWaitCount         lda TabWaitCount_Lo             ; 
.ChkDemoWaitCount_Lo_Min  bne .DecDemoWaitCount_Lo        ; 
                          
.DecDemoWaitCount_Hi      dec TabWaitCount_Hi             ; 
.ChkDemoWaitCount_Hi_Min  bmi .ChkDemoStart               ; start demo or leader board display
                          
.DecDemoWaitCount_Lo      dec TabWaitCount_Lo             ; 
                          
.Exit_NoKEy               rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; show high score table 1st / start demo from there
; -------------------------------------------------------------------------------------------------------------- ;
.ChkDemoStart             lda VMCSB                       ; VIC($D018) VIC Chip Memory Control
                          and #[$ff ^ VIC_VideoMask]      ; ####....
.TstShowHighScores        cmp #VIC_Video_0400             ; ...#....
.ChkShowHighScores        beq .GetScrnHighScore           ; 
                          
.GetFlagDemo              lda #TabFlagDemo_On             ; 
.IniStartDemo             sta TabFlagDemo                 ; 
                          bne .GetNumLives_Select         ; always
; -------------------------------------------------------------------------------------------------------------- ;
.ChkLevelSelect           lda #CIA_Key_None               ; 
                          sta TabKeyboardKeyGot           ; 
                          
                          cpx TabLevelSelect              ; 
                          bcs .DecDemoWaitCount           ; greater/equal
                          
                          stx TabSelectNumLevel           ; 
                          
.GetNumLives_Select       lda #SN_SnokieNumLivesCheat     ; 
.SetNumLives_Select       sta TabSelectNumLives           ; only one live after a main menu level select
                          
                          pla                             ; remove return address
                          pla                             ; 
                          
GameStartGetKeyHandlerX   rts                             ; return to MainInit.Start
; -------------------------------------------------------------------------------------------------------------- ;
TabHiScoreOrbMovDirOne    dc.b $00 ; 
TabHiScoreOrbMovDir_Ri       = $00 ; 
TabHiScoreOrbMovDir_Do       = $01 ; 
TabHiScoreOrbMovDir_Le       = $02 ; 
TabHiScoreOrbMovDir_Up       = $03 ; 
TabHiScoreOrbMovDir_Max      = $03 ; 
TabHiScoreOrbMovDirTwo    dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameHiScoreOrbSprtInit  Does   : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameHiScoreOrbSprtInit    subroutine                      ; 
.IniSprtSmallOrbsPosXY    lda #$19                        ; 
                          sta SP6X                        ; VIC($D00C) Sprite 6 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          lda #$33                        ; 
                          sta SP6Y                        ; VIC($D00D) Sprite 6 Y-Pos (Bits 0-7)
                          lda #$44                        ; 
                          sta SP7X                        ; VIC($D00E) Sprite 7 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          lda #$f1                        ; 
                          sta SP7Y                        ; VIC($D00F) Sprite 7 Y-Pos (Bits 0-7)
                          
                          lda MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
                          ora #$80                        ; #.......
                          sta MSIGX                       ; VIC($D010) MSB X-Pos Sprites 0-7
                          
.IniSprtSmallOrbsColor    lda #YELLOW                     ; 
                          sta SP6COL                      ; VIC($D02D) Color Sprite 6
                          sta SP7COL                      ; VIC($D02E) Color Sprite 7
                          
.IniSprtSmallOrbsMoveDir  lda #TabHiScoreOrbMovDir_Ri     ; 
                          sta TabHiScoreOrbMovDirOne      ; 
                          
                          lda #TabHiScoreOrbMovDir_Le     ; 
                          sta TabHiScoreOrbMovDirTwo      ; 
                          
.IniSprtSmallOrbsEnab     lda SPENA                       ; VIC($D015) Sprite Enable Register
                          ora #$c0                        ; ##......
                          sta SPENA                       ; VIC($D015) Sprite Enable Register
                          
GameHiScoreOrbSprtInitX   rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameTuneFinishHandler   Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameTuneFinishHandler     subroutine                      ; 
                          sei                             ; 
                          
                          jsr GameTuneFinishIniOsc_01     ; 
                          jsr GameTuneFinishIniOsc_02     ; 
                          
.GetFlag_EndOfTune_No     lda #TabIRQ_TuneEndFlag_No      ; 
.IniFlag_EndOfTune        sta TabIRQ_TuneEndFlag          ; 
                          
.GoPlayNextNote           jsr IRQ_PlayTune_Osc_01         ; 
                          jsr IRQ_PlayTune_Osc_02         ; 
                          
.IniWaitTime              ldx #$0c                        ; 
.DecNextWaitTime          dex                             ; 
.ChkWaitTime_Min          bne .DecNextWaitTime            ; 
                          
.GetFlag_EndOfTune        lda TabIRQ_TuneEndFlag          ; 
.ChkFlag_EndOfTune        beq .GoPlayNextNote             ; 
                          
                          cli                             ; 
                          
GameTuneFinishHandlerX    rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameTuneFinishIniOsc_01 Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameTuneFinishIniOsc_01   subroutine                      ; 
                          lda #$f0                        ; 
                          sta SUREL1                      ; SID($D406) Oscillator 1 Sustain/Release
                          
                          lda #$00                        ; 
                          sta ATDCY1                      ; SID($D405) Oscillator 1 Attack/Decay2
                          
                          lda #$0f                        ; 
                          sta SIGVOL                      ; SID($D418) Volume/Filter Select
                          
                          lda #<Tab_TuneFinalFreq_Osc_1   ; 
                          sta SNZ_Osc_1_TuneFreqTabPtr_Lo ; 
                          lda #>Tab_TuneFinalFreq_Osc_1   ; 
                          sta SNZ_Osc_1_TuneFreqTabPtr_Hi ; 
                          
                          lda #<Tab_TuneFinalTime         ; 
                          sta SNZ_Osc_1_TuneTimeTabPtr_Lo ; 
                          lda #>Tab_TuneFinalTime         ; 
                          sta SNZ_Osc_1_TuneTimeTabPtr_Hi ; 
                          
                          ldy #$00                        ; 
                          lda (SNZ_Osc_1_TuneTimeTabPtr),y; 
                          sta TabTune_Osc_1_Time_Lo       ; 
                          
                          lda (SNZ_Osc_1_TuneFreqTabPtr),y; 
                          sta FRELO1                      ; SID($D400) Oscillator 1 Freq Ctrl (low byte)
                          
                          iny                             ; 
                          
                          lda (SNZ_Osc_1_TuneTimeTabPtr),y; 
                          sta TabTune_Osc_1_Time_Hi       ; 
                          
                          lda (SNZ_Osc_1_TuneFreqTabPtr),y; 
                          sta FREHI1                      ; SID($D401) Oscillator 1 Freq Ctrl (high byte)
                          
                          lda #VcRegTRI                   ; ...#....
                          sta VCREG1                      ; SID($D404) Oscillator 1 Control
                          
                          lda #[VcRegTRI | VcRegGATE]     ; ...#...#
                          sta TabTune_Osc_1_Ctrl          ; 
                          sta VCREG1                      ; SID($D404) Oscillator 1 Control
                          
                          lda #VcRegTRI                   ; ...#....
                          sta VCREG1                      ; SID($D404) Oscillator 1 Control
                          
                          lda #$00                        ; 
                          sta TabTunePlayTime_Lo          ; 
                          
                          lda #$59                        ; 
                          sta TabTunePlayTime_Hi          ; 
                          
GameTuneFinishIniOsc_01X  rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameTuneFinishIniOsc_02 Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameTuneFinishIniOsc_02   subroutine                      ; 
                          lda #$f0                        ; 
                          sta SUREL2                      ; SID($D40D) Oscillator 2 Sustain/Release
                          
                          lda #$00                        ; 
                          sta ATDCY2                      ; SID($D40C) Oscillator 2 Attack/Decay
                          
                          lda #<Tab_TuneFinalFreq_Osc_2   ; 
                          sta SNZ_Osc_2_TuneFreqTabPtr_Lo ; 
                          lda #>Tab_TuneFinalFreq_Osc_2   ; 
                          sta SNZ_Osc_2_TuneFreqTabPtr_Hi ; 
                          
                          lda #<Tab_TuneFinalTime         ; 
                          sta SNZ_Osc_2_TuneTimeTabPtr_Lo ; 
                          lda #>Tab_TuneFinalTime         ; 
                          sta SNZ_Osc_2_TuneTimeTabPtr_Hi ; 
                          
                          ldy #$00                        ; 
                          lda (SNZ_Osc_2_TuneTimeTabPtr),y; 
                          sta TabTune_Osc_2_Time_Lo       ; 
                          
                          lda (SNZ_Osc_2_TuneFreqTabPtr),y; 
                          sta FRELO2                      ; SID($D407) Oscillator 2 Freq Ctrl (low byte)
                          
                          iny                             ; 
                          
                          lda (SNZ_Osc_2_TuneTimeTabPtr),y; 
                          sta TabTune_Osc_2_Time_Hi       ; 
                          
                          lda (SNZ_Osc_2_TuneFreqTabPtr),y; 
                          sta FREHI2                      ; SID($D408) Oscillator 2 Freq Ctrl (high byte)
                          
                          lda #VcRegTRI                   ; ...#....
                          sta VCREG2                      ; SID($D40B) Oscillator 2 Control
                          
                          lda #[VcRegTRI | VcRegGATE]     ; ...#...#
                          sta TabTune_Osc_2_Ctrl          ; 
                          sta VCREG2                      ; SID($D40B) Oscillator 2 Control
                          
                          lda #VcRegTRI                   ; ...#....
                          sta VCREG2                      ; SID($D40B) Oscillator 2 Control
                          
GameTuneFinishIniOsc_02X  rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameTuneDeath           Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameTuneDeath             subroutine                      ; 
                          sei                             ; 
                          
                          jsr GameTuneDeathIniOsc_01      ; 
                          jsr GameTuneDeathIniOsc_02      ; 
                          jsr GameTuneDeathIniOsc_03      ; 
                          
.GetFlag_EndOfTune_No     lda #TabIRQ_TuneEndFlag_No      ; 
.IniFlag_EndOfTune        sta TabIRQ_TuneEndFlag          ; 
                          
.GoPlayNextNote           jsr IRQ_PlayTune_Osc_01         ; 
                          jsr IRQ_PlayTune_Osc_02         ; 
                          jsr IRQ_PlayTune_Osc_03         ; 
                          
.GetFlag_EndOfTune        lda TabIRQ_TuneEndFlag          ; 
.ChkFlag_EndOfTune        beq .GoPlayNextNote             ; 
                          
                          cli                             ; 
                          
.OffAllOsc                jsr InitLockOscAll              ; Lock oscillators 1-3
                          
GameTuneDeathX            rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; IRQ_PlayTune_Osc_01     Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
IRQ_PlayTune_Osc_01       subroutine                      ; 
                          lda TabTune_Osc_1_Ctrl          ; 
                          sta VCREG1                      ; SID($D404) Oscillator 1 Control
                          
.GetTuneTime_Hi           lda TabTune_Osc_1_Time_Hi       ; 
.ChkTuneTime_Hi_Min       beq .GetTuneTime_Lo             ; 
                          
                          lda TabTune_Osc_1_Time_Lo       ; 
                          jmp .DecTunePlayTime            ; 
                          
.GetTuneTime_Lo           lda TabTune_Osc_1_Time_Lo       ; 
.ChkTuneTime_Lo_Min       beq .SetTuneOff                 ; 
                          
.DecTunePlayTime          sec                             ; 
                          sbc #$01                        ; 
                          sta TabTune_Osc_1_Time_Lo       ; 
                          
                          lda TabTune_Osc_1_Time_Hi       ; 
                          sbc #$00                        ; 
                          sta TabTune_Osc_1_Time_Hi       ; 
                          
.Exit_DecTime             rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.SetTuneOff               lda #VcRegTEST                  ; ....#... - 1=Lock Oscillator
                          sta VCREG1                      ; SID($D404) Oscillator 1 Control
                          sta VCREG2                      ; SID($D40B) Oscillator 2 Control
                          sta VCREG3                      ; SID($D412) Oscillator 3 Control
                          
                          lda TabTunePlayTime_Hi          ; 
                          ora TabTunePlayTime_Lo          ; 
.ChkTunePlayTime          bne .DecTunePlayTimeNext        ; 
                          
.GetFlag_EndOfTune_Yes    lda #TabIRQ_TuneEndFlag_Yes     ; 
.SetFlag_EndOfTune        sta TabIRQ_TuneEndFlag          ; 
                          
.Exit_TuneOff             rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.DecTunePlayTimeNext      lda TabTunePlayTime_Hi          ; 
                          sec                             ; 
                          sbc #$01                        ; 
                          sta TabTunePlayTime_Hi          ; 
                          
                          lda TabTunePlayTime_Lo          ; 
                          sbc #$00                        ; 
                          sta TabTunePlayTime_Lo          ; 
                          
                          jsr Osc_1_IncTuneDataPointer    ; 
                          
                          ldy #$01                        ; 
                          lda (SNZ_Osc_1_TuneFreqTabPtr),y ; 
                          cmp #$01                        ; 
                          bne .SetTuneFreq                ; 
                          
                          lda #VcRegTEST                  ; ....#... - 1=Lock Oscillator
                          sta TabTune_Osc_1_Ctrl          ; 
                          
.GetNextTuneTime          lda (SNZ_Osc_1_TuneTimeTabPtr),y ; 
                          sta TabTune_Osc_1_Time_Hi       ; 
                          
                          dey                             ; 
                          
                          lda (SNZ_Osc_1_TuneTimeTabPtr),y ; 
                          sta TabTune_Osc_1_Time_Lo       ; 
                          
.Exit_NextTime            rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.SetTuneFreq              sta FREHI1                      ; SID($D401) Oscillator 1 Freq Ctrl (high byte)
                          
                          lda (SNZ_Osc_1_TuneTimeTabPtr),y ; 
                          sta TabTune_Osc_1_Time_Hi       ; 
                          
                          dey                             ; 
                          
                          lda (SNZ_Osc_1_TuneTimeTabPtr),y ; 
                          sta TabTune_Osc_1_Time_Lo       ; 
                          
                          lda (SNZ_Osc_1_TuneFreqTabPtr),y ; 
                          sta FRELO1                      ; SID($D400) Oscillator 1 Freq Ctrl (low byte)
                          
                          lda #VcRegTRI                   ; ...#....
                          sta VCREG1                      ; SID($D404) Oscillator 1 Control
                          
                          lda #[VcRegTRI | VcRegGATE]     ; ...#...#
                          sta TabTune_Osc_1_Ctrl          ; 
                          sta VCREG1                      ; SID($D404) Oscillator 1 Control
                          
                          lda #VcRegTRI                   ; ...#....
                          sta VCREG1                      ; SID($D404) Oscillator 1 Control
                          
IRQ_PlayTune_Osc_01X      rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabTune_Osc_1_Time_Lo     dc.b $00 ; 
TabTune_Osc_1_Time_Hi     dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameTuneDeathIniOsc_01  Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameTuneDeathIniOsc_01    subroutine                      ; 
                          lda #$f0                        ; 
                          sta SUREL1                      ; SID($D406) Oscillator 1 Sustain/Release
                          
                          lda #$00                        ; 
                          sta ATDCY1                      ; SID($D405) Oscillator 1 Attack/Decay2
                          
                          lda #$0f                        ; 
                          sta SIGVOL                      ; SID($D418) Volume/Filter Select
                          
                          lda #<Tab_TuneDeathFreq_Osc_1   ; 
                          sta SNZ_Osc_1_TuneFreqTabPtr_Lo ; 
                          lda #>Tab_TuneDeathFreq_Osc_1   ; 
                          sta SNZ_Osc_1_TuneFreqTabPtr_Hi ; 
                          
                          lda #<Tab_TuneDeathTime_Osc_1   ; 
                          sta SNZ_Osc_1_TuneTimeTabPtr_Lo ; 
                          lda #>Tab_TuneDeathTime_Osc_1   ; 
                          sta SNZ_Osc_1_TuneTimeTabPtr_Hi ; 
                          
                          ldy #$00                        ; 
                          lda (SNZ_Osc_1_TuneTimeTabPtr),y ; 
                          sta TabTune_Osc_1_Time_Lo       ; 
                          
                          lda (SNZ_Osc_1_TuneFreqTabPtr),y ; 
                          sta FRELO1                      ; SID($D400) Oscillator 1 Freq Ctrl (low byte)
                          
                          iny                             ; 
                          
                          lda (SNZ_Osc_1_TuneTimeTabPtr),y ; 
                          sta TabTune_Osc_1_Time_Hi       ; 
                          
                          lda (SNZ_Osc_1_TuneFreqTabPtr),y ; 
                          sta FREHI1                      ; SID($D401) Oscillator 1 Freq Ctrl (high byte)
                          
                          lda #VcRegTRI                   ; ...#....
                          sta VCREG1                      ; SID($D404) Oscillator 1 Control
                          
                          lda #[VcRegTRI | VcRegGATE]     ; ...#...#
                          sta TabTune_Osc_1_Ctrl          ; 
                          sta VCREG1                      ; SID($D404) Oscillator 1 Control
                          
                          lda #VcRegTRI                   ; ...#....
                          sta VCREG1                      ; SID($D404) Oscillator 1 Control
                          
                          lda #$00                        ; 
                          sta TabTunePlayTime_Lo          ; 
                          
                          lda #$11                        ; 
                          sta TabTunePlayTime_Hi          ; 
                          
GameTuneDeathIniOsc_01X   rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabTunePlayTime_Hi        dc.b $00 ; 
TabTunePlayTime_Lo        dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
; Osc_1_IncTuneDataPointer Does   : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
Osc_1_IncTuneDataPointer  subroutine                      ; 
.IntFreqDataPtr           lda SNZ_Osc_1_TuneFreqTabPtr_Lo ; 
                          clc                             ; 
                          adc #$02                        ; 
                          sta SNZ_Osc_1_TuneFreqTabPtr_Lo ; 
                          
                          lda SNZ_Osc_1_TuneFreqTabPtr_Hi ; 
                          adc #$00                        ; 
                          sta SNZ_Osc_1_TuneFreqTabPtr_Hi ; 
                          
.InTimeDataPtr            lda SNZ_Osc_1_TuneTimeTabPtr_Lo ; 
                          clc                             ; 
                          adc #$02                        ; 
                          sta SNZ_Osc_1_TuneTimeTabPtr_Lo ; 
                          
                          lda SNZ_Osc_1_TuneTimeTabPtr_Hi ; 
                          adc #$00                        ; 
                          sta SNZ_Osc_1_TuneTimeTabPtr_Hi ; 
                          
Osc_1_IncTuneDataPointerX rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabTune_Osc_1_Ctrl        dc.b $00 ; 
TabTune_Osc_2_Ctrl        dc.b $00 ; 
TabTune_Osc_3_Ctrl        dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
; IRQ_PlayTune_Osc_02     Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
IRQ_PlayTune_Osc_02       subroutine                      ; 
                          lda TabTune_Osc_2_Ctrl          ; 
                          sta VCREG2                      ; SID($D40B) Oscillator 2 Control
                          
.GetTuneTime_Hi           lda TabTune_Osc_2_Time_Hi       ; 
.ChkTuneTime_Hi_Min       beq .GetTuneTime_Lo             ; 
                          
                          lda TabTune_Osc_2_Time_Lo       ; 
                          jmp .DecTunePlayTime            ; 
                          
.GetTuneTime_Lo           lda TabTune_Osc_2_Time_Lo       ; 
                          beq .SetTuneOff                 ; 
                          
.DecTunePlayTime          sec                             ; 
                          sbc #$01                        ; 
                          sta TabTune_Osc_2_Time_Lo       ; 
                          
                          lda TabTune_Osc_2_Time_Hi       ; 
                          sbc #$00                        ; 
                          sta TabTune_Osc_2_Time_Hi       ; 
                          
.Exit_DecTime             rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.SetTuneOff               lda #VcRegTEST                  ; ....#... - 1=Lock Oscillator
                          sta VCREG2                      ; SID($D40B) Oscillator 2 Control
                          jsr Osc_2_IncTuneDataPointer    ; 
                          
                          ldy #$01                        ; 
                          lda (SNZ_Osc_2_TuneFreqTabPtr),y ; 
                          cmp #$01                        ; 
                          bne .SetTuneFreq                ; 
                          
                          lda #VcRegTEST                  ; ....#... - 1=Lock Oscillator
                          sta TabTune_Osc_2_Ctrl          ; 
                          
                          lda (SNZ_Osc_2_TuneTimeTabPtr),y ; 
                          sta TabTune_Osc_2_Time_Hi       ; 
                          
                          dey                             ; 
                          
                          lda (SNZ_Osc_2_TuneTimeTabPtr),y ; 
                          sta TabTune_Osc_2_Time_Lo       ; 
                          
.Exit_TuneOff             rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.SetTuneFreq              sta FREHI2                      ; SID($D408) Oscillator 2 Freq Ctrl (high byte)
                          
                          lda (SNZ_Osc_2_TuneTimeTabPtr),y ; 
                          sta TabTune_Osc_2_Time_Hi       ; 
                          
                          dey                             ; 
                          
                          lda (SNZ_Osc_2_TuneTimeTabPtr),y ; 
                          sta TabTune_Osc_2_Time_Lo       ; 
                          
                          lda (SNZ_Osc_2_TuneFreqTabPtr),y ; 
                          sta FRELO2                      ; SID($D407) Oscillator 2 Freq Ctrl (low byte)
                          
                          lda #VcRegTRI                   ; ...#....
                          sta VCREG2                      ; SID($D40B) Oscillator 2 Control
                          
                          lda #[VcRegTRI | VcRegGATE]     ; ...#...#
                          sta TabTune_Osc_2_Ctrl          ; 
                          sta VCREG2                      ; SID($D40B) Oscillator 2 Control
                          
                          lda #VcRegTRI                   ; ...#....
                          sta VCREG2                      ; SID($D40B) Oscillator 2 Control
                          
IRQ_PlayTune_Osc_02X      rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabTune_Osc_2_Time_Lo     dc.b $00 ; 
TabTune_Osc_2_Time_Hi     dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameTuneDeathIniOsc_02  Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameTuneDeathIniOsc_02    subroutine                      ; 
                          lda #$f0                        ; 
                          sta SUREL2                      ; SID($D40D) Oscillator 2 Sustain/Release
                           
                          lda #$00                        ; 
                          sta ATDCY2                      ; SID($D40C) Oscillator 2 Attack/Decay
                          
                          lda #<Tab_TuneDeathFreq_Osc_2   ; 
                          sta SNZ_Osc_2_TuneFreqTabPtr_Lo ; 
                          lda #>Tab_TuneDeathFreq_Osc_2   ; 
                          sta SNZ_Osc_2_TuneFreqTabPtr_Hi ; 
                          
                          lda #<Tab_TuneDeathTime_Osc_2   ; 
                          sta SNZ_Osc_2_TuneTimeTabPtr_Lo ; 
                          lda #>Tab_TuneDeathTime_Osc_2   ; 
                          sta SNZ_Osc_2_TuneTimeTabPtr_Hi ; 
                          
                          ldy #$00                        ; 
                          lda (SNZ_Osc_2_TuneTimeTabPtr),y ; 
                          sta TabTune_Osc_2_Time_Lo       ; 
                          
                          lda (SNZ_Osc_2_TuneFreqTabPtr),y ; 
                          sta FRELO2                      ; SID($D407) Oscillator 2 Freq Ctrl (low byte)
                          
                          iny                             ; 
                          
                          lda (SNZ_Osc_2_TuneTimeTabPtr),y ; 
                          sta TabTune_Osc_2_Time_Hi       ; 
                          
                          lda (SNZ_Osc_2_TuneFreqTabPtr),y ; 
                          sta FREHI2                      ; SID($D408) Oscillator 2 Freq Ctrl (high byte)
                          
                          lda #VcRegTRI                   ; ...#....
                          sta VCREG2                      ; SID($D40B) Oscillator 2 Control
                          
                          lda #[VcRegTRI | VcRegGATE]     ; ...#...#
                          sta TabTune_Osc_2_Ctrl          ; 
                          sta VCREG2                      ; SID($D40B) Oscillator 2 Control
                          
                          lda #VcRegTRI                   ; ...#....
                          sta VCREG2                      ; SID($D40B) Oscillator 2 Control
                          
GameTuneDeathIniOsc_02X   rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; Osc_2_IncTuneDataPointer Does   : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
Osc_2_IncTuneDataPointer  subroutine                      ; 
                          lda SNZ_Osc_2_TuneFreqTabPtr_Lo ; 
                          clc                             ; 
                          adc #$02                        ; 
                          sta SNZ_Osc_2_TuneFreqTabPtr_Lo ; 
                          
                          lda SNZ_Osc_2_TuneFreqTabPtr_Hi ; 
                          adc #$00                        ; 
                          sta SNZ_Osc_2_TuneFreqTabPtr_Hi ; 
                          
                          lda SNZ_Osc_2_TuneTimeTabPtr_Lo ; 
                          clc                             ; 
                          adc #$02                        ; 
                          sta SNZ_Osc_2_TuneTimeTabPtr_Lo ; 
                          
                          lda SNZ_Osc_2_TuneTimeTabPtr_Hi ; 
                          adc #$00                        ; 
                          sta SNZ_Osc_2_TuneTimeTabPtr_Hi ; 
                          
Osc_2_IncTuneDataPointerX rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; IRQ_PlayTune_Osc_03     Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
IRQ_PlayTune_Osc_03       subroutine                      ; 
                          lda TabTune_Osc_3_Ctrl          ; 
                          sta VCREG3                      ; SID($D412) Oscillator 3 Control
                          
.GetTuneTime_Hi           lda TabTune_Osc_3_Time_Hi       ; 
.ChkTuneTime_Hi_Min       beq .GetTuneTime_Lo             ; 
                          
                          lda TabTune_Osc_3_Time_Lo       ; 
                          jmp .DecTunePlayTime            ; 
                          
.GetTuneTime_Lo           lda TabTune_Osc_3_Time_Lo       ; 
.ChkTuneTime_Lo_Min       beq .SetTuneOff                 ; 
                          
.DecTunePlayTime          sec                             ; 
                          sbc #$01                        ; 
                          sta TabTune_Osc_3_Time_Lo       ; 
                          
                          lda TabTune_Osc_3_Time_Hi       ; 
                          sbc #$00                        ; 
                          sta TabTune_Osc_3_Time_Hi       ; 
                          
.Exit_DecTime             rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.SetTuneOff               lda #VcRegTEST                  ; ....#... - 1=Lock Oscillator
                          sta VCREG3                      ; SID($D412) Oscillator 3 Control
                          jsr Osc_3_IncTuneDataPointer    ; 
                          
                          ldy #$01                        ; 
                          lda (SNZ_Osc_3_TuneFreqTabPtr),y ; 
                          cmp #$01                        ; 
                          bne .SetTuneFreq                ; 
                          
                          lda #VcRegTEST                  ; ....#... - 1=Lock Oscillator
                          sta TabTune_Osc_3_Ctrl          ; 
                          
.GetNextTuneTime          lda (SNZ_Osc_3_TuneTimeTabPtr),y ; 
                          sta TabTune_Osc_3_Time_Hi       ; 
                          
                          dey                             ; 
                          lda (SNZ_Osc_3_TuneTimeTabPtr),y ; 
                          sta TabTune_Osc_3_Time_Lo       ; 
                          
.Exit_NextTime            rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.SetTuneFreq              sta FREHI3                      ; SID($D40F) Oscillator 3 Freq Ctrl (high byte)
                          
                          lda (SNZ_Osc_3_TuneTimeTabPtr),y ; 
                          sta TabTune_Osc_3_Time_Hi       ; 
                          
                          dey                             ; 
                          
                          lda (SNZ_Osc_3_TuneTimeTabPtr),y ; 
                          sta TabTune_Osc_3_Time_Lo       ; 
                          
                          lda (SNZ_Osc_3_TuneFreqTabPtr),y ; 
                          sta FRELO3                      ; SID($D40E) Oscillator 3 Freq Ctrl (low byte)
                          
                          lda #VcRegTRI                   ; ...#....
                          sta VCREG3                      ; SID($D412) Oscillator 3 Control
                          
                          lda #[VcRegTRI | VcRegGATE]     ; ...#...#
                          sta TabTune_Osc_3_Ctrl          ; 
                          sta VCREG3                      ; SID($D412) Oscillator 3 Control
                          
                          lda #VcRegTRI                   ; ...#....
                          sta FREHI1                      ; SID($D401) Oscillator 1 Freq Ctrl (high byte)
                          
IRQ_PlayTune_Osc_03X      rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabTune_Osc_3_Time_Lo     dc.b $00 ; 
TabTune_Osc_3_Time_Hi     dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameTuneDeathIniOsc_03  Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameTuneDeathIniOsc_03    subroutine                      ; 
                          lda #$f0                        ; 
                          sta SUREL3                      ; SID($D414) Oscillator 3 Sustain/Release
                          
                          lda #$00                        ; 
                          sta ATDCY3                      ; SID($D413) Oscillator 3 Attack/Decay
                          
                          lda #<Tab_TuneDeathFreq_Osc_3   ; 
                          sta SNZ_Osc_3_TuneFreqTabPtr_Lo ; 
                          lda #>Tab_TuneDeathFreq_Osc_3   ; 
                          sta SNZ_Osc_3_TuneFreqTabPtr_Hi ; 
                          
                          lda #<Tab_TuneDeathTime_Osc_3   ; 
                          sta SNZ_Osc_3_TuneTimeTabPtr_Lo ; 
                          lda #>Tab_TuneDeathTime_Osc_3   ; 
                          sta SNZ_Osc_3_TuneTimeTabPtr_Hi ; 
                          
                          ldy #$00                        ; 
                          lda (SNZ_Osc_3_TuneTimeTabPtr),y ; 
                          sta TabTune_Osc_3_Time_Lo       ; 
                          
                          lda (SNZ_Osc_3_TuneFreqTabPtr),y ; 
                          sta FRELO3                      ; SID($D40E) Oscillator 3 Freq Ctrl (low byte)
                          
                          iny                             ; 
                          
                          lda (SNZ_Osc_3_TuneTimeTabPtr),y ; 
                          sta TabTune_Osc_3_Time_Hi       ; 
                          
                          lda (SNZ_Osc_3_TuneFreqTabPtr),y ; 
                          sta FREHI3                      ; SID($D40F) Oscillator 3 Freq Ctrl (high byte)
                          
                          lda #VcRegTRI                   ; ...#....
                          sta VCREG3                      ; SID($D412) Oscillator 3 Control
                          
                          lda #[VcRegTRI | VcRegGATE]     ; ...#...#
                          sta TabTune_Osc_3_Ctrl          ; 
                          sta VCREG3                      ; SID($D412) Oscillator 3 Control
                          
                          lda #VcRegTRI                   ; ...#....
                          sta VCREG3                      ; SID($D412) Oscillator 3 Control
                          
GameTuneDeathIniOsc_03x   rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; Osc_3_IncTuneDataPointer Does   : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
Osc_3_IncTuneDataPointer  subroutine                      ; 
                          lda SNZ_Osc_3_TuneFreqTabPtr_Lo ; 
                          clc                             ; 
                          adc #$02                        ; 
                          sta SNZ_Osc_3_TuneFreqTabPtr_Lo ; 
                          
                          lda SNZ_Osc_3_TuneFreqTabPtr_Hi ; 
                          adc #$00                        ; 
                          sta SNZ_Osc_3_TuneFreqTabPtr_Hi ; 
                          
                          lda SNZ_Osc_3_TuneTimeTabPtr_Lo ; 
                          clc                             ; 
                          adc #$02                        ; 
                          sta SNZ_Osc_3_TuneTimeTabPtr_Lo ; 
                          
                          lda SNZ_Osc_3_TuneTimeTabPtr_Hi ; 
                          adc #$00                        ; 
                          sta SNZ_Osc_3_TuneTimeTabPtr_Hi ; 
                          
Osc_3_IncTuneDataPointerX rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
TuneDeath                 include inc\SN_TuneDeath.asm    ; Death Blues
; -------------------------------------------------------------------------------------------------------------- ;
; InitGameStartTuneSprite Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitGameStartTuneSprite   subroutine                      ; 
                          jsr InitMemory                  ; 
                          jsr InitIniSpriteSunsoft        ; 
                          
                          sei                             ; 
                          
.GetSIDRegLen             ldy #SID_Len                    ; 
.GetSIDRegIni             lda #$00                        ; 
.IniNextSIDReg            sta [SID - $01],y               ; 
.DecSIDRegLen             dey                             ; 
.ChkSIDRegLen_Min         bne .IniNextSIDReg              ; 
                          
.IniTuneStartHandler      lda #<IRQ_TuneStart             ; 
                          sta CINV_LO                     ; Vector($0314/$0315): Hardware IRQ Interrupt address at ($EA31)
                          lda #>IRQ_TuneStart             ; 
                          sta CINV_HI                     ; Vector($0314/$0315): Hardware IRQ Interrupt address at ($EA31)
                          
                          lda #VIC_IrqSetRaster           ; 
                          sta IRQMASK                     ; VIC($D01A) IRQ Mask Register
                          
                          lda SCROLY                      ; VIC($D011) VIC Control Register 1 (Vert Fine Scroll)
                          and #VIC_RasterHi_Off           ; .#######
                          sta SCROLY                      ; VIC($D011) VIC Control Register 1 (Vert Fine Scroll)
                          
                          lda #$00                        ; 
                          sta RASTER                      ; VIC($D012) Read : Current Raster Scan Line (Bit 8 in SCROLY = $D011)
                          
                          lda #CIA_IntMaskAll             ; 
                          sta CIAICR                      ; CIA1($DC0D) Interrupt Control
                          
.IniTuneStart             jsr GameTuneStartIniOsc_01      ; 
                          jsr GameTuneStartIniOsc_02      ; 
                          
                          cli                             ; 
                          
.SetNextSpriteSunsoftPosY jsr InitMoveSpriteSunsoft       ; 
                          jsr InitBlinkTextPresents       ; 
                          
.IniWait_Hi               ldx #$14                        ; 
.IniNextWait_Lo           ldy #$ff                        ; 
.DecNextWait_Lo           dey                             ; 
                          bne .DecNextWait_Lo             ; 
                          dex                             ; 
                          bne .IniNextWait_Lo             ; 
                          
.DecSprtSunsoftPosY       dec TabSprtSunsoftPosY          ; 
.ChkSprtSunsoftPosY_Min   bne .SetNextSpriteSunsoftPosY   ; 
                          
                          sei                             ; 
InitGameStartTuneSpriteX  rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; IRQ_TuneStart           Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
IRQ_TuneStart             subroutine                      ; 
                          lda #$01                        ; 
                          sta VICIRQ                      ; VIC($D019) Interrupt Flags - Latched flags cleared if set to 1
                          
.GetFlag_EndOfTune_No     lda #TabIRQ_TuneEndFlag_No      ; 
.IniFlag_EndOfTune        sta TabIRQ_TuneEndFlag          ; 
                          
                          jsr IRQ_PlayTune_Osc_01         ; 
                          jsr IRQ_PlayTune_Osc_02         ; 
                          
.GetFlag_EndOfTune        lda TabIRQ_TuneEndFlag          ; 
.ChkFlag_EndOfTune        bne .SetInterruptOff            ; 
                          
                          jmp [_IRQ_MAIN + $4d]           ; read CIA1($DC0D) Interrupt Control and return from interrupt
                          
.SetInterruptOff          sei                             ; 
IRQ_TuneStartX            jmp [_IRQ_MAIN + $4d]           ; read CIA1($DC0D) Interrupt Control and return from interrupt
; -------------------------------------------------------------------------------------------------------------- ;
TabIRQ_TuneEndFlag        dc.b $00 ; 
TabIRQ_TuneEndFlag_No          = $00 ; 
TabIRQ_TuneEndFlag_Yes         = $05 ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameTuneStartIniOsc_01  Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameTuneStartIniOsc_01    subroutine                      ; 
                          lda #$f0                        ; 
                          sta SUREL1                      ; SID($D406) Oscillator 1 Sustain/Release
                          
                          lda #$00                        ; 
                          sta ATDCY1                      ; SID($D405) Oscillator 1 Attack/Decay2
                          
                          lda #$0f                        ; 
                          sta SIGVOL                      ; SID($D418) Volume/Filter Select
                          
                          lda #<Tab_TuneStartFreq_Osc_1   ; 
                          sta SNZ_Osc_1_TuneFreqTabPtr_Lo ; 
                          lda #>Tab_TuneStartFreq_Osc_1   ; 
                          sta SNZ_Osc_1_TuneFreqTabPtr_Hi ; 
                          
                          lda #<Tab_TuneStartTime         ; 
                          sta SNZ_Osc_1_TuneTimeTabPtr_Lo ; 
                          lda #>Tab_TuneStartTime         ; 
                          sta SNZ_Osc_1_TuneTimeTabPtr_Hi ; 
                          
                          ldy #$00                        ; 
                          lda (SNZ_Osc_1_TuneTimeTabPtr),y ; 
                          sta TabTune_Osc_1_Time_Lo       ; 
                          
                          lda (SNZ_Osc_1_TuneFreqTabPtr),y ; 
                          sta FRELO1                      ; SID($D400) Oscillator 1 Freq Ctrl (low byte)
                          
                          iny                             ; 
                          
                          lda (SNZ_Osc_1_TuneTimeTabPtr),y ; 
                          sta TabTune_Osc_1_Time_Hi       ; 
                          
                          lda (SNZ_Osc_1_TuneFreqTabPtr),y ; 
                          sta FREHI1                      ; SID($D401) Oscillator 1 Freq Ctrl (high byte)
                          
                          lda #VcRegTRI                   ; ...#....
                          sta VCREG1                      ; SID($D404) Oscillator 1 Control
                          
                          lda #[VcRegTRI | VcRegGATE]     ; ...#...#
                          sta TabTune_Osc_1_Ctrl          ; 
                          sta VCREG1                      ; SID($D404) Oscillator 1 Control
                          
                          lda #VcRegTRI                   ; ...#....
                          sta VCREG1                      ; SID($D404) Oscillator 1 Control
                          
                          lda #$00                        ; 
                          sta TabTunePlayTime_Lo          ; 
                          
                          lda #$16                        ; 
                          sta TabTunePlayTime_Hi          ; 
                          
GameTuneStartIniOsc_01X   rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; GameTuneStartIniOsc_02  Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameTuneStartIniOsc_02    subroutine                      ; 
                          lda #$f0                        ; 
                          sta SUREL2                      ; SID($D40D) Oscillator 2 Sustain/Release
                          
                          lda #$00                        ; 
                          sta ATDCY2                      ; SID($D40C) Oscillator 2 Attack/Decay
                          
                          lda #<Tab_TuneStartFreq_Osc_2   ; 
                          sta SNZ_Osc_2_TuneFreqTabPtr_Lo ; 
                          lda #>Tab_TuneStartFreq_Osc_2   ; 
                          sta SNZ_Osc_2_TuneFreqTabPtr_Hi ; 
                          
                          lda #<Tab_TuneStartTime         ; 
                          sta SNZ_Osc_2_TuneTimeTabPtr_Lo ; 
                          lda #>Tab_TuneStartTime         ; 
                          sta SNZ_Osc_2_TuneTimeTabPtr_Hi ; 
                          
                          ldy #$00                        ; 
                          lda (SNZ_Osc_2_TuneTimeTabPtr),y ; 
                          sta TabTune_Osc_2_Time_Lo       ; 
                          
                          lda (SNZ_Osc_2_TuneFreqTabPtr),y ; 
                          sta FRELO2                      ; SID($D407) Oscillator 2 Freq Ctrl (low byte)
                          
                          iny                             ; 
                          
                          lda (SNZ_Osc_2_TuneTimeTabPtr),y ; 
                          sta TabTune_Osc_2_Time_Hi       ; 
                          
                          lda (SNZ_Osc_2_TuneFreqTabPtr),y ; 
                          sta FREHI2                      ; SID($D408) Oscillator 2 Freq Ctrl (high byte)
                          
                          lda #VcRegTRI                   ; ...#....
                          sta VCREG2                      ; SID($D40B) Oscillator 2 Control
                          
                          lda #[VcRegTRI | VcRegGATE]     ; ...#...#
                          sta TabTune_Osc_2_Ctrl          ; 
                          sta VCREG2                      ; SID($D40B) Oscillator 2 Control
                          
                          lda #VcRegTRI                   ; ...#....
                          sta VCREG2                      ; SID($D40B) Oscillator 2 Control
                          
GameTuneStartIniOsc_02X   rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitIniSpriteSunsoft    Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitIniSpriteSunsoft      subroutine                      ; 
                          lda #$fa                        ; 
                          sta TabSprtSunsoftPosY          ; 
                          sta TabSprtSunsoftNum           ; 
                          
                          lda #$0f                        ; ....####
                          sta YXPAND                      ; VIC($D017) Sprite Scale Double Width
                          sta XXPAND                      ; VIC($D01D) Sprite Double Height
                          sta SPMC                        ; VIC($D01C) Sprite Multicolor
                          
                          lda #RED                        ; 
                          sta SP0COL                      ; VIC($D027) Color Sprite 0
                          sta SP1COL                      ; VIC($D028) Color Sprite 1
                          sta SP2COL                      ; VIC($D029) Color Sprite 2
                          sta SP3COL                      ; VIC($D02A) Color Sprite 3
                          
                          lda #BLUE                       ; 
                          sta SPMC0                       ; VIC($D025) Sprite Multicolor Register 0
                          
                          lda #YELLOW                     ; 
                          sta SPMC1                       ; VIC($D026) Sprite Multicolor Register 1
                          
                          lda #$56                        ; 
                          sta SP0X                        ; VIC($D000) Sprite 0 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          lda #$86                        ; 
                          sta SP1X                        ; VIC($D002) Sprite 1 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          lda #$b6                        ; 
                          sta SP2X                        ; VIC($D004) Sprite 2 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          lda #$e6                        ; 
                          sta SP3X                        ; VIC($D006) Sprite 3 X-Pos (Bits 0-7 - Bit 8 in MSIGX = $D010)
                          
                          lda #$fa                        ; 
                          sta SP0Y                        ; VIC($D001) Sprite 0 Y-Pos (Bits 0-7)
                          sta SP1Y                        ; VIC($D003) Sprite 1 Y-Pos (Bits 0-7)
                          sta SP2Y                        ; VIC($D005) Sprite 2 Y-Pos (Bits 0-7)
                          sta SP3Y                        ; VIC($D007) Sprite 3 Y-Pos (Bits 0-7)
                          
                          lda #$0f                        ; ....####
                          sta SPENA                       ; VIC($D015) Sprite Enable Register
                          
InitIniSpriteSunsoftX     rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitMoveSpriteSunsoft   Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitMoveSpriteSunsoft     subroutine                      ; 
.IncSprtSunsoftNum        inc TabSprtSunsoftNum           ; 
.GetSprtSunsoftNum        lda TabSprtSunsoftNum           ; 
.TstSprtSunsoftNum_Max    cmp #$03                        ; 
.ChkSprtSunsoftNum_Max    bcc .SetSprtSunsoftNum          ; lower
                          
.IniSprtSunsoftNum        lda #$00                        ; 
                          
.SetSprtSunsoftNum        sta TabSprtSunsoftNum           ; 
                          
                          asl a                           ; 
                          asl a                           ; 
                          clc                             ; 
                          adc #[SpriteData_Sunsoft / VIC_Sprt_BlockLen] ; 
                          tax                             ; 
                          stx SN_GameSpritePtr_00         ; 
                          inx                             ; 
                          stx SN_GameSpritePtr_01         ; 
                          inx                             ; 
                          stx SN_GameSpritePtr_02         ; 
                          inx                             ; 
                          stx SN_GameSpritePtr_03         ; 
                          
                          lda TabSprtSunsoftPosY          ; 
                          cmp #$64                        ; 
                          bcs .SetSprtSunsoftPosY         ; 
                          
.Exit_NoMove              rts                             ; 
                          
.SetSprtSunsoftPosY       dec SP0Y                        ; VIC($D001) Sprite 0 Y-Pos (Bits 0-7)
                          dec SP1Y                        ; VIC($D003) Sprite 1 Y-Pos (Bits 0-7)
                          dec SP2Y                        ; VIC($D005) Sprite 2 Y-Pos (Bits 0-7)
                          dec SP3Y                        ; VIC($D007) Sprite 3 Y-Pos (Bits 0-7)
                          
InitMoveSpriteSunsoftX    rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
; InitBlinkTextPresents   Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
InitBlinkTextPresents     subroutine                      ; 
.GetSprtChkPosYCount      ldx #$00                        ; 
.GetNextSprtChkPosY       lda TabPresentsTextPosY,x       ; 
                          cmp TabSprtSunsoftPosY          ; 
                          beq .GetByteCount_Text          ; 
                          
                          lda TabPresentsBlankPosY,x      ; 
                          cmp TabSprtSunsoftPosY          ; 
                          beq .GetByteCount_Blank         ; 
                          
.IncSprtChkPosYCount      inx                             ; 
.ChkSprtChkPosYCount_Max  cpx #[TabPresentsTextPosY_Len + $01] ; 
                          bcc .GetNextSprtChkPosY         ; lower
                          
.Exit_Ready               rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetByteCount_Text        ldy #TxtInfoScrn_Presents_Len   ; 
.GetNextTextByte          lda TxtInfoScrn_Presents,y      ; 
                          and #$bf                        ; #.######
                          sta [SN_Scrn_Game + [SN_Scrn_RowLen * $0e] + $0b],y ; 
.DecByteCount_Text        dey                             ; 
.ChkByteCount_Text_Min    bne .GetNextTextByte            ; 
                          
.Exit_Text                rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
.GetByteCount_Blank       ldy #TxtInfoScrn_Presents_Len   ; 
.GetByte_Blank            lda #" "                        ; 
.SetNextBlankByte         sta [SN_Scrn_Game + [SN_Scrn_RowLen * $0e] + $0b],y ; 
.DecByteCount_Blank       dey                             ; 
.ChkByteCount_Blank_Min   bne .SetNextBlankByte           ; 
                          
InitBlinkTextPresentsX    rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabSprtSunsoftNum         dc.b $00 ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabPresentsTextPosY       equ  *   ; 
                          dc.b $0f ; 
                          dc.b $23 ; 
                          dc.b $37 ; 
                          dc.b $4b ; 
                          dc.b $5f ; 
TabPresentsTextPosY_Len   equ  [* - TabPresentsTextPosY - $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabPresentsBlankPosY      equ  *   ; 
                          dc.b $03 ; 
                          dc.b $17 ; 
                          dc.b $2b ; 
                          dc.b $3f ; 
; -------------------------------------------------------------------------------------------------------------- ;
TxtInfoScrn_Presents      equ  *   ; 
                          dc.b $53 ; 
                          dc.b $2a ; *
                          dc.b $20 ; <space>
                          dc.b $50 ; p
                          dc.b $52 ; r
                          dc.b $45 ; e
                          dc.b $53 ; s
                          dc.b $45 ; e
                          dc.b $4e ; n
                          dc.b $54 ; t
                          dc.b $53 ; s
                          dc.b $20 ; <space>
                          dc.b $2a ; *
TxtInfoScrn_Presents_Len  equ  [* - TxtInfoScrn_Presents - $01] ; 
; -------------------------------------------------------------------------------------------------------------- ;
TuneStart                 include inc\SN_TuneStart.asm          ; Start Fanfare
TuneFinish                include inc\SN_TuneFinish.asm         ; Final Blues
; -------------------------------------------------------------------------------------------------------------- ;
; GameHiScoreFileScratch  Does    : 
;                         Expects : 
;                         Returns : 
; -------------------------------------------------------------------------------------------------------------- ;
GameHiScoreFileScratch    subroutine                      ; 
                          lda #$00                        ; 
                          jsr SETNAM                      ; Kernal($FFBD) Set filename parameters ($FDF9)
                          
                          lda #$0f                        ; set logical file number - command channel
                          ldx #$08                        ; set device number
                          ldy #$0f                        ; set secondary command number
                          jsr SETLFS                      ; Kernal($FFBA) Set logical file parameters ($FE00)
                          jsr OPEN                        ; Kernal($FFC0) Open a logical file (via $31A to $F34A)
                          
                          lda #$08                        ; set device number
                          jsr LISTEN                      ; Kernal($FFB1) Serial bus device to LISTEN ($ED0C)
                          
                          ldx #$0f                        ; command channel
                          jsr CHKOUT                      ; Kernal($FFC9) Define an output channel (via $320 to $F250)
                          
                          ldx #$00                        ; 
.GetNextDiskCmdByte       lda TabDiskCmd_Scratch,x        ; 
.ChkEndOfDiskCmd          beq .ClearChannels              ; 
                          
                          jsr CHROUT                      ; Kernal($FFD2) Output a character (via $326 to $F1CA)
                          
                          inx                             ; 
                          bne .GetNextDiskCmdByte         ; 
                          
.ClearChannels            jsr CLRCHN                      ; Kernal($FFCC) Restore default devices (via $322 to $F333)
                          
                          lda #$0f                        ; command channel
.CloseChannel_DiskCmd     jsr CLOSE                       ; Kernal($FFC3) Close a logical file (via $31C to $F291)
                          
GameHiScoreFileScratchX   rts                             ; 
; -------------------------------------------------------------------------------------------------------------- ;
TabDiskCmd_Scratch        equ  *   ; 
                          dc.b $53 ; s
                          dc.b $30 ; 0
                          dc.b $3a ; :
                          dc.b $48 ; h
                          dc.b $49 ; i
                          dc.b $47 ; g
                          dc.b $48 ; h
                          dc.b $20 ; <space>
                          dc.b $53 ; s
                          dc.b $43 ; c
                          dc.b $4f ; o
                          dc.b $52 ; r
                          dc.b $45 ; e
                          dc.b $53 ; s
TabDiskCmd_ScratchX       dc.b $00 ; flag: end of command
; -------------------------------------------------------------------------------------------------------------- ;
; unused
; -------------------------------------------------------------------------------------------------------------- ;
A_9fc0                    dc.b $00 ; ........
                          dc.b $03 ; ......##
                          dc.b $cb ; ##..#.##
                          dc.b $fe ; #######.
                          dc.b $54 ; .#.#.#..
                          dc.b $40 ; .#......
                          dc.b $40 ; .#......
                          dc.b $c8 ; ##..#...
                          dc.b $cc ; ##..##..
                          dc.b $88 ; #...#...
                          dc.b $18 ; ...##...
                          dc.b $40 ; .#......
                          dc.b $40 ; .#......
                          dc.b $80 ; #.......
                          dc.b $84 ; #....#..
                          dc.b $2e ; ..#.###.
A_9fd0                    dc.b $bc ; #.####..
                          dc.b $dc ; ##.###..
                          dc.b $08 ; ....#...
                          dc.b $4c ; .#..##..
                          dc.b $a4 ; #.#..#..
                          dc.b $f9 ; #####..#
                          dc.b $88 ; #...#...
                          dc.b $d0 ; ##.#....
                          dc.b $80 ; #.......
                          dc.b $dc ; ##.###..
                          dc.b $c8 ; ##..#...
                          dc.b $2a ; ..#.#.#.
                          dc.b $80 ; #.......
                          dc.b $44 ; .#...#..
                          dc.b $00 ; ........
                          dc.b $d6 ; ##.#.##.
A_9fe0                    dc.b $ff ; ########
                          dc.b $c8 ; ##..#...
                          dc.b $cc ; ##..##..
                          dc.b $40 ; .#......
                          dc.b $48 ; .#..#...
                          dc.b $00 ; ........
                          dc.b $84 ; #....#..
                          dc.b $08 ; ....#...
                          dc.b $84 ; #....#..
                          dc.b $dc ; ##.###..
                          dc.b $00 ; ........
                          dc.b $98 ; #..##...
                          dc.b $dc ; ##.###..
                          dc.b $cc ; ##..##..
                          dc.b $08 ; ....#...
                          dc.b $00 ; ........
A_9ff0                    dc.b $40 ; .#......
                          dc.b $80 ; #.......
                          dc.b $11 ; ...#...#
                          dc.b $80 ; #.......
                          dc.b $a9 ; #.#.#..#
                          dc.b $dd ; ##.###.#
                          dc.b $85 ; #....#.#
                          dc.b $41 ; .#.....#
                          dc.b $54 ; .#.#.#..
                          dc.b $53 ; .#.#..##
                          dc.b $4e ; .#..###.
                          dc.b $4f ; .#..####
                          dc.b $4b ; .#..#.##
                          dc.b $49 ; .#..#..#
                          dc.b $45 ; .#...#.#
A_9fff                    dc.b $ff ; ########
; -------------------------------------------------------------------------------------------------------------- ;
